<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009 - 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package components/ProjectPlanningCharter
 */

/**
 * ProjectPlanningCharter provides displaying of project planning charter component
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Leos Navratil <leos.navratil@honeywell.com>
 */
class ProjectPlanningCharterControl extends Control
{
	/** @var Project */
    private $project;
	
    /** @var int year */
    private $year;
	
	// year range used to zero Workable Hours for a new TG Person
	const WH_START_YEAR = 2010;
	const WH_END_YEAR = 2026;
	
	private $batch_id;
	
    /**
     * @param Project $project
     * @param int $year
     */
    public function __construct($project, $year)
    {
		$this->batch_id = time().'-'. rand(1, 9);
		Debugger::log($this->batch_id . "*** PPM - Constructor start - " . Environment::getUser()->getId(), Debugger::INFO);
        parent::__construct();
		$this->project = $project;
		$this->year = $year;
		Debugger::log($this->batch_id . "*** PPM - Constructor end - " . Environment::getUser()->getId(), Debugger::INFO);
    }


	public function createComponentProjectPlanningForm($name)
	{
		Debugger::log($this->batch_id . "*** PPM - reateComponentProjectPlanningForm start - " . Environment::getUser()->getId(), Debugger::INFO);
		if (!$this->presenter->isAjax() || isset($this->presenter->payload->loadPlanningDBFromProject))
		{
			// we aren't just editing our temporary planning, so
			// synchronize with this project's planning tasks
			$this->loadPlanningDBFromProject();
		}
		
		$form = new AppForm($this, $name);
		$form->getElementPrototype()->id = 'ProjectPlanningForm';
		
		// Adding new task/WP tables
		$form->addText('addTaskOrWPName', '');
		$form->addButton('addTaskOrWP', 'Add');

		
		// Tasks/WPs
		$tasksWPs = $this->presenter->context->projectPlanningCharterRepository->findByProjectIDAndYearAndType($this->project->ID, $this->year, ProjectPlanningCharter::PROJECT_PLANNING_CHARTER_TYPE_HRS);
		
		foreach ($tasksWPs as $projectPlanningCharter)
		{
			// User specified name for the group of people and OrgUnits
			$form->addText('taskWPLabel' . $projectPlanningCharter->ID)
					->setDefaultValue($projectPlanningCharter->Name);

			// First row - task/WP overview
			$form->addButton('deleteTaskWPTable' . $projectPlanningCharter->ID, 'Delete')
					->setAttribute('class', 'deleteTaskWPTable');
			
			// People and Organization Units
			$people = $this->presenter->context->projectPlanningCharterItemRepository->
					findByProjectPlanningCharterIDAndType($projectPlanningCharter->ID, ProjectPlanningCharterItem::PROJECT_PLANNING_CHARTER_ITEM_TYPE_PERSON);
			$orgUnits = $this->presenter->context->projectPlanningCharterItemRepository->
					findByProjectPlanningCharterIDAndType($projectPlanningCharter->ID, ProjectPlanningCharterItem::PROJECT_PLANNING_CHARTER_ITEM_TYPE_ORGANIZATION_UNIT);
			
			$peopleOrgUnits = array_merge($people, $orgUnits);
			
			foreach ($peopleOrgUnits as $item)
			{
				for ($m = 1; $m < 13; $m++)
				{
					// taskWPBudget . [Task/WP table ID] (= ProjectPlanningCharter ID) . i . [Person/OrgUnit ID] (= ProjectPlanningCharterItem ID) . m . [Month index]
					$form->addText('taskWPBudget' . $projectPlanningCharter->ID . 'i' . $item->ID . 'm' . $m, NULL, 5)
							->setDefaultValue(0)
							->addRule(FORM::RANGE, 'Budget must contain positive integer.', array(0, NULL))
							->addRule(FORM::PATTERN, 'Budget must contain positive integer.', '[0-9]*');
				}
				
				$form->addButton('deleteTaskWP' . $item->ID, 'Delete')
						->setAttribute('class', 'deleteTaskWP');
			}

			$form->addText('taskWPCostShare' . $projectPlanningCharter->ID, NULL, 5)
						->setDefaultValue(0)
						->addRule(FORM::RANGE, 'Cost share must contain positive integer.', array(0, 100))
						->addRule(FORM::PATTERN, 'Cost share must contain positive integer.', '[0-9]*');
			
			// Adding new workers/organization units to the task/WP table
			$form->addRadioList('assignmentType' . $projectPlanningCharter->ID, 'Assignment Type', array(0 => 'Worker', 1 => 'Technical Group'))
					->setDefaultValue(0)
					->getSeparatorPrototype()->setName('');
			
			// toggled selectboxes
			$form->addSelect('personSelect' . $projectPlanningCharter->ID, 'People')
					->setAttribute('style', 'width: 200px;')
					->setPrompt('*');
			
			// org unit select box with filtering capabilities
			$orgUnits = array();

			$techGroups = $this->presenter->context->organizationUnitRepository->findAllOrgUnitByType('TG');
			foreach ($techGroups as $techGroup)
			{
				$SBE = $this->presenter->context->organizationUnitRepository->findFirstAncestor($techGroup->ID);
				$SBEbracket = ($SBE === NULL) ? '' : ' (' . $SBE->Short . ')';
				if ($techGroup->Status === TRUE)
					$orgUnits[$techGroup->ID] = $techGroup->Name . $SBEbracket;
			}
			
			// don't allow addition of an Org Unit, which is already added to this Task/WP table
			$alreadyUsedOrgUnits = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterIDAndType($projectPlanningCharter->ID, ProjectPlanningCharterItem::PROJECT_PLANNING_CHARTER_ITEM_TYPE_ORGANIZATION_UNIT);
			
			foreach ($alreadyUsedOrgUnits as $projectPlanningCharterItem)
			{
				$orgUnit = $this->presenter->context->organizationUnitRepository->find($projectPlanningCharterItem->organizationUnitID);
				unset($orgUnits[$orgUnit->ID]);
			}
			
			$form->addSelect('orgUnitSelect' . $projectPlanningCharter->ID, 'Technical Groups', $orgUnits)
					->setAttribute('style', 'width: 200px;')
					->setPrompt('*');

			$form->addButton('addWorkerOrgUnit' . $projectPlanningCharter->ID, 'Add')
					->setAttribute('class', 'addWorkerOrgUnit');
		}

		// Subcontractors
		$subcontractors = array();
		$subcontractors = $this->presenter->context->subcontractorRepository->findAll();
		$alreadyUsedSubcontractors = $this->presenter->context->
				projectPlanningCharterRepository->findByProjectIDAndYearAndType($this->project->ID, $this->year, ProjectPlanningCharter::PROJECT_PLANNING_CHARTER_TYPE_SUB);
		
		foreach ($subcontractors as $subcontractor)
		{
			// don't add inactive subcontractors
			if ($subcontractor->Status)
				$subcontractorList[$subcontractor->ID] = $subcontractor->Name;
		}
		
		foreach ($alreadyUsedSubcontractors as $sub)
		{
			$item = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($sub->ID);
			$item = $item[0];
			if (array_key_exists($item->SubcontractorID, $subcontractorList))
				unset($subcontractorList[$item->SubcontractorID]);
		}
		
		$subCount = count($alreadyUsedSubcontractors);

		foreach ($alreadyUsedSubcontractors as $sub)
		{
			for ($j = 1; $j < 13; $j++)
			{
				$form->addText('sub' . $sub->ID . 'm' . $j, NULL, 5)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Subcontractor\'s budget must contain either a positive integer or zero.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Subcontractor\'s budget must contain either a positive integer or zero.', '[0-9]*');
			}
			$form->addText('subCostShare' . $sub->ID, NULL, 5)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Cost share must contain positive integer.', array(0, 100))
					->addRule(FORM::PATTERN, 'Cost share must contain positive integer.', '[0-9]*');
			
			$form->addButton('deleteSub' . $sub->ID, 'Delete')
					->setAttribute('class', 'deleteSub');
		}
		
		// Add Subcontractor line (row)
		$form->addSelect('selectSubcontractors', 'Subcontractors', $subcontractorList);
		$form->addButton('addSubcontractor', 'Add')
				->setAttribute('class', 'addSubcontractor');
		
		
		// MATERIAL & TRAVEL
		for ($i = 1; $i < 13; $i++)
		{
			$form->addText('mat' . $i, NULL, 5)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Material budget must contain either a positive integer or zero.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Material budget must contain either a positive integer or zero.', '[0-9]*');
			
			$form->addText('trv' . $i,  NULL, 5)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Travel budget must contain either a positive integer or zero.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Travel budget must contain either a positive integer or zero.', '[0-9]*');
		}
		
		$form->addText('matCostShare', NULL, 5)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Cost share must contain positive integer.', array(0, 100))
					->addRule(FORM::PATTERN, 'Cost share must contain positive integer.', '[0-9]*');
		
		$form->addText('trvCostShare', NULL, 5)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Cost share must contain positive integer.', array(0, 100))
					->addRule(FORM::PATTERN, 'Cost share must contain positive integer.', '[0-9]*');
		
		$form->addButton('create', 'Create Charter')
				->setAttribute('title', '')
				->setAttribute('class', 'create');
		
		$form->addButton('update', 'Update Charter')
				->setAttribute('title', '')
				->setAttribute('class', 'update');
		
		$form->addButton('discard', 'Discard Changes')
				->setAttribute('title', '')
				->setAttribute('class', 'discard');
		
		$form->addButton('saveAll', 'Save Project Plan')
				->setAttribute('title', 'Creates planning tasks under this project.')
				->setAttribute('class', 'saveAll');
		
		$form->addButton('finish', 'Finish Planning')
				->setAttribute('title', '')
				->setAttribute('class', 'finish');
				//->setAttribute('onlick', 'alert("Do you really want to finish this planning?");');
		
		$form->setDefaults(array('assignmentType' => 'Worker'));
		
		// set up buttons
		$planningState = $this->presenter->context->projectPlanningCharterStateRepository
			->find($this->project->ID, $this->year);
		
		if ($planningState !== NULL)
		{
			if ($planningState->State === ProjectPlanningCharterState::PROJECT_PLANNING_CHARTER_STATE_CHARTEREXISTS)
			{
				$form['create']->setDisabled(TRUE);
			}
			else
			{
				$form['update']->setDisabled(TRUE);
			}
		}
		
		// only for PMO
		if (!Environment::getUser()->isInRole('PMO'))
			$form['finish']->setDisabled(TRUE);
		
		$form->onSuccess[] = array($this, 'autosave');
		Debugger::log($this->batch_id . "*** PPM - reateComponentProjectPlanningForm end - " . Environment::getUser()->getId(), Debugger::INFO);
		return $form;
	}
	
	
    /**
     * Shows project planning charter
     */
    public function renderDefault()
    {
		if ($this->presenter->isAjax()) // is ajax request
		{
			Debugger::log($this->batch_id . "*** PPM - render default start - " . Environment::getUser()->getId(), Debugger::INFO);
			$this->flashMessage('To improve performance automatic recalculation has been disabled. Press recalculate button or save your changes to update values.', 'warning');
			$planningState = $this->presenter->context->projectPlanningCharterStateRepository
				->find($this->project->ID, $this->year);

			$template = $this->template;
			$template->year = $this->year;

			if ($planningState !== NULL && $planningState->State !== ProjectPlanningCharterState::PROJECT_PLANNING_CHARTER_STATE_FINISHED)
			{
				$form = $this['projectPlanningForm'];
				$template->planningForThisYear = TRUE;

				$template->months = array('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');

				$planningTables = $this->presenter->context->projectPlanningCharterRepository->findByProjectIDAndYear($this->project->ID, $this->year);
				if (count($planningTables) < 1)
				{
					$this->flashMessage('It seems someone else is editing planning for this project and year at the moment. Please, try again later.', 'error');
					$template->planningForThisYear = FALSE;
					$template->setFile(__DIR__ . '/ProjectPlanningCharterControl.latte');
					$template->render();
					return;
				}
				// mandatory piece of information, those will always render into <table></table>
				$mat = $trv = $template->mat = $template->trv = array();
				// option information (yet important), these could be deleted by user,
				// count($template->hrs) will be used to determine if we have
				// any data to display
				$template->hrs = $template->sub = $template->subTotals = array();

				$template->summaryTotal = $template->summaryRDE = array();

				// $table is a 'ProjectPlanningCharter' record,
				// which falls to one of these groups:
				// Tasks or WPs | Subcontractors | MAT | TRV
				foreach ($planningTables as $table)
				{
					switch ($table->Type)
					{
						case 'HRS':
							$newTaskWP = array();
							// name of the whole Task/WP <table></table>
							$newTaskWP['name'] = $table->Name;

							// Organization Units or Workers
							$items = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($table->ID);
							$subItems = array();
							foreach ($items as $item)
							{
								$newOrgUnitOrPerson = array();
								$newOrgUnitOrPerson['id'] = $item->ID;
								$newOrgUnitOrPerson['type'] = $item->Type;
								$newOrgUnitOrPerson['orgUnit'] = $item->OrganizationUnitID;
								$newOrgUnitOrPerson['person'] = $item->PersonID;
								$newOrgUnitOrPerson['costShare'] = $item->CostShare * 100;
								$newOrgUnitOrPerson['budget'] = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($item->ID);
								$SBE = $this->presenter->context->organizationUnitRepository->findFirstAncestor($item->OrganizationUnitID);
								$SBEbracket = ($SBE === NULL) ? '' : ' (' . $SBE->Short . ')';
								$newOrgUnitOrPerson['name'] = $item->OrganizationUnitID !== NULL ?
									$this->presenter->context->organizationUnitRepository->find($item->OrganizationUnitID)->Name . $SBEbracket :
									$this->presenter->context->personRepository->find($item->PersonID)->ReversedFullNameWithEID;
								$subItems[] = $newOrgUnitOrPerson;
							}

							$newTaskWP['subItems'] = $subItems;
							$template->hrs[$table->ID] = $newTaskWP;
							break;

						case 'SUB':
							$item = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($table->ID);
							$item = $item[0];

							$newSubcontractor = array();
							$newSubcontractor['name'] = $this->presenter->context->subcontractorRepository->find($item->subcontractorID)->Name;
							$newSubcontractor['costShare'] = $item->costShare * 100;
							$newSubcontractor['budget'] = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($item->ID);
							$newSubcontractor['budgetInTotal'] = 0;
							$template->sub[$table->ID] = $newSubcontractor;
							break;

						case 'MAT':
							$item = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($table->ID);
							$item = $item[0];
							$template->mat['costShare'] = $item->costShare * 100;
							$mat['budget'] = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($item->ID);
							$template->mat['budgetInTotal'] = 0;
							break;

						case 'TRV':
							$item = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($table->ID);
							$item = $item[0];
							$template->trv['costShare'] = $item->costShare * 100;
							$trv['budget'] = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($item->ID);
							$template->trv['budgetInTotal'] = 0;
							break;
					}

				}

				// fill all budgets with real data
				for ($i = 1; $i < 13; $i++)
				{
					// Tasks/WPs
					foreach ($template->hrs as $projectPlanningCharterID => $table)
					{
						foreach ($table['subItems'] as $subItem)
						{
							$form['taskWPBudget' . $projectPlanningCharterID . 'i' . $subItem['id'] . 'm' . $i]->setValue($subItem['budget'][$i]->budget);
						}
					}

					// Subcontractors
					foreach ($template->sub as $subID => $sub)
					{
						$form['sub' . $subID . 'm' . $i]->setValue($sub['budget'][$i]->budget);
					}

					// MAT & TRV
					if (isset($mat['budget']))
						$form['mat' . $i]->setValue($mat['budget'][$i]->budget);
					if (isset($trv['budget']))
						$form['trv' . $i]->setValue($trv['budget'][$i]->budget);
				}

				// Task/WP Cost Share
				foreach ($template->hrs as $taskWPTableID => $taskWPTable)
				{
					$firstSubItem = 0.0;
					// get the first element of the array without using reset()
					foreach ($taskWPTable['subItems'] as $subItem)
					{
						$firstSubItem = $subItem['costShare'];
						break;
					}
					$form['taskWPCostShare' . $taskWPTableID]->setValue($firstSubItem);
				}
				// Subcontrator Cost Share
				foreach ($template->sub as $subID => $sub)
				{
					$form['subCostShare' . $subID]->setValue($sub['costShare']);
				}

				$form['matCostShare']->setValue($template->mat['costShare']);
				$form['trvCostShare']->setValue($template->trv['costShare']);

				for ($i = 1; $i < 13; $i++)
				{
					$template->mat['budgetInTotal'] += $mat['budget'][$i]->budget;
					$template->trv['budgetInTotal'] += $trv['budget'][$i]->budget;
				}

				// also calculate total month values for each month and the total
				// of totalsHRS
				foreach ($template->hrs as $projectPlanningCharterID => $table)
				{
					// init
					$template->hrs[$projectPlanningCharterID]['totalsHRS'] = array();
					$template->hrs[$projectPlanningCharterID]['totalOfTotalsHRS'] = 0;
					$template->hrs[$projectPlanningCharterID]['totalOfTotalsUSD'] = 0;
					$template->hrs[$projectPlanningCharterID]['totalsUSDColumn'] = array();

					for ($i = 1; $i < 13; $i++)
						$template->hrs[$projectPlanningCharterID]['totalsHRS'][$i] = 0;


					// people and org units
					foreach ($table['subItems'] as $subItem)
					{
						$template->hrs[$projectPlanningCharterID]['totalHRSColumn'][$subItem['id']] = 0;
						$template->hrs[$projectPlanningCharterID][$subItem['id']]['labourRate'] = $this->calculateLabourRate($subItem);

						foreach ($subItem['budget'] as $monthNumber => $budget)
						{
							$template->hrs[$projectPlanningCharterID]['totalHRSColumn'][$subItem['id']] += $form['taskWPBudget' . $projectPlanningCharterID . 'i' . $subItem['id'] . 'm' . $monthNumber]->getValue($budget->budget);
							$template->hrs[$projectPlanningCharterID]['totalsHRS'][$monthNumber] += $budget->budget;
						}
						// for every row, 'Total (USD)' equals 'Total (HRS)' * labor rate
						$template->hrs[$projectPlanningCharterID]['totalsUSDColumn'][$subItem['id']] = $template->hrs[$projectPlanningCharterID]['totalHRSColumn'][$subItem['id']] * $template->hrs[$projectPlanningCharterID][$subItem['id']]['labourRate'];
						$template->hrs[$projectPlanningCharterID]['totalOfTotalsUSD'] += $template->hrs[$projectPlanningCharterID]['totalsUSDColumn'][$subItem['id']];
					}

					for ($i = 1; $i < 13; $i++)
						$template->hrs[$projectPlanningCharterID]['totalOfTotalsHRS'] += $template->hrs[$projectPlanningCharterID]['totalsHRS'][$i];
				}

				/* Subcontractor Totals */
				// init
				for ($i = 1; $i < 13; $i++)
				{
					$template->subTotals[$i] = 0;
				}

				// Subcontractor totals - calculate the sum
				$weightsMultipliedByValues = 0;
				$sumOfWeights = 0;
				foreach ($template->sub as $subID => $sub)
				{
					for ($i = 1; $i < 13; $i++)
					{
						$template->sub[$subID]['budgetInTotal'] += $sub['budget'][$i]->budget;
						$template->subTotals[$i] += $sub['budget'][$i]->budget;
					}
					$weightsMultipliedByValues += $template->sub[$subID]['budgetInTotal'] * $sub['costShare'];
					$sumOfWeights +=  $template->sub[$subID]['budgetInTotal'];
				}

				$template->subTotalOfTotals = 0;
				for ($i = 1; $i < 13; $i++)
				{
					$template->subTotalOfTotals += $template->subTotals[$i];
				}

				$template->subAverageCostShare = 0;
				if ($sumOfWeights != 0)
					$template->subAverageCostShare = round($weightsMultipliedByValues / $sumOfWeights, 2);
				/* Subcontractor Totals - END */

				/* Summary Total of everything & RDE */
				// init
				for ($i = 1; $i < 13; $i++)
				{
					$template->summaryTotal[$i] = 0;
					$template->summaryRDE[$i] = 0;
				}
				$template->summaryTotal['USD'] = 0;
				$template->summaryRDE['USD'] = 0;
				$template->summaryTotal['costShare'] = 0;

				// calculation part
				for ($i = 1; $i < 13; $i++)
				{
					// Task/WPs
					foreach ($template->hrs as $projectPlanningCharterID => $table)
					{
						// people and org units
						foreach ($table['subItems'] as $subItem)
						{
							$template->summaryTotal[$i] += $subItem['budget'][$i]->budget * $template->hrs[$projectPlanningCharterID][$subItem['id']]['labourRate'];
							$template->summaryRDE[$i] += $subItem['budget'][$i]->budget * $template->hrs[$projectPlanningCharterID][$subItem['id']]['labourRate'] * (1 - $subItem['costShare'] / 100);
						}
					}

					// Subcontractors
					$template->summaryTotal[$i] += $template->subTotals[$i];
					foreach ($template->sub as $subcontractor)
					{
						$template->summaryRDE[$i] += $subcontractor['budget'][$i]->budget * (1 - $subcontractor['costShare'] / 100);
					}

					// MAT & TRV
					$template->summaryTotal[$i] += $mat['budget'][$i]->budget;
					$template->summaryTotal[$i] += $trv['budget'][$i]->budget;

					$template->summaryRDE[$i] += $mat['budget'][$i]->budget * (1 - $template->mat['costShare'] / 100);
					$template->summaryRDE[$i] += $trv['budget'][$i]->budget * (1 - $template->trv['costShare'] / 100);
				}

				foreach ($template->hrs as $projectPlanningCharterID => $table)
				{
					$template->summaryTotal['USD'] += $template->hrs[$projectPlanningCharterID]['totalOfTotalsUSD'];

					// people and org units
					foreach ($table['subItems'] as $subItem)
					{
						$template->summaryTotal['costShare'] += $table['totalsUSDColumn'][$subItem['id']] * $subItem['costShare'];
					}
				}

				foreach ($template->sub as $subcontractor)
				{
					$template->summaryTotal['costShare'] += $subcontractor['budgetInTotal'] * $subcontractor['costShare'];
				}

				$template->summaryTotal['costShare'] += $template->mat['budgetInTotal'] * $template->mat['costShare'];
				$template->summaryTotal['costShare'] += $template->trv['budgetInTotal'] * $template->trv['costShare'];


				$template->summaryTotal['USD'] += $template->subTotalOfTotals;
				$template->summaryTotal['USD'] += $template->mat['budgetInTotal'];
				$template->summaryTotal['USD'] += $template->trv['budgetInTotal'];

				if ($template->summaryTotal['USD'] != 0)
					$template->summaryTotal['costShare'] = $template->summaryTotal['costShare'] / $template->summaryTotal['USD'];

				for ($i = 1; $i < 13; $i++)
				{
					$template->summaryRDE['USD'] += $template->summaryRDE[$i];
				}

				/* Summary Total of everything & RDE - END */
			}
			else
			{
				$template->planningForThisYear = FALSE;
			}
			Debugger::log($this->batch_id . "*** PPM - render default end - " . Environment::getUser()->getId(), Debugger::INFO);
			$template->setFile(__DIR__ . '/ProjectPlanningCharterControl.latte');
			$template->render();
		}
    }
	
	/**
	 * Function prepares labour rate for a person / an org unit.
	 * 
	 * @param array $item A person or an Organization unit.
	 * @return int Labour Rate.
	 */
	private function calculateLabourRate($item)
	{
		Debugger::log($this->batch_id . "*** PPM - calculate labour rate start - " . Environment::getUser()->getId(), Debugger::INFO);
		$result = 0;
		
		if ($item['type'] == 'Person')
		{
			$person = $this->presenter->context->personRepository->find($item['person']);
			
			// inactive person
			if (!$person->Status)
				return 0;

			$workCenter = $this->presenter->context->costCenterRepository->findCostCenter($person->CostCenterID);

			$laborRate = $workCenter->getYearLabourRate($this->year);

			for ($i = 1; $i < 13; $i++)
			{
				$result += $laborRate->getMonthlyHourRate($i);
			}

			$result /= 12;

		}
		else // org unit
		{
			$planningHelper = new PlanningHelper($this->presenter->context, $this->year);
			$result = $planningHelper->getOrgUnitLabourRate($item['orgUnit']);
			$result = $result['LabourRate'];
		}
		Debugger::log($this->batch_id . "*** PPM - calculate labour rate end - " . Environment::getUser()->getId(), Debugger::INFO);
		return $result;
	}
	
	/**
	 * Saves new task or work package with its name into the DB.
	 * 
	 * @param string $name The name of the Task/WP to be created.
	 */
	public function handleAddNewTaskWP($name)
	{
		Debugger::log($this->batch_id . "*** PPM - handle Add New Task WP start - " . Environment::getUser()->getId(), Debugger::INFO);
		if ($this->presenter->isAjax())
		{
			// init planning
			// Labour Task/WP
			$planningCharter = new ProjectPlanningCharter();
			$planningCharter->projectID = $this->project->ID;
			$planningCharter->year = $this->year;
			$planningCharter->name = $name;
			$planningCharter->type = 'HRS';

			$this->presenter->context->projectPlanningCharterRepository->save($planningCharter);
			
			$this->presenter->invalidateControl('planning');
		}
		Debugger::log($this->batch_id . "*** PPM - handle Add New Task WP end - " . Environment::getUser()->getId(), Debugger::INFO);
	}
	
	/**
	 * Adds new Person or Organization Unit to its Task/WP table.
	 * 
	 * @param int $tableID ID of the Task/WP table.
	 * @param bool $orgUnitFlag Determines if we're saving a person, or
	 * and org unit.
	 * @param int $IDToBeAdded ID of a Person or TG to be added.
	 */
	public function handleAddPersonOrOrgUnit($tableID, $orgUnitFlag, $IDToBeAdded)
	{
		Debugger::log($this->batch_id . "*** PPM - handle Add Person Or Org Unit start - " . Environment::getUser()->getId(), Debugger::INFO);
		if ($this->presenter->isAjax())
		{
			if ($tableID == '' || $IDToBeAdded == '')
			{
				$this->presenter->payload->data = '';
				$this->presenter->terminate();
			}
			
			if ($orgUnitFlag === 'true')
			{
				if ($this->presenter->context->organizationUnitRepository->find($IDToBeAdded) === NULL)
				{
					$this->presenter->payload->data = '';
					$this->presenter->terminate();
				}
				
				// add an org unit
				$newItem = new ProjectPlanningCharterItem;
				$newItem->OrganizationUnitID = $IDToBeAdded;
				$newItem->ProjectPlanningCharterID = $tableID;
				$newItem->Type = 'OrganizationUnit';
			}
			else
			{
				if ($this->presenter->context->personRepository->find($IDToBeAdded) === NULL)
				{
					$this->presenter->payload->data = '';
					$this->presenter->terminate();
				}
				
				// add a person
				$newItem = new ProjectPlanningCharterItem;
				$newItem->PersonID = $IDToBeAdded;
				$newItem->ProjectPlanningCharterID = $tableID;
				$newItem->Type = 'Person';
			}
			
			// ensure that the new person isn't already assigned in this Task/WP
			$projectPlanningCharter = $this->presenter->context->projectPlanningCharterRepository->find($tableID);
			
			if ($orgUnitFlag === 'true')
				$planningCharterItems = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterIDAndType($projectPlanningCharter->ID, 'OrganizationUnit');
			else
				$planningCharterItems = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterIDAndType($projectPlanningCharter->ID, 'Person');

			foreach ($planningCharterItems as $alreadyUsedItem)
			{
				if ($orgUnitFlag === 'true')
				{
					if ($IDToBeAdded == $alreadyUsedItem->OrganizationUnitID)
					{
						$this->flashMessage('This Item has already been added to the planning.', 'error');
						$this->presenter->invalidateControl('planning');
						return;
					}
				}
				else
				{
					if ($IDToBeAdded == $alreadyUsedItem->PersonID)
					{
						$this->flashMessage('This Item has already been added to the planning.', 'error');
						$this->presenter->invalidateControl('planning');
						return;
					}
				}
			}
			
			$this->presenter->context->projectPlanningCharterItemRepository->save($newItem);

			// link new budget
			for ($i = 1; $i < 13; $i++)
			{
				$budget = new ProjectPlanningCharterItemBudget();
				$budget->ProjectPlanningCharterItemID = $newItem->ID;
				$budget->Month = $i;
				$budget->Budget = 0;
				$budget->BudgetType = 'HRS';
				$this->presenter->context->projectPlanningCharterItemBudgetRepository->save($budget);
			}

			$this->presenter->invalidateControl('planning');
		}
		Debugger::log($this->batch_id . "*** PPM - handle Add Person Or Org Unit end - " . Environment::getUser()->getId(), Debugger::INFO);
	}
	
	/**
	 * Deletes Task or Work Package from the ProjectPlanningCharter DB.
	 * 
	 * @param int $id ProjectPlanningCharter ID
	 */
	public function handleDeleteTaskWP($id)
	{
		Debugger::log($this->batch_id . "*** PPM - handle Delete Task WP start - " . Environment::getUser()->getId(), Debugger::INFO);
		if ($this->presenter->isAjax())
		{
			if (!$this->presenter->context->projectPlanningCharterRepository->delete((int) $id))
				$this->flashMessage('Someone already removed this Task / Work Package. '
						. 'Form\'s data were synchronized to the current state.', 'error');
			
			$this->presenter->invalidateControl('planning');
		}
		Debugger::log($this->batch_id . "*** PPM - handle Delete Task WP end - " . Environment::getUser()->getId(), Debugger::INFO);
	}
	
	/**
	 * Deletes a Person or an Organization Unit from 
	 * 
	 * @param int $id ProjectPlanningCharterItem ID
	 */
	public function handleDeletePersonOrgUnit($id)
	{
		Debugger::log($this->batch_id . "*** PPM - handle Delete Person or org unit start - " . Environment::getUser()->getId(), Debugger::INFO);
		if ($this->presenter->isAjax())
		{
			if (!$this->presenter->context->projectPlanningCharterItemRepository->delete((int) $id))
				$this->flashMessage('Someone already removed this Person or Organization Unit. '
						. 'Form\'s data were synchronized to the current state.', 'error');
			
			$this->presenter->invalidateControl('planning');
		}
		Debugger::log($this->batch_id . "*** PPM - handle Delete Person or org unit end - " . Environment::getUser()->getId(), Debugger::INFO);
	}
	
	/**
	 * Adds a Subcontractor to the ProjectPlanningCharter DB.
	 * 
	 * @param int $id Subcontractor ID
	 */
	public function handleAddSubcontractor($id)
	{
		Debugger::log($this->batch_id . "*** PPM - handle add subcontractor start - " . Environment::getUser()->getId(), Debugger::INFO);
		$id = (int)$id;
		if ($this->presenter->isAjax())
		{
			$sub = $this->presenter->context->subcontractorRepository->find($id);
			
			if ($sub !== NULL)
			{
				// the subcontractor exists in our DB
				// we should check if the subcontractor isn't already assigned to this (ProjectID, Year), too
				$projectPlanningCharters = $this->presenter->context->projectPlanningCharterRepository->findByProjectIDAndYearAndType($this->project->ID, $this->year, 'SUB');
				
				foreach ($projectPlanningCharters as $planningCharter)
				{
					$planningCharterItem = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterIDAndType($planningCharter->ID, 'Subcontractor');
					foreach ($planningCharterItem as $alreadyUsedSub)
					{
						if ($sub->ID === $alreadyUsedSub->SubcontractorID)
						{
							$this->flashMessage('This Subcontractor has already been added to the planning.', 'error');
							$this->presenter->invalidateControl('planning');
							return;
						}
					}
				}
				
				$this->saveSubcontractor($sub->ID, $budgetArray = NULL);
			}
			else
			{
				$this->flashMessage('This Subcontractor couldn\'t be found in HON PT.', 'error');
				$this->presenter->invalidateControl('planning');
				return;
			}
			
			$this->presenter->invalidateControl('planning');
		}
		Debugger::log($this->batch_id . "*** PPM - handle add subcontractor end - " . Environment::getUser()->getId(), Debugger::INFO);
	}
	
	/**
	 * Deletes Subcontractor from the ProjectPlanningCharter DB.
	 * 
	 * @param int $id ProjectPlanningCharter ID
	 */
	public function handleDeleteSubcontractor($id)
	{
		Debugger::log($this->batch_id . "*** PPM - handle delete subcontractor start - " . Environment::getUser()->getId(), Debugger::INFO);
		if ($this->presenter->isAjax())
		{
			if (!$this->presenter->context->projectPlanningCharterRepository->delete((int) $id))
				$this->flashMessage('Someone already removed this subcontractor. '
						. 'Form\'s data were synchronized to the current state.', 'error');
			
			$this->presenter->invalidateControl('planning');
		}
		Debugger::log($this->batch_id . "*** PPM - handle delete subcontractor end - " . Environment::getUser()->getId(), Debugger::INFO);
	}
	
	/**
	 * Processes user's request and sends back the list of all people that could be
	 * possibly assigned to this Task/WP table, while it also takes care of the filtering.
	 * 
	 * Person list is a whitelist of people.
	 * 
	 * @param string $input The text that has been typed into search field by user.
	 * @param int $projectPlanningCharterID ID of Task/WP table
	 */
	public function handlePersonList($input, $projectPlanningCharterID)
	{
		Debugger::log($this->batch_id . "*** PPM - handle person list start - " . Environment::getUser()->getId(), Debugger::INFO);
		if ($this->presenter->isAjax())
		{
			// IDs of whitelisted SBE IDs
			// + bendix (ID == 334)
			$orgUnitWhitelist = array(1, 2, 3, 87, 121);
			$whitelistedTGs = array();
			
			// leaders need to be handled separately
			$leaders = $this->presenter->context->personRepository->getLeadersByOrgUnitTypeAndSBE(array('TG'), $orgUnitWhitelist);

			foreach ($orgUnitWhitelist as $id)
			{
				$orgUnit = $this->presenter->context->organizationUnitRepository->findAllChildrenByType($id, 'TG');
				$whitelistedTGs += $orgUnit;
			}
			$whitelistedTGs = array_keys($whitelistedTGs);
			// append Bendix King TG
			$whitelistedTGs[] = 334;
			$whitelistedTGs[] = 375;
			$whitelistedTGs[] = 328;
			
			$text = trim($input);
			if ($text !== '')
			{
				$filter = new PersonFilter();
				$filter->setFirstname($text)->setSurname($text)->setID($text);
				$people = $this->presenter->context->personRepository->getPersonsByOrFilter($filter);
			}
			else
			{
				$people = $this->presenter->context->personRepository->findAll();
			}
			
			// pop people without one of the whitelistedTGs as OrganizationUnitID
			foreach ($people as $person)
				if (!in_array($person->OrganizationUnitID, $whitelistedTGs) && !array_key_exists($person->ID, $leaders))
					unset($people[$person->ID]);
			
			// $alreadyAssignedPeople are people that we're added to this Task/WP table
			// so they can't be added again (until removal)
			$alreadyAssignedPeople = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterIDAndType($projectPlanningCharterID, ProjectPlanningCharterItem::PROJECT_PLANNING_CHARTER_ITEM_TYPE_PERSON);

			foreach ($alreadyAssignedPeople as $assigned)
			{
				if (isset($people[$assigned->PersonID]))
					unset($people[$assigned->PersonID]);
			}

			$list = array();
			foreach ($people as $person)
			{
				// get rid of people we can't assign anyway
				$peopleAccessControl = new PlanningControl($person, $this->project);
				// skip people that aren't assignable according to the PlanningControl class,
				// and also skip people shadowing TGs
				if ($peopleAccessControl->assignable === FALSE || substr($person->ID, 0, 2) === 'TG')
					continue;
			
				$personItem = array('id' => strtoupper(trim($person->ID)), 'text' => $person->getReversedFullNameWithEID(), 'disabled' => false);
				$list[] = $personItem;
			}
			
			Debugger::log($this->batch_id . "*** PPM - handle person list end - " . Environment::getUser()->getId(), Debugger::INFO);
			$this->presenter->payload->items = $list;
			$this->presenter->terminate();
		}
	}
	
	/**
	 * Saves all changed values into our ProjectPlanningCharter tables.
	 * We can only update the values, which are currently stored in the DB,
	 * because we expect multiple user interaction and the item being updated could
	 * be already deleted.
	 * 
	 * $param AppForm $form Submitted form.
	 */
	public function autosave($form)
	{
		Debugger::log($this->batch_id . "*** PPM - autosave start - " . Environment::getUser()->getId(), Debugger::INFO);
		$entry = $form->values;
		
		// MAT and TRV
		$matProjectPlanningCharter = $this->presenter->context->projectPlanningCharterRepository->findByProjectIDAndYearAndType($this->project->ID, $this->year, 'MAT');
		$matProjectPlanningCharter = $matProjectPlanningCharter[0];
		$matProjectPlanningCharterItem = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($matProjectPlanningCharter->ID);
		$matProjectPlanningCharterItem = $matProjectPlanningCharterItem[0];
		$matBudget = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($matProjectPlanningCharterItem->ID);
		
		$trvProjectPlanningCharter = $this->presenter->context->projectPlanningCharterRepository->findByProjectIDAndYearAndType($this->project->ID, $this->year, 'TRV');
		$trvProjectPlanningCharter = $trvProjectPlanningCharter[0];
		$trvProjectPlanningCharterItem = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($trvProjectPlanningCharter->ID);
		$trvProjectPlanningCharterItem = $trvProjectPlanningCharterItem[0];
		$trvBudget = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($trvProjectPlanningCharterItem->ID);
		
		for ($i = 1; $i < 13; $i++)
		{
			if ($matBudget[$i]->budget != $entry['mat' . $i])
			{
				// update changed budget
				$matBudget[$i]->budget = $entry['mat' . $i];
				$this->presenter->context->projectPlanningCharterItemBudgetRepository->update($matBudget[$i]);
			}
			
			if ($trvBudget[$i]->budget != $entry['trv' . $i])
			{
				// update changed budget
				$trvBudget[$i]->budget = $entry['trv' . $i];
				$this->presenter->context->projectPlanningCharterItemBudgetRepository->update($trvBudget[$i]);
			}
		}
		
		// it isn't necessary to query DB for every month
		$cachedBudgets = array();
		$taskWPCostShare = 0.0;
		
		foreach ($entry as $fieldName => $value)
		{
			// Task/WP names
			if (substr($fieldName, 0, 11) === 'taskWPLabel')
			{
				$id = substr($fieldName, 11);
				$taskWP = $this->presenter->context->projectPlanningCharterRepository->find($id);
				
				if ($taskWP !== NULL && $taskWP->Name !== $value)
				{
					$taskWP->Name = $value;
					$this->presenter->context->projectPlanningCharterRepository->update($taskWP);
				}
				
				// Cost Share for whole Task/WP
				$taskWPCostShare = $entry['taskWPCostShare' . $id];
			}
			// People and Organization Units (under Task/WP table)
			else if (substr($fieldName, 0, 12) === 'taskWPBudget')
			{
				$dataPart = substr($fieldName, 12);
				
				// $dataPart indexes:					0													1											2
				// taskWPBudget . [Task/WP table ID] (= ProjectPlanningCharter ID) . i . [Person/OrgUnit ID] (= ProjectPlanningCharterItem ID) . m . [Month index]
				$pattern = '/[im]/';
				$dataPart = preg_split($pattern, $dataPart);
				
				$personOrgUnitItem = $this->presenter->context->projectPlanningCharterItemRepository->find($dataPart[1]);
				if ($personOrgUnitItem !== NULL)
				{
					// if we don't have this budget in cache
					if (!array_key_exists($personOrgUnitItem->ID, $cachedBudgets))
					{
						$cachedBudgets[$personOrgUnitItem->ID] = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($personOrgUnitItem->ID);
					}
					
					if ($cachedBudgets[$personOrgUnitItem->ID][$dataPart[2]]->budget != $value)
					{
						$cachedBudgets[$personOrgUnitItem->ID][$dataPart[2]]->budget = $value;
						$this->presenter->context->projectPlanningCharterItemBudgetRepository->update($cachedBudgets[$personOrgUnitItem->ID][$dataPart[2]]);
					}
					
					// Cost Share
					if ($personOrgUnitItem->costShare != $taskWPCostShare / 100)
					{
						$personOrgUnitItem->costShare = $taskWPCostShare / 100;
						$this->presenter->context->projectPlanningCharterItemRepository->update($personOrgUnitItem);
					}
				}
			}
			// Cost Share for MAT
			else if (substr($fieldName, 0, 12) === 'matCostShare')
			{
				if ($matProjectPlanningCharterItem !== NULL)
				{
					if ($matProjectPlanningCharterItem->costShare != $value / 100)
					{
						$matProjectPlanningCharterItem->costShare = $value / 100;
						$this->presenter->context->projectPlanningCharterItemRepository->update($matProjectPlanningCharterItem);
					}
				}
			}
			// Cost Share for TRV
			else if (substr($fieldName, 0, 12) === 'trvCostShare')
			{
				if ($trvProjectPlanningCharterItem !== NULL)
				{
					if ($trvProjectPlanningCharterItem->costShare != $value / 100)
					{
						$trvProjectPlanningCharterItem->costShare = $value / 100;
						$this->presenter->context->projectPlanningCharterItemRepository->update($trvProjectPlanningCharterItem);
					}
				}
			}
			// Subcontractors
			// Cost Share
			else if (substr($fieldName, 0, 12) === 'subCostShare')
			{
				$projectPlanningCharterID = substr($fieldName, 12);
				
				$subcontractorItem = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($projectPlanningCharterID);
				$subcontractorItem = $subcontractorItem[0];
				if ($subcontractorItem !== NULL && $subcontractorItem->CostShare != $value / 100)
				{
					$subcontractorItem->costShare = $value / 100;
					$this->presenter->context->projectPlanningCharterItemRepository->update($subcontractorItem);
				}
			}
			else if (substr($fieldName, 0, 3) === 'sub')
			{
				$dataPart = substr($fieldName, 3);
				
				// $dataPart indexes:					0							1
				// sub . [Task/WP table ID] (= ProjectPlanningCharter ID) . m . [Month index]
				$pattern = '/[m]/';
				$dataPart = preg_split($pattern, $dataPart);

				$subcontractorItem = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($dataPart[0]);
				$subcontractorItem = $subcontractorItem[0];
				if ($subcontractorItem !== NULL)
				{
					// if we don't have this budget in cache
					if (!array_key_exists($subcontractorItem->ID, $cachedBudgets))
					{
						$cachedBudgets[$subcontractorItem->ID] = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($subcontractorItem->ID);
					}
					
					if ($cachedBudgets[$subcontractorItem->ID][$dataPart[1]]->budget != $value)
					{
						$cachedBudgets[$subcontractorItem->ID][$dataPart[1]]->budget = $value;
						$this->presenter->context->projectPlanningCharterItemBudgetRepository->update($cachedBudgets[$subcontractorItem->ID][$dataPart[1]]);
					}
				}
			}
		}
		
		Debugger::log($this->batch_id . "*** PPM - autosave end - " . Environment::getUser()->getId(), Debugger::INFO);
		// there may be multiple users editing this planning at the same time
		$this->presenter->invalidateControl('planning');
	}
	
	/**
	 * Refreshes Project Charter Planning table. Unsaved changes will be lost,
	 * because we will obtain new data from Project's planning tasks or initialize
	 * an empty planning structure in the beginning of the life of presenter.
	 */
	public function handleDiscard()
	{
		Debugger::log($this->batch_id . "*** PPM - discard start - " . Environment::getUser()->getId(), Debugger::INFO);
		if ($this->presenter->isAjax())
		{
			$this->presenter->redirect('this');
		}
		Debugger::log($this->batch_id . "*** PPM - discard end - " . Environment::getUser()->getId(), Debugger::INFO);
	}
	
	/**
	 * Creates new ProjectCharter according to the saved planning tasks in the WBS
	 * and binds it to ProjectPlanningCharterState table.
	 */
	public function handleCreate()
	{
		Debugger::log($this->batch_id . "*** PPM - create charter start - " . Environment::getUser()->getId(), Debugger::INFO);
		/*if ($this->presenter->isAjax())
		{
			$this->presenter->redirect('this');
		}*/
		
		// save planning
		$this->trasferDataFromPlanningDBToProjectDB();
		
		// make sure we haven't created the charter yet
		$planningState = $this->presenter->context->projectPlanningCharterStateRepository->find($this->project->ID, $this->year);
		
		if ($planningState === NULL)
		{
			$this->flashMessage('Creating of Project Charter from planning failed, because the Planning module wasn\'t in the proper state.', 'error');
			Debugger::log('Creating of Project Charter from planning failed, because the Planning module wasn\'t in the proper state. '
					. 'There was no record in ProjectPlanningCharterState table.', Debugger::ERROR);
			$this->presenter->invalidateControl('planning');
			return;
		}
		else if ($planningState->ProjectCharterID !== NULL)
		{
			$this->flashMessage('Creating of Project Charter from planning failed, because the Project Charter to be created already exists.', 'error');
			Debugger::log('Creating of Project Charter from planning failed, because the Project Charter to be created already exists.', Debugger::ERROR);
			$this->presenter->invalidateControl('planning');
			return;
		}
		else if ($planningState->WorkpackageID === NULL)
		{
			$this->flashMessage('Creating of Project Charter from planning failed, because we were unable to find planning work package.', 'error');
			Debugger::log('Creating of Project Charter from planning failed, because we were unable to find planning work package.', Debugger::ERROR);
			$this->presenter->invalidateControl('planning');
			return;
		}
		Debugger::log($this->batch_id . "*** PPM - create charter end - " . Environment::getUser()->getId(), Debugger::INFO);
		$this->presenter->redirect('ProjectCharter:createStepII', array('id' => $this->project->ID, 'year' => $this->year));
	}
	
	/**
	 * Updates the values of the charter attached to the planning from
	 * ProjectPlanningCharterState table.
	 */
	public function handleUpdate()
	{
		Debugger::log($this->batch_id . "*** PPM - update charter start - " . Environment::getUser()->getId(), Debugger::INFO);
		if ($this->presenter->isAjax())
		{
			// save planning
			$this->trasferDataFromPlanningDBToProjectDB();


			// state must be charter_exists or finished
			$planningState = $this->presenter->context->projectPlanningCharterStateRepository->find($this->project->ID, $this->year);

			if ($planningState === NULL || $planningState->State !== ProjectPlanningCharterState::PROJECT_PLANNING_CHARTER_STATE_CHARTEREXISTS)
			{
				$this->flashMessage('Updating of project charter failed, because the Planning module wasn\'t in the proper state.', 'error');
				Debugger::log('Updating of project charter failed, because the Planning module wasn\'t in the proper state.', Debugger::ERROR);
				$this->presenter->invalidateControl('planning');
				return;
			}

			if ($planningState->ProjectCharterID === NULL || $planningState->WorkpackageID === NULL)
			{
				$this->flashMessage('Updating of project charter failed, because the charter or planning workpackage wasn\'t found.', 'error');
				Debugger::log('Updating of project charter failed, because the charter or planning workpackage wasn\'t found.', Debugger::ERROR);
				$this->presenter->invalidateControl('planning');
				return;
			}

			// is charter in definition?
			$charter = $this->presenter->context->projectCharterRepository->find($planningState->ProjectCharterID);
			if ($charter->Status !== ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION)
			{
				$this->flashMessage('Updating of project charter failed, because the charter wasn\'t in the definition state.', 'error');
				$this->presenter->invalidateControl('planning');
				return;
			}

			if ($charter->Owner !== strtoupper($this->presenter->getUser()->getID()))
			{
				$this->flashMessage('Updating of project charter failed, because you are not the charter\'s owner.', 'error');
				$this->presenter->invalidateControl('planning');
				return;
			}

			// init
			$planningHelper = new PlanningHelper($this->presenter->context, $this->year);
			$costShare = 0;
			$totalUSD = 0;

			// sync charter's header from project
			$charter->SAP_ID = $this->project->SAP_ID;
			if ($this->project->Customer !== NULL)
				$charter->CustomerID = $this->project->Customer;
			if ($this->project->Responsibility !== NULL)
				$charter->ResponsibilityID = $this->project->Responsibility;
			if ($this->project->ECLicenceID !== NULL)
				$charter->ECLicenceID = $this->project->ECLicenceID;
//			if ($this->project->TM !== NULL)
//				$charter->LineManager = $this->project->TM;
			$charter->ProgramID = $this->project->ProgramID;

			// update charter's budget and frozen LR
			// delete previous budget
			$budgetHistoryOwner = $this->presenter->context->projectCharterBudgetHistoryRepository->findLatestVersionByEID($charter->ID, $charter->Owner);
			$budgetHistoryLM = $this->presenter->context->projectCharterBudgetHistoryRepository->findLatestVersionByEID($charter->ID, $charter->LineManager);
			$lmHistoryExists = false;
			if($budgetHistoryLM)
			{
				$lmHistoryExists = true;
			}
			
			$this->presenter->context->projectCharterBudgetRepository->deleteByProjectCharterIDAndVersion($charter->ID, $budgetHistoryOwner->Version);
			if($lmHistoryExists)
			{
				$this->presenter->context->projectCharterBudgetRepository->deleteByProjectCharterIDAndVersion($charter->ID, $budgetHistoryLM->Version);
			}

			$planningWPs = $this->presenter->context->workPackageRepository->findByParentID($planningState->WorkpackageID);
			$TGs = array();
			foreach ($planningWPs as $WP)
			{
				$tasks = $this->presenter->context->taskRepository->findAllWorkPackageTasks($WP->ID);
				switch ($WP->name)
				{
					case 'Subcontractors':
						// each task means one subcontractor
						foreach ($tasks as $task)
						{
							$budgetArray = array();
							$assignments = $this->presenter->context->assignmentsCostRepository->findTaskYearAssignments($task->ID, $this->year);
							foreach ($assignments as $assignment)
							{
								$budgetArray[$assignment->month] = $assignment->cost;
								$costShare += $assignment->cost * $task->costShare;
								$totalUSD += $assignment->cost;
							}

							$subID = explode(' ', $task->SAPID);
							$subID = $subID[1];

							$subcontractor = $this->presenter->context->subcontractorRepository->find($subID);
							if ($subcontractor === NULL)
							{
								$this->flashMessage('Couldn\'t preload subcontractor from Task with Task SAP ID: '
									. $task->SAPID, 'error');
								continue;
							}
							
							$budgetItem = new ProjectCharterBudget();
							$budgetItem->ProjectCharterID = $charter->ID;
							$budgetItem->Version = $budgetHistoryOwner->Version;
							$budgetItem->ResourceType = ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING;
							$budgetItem->Subcontractor = $subcontractor->ID;

							$budgetItem = $planningHelper->convertToQuartals($budgetArray, $budgetItem);
							$this->presenter->context->projectCharterBudgetRepository->save($budgetItem);
							
							if($lmHistoryExists)
							{
								$budgetItem->Version = $budgetHistoryLM->Version;
								$this->presenter->context->projectCharterBudgetRepository->save($budgetItem);
							}
						}
						break;

					case 'MAT & TRV':
						foreach ($tasks as $task)
						{
							$budgetArray = array();
							$assignments = $this->presenter->context->assignmentsCostRepository->findTaskYearAssignments($task->ID, $this->year);
							foreach ($assignments as $assignment)
							{
								$budgetArray[$assignment->month] = $assignment->cost;
								$costShare += $assignment->cost * $task->costShare;
								$totalUSD += $assignment->cost;
							}

							$budgetItem = new ProjectCharterBudget();
							$budgetItem->ProjectCharterID = $charter->ID;
							$budgetItem->Version = $budgetHistoryOwner->Version;

							if ($task->Name == 'MAT ' . $this->year)
							{
								$budgetItem->ResourceType = ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL;
							}
							else if ($task->Name == 'TRV ' . $this->year)
							{
								$budgetItem->ResourceType = ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL;
							}
							else
							{
								$this->flashMessage('Couldn\'t preload material or travel from Task with Task SAP ID: '
									. $task->SAPID, 'error');
							}

							$budgetItem = $planningHelper->convertToQuartals($budgetArray, $budgetItem);
							$this->presenter->context->projectCharterBudgetRepository->save($budgetItem);
							
							if($lmHistoryExists)
							{
								$budgetItem->Version = $budgetHistoryLM->Version;
								$this->presenter->context->projectCharterBudgetRepository->save($budgetItem);
							}
						}
						break;

					default:
						// Task/WP tables with custom names
						foreach ($tasks as $task)
						{
							$assignments = $this->presenter->context->personTaskAssignmentRepository->findTaskYearAssignments($task->ID, $this->year);

							foreach ($assignments as $EID => $assignmentArray)
							{
								$person = $this->presenter->context->personRepository->find($EID);
//								$orgUnit = $this->presenter->context->organizationUnitRepository->find($person->OrganizationUnitID);
								$orgUnitID = $person->OrganizationUnitID;

//								if (!$orgUnit || $orgUnit->Type !== OrganizationUnit::ORG_UNIT_TYPE_TG)
//								{
//									$this->flashMessage('Couldn\'t find Technical Group associated with EID: ' . $person->ID . '.', 'warning');
//									continue;
//								}

								$result = $planningHelper->getOrgUnitLabourRate($orgUnitID, $this->project->ID);
								$labourRate = $result['LabourRate'];

								$hourArray = array();
								foreach ($assignmentArray as $assignment)
								{
									$hourArray[$assignment->month] = $assignment->hours;
									$USD = $assignment->hours * $labourRate;
									$costShare += $USD * $task->costShare;
									$totalUSD += $USD;
								}

								if (isset($TGs[$orgUnitID  . 'Owner']))
								{
									// we will be adding values to this TG
									$budgetItem = $planningHelper->convertToQuartals($hourArray, $TGs[$orgUnitID . 'Owner']);
									
									$this->presenter->context->projectCharterBudgetRepository->update($budgetItem);
									if($lmHistoryExists)
									{
										$budgetItem = $planningHelper->convertToQuartals($hourArray, $TGs[$orgUnitID . 'LM']);
										$this->presenter->context->projectCharterBudgetRepository->update($budgetItem);
									}
								}
								else
								{
									// we'll be creating a new TG-row
									$budgetItem = new ProjectCharterBudget();
									$budgetItem->ProjectCharterID = $charter->ID;
									$budgetItem->Version = $budgetHistoryOwner->Version;
									$budgetItem->ResourceType = ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS;
									$budgetItem->TechnicalGroup = $orgUnitID;
									$budgetItem->Rate = $labourRate;

									$budgetItem = $planningHelper->convertToQuartals($hourArray, $budgetItem);

									$this->presenter->context->projectCharterBudgetRepository->save($budgetItem);
									$TGs[$orgUnitID . 'Owner'] = $budgetItem;
									
									if($lmHistoryExists)
									{
										// we'll be creating a new TG-row
										$budgetItem = new ProjectCharterBudget();
										$budgetItem->ProjectCharterID = $charter->ID;
										$budgetItem->Version = $budgetHistoryLM->Version;
										$budgetItem->ResourceType = ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS;
										$budgetItem->TechnicalGroup = $orgUnitID;
										$budgetItem->Rate = $labourRate;

										$budgetItem = $planningHelper->convertToQuartals($hourArray, $budgetItem);

										$this->presenter->context->projectCharterBudgetRepository->save($budgetItem);
										$TGs[$orgUnitID . 'LM'] = $budgetItem;
									}
								}
							}
						}
				}
			}

			// Cost Share
			if ($totalUSD != 0)
				$charter->CostShare = round($costShare / $totalUSD, 2);
			else
				$charter->CostShare = 0;

			if (!$this->presenter->context->projectCharterRepository->update($charter))
			{
				$this->flashMessage('Updating of project charter failed, because the charter couldn\'t be saved.', 'error');
				Debugger::log('Updating of project charter failed, because the charter couldn\'t be saved.', Debugger::ERROR);
				$this->presenter->invalidateControl('planning');
				return;
			}
			Debugger::log($this->batch_id . "*** PPM - update charter end - " . Environment::getUser()->getId(), Debugger::INFO);
			$this->flashMessage('Project charter' . $charter->Name . ' has been updated.', 'success');
			$this->presenter->redirect('ProjectCharter:detail', array('id' => $charter->ID, 'year' => $this->year));
		}
	}
	
	/**
	 * Changes planning tasks from 'type 2 - planning' to 'type 1 - placeholder'.
	 * And additionally unlocks the editing of ProjectCharter forever.
	 */
	public function handleFinish()
	{
		if ($this->presenter->isAjax())
		{
			// only for PMO
			if (!Environment::getUser()->isInRole('PMO'))
				return;

			$planningState = $this->presenter->context->projectPlanningCharterStateRepository->find($this->project->ID, $this->year);

			if ($planningState === NULL)
			{
				$this->flashMessage('Finishing of planning failed, because the Planning module wasn\'t in the proper state.', 'error');
				Debugger::log('Finishing of planning failed, because the Planning module wasn\'t in the proper state. '
						. 'There was no record in ProjectPlanningCharterState table.', Debugger::ERROR);
				$this->presenter->invalidateControl('planning');
				return;
			}

			if ($planningState->WorkpackageID === NULL)
			{
				$this->flashMessage('There aren\'t any planning tasks to be finished in this year. First, you must save the planning.', 'notice');
				$this->presenter->invalidateControl('planning');
				return;
			}
			else
			{
				// mark planning WP as PH
				$planningWP = $this->presenter->context->workPackageRepository->find($planningState->WorkpackageID);
				$newSAPID = str_replace('Planning', 'Placeholder', $planningWP->SAPID);
				$index = 1;

				while ($this->presenter->context->workPackageRepository->findBySAPID($newSAPID))
				{
					$index++;
					$newSAPID = str_replace('Placeholder', 'Placeholder-' . $index, $newSAPID);
					$planningWP->Name = $planningWP->Name . '-' . $index;
				}

				$planningWP->SAPID = $newSAPID;
				$this->presenter->context->workPackageRepository->update($planningWP);

				// change planning tasks to placeholders 
				$planningWPs = $this->presenter->context->workPackageRepository->findByParentID($planningState->WorkpackageID);
				foreach ($planningWPs as $WP)
				{
					$WP->SAPID = str_replace('Planning', 'Placeholder-' . $index, $WP->SAPID);
					$this->presenter->context->workPackageRepository->update($WP);

					$tasks = $this->presenter->context->taskRepository->findAllWorkPackageTasks($WP->ID);
					foreach ($tasks as $task)
					{
						if ($task->Fake === Task::TASK_TYPE_PLANNING)
						{
							$task->Fake = Task::TASK_TYPE_PLACEHOLDER;
							$this->presenter->context->taskRepository->update($task);
						}
					}
				}

				$planningState->State = ProjectPlanningCharterState::PROJECT_PLANNING_CHARTER_STATE_FINISHED;
				$planningState->WorkpackageID = NULL;
				$planningState->AtLeastOnceFinished = TRUE;

				$this->presenter->context->projectPlanningCharterStateRepository->update($planningState);
				$this->flashMessage('Planning has been finished successfully. Planning tasks are converted to placeholders.', 'success');
			}
		}
		
		$this->presenter->redirect('this');
	}
	
	/**
	 * Regular editing changes are saved automatically using AJAX calls
	 * (handle methods). The planning form is save to the DB.
	 * 
	 * However, if user clicks on the 'Save Project Planning' button,
	 * the planning will be transformed into work package structure.
	 */
	public function handleSavePlanningForm()
	{
		Debugger::log($this->batch_id . "*** PPM - handle save Planning start - " . Environment::getUser()->getId(), Debugger::INFO);
		if ($this->presenter->isAjax())
		{
			// TODO: check if the data in form are coresponding to the DB

			$this->trasferDataFromPlanningDBToProjectDB();
			Debugger::log($this->batch_id . "*** PPM - handle save Planning end - " . Environment::getUser()->getId(), Debugger::INFO);
			$this->presenter->redirect('this');
		}
	}
	
	/**
	 * Creates new planning tasks in project WBS,
	 * 
	 */
	private function trasferDataFromPlanningDBToProjectDB()
	{
		Debugger::log($this->batch_id . "*** PPM - trasferDataFromPlanningDBToProjectDB start - " . Environment::getUser()->getId(), Debugger::INFO);
		// delete all planning tasks already present there, according to the
		// planning state DB table (Project_ID, Year)
		$planningState = $this->presenter->context->projectPlanningCharterStateRepository->find($this->project->ID, $this->year);
		if ($planningState !== NULL && $planningState->WorkpackageID !== NULL)
		{
			$this->presenter->context->workPackageRepository->delete($planningState->WorkpackageID);
		}
		
		$rootWP = $this->presenter->context->workPackageRepository->findBySAPID($this->project->SAPID);
		if ($rootWP === NULL && $this->project->Fake)
		{
			// a placeholder project without root WP
			$rootWP = new WorkPackage();
			$rootWP->ProjectID = $this->project->ID;
			$rootWP->SAPID = $this->project->SAP_ID;
			$rootWP->ParentID = 0;
			$rootWP->Name = $this->project->Name;
			$rootWP->Date = date('Y-m-d H:i:s');
			
			$this->presenter->context->workPackageRepository->save($rootWP);

			// Placeholder WP
			$placeholderWP = new WorkPackage();
			$placeholderWP->ProjectID = 0;
			if ($this->project->SAP_ID)
				$placeholderWP->SAPID = $this->project->SAP_ID . '-Placeholder';
			else
				$placeholderWP->SAPID = $this->project->Name . '-Placeholder';

			$placeholderWP->ParentID = $rootWP->ID;
			$placeholderWP->Name = $rootWP->Name;
			$placeholderWP->Date = date('Y-m-d H:i:s');
			
			$this->presenter->context->workPackageRepository->save($placeholderWP);
		}
		
		
		$sapIdEndingSubPart = 0;
		
		$planningWP = new WorkPackage();
		$planningWP->Name = 'Project Planning ' . $this->year;
		$planningWP->ParentID = $rootWP->ID;
		$planningWP->SAPID = $rootWP->SAPID . '-Planning-' . $this->year;
		$planningWP->Date = date('Y-m-d H:i:s');
		$this->presenter->context->workPackageRepository->save($planningWP);
		
		// update planning state - insert WorkPackage ID
		$planningState->workpackageID = $planningWP->ID;
		$this->presenter->context->projectPlanningCharterStateRepository->update($planningState);
		
		// Subcontractor subWP
		$subWP = new WorkPackage();
		$subWP->Name = 'Subcontractors';
		$subWP->ParentID = $planningWP->ID;
		$subWP->SAPID = $rootWP->SAPID . '-' . 'Planning-' . $this->year . '-' . ++$sapIdEndingSubPart;
		$subWP->Date = date('Y-m-d H:i:s');
		$this->presenter->context->workPackageRepository->save($subWP);
		
		// MAT & TRV subWP
		$matAndTrvWP = new WorkPackage();
		$matAndTrvWP->Name = 'MAT & TRV';
		$matAndTrvWP->ParentID = $planningWP->ID;
		$matAndTrvWP->SAPID = $rootWP->SAPID . '-' . 'Planning-' . $this->year . '-' . ++$sapIdEndingSubPart;
		$matAndTrvWP->Date = date('Y-m-d H:i:s');
		$this->presenter->context->workPackageRepository->save($matAndTrvWP);
		
		$taskWPs = $this->presenter->context->projectPlanningCharterRepository->findByProjectIDAndYear($this->project->ID, $this->year);
		/* Contains a map for remembering Tasks, which shadow each WC.
		 * The structure is following:
		 * 
		 * ID of the planning table in GUI (filled with people/org units) => an array of workcenter IDs
		 *				an array of workcenter ID => ID of a Task used to shadow this WC
		 */
		$workcenterGroupings = array();
		
		foreach ($taskWPs as $projectPlanningCharter)
		{
			switch ($projectPlanningCharter->Type)
			{
				case 'HRS':
					// WP for each table (people or org unit planning)
					$tableWP = new WorkPackage();
					$tableWP->Name = $projectPlanningCharter->Name;
					$tableWP->ParentID = $planningWP->ID;
					$tableWP->SAPID = $rootWP->SAPID . '-' . 'Planning-' . $this->year . '-' . ++$sapIdEndingSubPart;
					$tableWP->Date = date('Y-m-d H:i:s');
					$this->presenter->context->workPackageRepository->save($tableWP);
					
					// ini this GUI WP/table in our map
					$workcenterGroupings[$projectPlanningCharter->ID] = array();
					
					$people = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterIDAndType($projectPlanningCharter->ID, ProjectPlanningCharterItem::PROJECT_PLANNING_CHARTER_ITEM_TYPE_PERSON);

					foreach ($people as $projectPlanningCharterItem)
					{
						$person = $this->presenter->context->personRepository->find($projectPlanningCharterItem->PersonID);
						if ($person === NULL)
							continue;
						
						// check if a task for this WC exists, as we are grouping people and
						// org units by WCs
						if (isset($workcenterGroupings[$projectPlanningCharter->ID][$person->CostCenterID]))
						{
							$planningTaskIDShadowingWC = $workcenterGroupings[$projectPlanningCharter->ID][$person->CostCenterID];
						}
						else
						{
							$WC = $this->presenter->context->costCenterRepository->findCostCenter($person->CostCenterID);
							
							// create WC shadowing/mirroring task
							$task = new Task();
							$task->Name = $WC->Name . ' ' . $this->year;
							$task->ProjectID = $this->project->ID;
							$task->WorkPackageID = $tableWP->ID;
							$task->CostCenterID = $person->CostCenterID;
							$task->SAPID = $this->presenter->context->taskRepository->getNextTaskID() . ' 0010';
							$task->Remark = NULL;
							$task->CostShare = $projectPlanningCharterItem->CostShare;
							$task->LabourTaskFlag = 1;
							$task->CostAsLabour = 0;
							$task->ActualsImport = 0;
							$task->ActualsImportDataSource = Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC;
							$task->Date = date('Y-m-d');
							$task->Fake = 2;

							$this->presenter->context->taskRepository->save($task);
							$planningTaskIDShadowingWC = $task->ID;
							
							$workcenterGroupings[$projectPlanningCharter->ID][$person->CostCenterID] = $task->ID;
						}
						
						// assign the person
						$hours = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($projectPlanningCharterItem->ID);
						
						// first, we must create a TaskPlan
						if ($this->presenter->context->taskPlanRepository->findTaskYearPlan($planningTaskIDShadowingWC, $this->year) === NULL)
						{
							$taskPlan = new TaskYearPlan($this->year);
							$taskPlan->TaskID = $planningTaskIDShadowingWC;
							for ($i = 1; $i < 13; $i++)
								$taskPlan->setMonthPlan($i, new MonthPlan(0, 0));

							$this->presenter->context->taskPlanRepository->save($taskPlan);
						}
						
						for ($i = 1; $i < 13; $i++)
						{
							$assignment = new PersonTaskMonthAssignment();
							$assignment->TaskID = $planningTaskIDShadowingWC;
							// in app, Person-Task assignment is indexed by range <1, 12>,
							// and so is PlanningCharterItemBudget
							$assignment->Month = $i;
							$assignment->Hours = $hours[$i]->Budget;
							$assignment->Year = $this->year;
							$assignment->PersonID = $person->ID;
							
							$this->presenter->context->personTaskAssignmentRepository->save($assignment);
						}
					}
					
					$orgUnits = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterIDAndType($projectPlanningCharter->ID, ProjectPlanningCharterItem::PROJECT_PLANNING_CHARTER_ITEM_TYPE_ORGANIZATION_UNIT);
					
					foreach ($orgUnits as $projectPlanningCharterItem)
					{
						$orgUnit = $this->presenter->context->organizationUnitRepository->find($projectPlanningCharterItem->organizationUnitID);
						if ($orgUnit === NULL)
							continue;

						$planningHelper = new PlanningHelper($this->presenter->context, $this->year);
						$tmpResult = $planningHelper->getOrgUnitLabourRate($orgUnit->ID);
						$workCenterID = $tmpResult['WorkcenterID'];
						
						if ($workCenterID === NULL)
						{
							$this->flashMessage('Couldn\'t calculate labor rate for technical group named ' . $orgUnit->Name .
								'. Corresponding planning task has been skipped. Please, make sure labor rates for this year exist.', 'warning');
							continue;
						}
						
						// check if a task for this WC exists, as we are grouping people and
						// org units by WCs
						if (isset($workcenterGroupings[$projectPlanningCharter->ID][$workCenterID]))
						{
							$planningTaskIDShadowingWC = $workcenterGroupings[$projectPlanningCharter->ID][$workCenterID];
						}
						else
						{
							$WC = $this->presenter->context->costCenterRepository->findCostCenter($workCenterID);
							
							// create WC shadowing/mirroring task
							$task = new Task();
							$task->Name = $WC->Name . ' ' . $this->year;
							$task->ProjectID = $this->project->ID;
							$task->WorkPackageID = $tableWP->ID;
							$task->CostCenterID = $workCenterID;
							$task->SAPID = $this->presenter->context->taskRepository->getNextTaskID() . ' 0010';
							$task->Remark = NULL;
							$task->CostShare = $projectPlanningCharterItem->CostShare;
							$task->LabourTaskFlag = 1;
							$task->CostAsLabour = 0;
							$task->ActualsImport = 0;
							$task->ActualsImportDataSource = Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC;
							$task->Date = date('Y-m-d');
							$task->Fake = 2;

							$this->presenter->context->taskRepository->save($task);
							$planningTaskIDShadowingWC = $task->ID;
							
							$workcenterGroupings[$projectPlanningCharter->ID][$workCenterID] = $task->ID;
						}
						
						// assign the person, who is shadowing an assignment of a technical group
						$hours = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($projectPlanningCharterItem->ID);
						
						// first, we must have create a TaskPlan
						if ($this->presenter->context->taskPlanRepository->findTaskYearPlan($planningTaskIDShadowingWC, $this->year) === NULL)
						{
							$taskPlan = new TaskYearPlan($this->year);
							$taskPlan->TaskID = $planningTaskIDShadowingWC;
							for ($i = 1; $i < 13; $i++)
								$taskPlan->setMonthPlan($i, new MonthPlan(0, 0));

							$this->presenter->context->taskPlanRepository->save($taskPlan);
						}

						// find / create a person shadowing the TG
						$person = NULL;
						$personAsTG = $this->presenter->context->personRepository->findAllTGs();
						
						$maxTGIndex = 0;
						foreach ($personAsTG as $p)
						{
							$index = substr($p->ID, 2);
							$maxTGIndex = ($index > $maxTGIndex) ? $index : $maxTGIndex;
						}
						
						if (isset($personAsTG[$orgUnit->ID]))
						{
							$person = $personAsTG[$orgUnit->ID];
							$person->CostCenterID = $workCenterID;
							
							$this->presenter->context->personRepository->update($person);
						}
						else
						{
							// create a new TG shadowing person
							$person = new Person();
							$person->ID = 'TG' . str_pad(($maxTGIndex + 1), 5, '0', STR_PAD_LEFT);
							$person->Firstname = 'TG placeholder';
							$person->Surname = $orgUnit->Name;
							$person->CostCenterID = $workCenterID;
							$person->OrganizationUnitID = $orgUnit->ID;
							$person->NationalityID = 3;
							
							$this->presenter->context->personRepository->save($person);
							
							// make sure that person's workable hours are zeroed
							for ($i = self::WH_START_YEAR; $i <= self::WH_END_YEAR; $i++)
							{
								$workableHours = new WorkableHour();
								$workableHours->ID = $person->ID;
								$workableHours->Year = $i;
								$workableHours->Yield = 0.0;
								$workableHours->Contract = 0.0;
								$workableHours->IsAOP = FALSE;
								for ($j = 1; $j < 13; $j++)
									$workableHours->setMonthHours(0, $j);

								$this->presenter->context->workableHourRepository->create($workableHours);
							}
						}
						
						for ($i = 1; $i < 13; $i++)
						{
							$assignment = new PersonTaskMonthAssignment();
							$assignment->TaskID = $planningTaskIDShadowingWC;
							// in app, Person-Task assignment is indexed by range <1, 12>,
							// and so is PlanningCharterItemBudget
							$assignment->Month = $i;
							$assignment->Hours = $hours[$i]->Budget;
							$assignment->Year = $this->year;
							$assignment->PersonID = $person->ID;
							
							$this->presenter->context->personTaskAssignmentRepository->save($assignment);
						}
					}

					break;
				
				case 'SUB':
					$sub = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($projectPlanningCharter->ID);
					$sub = $sub[0];
					
					$subcontractor = $this->presenter->context->subcontractorRepository->find($sub->subcontractorID);
					
					$task = new Task();
					$task->Name = $subcontractor->Short . ' ' . $subcontractor->Name . ' ' . $this->year;
					$task->ProjectID = $this->project->ID;
					$task->WorkPackageID = $subWP->ID;
					$task->CostCenterID = NULL;
					$task->SAPID = $this->presenter->context->taskRepository->getNextTaskID() . ' ' . str_pad($subcontractor->ID, 4, '0', STR_PAD_LEFT);
					$task->Remark = NULL;
					$task->CostShare = $sub->CostShare;
					$task->LabourTaskFlag = 0;
					$task->CostAsLabour = 0;
					$task->ActualsImport = 0;
					$task->ActualsImportDataSource = Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC;
					$task->Date = date('Y-m-d');
					$task->Fake = 2;
					
					$this->presenter->context->taskRepository->save($task);
					
					// cost assignment
					$usd = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($sub->ID);
					
					// first, we must have create a TaskPlan
					if ($this->presenter->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year) === NULL)
					{
						$taskPlan = new TaskYearPlan($this->year);
						$taskPlan->TaskID = $task->ID;
						for ($i = 1; $i < 13; $i++)
							$taskPlan->setMonthPlan($i, new MonthPlan(0, 0));

						$this->presenter->context->taskPlanRepository->save($taskPlan);
					}

					for ($i = 1; $i < 13; $i++)
					{
						$assignmentsCost = new AssignmentsCost();
						$assignmentsCost->TaskID = $task->ID;
						$assignmentsCost->Month = $i;
						$assignmentsCost->Cost = $usd[$i]->Budget;
						$assignmentsCost->Year = $this->year;
						$this->presenter->context->assignmentsCostRepository->save($assignmentsCost);
					}
					break;
				
				case 'MAT':
					$mat = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($projectPlanningCharter->ID);
					$mat = $mat[0];
					
					$task = new Task();
					$task->Name = 'MAT ' . $this->year;
					$task->ProjectID = $this->project->ID;
					$task->WorkPackageID = $matAndTrvWP->ID;
					$task->CostCenterID = NULL;
					$task->SAPID = $this->presenter->context->taskRepository->getNextTaskID() . ' 0010';
					$task->Remark = NULL;
					$task->CostShare = $mat->CostShare;
					$task->LabourTaskFlag = 0;
					$task->CostAsLabour = 0;
					$task->ActualsImport = 0;
					$task->ActualsImportDataSource = Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC;
					$task->Date = date('Y-m-d');
					$task->Fake = 2;
					
					$this->presenter->context->taskRepository->save($task);
					
					// cost assignment
					$usd = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($mat->ID);
					
					// first, we must have create a TaskPlan
					if ($this->presenter->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year) === NULL)
					{
						$taskPlan = new TaskYearPlan($this->year);
						$taskPlan->TaskID = $task->ID;
						for ($i = 1; $i < 13; $i++)
							$taskPlan->setMonthPlan($i, new MonthPlan(0, 0));

						$this->presenter->context->taskPlanRepository->save($taskPlan);
					}

					for ($i = 1; $i < 13; $i++)
					{
						$assignmentsCost = new AssignmentsCost();
						$assignmentsCost->TaskID = $task->ID;
						$assignmentsCost->Month = $i;
						$assignmentsCost->Cost = $usd[$i]->Budget;
						$assignmentsCost->Year = $this->year;
						$this->presenter->context->assignmentsCostRepository->save($assignmentsCost);
					}
					break;
				
				case 'TRV':
					$trv = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($projectPlanningCharter->ID);
					$trv = $trv[0];
					
					$task = new Task();
					$task->Name = 'TRV ' . $this->year;
					$task->ProjectID = $this->project->ID;
					$task->WorkPackageID = $matAndTrvWP->ID;
					$task->CostCenterID = NULL;
					$task->SAPID = $this->presenter->context->taskRepository->getNextTaskID() . ' 0010';
					$task->Remark = NULL;
					$task->CostShare = $trv->CostShare;
					$task->LabourTaskFlag = 0;
					$task->CostAsLabour = 0;
					$task->ActualsImport = 0;
					$task->ActualsImportDataSource = Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC;
					$task->Date = date('Y-m-d');
					$task->Fake = 2;
					
					$this->presenter->context->taskRepository->save($task);
					
					// cost assignment
					$usd = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($trv->ID);
					
					// first, we must have create a TaskPlan
					if ($this->presenter->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year) === NULL)
					{
						$taskPlan = new TaskYearPlan($this->year);
						$taskPlan->TaskID = $task->ID;
						for ($i = 1; $i < 13; $i++)
							$taskPlan->setMonthPlan($i, new MonthPlan(0, 0));

						$this->presenter->context->taskPlanRepository->save($taskPlan);
					}

					for ($i = 1; $i < 13; $i++)
					{
						$assignmentsCost = new AssignmentsCost();
						$assignmentsCost->TaskID = $task->ID;
						$assignmentsCost->Month = $i;
						$assignmentsCost->Cost = $usd[$i]->Budget;
						$assignmentsCost->Year = $this->year;
						$this->presenter->context->assignmentsCostRepository->save($assignmentsCost);
					}
					break;
				
				default:
					break;
			}
		}
		Debugger::log($this->batch_id . "*** PPM - trasferDataFromPlanningDBToProjectDB end - " . Environment::getUser()->getId(), Debugger::INFO);
	}


	/**
	 * This function will take current project's tasks and select only the planning ones.
	 * Then, it'll transfer this information into planning tables.
	 * 
	 * That way, we'll reset the planning module to its default state, for
	 * instance, when user closes the page and then loads it again.
	 * 
	 * If we don't have any planning tasks related to the current project in current
	 * year, it'll load an empty planning structure (initialize it).
	 */
	private function loadPlanningDBFromProject()
	{
		Debugger::log($this->batch_id . "*** PPM - loadPlanningDBFromProject start - " . Environment::getUser()->getId(), Debugger::INFO);
		// do we have any planning tasks (fake == 2) for this project?
		$planningState = $this->presenter->context->projectPlanningCharterStateRepository
			->find($this->project->ID, $this->year);
		
		$this->initEmptyPlanningDB(TRUE);
		
		if ($planningState !== NULL)
		{
			if ($planningState->WorkpackageID === NULL)
			{
				// recover
				$this->initEmptyPlanningDB();
				return;
			}
			
			// load temporary 'project charter planning' database from our planning tasks
			$planningWPs = $this->presenter->context->workPackageRepository->findByParentID($planningState->WorkpackageID);
			foreach ($planningWPs as $WP)
			{
				$tasks = $this->presenter->context->taskRepository->findAllWorkPackageTasks($WP->ID);
				switch ($WP->name)
				{
					case 'Subcontractors':
						// each task means one subcontractor
						// subcontractors never exist after init, so we can add them
						foreach ($tasks as $task)
						{
							$budgetArray = array();
							$assignments = $this->presenter->context->assignmentsCostRepository->findTaskYearAssignments($task->ID, $this->year);
							foreach ($assignments as $assignment)
							{
								$budgetArray[$assignment->month] = $assignment->cost;
							}
							
							$subID = explode(' ', $task->SAPID);
							$subID = $subID[1];
							
							$subcontractor = $this->presenter->context->subcontractorRepository->find($subID);
							if ($subcontractor === NULL)
							{
								$this->flashMessage('Couldn\'t preload subcontractor from Task with Task SAP ID: '
									. $task->SAPID, 'error');
								continue;
							}
							
							$this->saveSubcontractor($subcontractor->ID, $budgetArray, $task->CostShare);
						}
						break;
					
					case 'MAT & TRV':
						// MAT & TRV always exist after init
						foreach ($tasks as $task)
						{
							$budgetArray = array();
							$assignments = $this->presenter->context->assignmentsCostRepository->findTaskYearAssignments($task->ID, $this->year);
							foreach ($assignments as $assignment)
							{
								$budgetArray[$assignment->month] = $assignment->cost;
							}
							
							if ($task->Name == 'MAT ' . $this->year)
							{
								$this->saveMATorTRV('MAT', $budgetArray, TRUE, $task->CostShare);
							}
							else if ($task->Name == 'TRV ' . $this->year)
							{
								$this->saveMATorTRV('TRV', $budgetArray, TRUE, $task->CostShare);
							}
							else
							{
								$this->flashMessage('Couldn\'t preload material or travel from Task with Task SAP ID: '
									. $task->SAPID, 'error');
							}
						}
						break;
					
					default:
						// Task/WP tables with custom names
						$projectPlanningCharter = new ProjectPlanningCharter();
						$projectPlanningCharter->ProjectID = $this->project->ID;
						$projectPlanningCharter->Year = $this->year;
						$projectPlanningCharter->Type = 'HRS';
						$projectPlanningCharter->Name = $WP->Name;
						$this->presenter->context->projectPlanningCharterRepository->save($projectPlanningCharter);
						
						foreach ($tasks as $task)
						{
							$assignments = $this->presenter->context->personTaskAssignmentRepository->findTaskYearAssignments($task->ID, $this->year);
							
							foreach ($assignments as $EID => $assignmentArray)
							{
								$person = $this->presenter->context->personRepository->find($EID);

								$projectPlanningCharterItem = new ProjectPlanningCharterItem();
								if (substr($person->ID, 0, 2) === 'TG')
								{
									// the person on this assignment is shadowing a TG
									$projectPlanningCharterItem->OrganizationUnitID = $person->OrganizationUnitID;
									$projectPlanningCharterItem->Type = 'OrganizationUnit';
								}
								else
								{
									$projectPlanningCharterItem->PersonID = $person->ID;
									$projectPlanningCharterItem->Type = 'Person';
								}
								$projectPlanningCharterItem->ProjectPlanningCharterID = $projectPlanningCharter->ID;
								$projectPlanningCharterItem->CostShare = $task->CostShare;
								$this->presenter->context->projectPlanningCharterItemRepository->save($projectPlanningCharterItem);

								// link new budget
								for ($i = 1; $i < 13; $i++)
								{
									$budget = new ProjectPlanningCharterItemBudget();
									$budget->ProjectPlanningCharterItemID = $projectPlanningCharterItem->ID;
									$budget->Month = $i;
									$budget->Budget = $assignmentArray[$i]->Hours;
									$budget->BudgetType = 'HRS';
									$this->presenter->context->projectPlanningCharterItemBudgetRepository->save($budget);
								}
							}
						}
				}
			}
			$this->flashMessage('Planning form has been synchronized with planning tasks from the work breakdown structure.', 'success');
		}
		Debugger::log($this->batch_id . "*** PPM - loadPlanningDBFromProject end - " . Environment::getUser()->getId(), Debugger::INFO);
	}
	
	/**
	 * Sets up an empty ProjectPlanningCharter DB with all its associated tables.
	 * 
	 * @param bool $omitLabour Omits the creation of ProjectPlanningCharter with name 'Labor'.
	 */
	private function initEmptyPlanningDB($omitLabour = FALSE)
	{
		Debugger::log($this->batch_id . "*** PPM - initEmptyPlanningDB start - " . Environment::getUser()->getId(), Debugger::INFO);
		// first delete all remaining rows
		$records = $this->presenter->context->projectPlanningCharterRepository->findByProjectIDAndYear($this->project->ID, $this->year);
		if (count($records) > 0)
			$this->presenter->context->projectPlanningCharterRepository->deleteByProjectIDAndYear($this->project->ID, $this->year);
		
		$projectID = $this->project->ID;
		$planningForYear = $this->year;
		
		// check again if planning for this year doesn't exist -
		// prevent possible DB inconsistencies
		if (in_array($planningForYear, $this->presenter->context->projectPlanningCharterRepository->findYearsByProjectID($projectID)))
			$this->presenter->redirect('Project:detail', array('id' => $projectID, 'year' => $planningForYear));
		
		// init planning
		// Labour Task/WP
		if (!$omitLabour)
		{
			$planningCharter = new ProjectPlanningCharter();
			$planningCharter->projectID = $projectID;
			$planningCharter->year = (int)$planningForYear;
			$planningCharter->name = 'Labor';
			$planningCharter->type = 'HRS';

			$this->presenter->context->projectPlanningCharterRepository->save($planningCharter);
		}
		
		// Other - MAT
		$this->saveMATorTRV('MAT');
		
		// Other - TRV
		$this->saveMATorTRV('TRV');
		Debugger::log($this->batch_id . "*** PPM - initEmptyPlanningDB end - " . Environment::getUser()->getId(), Debugger::INFO);
	}
	
	
	/**
	 * Inserts new subcontractor into planning temporary (as it only holds unsaved data)
	 * database.
	 * 
	 * @param int $subcontractorID Subcontractor's ID.
	 * @param array $budgetArray An array indexed like this: <1, 12> => a value in $
	 * @param int Cost Share.
	 */
	private function saveSubcontractor($subcontractorID, $budgetArray = NULL, $costShare = NULL)
	{
		Debugger::log($this->batch_id . "*** PPM - saveSubcontractor start - " . Environment::getUser()->getId(), Debugger::INFO);
		$projectPlanningCharter = new ProjectPlanningCharter;
		$projectPlanningCharter->ProjectID = $this->project->ID;
		$projectPlanningCharter->Year = $this->year;
		$projectPlanningCharter->Name = '';
		$projectPlanningCharter->Type = ProjectPlanningCharter::PROJECT_PLANNING_CHARTER_TYPE_SUB;

		$this->presenter->context->projectPlanningCharterRepository->save($projectPlanningCharter);

		$projectPlanningCharterItem = new ProjectPlanningCharterItem;
		$projectPlanningCharterItem->ProjectPlanningCharterID = $projectPlanningCharter->ID;
		$projectPlanningCharterItem->Type = ProjectPlanningCharterItem::PROJECT_PLANNING_CHARTER_ITEM_TYPE_SUBCONTRACTOR;
		$projectPlanningCharterItem->CostShare = ($costShare === NULL) ? 0.0 : $costShare;
		$projectPlanningCharterItem->SubcontractorID = $subcontractorID;

		$this->presenter->context->projectPlanningCharterItemRepository->save($projectPlanningCharterItem);


		for ($i = 1; $i < 13; $i++)
		{
			$budget = new ProjectPlanningCharterItemBudget;
			$budget->ProjectPlanningCharterItemID = $projectPlanningCharterItem->ID;
			$budget->Month = $i;
			$budget->Budget = ($budgetArray === NULL) ? 0.0 : $budgetArray[$i];
			$budget->BudgetType = ProjectPlanningCharterItemBudget::PROJECT_PLANNING_CHARTER_ITEM_BUDGET_TYPE_USD;
			$this->presenter->context->projectPlanningCharterItemBudgetRepository->save($budget);
		}
		Debugger::log($this->batch_id . "*** PPM - saveSubcontractor end - " . Environment::getUser()->getId(), Debugger::INFO);
	}
	
	/**
	 * Inserts new material or travel into planning temporary (as it only holds unsaved data)
	 * database.
	 * 
	 * @param string $type 'MAT' or 'TRV'
	 * @param array $budgetArray An array indexed like this: <1, 12> => a value in $
	 * @param bool $updateMode Specifies whether to save a new or just update an existing item.
	 */
	private function saveMATorTRV($type, $budgetArray = NULL, $updateMode = FALSE, $costShare = NULL)
	{
		Debugger::log($this->batch_id . "*** PPM - saveMATorTRV start - " . Environment::getUser()->getId(), Debugger::INFO);
		if ($type !== 'MAT' && $type !== 'TRV')
			throw new InvalidArgumentException;
		
		if (!$updateMode)
		{
			$planningCharter = new ProjectPlanningCharter();
			$planningCharter->projectID = $this->project->ID;
			$planningCharter->year = $this->year;
			$planningCharter->name = '';
			$planningCharter->type = $type;
			$this->presenter->context->projectPlanningCharterRepository->save($planningCharter);

			$projectPlanningCharterItem = new ProjectPlanningCharterItem();
			$projectPlanningCharterItem->projectPlanningCharterID = $planningCharter->ID;
			$projectPlanningCharterItem->type = 'Other';
			$projectPlanningCharterItem->costShare = 0;
			$this->presenter->context->projectPlanningCharterItemRepository->save($projectPlanningCharterItem);
		}
		else
		{
			$planningCharter = $this->presenter->context->projectPlanningCharterRepository->findByProjectIDAndYearAndType($this->project->ID, $this->year, $type);
			$planningCharter = $planningCharter[0];
			$projectPlanningCharterItem = $this->presenter->context->projectPlanningCharterItemRepository->findByProjectPlanningCharterID($planningCharter->ID);
			$projectPlanningCharterItem = $projectPlanningCharterItem[0];
			$projectPlanningCharterItem->costShare = ($costShare === NULL) ? 0 : $costShare;
			$this->presenter->context->projectPlanningCharterItemRepository->update($projectPlanningCharterItem);
			
			$projectPlanningCharterItemBudgetArr = $this->presenter->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($projectPlanningCharterItem->ID);
		}

		for ($i = 1; $i < 13; $i++)
		{
			if (!$updateMode)
				$projectPlanningCharterItemBudget = new ProjectPlanningCharterItemBudget();
			else
				$projectPlanningCharterItemBudget = $projectPlanningCharterItemBudgetArr[$i];
			$projectPlanningCharterItemBudget->projectPlanningCharterItemID = $projectPlanningCharterItem->ID;
			$projectPlanningCharterItemBudget->month = $i;
			$projectPlanningCharterItemBudget->budget = ($budgetArray === NULL) ? 0.0 : $budgetArray[$i];
			$projectPlanningCharterItemBudget->budgetType = 'USD';
			if (!$updateMode)
				$this->presenter->context->projectPlanningCharterItemBudgetRepository->save($projectPlanningCharterItemBudget);
			else
				$this->presenter->context->projectPlanningCharterItemBudgetRepository->update($projectPlanningCharterItemBudget);
		}
		Debugger::log($this->batch_id . "*** PPM - saveMATorTRV end - " . Environment::getUser()->getId(), Debugger::INFO);
	}
	
    /**
     * Load values to projectPlanning component
     */
    public function handleAjaxCall()
    {
		$this->presenter->payload->loadPlanningDBFromProject = true;
		$this->presenter->invalidateControl('ProjectPlanningComponent');
    }
}