<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for administration of organization units.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class OrganizationPresenter extends SecuredPresenter
{
	/** @var session */
    protected $session;
	/** @var */
	private $load_form_state;
	/** Array Array of all cost centers*/
	private $cachedWWorkCenters;
	
	/** @const Setting form options */
	const SETTING_HIGHLIGHT = 'Highlight',
		  SETTING_SHOW_LEADERS = 'ShowLeaders',
		  SETTING_SHOW_STAFF = 'ShowStaff',
		  SETTING_SHOW_INACTIVE = 'ShowInactive';
	
	public function startup()
	{
		parent::startup();
		$this->cachedWWorkCenters = $this->context->costCenterRepository->findAllCostCenters();
		$this->session = $this->getSession('Organization');
        $this->session->setExpiration(0);
		if (!isset($this->session->visibleNodes))
			$this->session->visibleNodes = array();
	}


	/**
	 * Service for autocomplete
	 */
	public function handleAutoComplete()
    {
		$this->payload->autoComplete = array();

		$text = trim($_GET['q']);

		if ($text !== '')
		{
			$filter = new PersonFilter();
			$filter->setFirstname($text)->setSurname($text)->setID($text);
			$persons = $this->context->personRepository->getPersonsByOrFilter($filter);
			foreach ($persons as $person) 
			{
				$person_item = array($person->getReversedFullNameWithEID(), strtoupper(trim($person->ID)), $person->getReversedFullName());
				$this->payload->autoComplete[] = $person_item;
			}
		}

		$this->terminate();
	}
	
	/**
	 * Sets visibility of staff
	 * @param int $id Organization unit id
	 */
	public function handleSetVisibility($id)
	{
		if (in_array($id, $this->session->visibleNodes))
		{
			$key = array_search($id, $this->session->visibleNodes);
			if ($key !== FALSE)
				unset($this->session->visibleNodes[$key]);
		}
		else
			$this->session->visibleNodes[] = $id;
		
		$this->load_form_state = TRUE;
	}
	
	/**
	 * Shows organization unit chart
	 */
	public function renderChart()
	{
		$this->template->form = $this['chartSettingForm'];
		if ($this->isAjax() && !$this->load_form_state)
		{
			$this->context->formStateRepository->save('chartSettingForm', $this->template->form); // save setting state
		}
		else
		{
			if ($this->context->formStateRepository->exists('chartSettingForm')) // exists saved filter
				$this->context->formStateRepository->load('chartSettingForm', $this->template->form);
		}
		
		// Show staff?
		if ($this->template->form['ShowStaff']->getValue())
		{
			$this->session->visibleNodes = array();
			$all_org_units = $this->context->organizationUnitRepository->findAll(true);
			foreach ($all_org_units as $org_unit)
		        $this->session->visibleNodes[] = $org_unit->ID;
			
			$this->template->form['ShowStaff']->setValue(FALSE);
			$this->context->formStateRepository->save('chartSettingForm', $this->template->form);
		}
		
		$showInactiveOrgUnits = false;
		if ($this->template->form['ShowInactiveOrgUnits']->getValue())
			$showInactiveOrgUnits = true;
		
		$chart = array();		
		// fetch all SBE
		$all_SBE = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE, $showInactiveOrgUnits);
		foreach ($all_SBE as $sbe)
		{
			// init number of end nodes under current node
			$sbe_end_nodes = 0;
			
			// fetch related departments
			$departments = array();
			$related_departments = $this->context->organizationUnitRepository->findAllChildren($sbe->ID, $showInactiveOrgUnits);
			foreach ($related_departments as $department)
			{
				if ($sbe->ID == $department->ID) // avoid of duplication
					continue;
				
				// init number of end nodes under current node
			    $department_end_nodes = 0;
				
				// fetch related clusters
				$clusters = array();
				$related_clusters = $this->context->organizationUnitRepository->findAllChildren($department->ID, $showInactiveOrgUnits);
				foreach ($related_clusters as $cluster)
				{
					// fetch related groups
					$groups = array();
					$related_groups = $this->context->organizationUnitRepository->findAllChildren($cluster->ID, $showInactiveOrgUnits);
					foreach ($related_groups as $group)
					{
						$load_staff = in_array($group->ID, $this->session->visibleNodes);
						$groups[] = new OrganizationChartNode($group, $load_staff);
					}
					
					$cluster_end_nodes = count($groups); // init number of end nodes under current node
					$department_end_nodes += ($cluster_end_nodes) ? $cluster_end_nodes : 1; // cluster has no successor, but department has at least this cluster
					$load_staff = in_array($cluster->ID, $this->session->visibleNodes);
					$clusters[] = new OrganizationChartNode($cluster, $load_staff, $groups, $cluster_end_nodes);
				}
				
				$sbe_end_nodes += ($department_end_nodes) ? $department_end_nodes : 1; // department has no successor, but sbe has at least this department
				$load_staff = in_array($department->ID, $this->session->visibleNodes);
				$departments[] = new OrganizationChartNode($department, $load_staff, $clusters, $department_end_nodes);
			}
			
			$load_staff = in_array($sbe->ID, $this->session->visibleNodes);
			$chart[] = new OrganizationChartNode($sbe, $load_staff, $departments, $sbe_end_nodes);
		}
		
		$this->template->chart = $chart;
		$this->template->visibleNodes = $this->session->visibleNodes;
		$this->template->showInactive = $this->template->form['ShowInactive']->getValue();
		
		if ($this->isAjax())
			$this->invalidateControl('chart');
	}
	
	/**
	 * Shows detail of organization unit
	 * @param int $id Should be ID of organization unit
	 */
	public function renderDetail($id)
	{
		$orgUnit = $this->context->organizationUnitRepository->find($id);
		if (!$orgUnit)
		{
			$this->flashMessage('Invalid organization unit ID. You have been redirected.', 'warning');
			$this->redirect('Organization:chart');
		}
		
		$form = $this['orgUnitForm'];
		
		// set form default values
		$defaults = array(
			'ID' => $orgUnit->ID,
			'Virtual' => $orgUnit->Virtual,
			'Type' => $orgUnit->Type,
			'Name' => $orgUnit->Name,
			'Short' => $orgUnit->Short,
			'Leader' => $orgUnit->Leader,
			'ActingLeader' => $orgUnit->ActingLeader
			//'CostCenter' => $orgUnit->CostCenterID,
		);
		if($orgUnit->Status)
			$defaults['Status'] = 'Active';
		else
			$defaults['Status'] = 'Inactive';

		// get leader info
		$leader = $this->context->personRepository->find($orgUnit->Leader);
		$defaults['Leader_info'] = ($leader) ? $leader->ReversedFullName : NULL;
		// determine parent organization unit
		$parentOrgUnit = $this->context->organizationUnitRepository->find($orgUnit->Parent_ID);
		$type = ($parentOrgUnit) ? $parentOrgUnit->Type : NULL;
		switch ($type)
		{
			case OrganizationUnit::ORG_UNIT_TYPE_SBE:
				$defaults['OrganizationUnitSelector']['SBE'] = $orgUnit->Parent_ID;
				break;
			case OrganizationUnit::ORG_UNIT_TYPE_DEP:
				$defaults['OrganizationUnitSelector']['Department'] = $orgUnit->Parent_ID;
				break;
			case OrganizationUnit::ORG_UNIT_TYPE_CLUS:
				$defaults['OrganizationUnitSelector']['Cluster'] = $orgUnit->Parent_ID;
				break;
			default: break;
		}
		$form->setDefaults($defaults);
		
		$this->template->chartNode = new OrganizationChartNode($orgUnit);
	}


	/**
     * Creates form for setting of organization chart.
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
	public function createComponentChartSettingForm($name)
	{
		$form = new AppForm($this, $name);
		
		$form->addCheckbox('Highlight', 'Highlight Relations');
		$form->addCheckbox('ShowStaff', 'Show Staff');
		$form->addCheckbox('ShowInactive', 'Show Inactive Staff');
		$form->addCheckbox('ShowInactiveOrgUnits', 'Show Inactive Organization Units');
		
		$form->addSubmit('Reset', 'Reset Settings')
				->onClick[] = callback($this, 'resetChartSettingFormSubmitted');
		
		$form->addSubmit('Create', 'Create New Organization Unit')
				->onClick[] = callback($this, 'createChartSettingFormSubmitted');
		if (!$this->isAllowed(self::RESOURCE_ORGANIZATION, self::ACTION_CREATE))
            $form['Create']->setDisabled(TRUE);
		
		$form->onSuccess[] = array($this, 'chartSettingFormSubmitted');
		
		return $form;
	}
	
	/**
	 * Service for Reset button of ChartSettingForm
	 * @param SubmitButton $form
	 */
	public function resetChartSettingFormSubmitted(SubmitButton $button)
	{
		$this->context->formStateRepository->drop('chartSettingForm');
		unset($this->session->visibleNodes);
		$this->redirect('Organization:chart');
	}
	
	/**
	 * Service for Create button of ChartSettingForm
	 * @param SubmitButton $form
	 */
	public function createChartSettingFormSubmitted(SubmitButton $form)
	{
		$this->redirect('Organization:create');
	}
	
	/**
	 * Service for auto-submit of ChartSettingForm
	 * @param AppForm $form
	 */
	public function chartSettingFormSubmitted(AppForm $form)
	{
		if ($form->isValid())
		{
			if ($this->isAjax())
			{
			    $this->payload->setting = $form->getValues();
				$this->invalidateControl('chart');
			}
			else
				$this->redirect('Organization:chart');
		}
	}
	
	/**
     * Creates form for creation of new organization unit.
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
	public function createComponentOrgUnitForm($name)
	{
		$form = new AppForm($this, $name);
		
		$form->addHidden('ID');
		
		$form->addCheckbox('Virtual', HPTForm::createLabel('Virtual', 'Determines fact, that unit does not exist in real organizational structure. Virtual unit cannot be used for project creation and cannot be set as Displayed content in person detail.'));
		
		$form->addSelect('Type', 'Type', OrganizationUnit::getFullTypeNames())
				->setPrompt('')
				->addRule(Form::FILLED, 'Type must be filled.');
		
		$form['Type']->addCondition(Form::EQUAL, OrganizationUnit::ORG_UNIT_TYPE_DEP)
				->toggle('OrganizationUnitSelector-SBE');
		
		$form['Type']->addCondition(Form::EQUAL, OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				->toggle('OrganizationUnitSelector-Department');
		
		$form['Type']->addCondition(Form::EQUAL, OrganizationUnit::ORG_UNIT_TYPE_TG)
				->toggle('OrganizationUnitSelector-Cluster');
		
		$form['OrganizationUnitSelector'] = new OrganizationUnitSelectorContainer(215, FALSE);
		
		$form['OrganizationUnitSelector']['SBE']->addConditionOn($form['Type'], Form::EQUAL, OrganizationUnit::ORG_UNIT_TYPE_DEP)
				->addRule(Form::FILLED, 'Parent SBE must be filled.');
		
		$form['OrganizationUnitSelector']['Department']->addConditionOn($form['Type'], Form::EQUAL, OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				->addRule(Form::FILLED, 'Parent Department must be filled.');
		
		$form['OrganizationUnitSelector']['Cluster']->addConditionOn($form['Type'], Form::EQUAL, OrganizationUnit::ORG_UNIT_TYPE_TG)
				->addRule(Form::FILLED, 'Parent Cluster must be filled.');
		
		$form->addText('Name', 'Name', 31, 100)
				->addRule(Form::FILLED, 'Name must be filled.');
		
		$form->addText('Short', 'Short Name', 31, 20);
		
		$form->addText('Leader', 'Leader', 7, 100)
				->addRule(Form::FILLED, 'Leader must be filled.');
		        $form['Leader']->setAttribute('class', 'autocomplete');
		$form->addText('Leader_info', NULL)
			->setAttribute('readonly', 'readonly');

		$form->addCheckbox('ActingLeader', HPTForm::createLabel('Acting Leader', 'Informative mark, used in case that selected leader of organization unit acts temporarily as leader on behalf of someone else.'));
		
		$form->addRadioList('Status', HPTForm::createLabel('Status', 'Status change will be done recursively for all subordinate units.'), array('Active' => 'Active', 'Inactive' => 'Inactive'));
		$form['Status']->getSeparatorPrototype()->setName(NULL);
        
//        $cost_centers = array();
//        $all_cost_centers = $this->context->costCenterRepository->findAllCostCenters();
//        foreach ($all_cost_centers as $cost_center)
//        {
//            $cost_centers[$cost_center->ID] = $cost_center->Name . " (" . $cost_center->ID . ")";
//        }
//        $form->addSelect('CostCenter', 'Work center', $cost_centers)
//                ->setPrompt('')
//				->addRule(Form::FILLED, 'Work center be filled.');
//        $form['CostCenter']->setAttribute('style', 'width: 215px;');
		
		$form->addSubmit('Back', 'Back To Organization Chart')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'backOrgUnitFormSubmitted');
		
		$form->addSubmit('Create', 'Create')
				->onClick[] = array($this, 'createOrgUnitFormSubmitted');
		if (!$this->isAllowed(self::RESOURCE_ORGANIZATION, self::ACTION_CREATE))
            $form['Create']->setDisabled(TRUE);
		
		$form->addSubmit('Save', 'Save')
				->onClick[] = array($this, 'saveOrgUnitFormSubmitted');
		if (!$this->isAllowed(self::RESOURCE_ORGANIZATION, self::ACTION_EDIT))
            $form['Save']->setDisabled(TRUE);
		
		$form->addSubmit('Delete', 'Delete')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'deleteOrgUnitFormSubmitted');
		        $form['Delete']->setAttribute('onclick', 'return confirm("Are you sure to delete this Organization Unit?");');
		if (!$this->isAllowed(self::RESOURCE_ORGANIZATION, self::ACTION_DELETE))
            $form['Delete']->setDisabled(TRUE);
		
		return $form;
	}
	
	/**
	 * Service for Back button of OrgUnitForm
	 * @param SubmitButton $form
	 */
	public function backOrgUnitFormSubmitted(SubmitButton $button)
	{
		$this->redirect('Organization:chart');
	}
	
	/**
	 * Service for Create button of OrgUnitForm
	 * @param SubmitButton $form
	 */
	public function createOrgUnitFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues();
			
			$orgUnit = new OrganizationUnit();
			$orgUnit->Virtual = ($entry->Virtual) ? 1 : 0;
			$orgUnit->Type = $entry->Type; 
			$orgUnit->Name = $entry->Name;
			$orgUnit->Short = $entry->Short;
			$orgUnit->Leader = ($entry->Leader) ? $entry->Leader : NULL;
			$orgUnit->ActingLeader = $entry->ActingLeader;
			//$orgUnit->CostCenterID = ($entry->CostCenter) ? $entry->CostCenter : NULL;
			
			switch ($entry->Type)
			{
				case OrganizationUnit::ORG_UNIT_TYPE_DEP:
					$orgUnit->Parent_ID = $entry->OrganizationUnitSelector->SBE;
					break;
				case OrganizationUnit::ORG_UNIT_TYPE_CLUS:
					$orgUnit->Parent_ID = $entry->OrganizationUnitSelector->Department;
					break;
				case OrganizationUnit::ORG_UNIT_TYPE_TG:
					$orgUnit->Parent_ID = $entry->OrganizationUnitSelector->Cluster;
					break;
				default: break;
			}

			if ($this->context->organizationUnitRepository->save($orgUnit))
			{
				$sbe_parent = TRUE;
				if ($entry->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE)
				{
					$orgUnit->Parent_ID = $orgUnit->ID;
					$sbe_parent = $this->context->organizationUnitRepository->update($orgUnit);
				}
				
				if ($sbe_parent)
				    $this->flashMessage('Organization Unit ' . $orgUnit->Name . ' saved.', 'success');
				else
					$this->flashMessage('Can not assigned itself as parent organization.', 'error');
				// set display content for persons belonginf under 3320 or 3330 plants
				if ($entry->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE && !$orgUnit->Virtual)
				{
					$persons = $this->context->personRepository->findAll();
					$success = TRUE;
					foreach ($persons as $person)
					{	
						if($this->cachedWWorkCenters[$person->CostCenterID]->Plant == 3330 || $this->cachedWWorkCenters[$person->CostCenterID]->Plant == 3320)
						{
							$setting = new SettingDisplayedContent();
							$setting->EID = $person->ID;
							$setting->OrganizationUnitID = $orgUnit->ID;
							$success = $success && $this->context->settingDisplayedContentRepository->save($setting);
						}
					}
				}
				$this->redirect('Organization:detail', array('id' => $orgUnit->ID));
			}
			else
			{
				$this->flashMessage('Saving failed.', 'error');
				return;
			}
		}
		$this->flashMessage('Invalid form values.', 'error');
		return;
	}
	
	/**
	 * Service for Save button of OrgUnitForm
	 * @param SubmitButton $form
	 */
	public function saveOrgUnitFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues();
			
			$orgUnit = $this->context->organizationUnitRepository->find($entry->ID);
			if (!$orgUnit)
			{
				$this->flashMessage('Invalid organization unit ID. You have been redirected.', 'warning');
			    $this->redirect('Organization:chart');
			}
			$orgUnit->Virtual = ($entry->Virtual) ? 1 : 0;
			$orgUnit->Type = $entry->Type; 
			$orgUnit->Name = $entry->Name;
			$orgUnit->Short = $entry->Short;
			$orgUnit->Leader = ($entry->Leader) ? $entry->Leader : NULL;
			$orgUnit->ActingLeader = $entry->ActingLeader;
			
			if($entry['Status'] == 'Active')
				$newStatus = 1;
			else
				$newStatus = 0;
			if($newStatus ==! (int)$orgUnit->Status)
			{
				$orgUnit->Status = $newStatus;
				// for SBE check active or planned projects.
				if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE)
				{
					// check if org units have some active or planned project. Org units status cannot be changed, if active or planned projets have been found.
					if($this->context->projectRepository->hasOrgUnitActiveOrPlannedProject($orgUnit->ID))
					{
						$flashMessage = 'Organization unit status cannot be changed. Organization Unit: ' . $orgUnit->Name . ' has active or planned project/projects.';
						$this->flashMessage($flashMessage, 'error');
						$this->redirect('Organization:detail', array('id' => $orgUnit->ID));
					}
				}
				$orgUnitChildren = $this->context->organizationUnitRepository->findAllChildrenByPath($orgUnit->Path, true); // Returns also parent unit
				// check if org units have some active person. Org units status cannot be changed, if active persons have been found.
				$proceed = true;
				
				$orgUnitsName = array();
				foreach($orgUnitChildren as $childUnit)
				{
					if($this->hasOrgUnitActivePerson($childUnit))
					{
						$proceed = false;
						$orgUnitsName[] = $childUnit->Name;
					}
				}
				if(!$proceed)
				{
					$flashMessage = 'Organization unit status cannot be changed. Organization Unit(s): ' . implode(', ', $orgUnitsName) . ' has/have person assigned to the unit.';
					$this->flashMessage($flashMessage, 'error');
					$this->redirect('Organization:detail', array('id' => $orgUnit->ID));
				}
				unset($orgUnitChildren[$orgUnit->ID]);
				if($orgUnitChildren)
				{
					$multilog = new MultiLogger();
					$batch_id = 'ORG-UNIT-STATUS-'.time().'-'. rand(1, 9);
					$multilog->add($this->context->createLogger($batch_id, "Organization Unit"));
					foreach($orgUnitChildren as $childUnit)
					{
						$childUnit->Status = $orgUnit->Status;
						if($this->context->organizationUnitRepository->update($childUnit))
						{
							$multilog->log('Recursive organization unit: ' . $childUnit->Name . ' status change was successful.', ILogger::INFO);
						}
						else
						{
							$multilog->log('Recursive organization unit: ' . $childUnit->Name . 'status change was not successful.', ILogger::WARNING);
						}
					}
					$this->flashMessage("Recursive organization units status were changed. See log for more information.", 'notice');
				}
				else
				{
					$this->flashMessage("No recursive organization units have been found.", 'notice');
				}
			}

			//$orgUnit->CostCenterID = ($entry->CostCenter) ? $entry->CostCenter : NULL;
			
			switch ($entry->Type)
			{
				case OrganizationUnit::ORG_UNIT_TYPE_SBE:
					$orgUnit->Parent_ID = $orgUnit->ID;
					break;
				case OrganizationUnit::ORG_UNIT_TYPE_DEP:
					$orgUnit->Parent_ID = $entry->OrganizationUnitSelector->SBE;
					break;
				case OrganizationUnit::ORG_UNIT_TYPE_CLUS:
					$orgUnit->Parent_ID = $entry->OrganizationUnitSelector->Department;
					break;
				case OrganizationUnit::ORG_UNIT_TYPE_TG:
					$orgUnit->Parent_ID = $entry->OrganizationUnitSelector->Cluster;
					break;
				default: break;
			}
			
			if ($this->context->organizationUnitRepository->update($orgUnit))
			{
				// set display content for persons belonginf under 3320 or 3330 plants
				if ($entry->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE && !$orgUnit->Virtual)
				{
					$persons = $this->context->personRepository->findAll();
					$success = TRUE;
					foreach ($persons as $person)
					{	
						if($this->cachedWWorkCenters[$person->CostCenterID]->Plant == 3330 || $this->cachedWWorkCenters[$person->CostCenterID]->Plant == 3320)
						{
							$setting = new SettingDisplayedContent();
							$setting->EID = $person->ID;
							$setting->OrganizationUnitID = $orgUnit->ID;
							$success = $success && $this->context->settingDisplayedContentRepository->save($setting);
						}
					}
				}
				$this->flashMessage('Organization Unit ' . $orgUnit->Name . ' updated.', 'success');
				$this->redirect('Organization:detail', array('id' => $orgUnit->ID));
			}
			else
			{
				$this->flashMessage('Updating failed.', 'error');
				return;
			}
		}
		$this->flashMessage('Invalid form values.', 'error');
		return;
	}
	
	/**
	 * Service for Delete button of OrgUnitForm
	 * @param SubmitButton $form
	 */
	public function deleteOrgUnitFormSubmitted(SubmitButton $button)
	{
		$entry = $button->form->getValues();
		
		$selectedOrgUnit = $this->context->organizationUnitRepository->find($entry->ID);
		if (!$selectedOrgUnit)
		{
			$this->flashMessage('Invalid organization unit ID. You have been redirected.', 'warning');
		    $this->redirect('Organization:chart');
		}
		$proceedActivePerson = true;
		$orgUnitsName = array();
		
		$orgUnitChildren = $this->context->organizationUnitRepository->findAllChildrenByPath($selectedOrgUnit->Path, true);
		foreach($orgUnitChildren as $orgUnit)
		{
			switch ($orgUnit->Type)
			{
				case OrganizationUnit::ORG_UNIT_TYPE_SBE:
					// check if org units have some active or planned project. Org units cannot be deleted, if active or planned projets have been found.
					if($this->context->projectRepository->hasOrgUnitActiveOrPlannedProject($orgUnit->ID))
					{
						$flashMessage = 'Organization unit cannot be deleted. Organization Unit: ' . $orgUnit->Name . ' has active or planned project/projects.';
						$this->flashMessage($flashMessage, 'error');
						$this->redirect('Organization:detail', array('id' => $selectedOrgUnit->ID));
					}
					if($this->hasOrgUnitPerson($orgUnit))
					{
						$proceedActivePerson = false;
						$orgUnitsName[] = $orgUnit->Name;
					}
					break;
				case OrganizationUnit::ORG_UNIT_TYPE_DEP:
					if($this->hasOrgUnitPerson($orgUnit))
					{
						$proceedActivePerson = false;
						$orgUnitsName[] = $orgUnit->Name;
					}
					break;
				case OrganizationUnit::ORG_UNIT_TYPE_CLUS:
					if($this->hasOrgUnitPerson($orgUnit))
					{
						$proceedActivePerson = false;
						$orgUnitsName[] = $orgUnit->Name;
					}
					break;
				case OrganizationUnit::ORG_UNIT_TYPE_TG:
					if($this->hasOrgUnitPerson($orgUnit))
					{
						$proceedActivePerson = false;
						$orgUnitsName[] = $orgUnit->Name;
					}
					//if org unit is TG - check project charter budgets and STRAP Items
					if($this->context->projectCharterBudgetRepository->findProjectCharterBudgetByOrgUnitId($orgUnit->ID))
					{
						$flashMessage = 'Organization unit cannot be deleted. Organization Unit: ' . $orgUnit->Name . ' is used by Project Charter Tool.';
						$this->flashMessage($flashMessage, 'error');
						$this->redirect('Organization:detail', array('id' => $selectedOrgUnit->ID));
					}
					if($this->context->itemRepository->findAllByTechnicalGroupID($orgUnit->ID))
					{
						$flashMessage = 'Organization unit cannot be deleted. Organization Unit: ' . $orgUnit->Name . ' is used by STRAP Tool.';
						$this->flashMessage($flashMessage, 'error');
						$this->redirect('Organization:detail', array('id' => $selectedOrgUnit->ID));
					}
			}
		}

		if(!$proceedActivePerson)
		{
			$flashMessage = 'Organization unit cannot be deleted. Organization Unit(s): ' . implode(', ', $orgUnitsName) . ' has/have person assigned to the unit.';
			$this->flashMessage($flashMessage, 'error');
			$this->redirect('Organization:detail', array('id' => $selectedOrgUnit->ID));
		}
		
		foreach($orgUnitChildren as $orgUnit)
		{
			if ($this->context->organizationUnitRepository->delete($orgUnit->ID))
			{
				$this->flashMessage('Organization Unit ' . $orgUnit->Name . ' deleted.', 'success');
			}
			else
			{
				$this->flashMessage('Deleting ' . $orgUnit->Name . ' failed.', 'error');
			}
		}
		$this->redirect('Organization:chart');
	}
	
	/**
	 * Returns list of ids of subordinate organization units of given type.
	 * @param int $id organization unit ID.
	 * @param string $type Type of organization unit in range of OrganizationUnit::ORG_UNIT_TYPE_*.
	 * @return array Of children ids or empty array.
	 */
	private function getChildrenIDList($id, $type)
	{
		$all_children = $this->context->organizationUnitRepository->findAllChildrenByType($id, $type);
		$children_id_list = array();
		foreach ($all_children as $child)
			$children_id_list[] = $child->ID;
		
		return $children_id_list;
	}
	
	/**
	 * Finds out if org unit has some active person.
	 * 
	 * @param OrganizationUnit $orgUnit Organization unit object
	 * @return TRUE if any active person has been found, otherwise false.
	 */
	private function hasOrgUnitActivePerson($orgUnit)
	{
		$persons = $this->context->personRepository->findByOrganizationUnitID($orgUnit->ID);
		foreach($persons as $person)
		{
			if($person->Status)
			{
				return TRUE;
			}
		}
		return FALSE;
	}
	
	/**
	 * Finds out if org unit has some person.
	 * 
	 * @param OrganizationUnit $orgUnit Organization unit object
	 * @return TRUE if any person has been found, otherwise false.
	 */
	private function hasOrgUnitPerson($orgUnit)
	{
		$persons = $this->context->personRepository->findByOrganizationUnitID($orgUnit->ID);
		foreach($persons as $person)
		{
			return TRUE;
		}
		return FALSE;
	}
}
