<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for administration of project charters approvals.
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterApprovalPresenter extends SecuredPresenter
{
    /** @var session */
    protected $session;
	/** @var string Prefix for main directory of one project charter. */
    protected $charter_dir_prefix;
    /** @var string Prefix for directory containing one version of project charter file. */
    protected $version_dir_prefix;
    /** @var string Prefix for temporary file name of charter. */
    protected $charter_temp_prefix;
    /** @var string Path to storage of project charter files. */
    protected $charters_storage_dir;
    /** @var string Path to temporary storage of project charter files. */
    protected $charters_temp_storage_dir;
    /** @const DEFAULT_APPROVERROWS_NUMBER Default number of available rows for approver definition form */
    const DEFAULT_APPROVERROWS_NUMBER = 0;
	
    /**
     * Returns path to file.
     * @param int $projectCharterID
     * @param int $version File version
     * @param string $fileName File name
     * @return string
    */
    protected function getPathToFile($projectCharterID, $version, $fileName)
    {
        $storage = 
            '/data/actual/charters/' .
            $this->charter_dir_prefix . $projectCharterID . '/' .
            $this->version_dir_prefix . $version . '/'
        ;

        return ($storage . $fileName);
    }
	
	/**
	 * Returns path to summary file.
	 * @param int $projectCharterID
	 * @param int $version File version
	 * @param string $fileName File name
	 * @return string
	 */
	protected function getPathToSummaryFile($projectCharterID, $version, $fileName)
	{
		$storage =
			'/data/actual/charters/' .
			$this->charter_dir_prefix . $projectCharterID . '/summary/' .
			$this->version_dir_prefix . $version . '/'
		;

		return ($storage . $fileName);
	}
	
	/**
	 * Restricts access to methods which requires specific combination of user role and project charter status.
	 * @param int $id Must be positive integer. It should be ID of an existing project charter.
	 * @param array $statuses Array of possible statuses for access.
	 * @param bool $isOwner Determine if currently logged in user must be in owner role or in role of line manager.
	*/
	public function secureAccess($id, $statuses, $isOwner)
	{
		$charter = $this->context->projectCharterRepository->find($id);
		if ($charter)
		{
			$user = strtolower($this->getUser()->getId());
			$watchdog = FALSE;

			if ($isOwner)
			{
				// must be owner
				if ($user !== strtolower($charter->getOwner()))
				    $watchdog = TRUE;
			}
			else
			{
				// must be line manager
				if ($user !== strtolower($charter->getLineManager()))
				    $watchdog = TRUE;
			}

			if (!in_array($charter->getStatus(), $statuses))
				$watchdog = TRUE;

			if ($watchdog)
			{
				$this->flashMessage('Process is not in the right phase or you have invalid credentials for manipulation with this project charter. You have been redirected.', 'warning');
				$this->redirect('Dashboard:default');
			}
		}
		else
		{
			$this->flashMessage('Invalid project charter ID. You have been redirected.', 'warning');
		    $this->redirect('Dashboard:default');
	    }			
    }
    
	/**
     * Initializes common variables (e.g. paths, sessions)
     */
    public function startup()
    {
        parent::startup();
        // initialization of session
        $this->session = $this->getSession('ProjectCharterApproval');
        $this->session->setExpiration(0);
        if (!isset($this->session['ApproverRows']))
            $this->session['ApproverRows'] = self::DEFAULT_APPROVERROWS_NUMBER;
		if (!isset($this->session['Form']))
            $this->session['Form'] = array();
		// initialization of directory prefixies for storage of project charter files
        $this->charter_dir_prefix = Environment::getConfig('charter_dir_prefix');
        $this->version_dir_prefix = Environment::getConfig('version_dir_prefix');
        // initialization of directory prefix for temporary file name of charter
        $this->charter_temp_prefix = Environment::getConfig('charter_temp_prefix');
        // initialization of path to storage of project charter files
        $this->charters_storage_dir = Environment::getConfig('charters_storage_dir');
        // initialization of path to temporary storage of project charter files
        $this->charters_temp_storage_dir = Environment::getConfig('charters_temp_storage_dir'); 
    }
	
    /**
	 * Loads personal info according to given EID
	 * @param string $eid Should be EID of person.
	 */
	public function actionLoadInfo($eid)
	{		
		try
		{
			$personalInfo = $this->context->ldapQuery->getPersonalInfo($eid);
			$arrPersonalInfo = array();
			$arrPersonalInfo['EID'] = $personalInfo->getEID();
			$arrPersonalInfo['Name'] = $personalInfo->getName();
			$arrPersonalInfo['Surname'] = $personalInfo->getSurname();
			$arrPersonalInfo['FullName'] = $personalInfo->getFullName();
			$arrPersonalInfo['Email'] = $personalInfo->getEmail();
			// email not found
			if (!$personalInfo->getEmail())
				$this->payload->error = TRUE;
			else				
			    $this->payload->personInfo = $arrPersonalInfo;
		}
		catch (Exception $e)
		{
			$this->payload->error = TRUE;
		}
		$this->terminate();
	}
	
	/**
	 * Sets common content of templates
	 */
	public function beforeRender()
    {
        parent::beforeRender();
        // form tool tips
        $this->template->TIP_COST_SHARE = 'Cost share is percentage of costs covered by the customer.';
        $this->template->TIP_AMOUNT_HRS = 'Working hours of technical group per year quarters.';
        $this->template->TIP_SUBCONTRACTORS = 'Amounts for subcontractors per year quarters.';
        $this->template->TIP_OWNER = 'User responsible for creation of project charter.';
        $this->template->TIP_LINE_MANAGER = 'Should be cluster leader (CL) in common case.';
        $this->template->TIP_EMAIL_PREVIEW = 'You can add your own comments to the email. Use the preview button below to view the content of the resulting email.';	
        $this->template->WARNING_EMAIL_PREVIEW = 'Do not forget to preview the email before it is sent to the list of approvers!';	
        $this->template->WARNING_APPROVAL_LOCKED = 'Sending of charters is locked until moment, that is dependent on the policy of charter approval process.';	
		// form labels
        $this->template->percent = '%';
        $this->template->unitHRS = '(HRS)';
        $this->template->unitUSD = '(USD)';
        // project charter statuses
        $this->template->PROJECT_CHARTER_STATUS_APPROVED = ProjectCharter::PROJECT_CHARTER_STATUS_APPROVED;
        $this->template->PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION = ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION;
        $this->template->PROJECT_CHARTER_STATUS_CANCELLED = ProjectCharter::PROJECT_CHARTER_STATUS_CANCELLED;
        $this->template->PROJECT_CHARTER_STATUS_DEFINITION = ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION;
        $this->template->PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL = ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL;
        $this->template->PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL = ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL;
        $this->template->PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL = ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL;
    }
    
    /**
     * Shows page for definition of final approvers
     */
    public function renderDefinition($id = NULL)
    {
		// check accessibility for user
		$this->secureAccess($id, array(ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION), TRUE);
		
        $charter = $this->context->projectCharterRepository->find($id);
        if ($charter == NULL)
        {
            $this->flashMessage('Invalid project charter ID. You have been redirected.', 'warning');
            $this->redirect('Dashboard:default');
        }
        
        $arrCharter = array();
		$arrCharter['Name'] = $charter->getName();
        $arrCharter['SAP_ID'] = $charter->getSAP_ID();
            $owner = $this->context->personRepository->find($charter->getOwner());
		$arrCharter['Owner'] = $owner->getReversedFullNameWithEID();
            $lineManager = $this->context->personRepository->find($charter->getLineManager());
		$arrCharter['LineManager'] = $lineManager->getReversedFullNameWithEID();
			$strapItem = $this->context->itemRepository->find($charter->StrapItemID);
		$arrCharter['StrapItem'] = ($strapItem) ? $strapItem->Name : '-';
		$arrCharter['StrapItemID'] = ($strapItem) ? $strapItem->ID : NULL;
        $arrCharter['Status'] = ProjectCharter::getFullStatusName($charter->getStatus());
        $arrCharter['CreationDateTime'] = $charter->getCreationDateTime();
        $arrCharter['CostShare'] = $charter->getCostSharePercentage();
        $arrCharter['Customer_ID'] = $this->context->customerRepository->find($charter->getCustomerID())->getName();
		    $program = $this->context->customerProgramRepository->find($charter->getProgramID());
        $arrCharter['ProgramID'] = ($program) ? $program->getProgram() : NULL;
		    $call = $this->context->customerProgramCallRepository->find($charter->getCallID());
		$arrCharter['CallID'] = ($call) ? $call->getCall() : NULL;
        $arrCharter['Responsibility_ID'] = $this->context->responsibilityRepository->find($charter->getResponsibilityID())->getName();
        $arrCharter['Year'] = $charter->getYear();
		
		$file = $this->context->projectCharterFileHistoryRepository->findLatestVersion($charter->getID());
        if ($file)
        {
            $this->template->filePath = $this->getPathToFile($file->getProjectCharterID(), $file->getVersion(), $file->getFileName());
            $this->template->fileName = $file->getFileName();
        }
		
		$summaryFile = $this->context->projectCharterSummaryFileHistoryRepository->findLatestVersion($charter->getID());
        if ($summaryFile)
        {
            $this->template->summaryFilePath = $this->getPathToSummaryFile($summaryFile->getProjectCharterID(), $summaryFile->getVersion(), $summaryFile->getFileName());
            $this->template->summaryFileName = $summaryFile->getFileName();
        }
		
		$form = $this['approversDefinitionForm'];
		if (count($this->session['Form']))
        {
            $form->setDefaults($this->session['Form']);
            $this->session['Form'] = array();
        }
		
		$info = array();
		
		// pre fill empty form
		if (!$form['ProjectCharterID']->getValue())
		{
			$row = 0;		
			$this->session['ApproverRows'] = self::DEFAULT_APPROVERROWS_NUMBER;
			$approverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($charter->getID());
			if ($approverHistory)
			{
				$form['Version']->setValue($approverHistory->getVersion());
				
				$approvers = $this->context->projectCharterApproverRepository->findAllApprover($approverHistory->getProjectCharterID(), $approverHistory->getVersion());
				if ($approvers)
				{
					$row = 0;
					foreach ($approvers as $approver)
					{
						$arrApprover = array();
						switch ($approver->getRole())
						{
							case ProjectCharterApprover::APPROVER_ROLE_DEP_LEADER:
								$arrApprover['EID'] = $approver->getApproverEID();
								$arrApprover['Name'] = $approver->getName();
								$arrApprover['Surname'] = $approver->getSurname();
								$arrApprover['FullName'] = $approver->getSurname() . ', ' . $approver->getName();
								$arrApprover['Email'] = $approver->getEmail() ? $approver->getEmail() : 'Not found.';

								$form['DEPLeader_ID']->setValue($approver->getProjectCharterApproverID());
								$form['DEPLeader']->setValue($approver->getApproverEID());
								$form['DEPLeader_info']->setValue(json_encode($arrApprover));
								break;		
							case ProjectCharterApprover::APPROVER_ROLE_ATE_DIRECTOR:
								$arrApprover['EID'] = $approver->getApproverEID();
								$arrApprover['Name'] = $approver->getName();
								$arrApprover['Surname'] = $approver->getSurname();
								$arrApprover['FullName'] = $approver->getSurname() . ', ' . $approver->getName();
								$arrApprover['Email'] = $approver->getEmail() ? $approver->getEmail() : 'Not found.';

								$form['ATEDirector_ID']->setValue($approver->getProjectCharterApproverID());
								$form['ATEDirector']->setValue($approver->getApproverEID());
								$form['ATEDirector_info']->setValue(json_encode($arrApprover));
								break;		
							case ProjectCharterApprover::APPROVER_ROLE_PEC:
							case ProjectCharterApprover::APPROVER_ROLE_PROGRAM_MANAGER:
								$arrApprover['EID'] = $approver->getApproverEID();
								$arrApprover['Name'] = $approver->getName();
								$arrApprover['Surname'] = $approver->getSurname();
								$arrApprover['FullName'] = $approver->getSurname() . ', ' . $approver->getName();
								$arrApprover['Email'] = $approver->getEmail() ? $approver->getEmail() : 'Not found.';

								$form['PEC_ID']->setValue($approver->getProjectCharterApproverID());
								$form['PEC']->setValue($approver->getApproverEID());
								$form['PEC_info']->setValue(json_encode($arrApprover));
								break;		
							case ProjectCharterApprover::APPROVER_ROLE_PMO:
								$arrApprover['EID'] = $approver->getApproverEID();
								$arrApprover['Name'] = $approver->getName();
								$arrApprover['Surname'] = $approver->getSurname();
								$arrApprover['FullName'] = $approver->getSurname() . ', ' . $approver->getName();
								$arrApprover['Email'] = $approver->getEmail() ? $approver->getEmail() : 'Not found.';

								$form['PMO_ID']->setValue($approver->getProjectCharterApproverID());
								$form['PMO']->setValue($approver->getApproverEID());
								$form['PMO_info']->setValue(json_encode($arrApprover));
								break;
							default: 
								$this->addApproverToForm($form, $row);
							
								$arrApprover['EID'] = $approver->getApproverEID();
								$arrApprover['Name'] = $approver->getName();
								$arrApprover['Surname'] = $approver->getSurname();
								$arrApprover['FullName'] = $approver->getSurname() . ', ' . $approver->getName();
								$arrApprover['Email'] = $approver->getEmail();

								$form['Approver' . $row . '_ID']->setValue($approver->getProjectCharterApproverID());
								$form['Approver' . $row . '_role']->setValue($approver->getRole());
								$form['Approver' . $row . '_info']->setValue(json_encode($arrApprover));
								$form['Approver' . $row]->setValue($approver->getApproverEID());

								$row++;
								$this->session['ApproverRows'] = $row;
								break;
						}
					}
				}	
				
			} // probably first definition of approvers for this charter, so pre fill empty rows for M&PM and RTC Director
			else
			{	
				
				// pre fill DEP Leader info
				$parentOrgUnit = NULL;
				$arrOrgUnit = $this->context->organizationUnitRepository->findByTypeAndLeader(OrganizationUnit::ORG_UNIT_TYPE_CLUS, $charter->getLineManager());
				foreach ($arrOrgUnit as $orgUnit)
				{
					$parentOrgUnit = $this->context->organizationUnitRepository->find($orgUnit->getParent_ID());
					break; // first organization unit is enough
				}
				if ($parentOrgUnit)
				{
					try
					{
						$personalInfo = $this->context->ldapQuery->getPersonalInfo($parentOrgUnit->getLeader());
						$depLeader['EID'] = $personalInfo->getEID();
						$depLeader['Name'] = $personalInfo->getName();
						$depLeader['Surname'] = $personalInfo->getSurname();
						$depLeader['FullName'] = $personalInfo->getFullName();
						$depLeader['Email'] = $personalInfo->getEmail() ? $personalInfo->getEmail():'Not found.';
						
						$form['DEPLeader']->setValue($personalInfo->getEID());
						$form['DEPLeader_info']->setValue(json_encode($depLeader));
					} catch (Exception $e) { /* if error occured DEP leader won't be pre filled */ }
				}
				
				try // pre fill ATE Director info
				{
					$personalInfo = $this->context->ldapQuery->getPersonalInfo('E164534');
					$ateDirector['EID'] = $personalInfo->getEID();
					$ateDirector['Name'] = $personalInfo->getName();
					$ateDirector['Surname'] = $personalInfo->getSurname();
					$ateDirector['FullName'] = $personalInfo->getFullName();
					$ateDirector['Email'] = $personalInfo->getEmail() ? $personalInfo->getEmail():'Not found.';
					
					$form['ATEDirector']->setValue($personalInfo->getEID());		
					$form['ATEDirector_info']->setValue(json_encode($ateDirector));
				} catch (Exception $e) { /* if error occured SBE leader won't be pre filled */ }
				
				
				
				$this->addApproverToForm($form, $row);
				$form['Approver' . $row . '_role']->setValue(ProjectCharterApprover::APPROVER_ROLE_MandPM);
				$row++;
				$this->addApproverToForm($form, $row);
				$form['Approver' . $row . '_role']->setValue(ProjectCharterApprover::APPROVER_ROLE_RTC_DIRECTOR);
				$row++;
				$this->session['ApproverRows'] = $row;
			}
			
			// lock form against next pre filling
			$form['ProjectCharterID']->setValue($charter->getID());
		}
		
		// get readonly info
		if ($form['DEPLeader_info']->getValue())
		    $info['DEPLeader'] = (array) json_decode($form['DEPLeader_info']->getValue());
		if ($form['ATEDirector_info']->getValue())
		    $info['ATEDirector'] = (array) json_decode($form['ATEDirector_info']->getValue());
		if ($form['PEC_info']->getValue())
		    $info['PEC'] = (array) json_decode($form['PEC_info']->getValue());
		if ($form['PMO_info']->getValue())
		    $info['PMO'] = (array) json_decode($form['PMO_info']->getValue());

		for ($row = 0; $row < $this->session['ApproverRows']; $row++)
		{
			if ($form['Approver' . $row . '_info']->getValue())
			    $info['Approver' . $row] = (array) json_decode($form['Approver' . $row . '_info']->getValue());
		}		
		
		$this->template->charter = $arrCharter;
		$this->template->info = $info;
		$this->template->approverRows = $this->session['ApproverRows'];
    }
	
	/**
	 * Shows page with form for editation of email
	 * @param int $id Should be project charter ID.
	 */
	public function renderEmail($id = NULL)
	{
		// check accessibility for user
		$this->secureAccess($id, array(ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION), TRUE);
		
		$approverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($id);
		if ($approverHistory == NULL)
		{
			$this->flashMessage('Invalid project charter ID. You have been redirected.', 'warning');
            $this->redirect('Dashboard:default');
		}
		$charter = $this->context->projectCharterRepository->find($approverHistory->getProjectCharterID());
		if ($charter == NULL)
		{
			$this->flashMessage('Invalid project charter ID. You have been redirected.', 'warning');
            $this->redirect('Dashboard:default');
		}	
		
		$form = $this['emailForm'];
		if (!$form['ProjectCharterID']->getValue())
		{
			$form['ProjectCharterID']->setValue($approverHistory->getProjectCharterID());
			$form['EmailContent']->setValue($approverHistory->getEmailContent());			
		}
		
		$arrApprovers = array();
		$approvers = $this->context->projectCharterApproverRepository->findAllApprover($approverHistory->getProjectCharterID(), $approverHistory->getVersion());
		if ($approvers)
		{
			$i = 0;
			foreach ($approvers as $approver)
			{
				$arrApprovers[$i]['Role'] = $approver->getRole();
				$arrApprovers[$i]['Name'] = $approver->getSurname() . ', ' . $approver->getName();
				$arrApprovers[$i]['Name_Full'] = $approver->getSurname() . ', ' . $approver->getName() . ' (' . strtoupper($approver->getApproverEID()) . ')';
				$arrApprovers[$i]['Email'] = $approver->getEmail();
				$i++;
			}
		}
		
		// lock request for final approval in case it is not allowed
		try
		{
		    $approvalLocked = $this->context->globalSetting->get('charters.finalApprovalLocked.' . $charter->getYear());
		}
		catch (RangeException $e)
		{
		    $approvalLocked = TRUE; // unknown key means approval is not allowed
		} 
		// if approval is not allowed, disable button and show message    
		if ($approvalLocked)
		{
			$form['submit']->setDisabled(TRUE);
			$this->template->approvalLocked = TRUE;
		}
		
		
		$user = $this->context->personRepository->find($this->getUser()->getId());

		$arrCharter = array();
		$arrCharter['Name'] = $charter->getName();
		$arrCharter['Owner'] = $user->getFullName();
		$arrCharter['Owner_Full'] = $user->getReversedFullNameWithEID();
		$arrCharter['Year'] = $charter->getYear();
		
		$this->template->arrApprovers = $arrApprovers;
		$this->template->charter = $arrCharter;
		$this->template->emailContent = ''; // must be empty
		$charterMailerOptions = Environment::getConfig('charter_mailer_options');
		$this->template->emailTemplate = __DIR__ . $charterMailerOptions['template_storage_dir_from_presenters']  . $charterMailerOptions['template_approval_request'];
	}

	/**
     * Shows detail of project charter and confirm form for final approver.
     * 
     * @param int $id ProjectCharter ID.
     * 
     */
	public function renderApprover($id = null)
    {
        // check project charter ID
        $charter = $this->context->projectCharterRepository->find($id);
        if ($charter == NULL)
        {
            $this->flashMessage('Invalid project charter ID. You have been redirected.', 'warning');
            $this->redirect('Dashboard:default');
        }
        
        /* Project Charter deatil */
        $this->template->currentCharterStatus = $charter->getStatus();
        $arrCharter = array();
        $arrCharter['SAP_ID'] = $charter->getSAP_ID();
        $arrCharter['Name'] = $charter->getName();
            $owner = $this->context->personRepository->find($charter->getOwner());
		$arrCharter['Owner'] = ($owner) ? $owner->getReversedFullNameWithEID() : '-';
            $lineManager = $this->context->personRepository->find($charter->getLineManager());
		$arrCharter['LineManager'] = ($lineManager) ?  $lineManager->getReversedFullNameWithEID() : '-';
			$strapItem = $this->context->itemRepository->find($charter->StrapItemID);
		$arrCharter['StrapItem'] = ($strapItem) ? $strapItem->Name : '-';
		$arrCharter['StrapItemID'] = ($strapItem) ? $strapItem->ID : NULL;
        $arrCharter['Status'] = ProjectCharter::getFullStatusName($charter->getStatus());
        $arrCharter['CreationDateTime'] = $charter->getCreationDateTime();
        $arrCharter['CostShare'] = $charter->getCostSharePercentage();
        $arrCharter['Customer_ID'] = $this->context->customerRepository->find($charter->getCustomerID())->getName();
		    $program = $this->context->customerProgramRepository->find($charter->getProgramID());
        $arrCharter['ProgramID'] = ($program) ? $program->getProgram() : NULL;
		    $call = $this->context->customerProgramCallRepository->find($charter->getCallID());
		$arrCharter['CallID'] = ($call) ? $call->getCall() : NULL;
        $arrCharter['Responsibility_ID'] = $this->context->responsibilityRepository->find($charter->getResponsibilityID())->getName();
        $arrCharter['Year'] = $charter->getYear();
        
        
        $file = $this->context->projectCharterFileHistoryRepository->findLatestVersion($charter->getID());
        if ($file)
        {
            $this->template->filePath = $this->getPathToFile($file->getProjectCharterID(), $file->getVersion(), $file->getFileName());
            $this->template->fileName = $file->getFileName();
        }

		$summaryFile = $this->context->projectCharterSummaryFileHistoryRepository->findLatestVersion($charter->getID());
        if ($summaryFile)
        {
            $this->template->summaryFilePath = $this->getPathToSummaryFile($summaryFile->getProjectCharterID(), $summaryFile->getVersion(), $summaryFile->getFileName());
            $this->template->summaryFileName = $summaryFile->getFileName();
        }
        
        $groups = array(); $subcontractors = array(); $others = array(); $sumOfTotals = 0;
        $budget = $this->context->projectCharterBudgetHistoryRepository->findLatestVersion($charter->getID());
        if ($budget != NULL)
        {
            $budgetItems = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($budget->getProjectCharterID(), $budget->getVersion());
            if ($budgetItems != NULL)
            {
                foreach ($budgetItems as $budgetItem)
                {
                    if ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS)
                    {
						$orgunit = $this->context->organizationUnitRepository->find($budgetItem->getTechnicalGroup());
						if ($orgunit)
						{
							$groups[$budgetItem->getID()]['Name'] = $orgunit->getName();
							$groups[$budgetItem->getID()]['Q1'] = $budgetItem->getBudgetQ1();
							$groups[$budgetItem->getID()]['Q2'] = $budgetItem->getBudgetQ2();
							$groups[$budgetItem->getID()]['Q3'] = $budgetItem->getBudgetQ3();
							$groups[$budgetItem->getID()]['Q4'] = $budgetItem->getBudgetQ4();
							$groups[$budgetItem->getID()]['Total'] = $budgetItem->getBudgetQ1() + $budgetItem->getBudgetQ2() + $budgetItem->getBudgetQ3() + $budgetItem->getBudgetQ4();
							$sumOfTotals += $groups[$budgetItem->getID()]['Total'] * $budgetItem->getRate();
						}
                    }
                    elseif ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING)
                    {
                        $subcontractor = $this->context->subcontractorRepository->find($budgetItem->getSubcontractor());
                        if ($subcontractor)
                        {
                            $subcontractors[$budgetItem->getID()]['Name'] = $subcontractor->getName();
                            $subcontractors[$budgetItem->getID()]['Q1'] = $budgetItem->getBudgetQ1();
                            $subcontractors[$budgetItem->getID()]['Q2'] = $budgetItem->getBudgetQ2();
                            $subcontractors[$budgetItem->getID()]['Q3'] = $budgetItem->getBudgetQ3();
                            $subcontractors[$budgetItem->getID()]['Q4'] = $budgetItem->getBudgetQ4();
                            $subcontractors[$budgetItem->getID()]['Total'] = $budgetItem->getBudgetQ1() + $budgetItem->getBudgetQ2() + $budgetItem->getBudgetQ3() + $budgetItem->getBudgetQ4();
                            $sumOfTotals += $subcontractors[$budgetItem->getID()]['Total'];
                        }
                    }
                    else
                    {
                        switch ($budgetItem->getResourceType())
                        {
                            case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL:
                                $name = 'Travel';
                                break;
                            case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL:
                                $name = 'Material';
                                break;
                            default: break;
                        }
                        $others[$budgetItem->getID()]['Name'] = $name;
                        $others[$budgetItem->getID()]['Q1'] = $budgetItem->getBudgetQ1();                    
                        $others[$budgetItem->getID()]['Q2'] = $budgetItem->getBudgetQ2();                    
                        $others[$budgetItem->getID()]['Q3'] = $budgetItem->getBudgetQ3();                    
                        $others[$budgetItem->getID()]['Q4'] = $budgetItem->getBudgetQ4();                    
                        $others[$budgetItem->getID()]['Total'] = $budgetItem->getBudgetQ1() + $budgetItem->getBudgetQ2() + $budgetItem->getBudgetQ3() + $budgetItem->getBudgetQ4();
                        $sumOfTotals += $others[$budgetItem->getID()]['Total'];
                    }
                }
            }
        }
        
        $this->template->arrCharter = $arrCharter;
        $this->template->groups = $groups;
        $this->template->others = $others;
        $this->template->subcontractors = $subcontractors;
        $this->template->sumOfTotals = $sumOfTotals;

        /* List of final approvers */
        $version = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($id);
        if($version)
        {
            $approvers = array();
            $approvers = $this->context->projectCharterApproverRepository->findAllApprover($id, $version->Version);
            $this->template->approvers = $approvers;
        }
        
        /* My action form */
        $projectCharterApprover = $this->context->projectCharterApproverRepository->findByApprover($id, $version->Version, Environment::getUser()->getId());        
        $confirmForm = $this['confirmForm'];
        $defaults = array();
        $defaults['projectCharterID'] = $id;
        $defaults['version'] = $version->Version;
        $this->template->renderForm = true;
		$this->template->decision = ProjectCharterApprover::DECISION_PENDING;
		if(count($projectCharterApprover) > 0)
		{
			if (current($projectCharterApprover)->Approval != ProjectCharterApprover::DECISION_PENDING)
			{
				$this->template->renderForm = false;
				if(current($projectCharterApprover)->Approval == ProjectCharterApprover::DECISION_APPROVED)
				{
					$this->template->decision = 'approved';
				}
				else
				{
					$this->template->decision = 'disapproved';
				}
				$defaults['Comment'] = current($projectCharterApprover)->Comment;
				$confirmForm['Approve']->setDisabled(true);
				$confirmForm['Disapprove']->setDisabled(true);
			}
		}
		else
		{
            $this->flashMessage('You are not in final approvers list. You have been redirected.', 'warning');
            $this->redirect('Dashboard:default');
		}
        $confirmForm->setDefaults($defaults);
        $this->template->form = $confirmForm;
    }
    
    /**
     * Shows detail of project charter and buttons for next status of charter.
     * 
     * @param int $id ProjectCharter ID.
     * 
     */
	public function renderMissingApproval($id = null)
    {
		// check accessibility for user
		$this->secureAccess($id, array(ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED), TRUE);
		
        // check project charter ID
        $charter = $this->context->projectCharterRepository->find($id);
        if ($charter == NULL)
        {
            $this->flashMessage('Invalid project charter ID. You have been redirected.', 'warning');
            $this->redirect('Dashboard:default');
        }
        
        /* Project Charter deatil */
        $this->template->currentCharterStatus = $charter->getStatus();
        $arrCharter = array();
        $arrCharter['SAP_ID'] = $charter->getSAP_ID();
        $arrCharter['Name'] = $charter->getName();
            $owner = $this->context->personRepository->find($charter->getOwner());
		$arrCharter['Owner'] = ($owner) ? $owner->getReversedFullNameWithEID() : '-';
            $lineManager = $this->context->personRepository->find($charter->getLineManager());
		$arrCharter['LineManager'] = ($lineManager) ?  $lineManager->getReversedFullNameWithEID() : '-';
			$strapItem = $this->context->itemRepository->find($charter->StrapItemID);
		$arrCharter['StrapItem'] = ($strapItem) ? $strapItem->Name : '-';
		$arrCharter['StrapItemID'] = ($strapItem) ? $strapItem->ID : NULL;
        $arrCharter['Status'] = ProjectCharter::getFullStatusName($charter->getStatus());
        $arrCharter['CreationDateTime'] = $charter->getCreationDateTime();
        $arrCharter['CostShare'] = $charter->getCostSharePercentage();
        $arrCharter['Customer_ID'] = $this->context->customerRepository->find($charter->getCustomerID())->getName();
        $arrCharter['Responsibility_ID'] = $this->context->responsibilityRepository->find($charter->getResponsibilityID())->getName();
        $arrCharter['Year'] = $charter->getYear();
        
        $file = $this->context->projectCharterFileHistoryRepository->findLatestVersion($charter->getID());
        if ($file)
        {
            $this->template->filePath = $this->getPathToFile($file->getProjectCharterID(), $file->getVersion(), $file->getFileName());
            $this->template->fileName = $file->getFileName();
        }
        
        $groups = array(); $subcontractors = array(); $others = array(); $sumOfTotals = 0;
        $budget = $this->context->projectCharterBudgetHistoryRepository->findLatestVersion($charter->getID());
        if ($budget != NULL)
        {
            $budgetItems = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($budget->getProjectCharterID(), $budget->getVersion());
            if ($budgetItems != NULL)
            {
                foreach ($budgetItems as $budgetItem)
                {
                    if ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS)
                    {
						$orgunit = $this->context->organizationUnitRepository->find($budgetItem->getTechnicalGroup());
						if ($orgunit)
						{
							$groups[$budgetItem->getID()]['Name'] = $orgunit->getName();
							$groups[$budgetItem->getID()]['Q1'] = $budgetItem->getBudgetQ1();
							$groups[$budgetItem->getID()]['Q2'] = $budgetItem->getBudgetQ2();
							$groups[$budgetItem->getID()]['Q3'] = $budgetItem->getBudgetQ3();
							$groups[$budgetItem->getID()]['Q4'] = $budgetItem->getBudgetQ4();
							$groups[$budgetItem->getID()]['Total'] = $budgetItem->getBudgetQ1() + $budgetItem->getBudgetQ2() + $budgetItem->getBudgetQ3() + $budgetItem->getBudgetQ4();
							$sumOfTotals += $groups[$budgetItem->getID()]['Total'] * $budgetItem->getRate();
						}
                    }
                    elseif ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING)
                    {
                        $subcontractor = $this->context->subcontractorRepository->find($budgetItem->getSubcontractor());
                        if ($subcontractor)
                        {
                            $subcontractors[$budgetItem->getID()]['Name'] = $subcontractor->getName();
                            $subcontractors[$budgetItem->getID()]['Q1'] = $budgetItem->getBudgetQ1();
                            $subcontractors[$budgetItem->getID()]['Q2'] = $budgetItem->getBudgetQ2();
                            $subcontractors[$budgetItem->getID()]['Q3'] = $budgetItem->getBudgetQ3();
                            $subcontractors[$budgetItem->getID()]['Q4'] = $budgetItem->getBudgetQ4();
                            $subcontractors[$budgetItem->getID()]['Total'] = $budgetItem->getBudgetQ1() + $budgetItem->getBudgetQ2() + $budgetItem->getBudgetQ3() + $budgetItem->getBudgetQ4();
                            $sumOfTotals += $subcontractors[$budgetItem->getID()]['Total'];
                        }
                    }
                    else
                    {
                        switch ($budgetItem->getResourceType())
                        {
                            case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL:
                                $name = 'Travel';
                                break;
                            case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL:
                                $name = 'Material';
                                break;
                            default: break;
                        }
                        $others[$budgetItem->getID()]['Name'] = $name;
                        $others[$budgetItem->getID()]['Q1'] = $budgetItem->getBudgetQ1();                    
                        $others[$budgetItem->getID()]['Q2'] = $budgetItem->getBudgetQ2();                    
                        $others[$budgetItem->getID()]['Q3'] = $budgetItem->getBudgetQ3();                    
                        $others[$budgetItem->getID()]['Q4'] = $budgetItem->getBudgetQ4();                    
                        $others[$budgetItem->getID()]['Total'] = $budgetItem->getBudgetQ1() + $budgetItem->getBudgetQ2() + $budgetItem->getBudgetQ3() + $budgetItem->getBudgetQ4();
                        $sumOfTotals += $others[$budgetItem->getID()]['Total'];
                    }
                }
            }
        }
        
        $this->template->arrCharter = $arrCharter;
        $this->template->groups = $groups;
        $this->template->others = $others;
        $this->template->subcontractors = $subcontractors;
        $this->template->sumOfTotals = $sumOfTotals;
        
        /* List of final approvers */
        $version = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($id);
        if($version)
        {
            $approvers = array();
            $approvers = $this->context->projectCharterApproverRepository->findAllApprover($id, $version->Version);
            $this->template->approvers = $approvers;
        }
        
        /* disapproved received form */
        $disapproveReceivedForm = $this['disapproveReceivedForm'];
        $this->template->form = $disapproveReceivedForm;
        
        /* file update form*/
        $fileForm = $this['fileForm'];
        $this->template->fileForm = $fileForm;
    }
    
	/**
	 * Creates general form for adding new approvers.
	 * @param atring $name Name of the form.
	 * @return \AppForm
	 */
	public function createComponentApproversDefinitionForm($name)
	{
		$form = new AppForm($this, $name);		
		
		$form->addHidden('ProjectCharterID');
		$form->addHidden('Version');
		
		$form->addHidden('DEPLeader_ID');
		$form->addHidden('DEPLeader_info');
		$form->addText('DEPLeader', 'Department Leader', 10, 7)
				->addRule(Form::FILLED, 'Department leader must be filled.')
				->addRule(Form::PATTERN, 'EID must have valid format (e.g. E123456).', '[E|e|H|h|T|t][0-9]{6}')
				->addConditionOn($form['DEPLeader_info'], ~Form::FILLED)
				        ->addRule(~Form::FILLED, 'EID E000000 is not acceptable. This may be caused by invalid EID (deleted or missing user account) or by connection error. Delete corrupt EID and try to input it again.');
				
	    $form->addHidden('ATEDirector_ID');
	    $form->addHidden('ATEDirector_info');
		$form->addText('ATEDirector', 'ATE Director', 10, 7)
				->addRule(Form::FILLED, 'ATE director must be filled.')
				->addRule(Form::PATTERN, 'EID must have valid format (e.g. E123456).', '[E|e|H|h|T|t][0-9]{6}')
				->addConditionOn($form['ATEDirector_info'], ~Form::FILLED)
				        ->addRule(~Form::FILLED, 'EID E000000 is not acceptable. This may be caused by invalid EID (deleted or missing user account) or by connection error. Delete corrupt EID and try to input it again.');
//	    
		$form->addHidden('PEC_ID');
		$form->addHidden('PEC_info');
		$form->addText('PEC', 'Program Manager', 10, 7)
				->addRule(Form::FILLED, 'PEC must be filled.')
				->addRule(Form::PATTERN, 'EID must have valid format (e.g. E123456).', '[E|e|H|h|T|t][0-9]{6}')
				->addConditionOn($form['PEC_info'], ~Form::FILLED)
				        ->addRule(~Form::FILLED, 'EID E000000 is not acceptable. This may be caused by invalid EID (deleted or missing user account) or by connection error. Delete corrupt EID and try to input it again.');
	    
		$form->addHidden('PMO_ID');
		$form->addHidden('PMO_info');
		$form->addText('PMO', 'PMO', 10, 7)
				->addRule(Form::FILLED, 'PMO must be filled.')
				->addRule(Form::PATTERN, 'EID must have valid format (e.g. E123456).', '[E|e|H|h|T|t][0-9]{6}')
				->addConditionOn($form['PMO_info'], ~Form::FILLED)
				        ->addRule(~Form::FILLED, 'EID E000000 is not acceptable. This may be caused by invalid EID (deleted or missing user account) or by connection error. Delete corrupt EID and try to input it again.');
	    
		for ($row = 0; $row < $this->session['ApproverRows']; $row++)
		{
			$this->addApproverToForm($form, $row);
		}
		
		$form->addSubmit('AddApprover', 'Add')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'addApproverSubmitted');
		
		$form->addSubmit('save', 'Save')
				->onClick[] = array($this, 'approversDefinitionFormSubmitted');
		
		$form->addSubmit('continue', 'Save and continue')
				->onClick[] = array($this, 'approversDefinitionFormSubmitted');
		
		$form->addSubmit('restore', 'Redefine charter')
				->onClick[] = array($this, 'restoreDetailCharterFormSubmitted');
				$form['restore']->setAttribute('onclick',
						'return confirm("Do you really want to restore the charter for redefinition?");'
					);
		
		return $form;
	}
	
	/**
     * Adds another field for new approver.
     * @param AppForm $form Destination form.
     * @param int $row It should be positive integer. Identification of the row.
     */
	protected function addApproverToForm(AppForm $form, $row)
	{
		// if field already exists do nothing
        if (isset($form['Approver' . $row]))
            return;
		
		$form->addHidden('Approver' . $row . '_ID');
		$form->addHidden('Approver' . $row . '_info');
		$form->addText('Approver' . $row . '_role', NULL, 35, 50)
				->addRule(Form::FILLED, 'Either define roles and EIDs for all approver lines or delete them.');
		$form->addText('Approver' . $row, NULL, 10, 7)
				->addRule(Form::FILLED, 'Either define roles and EIDs for all approver lines or delete them.')
				->addRule(Form::PATTERN, 'EID must have valid format (e.g. E123456).', '[E|e|H|h|T|t][0-9]{6}')
				->addConditionOn($form['Approver' . $row . '_info'], ~Form::FILLED)
				        ->addRule(~Form::FILLED, 'EID E000000 is not acceptable. This may be caused by invalid EID (deleted or missing user account) or by connection error. Delete corrupt EID and try to input it again.');
		
		$form->addSubmit('DeleteApprover' . $row, 'Delete')
                ->setValidationScope(FALSE)
                ->onClick[] = array($this, 'deleteApproverSubmitted');
	}


	public function addApproverSubmitted(SubmitButton $button)
	{
		$form = $button->form;		
		$this->addApproverToForm($form, $this->session['ApproverRows']);				
		$this->session['ApproverRows'] = $this->session['ApproverRows'] + 1;
        // save form and redirect
        $this->session['Form'] = $form->getValues(TRUE);
        $this->redirect('this#Approvers');
	}
	
	public function deleteApproverSubmitted(SubmitButton $button)
	{
		$entry = $button->form->getValues(TRUE);
        $form = $button->form;        
        // delete given row
        if ($this->session['ApproverRows'] != 1)
        {
			$row = preg_replace('/DeleteApprover/', '', $button->getHtmlName());
            // for each row behind deleted row, copy data to the row ahead of itself
            for ($row; $row < $this->session['ApproverRows']; $row++)
            {
                // is there any following row?
                $nextRow = $row + 1;
                if (!isset($entry['Approver' . $nextRow]))
                    break;

				$form['Approver' . $row . '_ID']->setValue($entry['Approver' . $nextRow . '_ID']);
                $form['Approver' . $row . '_role']->setValue($entry['Approver' . $nextRow . '_role']);
                $form['Approver' . $row . '_info']->setValue($entry['Approver' . $nextRow . '_info']);
                $form['Approver' . $row]->setValue($entry['Approver' . $nextRow]);
            }
        }
        
        // decrease number of available rows
        $this->session['ApproverRows'] = $this->session['ApproverRows'] - 1;
        // save form and redirect
        $this->session['Form'] = $form->getValues(TRUE);
        $this->redirect('this#Approvers');
	} 
	
	public function approversDefinitionFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
		    $entry = $button->form->getValues(TRUE);
			
			// check accessibility for user
		    $this->secureAccess($entry['ProjectCharterID'], array(ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION), TRUE);
			
			if ($entry['Version'])
			{
				$latestApproverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($entry['ProjectCharterID']);
				if ($latestApproverHistory->version !== (int) $entry['Version'])
				{
					// the submitted form has older ['Version'] than currently in the DB
					// it would overwrite previous version of Approvers
					$this->flashMessage("It seems that the charter has been redefined in the meantime. We have refreshed this page for you.", 'warning');
					$this->redirect('this');
				}
				
				if ($latestApproverHistory)
				{
					$approverHistory = $latestApproverHistory;
					$approverHistory->setCreationDateTime(new DateTime);
					$approverHistory->setResponsible($this->getUser()->getId());
					
					if (!$this->context->projectCharterApproverHistoryRepository->update($approverHistory))
					{
						$this->flashMessage("Saving list of responsible persons for final approval failed.", 'error');
						return;
					}
				}
			}
			else
			{
				if ($this->context->projectCharterApproverHistoryRepository->findLatestVersion($entry['ProjectCharterID']) !== NULL)
				{
					$this->flashMessage("It seems you are defining Approvers in more than one window at once. Sorry, this is not supported.", 'warning');
					$this->redirect('this');
				}
				
				$approverHistory = new ProjectCharterApproverHistory();
				$approverHistory->setProjectCharterID($entry['ProjectCharterID']);
				$approverHistory->setVersion(1);
				$approverHistory->setCreationDateTime(new DateTime);
				$approverHistory->setResponsible($this->getUser()->getId());
				$approverHistory->setLatestVersion(1);
				// others are null by default
				
				$this->context->projectCharterApproverHistoryRepository->initLatestVersion($entry['ProjectCharterID']);
				if (!$this->context->projectCharterApproverHistoryRepository->save($approverHistory))
				{
					$this->flashMessage("Saving list of responsible persons for final approval failed.", 'error');
					return;
				}
			}
			
			$arrApproverSource = array('DEPLeader', 'ATEDirector', 'PEC', 'PMO');
			for ($row = 0; $row < $this->session['ApproverRows']; $row++)
				array_push($arrApproverSource, 'Approver' . $row);
			
			// delete difference
            if ($entry['Version'])
            {
                // get saved
                $currentlySaved = $this->context->projectCharterApproverRepository->findAllApprover($entry['ProjectCharterID'], $entry['Version']);           
                $currentlySavedID = array();
                foreach ($currentlySaved as $savedApprover)
                    $currentlySavedID[] = $savedApprover->getProjectCharterApproverID();
                
                // get approvers from form
                $fromFormID = array();
                foreach ($arrApproverSource as $source)
                    $fromFormID[] = $entry[$source . '_ID'];
                
                // delete approvers
                $toDeleteID = array_diff($currentlySavedID, $fromFormID);
                foreach ($toDeleteID as $id)
                    $this->context->projectCharterApproverRepository->delete($id);
            }
			
			// parse information for each responsible person
			$i = 0;
			foreach ($arrApproverSource as $source)
			{
				$info = (array) json_decode($entry[$source . '_info']);
				if (isset($info['EID']))
				{
				    $approvers[$i] = $info;
				    $approvers[$i]['ProjectCharterApprover_ID'] = $entry[$source . '_ID'];
					switch ($source)
					{
						case 'DEPLeader':
							$role = ProjectCharterApprover::APPROVER_ROLE_DEP_LEADER;
							break;
						case 'ATEDirector':
							$role = ProjectCharterApprover::APPROVER_ROLE_ATE_DIRECTOR;
							break;
						case 'PEC':
							$role = ProjectCharterApprover::APPROVER_ROLE_PROGRAM_MANAGER;
							break;
						case 'PMO':
							$role = ProjectCharterApprover::APPROVER_ROLE_PMO;
							break;
						default:
							$role = $entry[$source . '_role'];
							break;
					}
				    $approvers[$i]['Role'] = $role;
				}
				$i++;
			}
			
			//
			$approver = new ProjectCharterApprover();
			$approver->setProjectCharterID($entry['ProjectCharterID']);
			$approver->setVersion($approverHistory->getVersion());
			$approver->setApproval(ProjectCharterApprover::DECISION_PENDING);
			$approver->setAutoApproved(false);
			// others are null by default
			foreach ($approvers as $person)
			{
				$approver->setRole($person['Role']);
				$approver->setApproverEID($person['EID']);
				$approver->setSurname($person['Surname']);
				$approver->setName($person['Name']);
				$approver->setEmail($person['Email']);
				if ($person['ProjectCharterApprover_ID'])
				{
					$approver->setProjectCharterApproverID($person['ProjectCharterApprover_ID']);
					if (!$this->context->projectCharterApproverRepository->update($approver))
					{
						$this->flashMessage("Saving person " . $person['FullName'] . " failed.", 'error');
						return;
					}
					continue;
				}
				
				if (!$this->context->projectCharterApproverRepository->save($approver))
				{
					$this->flashMessage("Saving person " . $person['FullName'] . " failed.", 'error');
					return;
				}
			}
			
			if ($button->getHtmlName() == 'save')
			{
				$this->flashMessage("List of responsible persons for final approval was saved.", 'success');
				$this->redirect('this');
			}
			else
			{
				$this->flashMessage("List of responsible persons for final approval was saved.", 'success');
				$this->redirect('ProjectCharterApproval:email', array('id' => $entry['ProjectCharterID']));
			}
		}
		
		$this->flashMessage("Error occured during submitting the form.", 'error');
        return;
	}
	
    /**
     * Creates form for project charet confirmation by final approvers.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentConfirmForm($name)
    {
        $form = new AppForm($this, $name);

        $form->addHidden('projectCharterID');
        $form->addHidden('version');
        
        $form->addSubmit('Approve', 'Approve')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'approveClicked');
        $form['Approve']->setAttribute('onclick', '
                    return confirm("Are you sure to approve the charter?");
                ');
        $form->addSubmit('Disapprove', 'Disapprove')
            ->onClick[] = callback($this, 'disapproveClicked');
        $form['Disapprove']->setAttribute('onclick', '
                    return confirm("Are you sure to disapprove the charter?");
                ');
        $form->addTextArea('Comment', 'Comment', 100, 10)
                ->addRule(Form::FILLED, 'Please, state reasons for your disapproval into the comment text area.');

        return $form;
    }
    
    /**
     * Approve button.
     * 
     * @param SubmitButton $button Approve button.
     */
    public function approveClicked(SubmitButton $button)
    {
        $approver_date = date("Y-m-d H:i:s");
        $approverEID = Environment::getUser()->getId();
        $values = $button->getForm()->getValues();
        $projectCharterApprovers = $this->context->projectCharterApproverRepository->findByApprover($values['projectCharterID'], $values['version'], strtoupper($approverEID));
		foreach($projectCharterApprovers as $projectCharterApprover)
		{
			if (!$projectCharterApprover->getDecisionDateTime())
			{
				$projectCharterApprover->Comment = $values['Comment'];
				$projectCharterApprover->Approval = ProjectCharterApprover::DECISION_APPROVED;
				$projectCharterApprover->DecisionDateTime = $approver_date;
				$projectCharterApprover->AutoApproved = false;
				if($this->context->projectCharterApproverRepository->update($projectCharterApprover))
					$this->flashMessage("Project Charter was approved.", 'success');
				else
					$this->flashMessage("Approval of project charter failed.", 'error');
			}
			else
			{
				$this->flashMessage("You have already made decision to this charter.", 'warning');
				$this->redirect('this');
			}
		}
		$missing_approvals = 0;
		$pending_approvals = 0;
        $approvers = $this->context->projectCharterApproverRepository->findAllApprover($values['projectCharterID'], $values['version']);
        foreach($approvers as $approver)
        {
			switch ($approver->Approval) {
				case ProjectCharterApprover::DECISION_PENDING :
					$pending_approvals++;
					break;
				case ProjectCharterApprover::DECISION_DISAPPROVED :
					$missing_approvals++;
					break;
			}
        }
		if ($pending_approvals == 0) 
		{
			$charter = $this->context->projectCharterRepository->find($values['projectCharterID']);
			if ($missing_approvals == 0)
				$charter->Status = ProjectCharter::PROJECT_CHARTER_STATUS_APPROVED;
			else
				$charter->Status = ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED;
			$this->context->projectCharterRepository->update($charter);
			// send email to project charter owner???
			$this->context->projectCharterMailer->sendNoticeStatusChange($charter, $this->link('//ProjectCharter:detail', array('id' => $charter->getID())));
		}
        $this->redirect('approver', $values['projectCharterID']);
    }
    
    /**
     * Dissapprove button.
     * 
     * @param SubmitButton $button Disapprove button.
     */
    public function disapproveClicked(SubmitButton $button)
    {		
        $approver_date = date("Y-m-d H:i:s");
        $approverEID = Environment::getUser()->getId();
        $values = $button->getForm()->getValues();
        $projectCharterApprovers = $this->context->projectCharterApproverRepository->findByApprover($values['projectCharterID'], $values['version'], strtoupper($approverEID));
		foreach($projectCharterApprovers as $projectCharterApprover)
		{
			if (!$projectCharterApprover->getDecisionDateTime())
			{
				$projectCharterApprover->Comment = $values['Comment'];
				$projectCharterApprover->Approval = ProjectCharterApprover::DECISION_DISAPPROVED;
				$projectCharterApprover->DecisionDateTime = $approver_date;
				$projectCharterApprover->AutoApproved = false;
				if($button->getForm()->isValid())
					if($this->context->projectCharterApproverRepository->update($projectCharterApprover))
						$this->flashMessage("Project Charter was disapproved.", 'success');
					else
						$this->flashMessage("Disapproving project charter failed.", 'error');
				else
					$this->flashMessage("Invalid form values.", 'error');
			}
			else
			{
				$this->flashMessage("You have already made decision to this charter.", 'warning');
				$this->redirect('this');
			}
		}
		$pending_approvals = 0;
        $approvers = $this->context->projectCharterApproverRepository->findAllApprover($values['projectCharterID'], $values['version']);
        foreach($approvers as $approver)
        {
			if ($approver->Approval == ProjectCharterApprover::DECISION_PENDING) 
			{
				$pending_approvals++;
				break;
			}
        }
		if ($pending_approvals == 0) 
        {
            $charter = $this->context->projectCharterRepository->find($values['projectCharterID']);
            $charter->Status = ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED;
            $this->context->projectCharterRepository->update($charter);
            // send email to project charter owner???
			$this->context->projectCharterMailer->sendNoticeStatusChange($charter, $this->link('//ProjectCharter:detail', array('id' => $charter->getID())));
        }
        $this->redirect('approver', $values['projectCharterID']);
    }
	
	public function createComponentEmailForm($name)
	{
		$form = new AppForm($this, $name);
		
		$form->addHidden('ProjectCharterID');
		$form->addTextArea('EmailContent', 'Comment', 80, 10);
		$form->addCheckbox('Copy', 'Send me a copy')
				->setDefaultValue(1);
		
		$form->addSubmit('back', 'Go back')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'backEmailFormSubmitted');
		
		$form->addButton('preview', 'Preview');
		
		$form->addSubmit('save', 'Save')
				->onClick[] = array($this, 'emailFormSubmitted');
		
		$form->addSubmit('submit', 'Send for Approval')
				->onClick[] = array($this, 'emailFormSubmitted');
		        $form['submit']->setAttribute('onclick', '
                    return confirm("Are you sure to send request for approval?");
                ');
	}
	
	public function backEmailFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues(TRUE);
			$this->redirect('ProjectCharterApproval:definition', array('id' => $entry['ProjectCharterID']));
		}
	}

	public function emailFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues(TRUE);
			
			// check accessibility for user
		    $this->secureAccess($entry['ProjectCharterID'], array(ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION), TRUE);
			
		    $approverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($entry['ProjectCharterID']);
			if ($approverHistory)
			{
			    $approverHistory->setEmailContent($entry['EmailContent']);
				
				if (!$this->context->projectCharterApproverHistoryRepository->update($approverHistory))
				{
					$this->flashMessage("Saving email content failed.", 'error');
					return;
				}
			}
			else
			{
				$this->flashMessage("Saving email content failed.", 'error');
				return;
			}
			
			if ($button->getHtmlName() == 'save')
			{
			    $this->flashMessage('Email content saved.', 'success');
			    $this->redirect('this');
			}
			else
			{
				
				$copy = FALSE;
				if ($entry['Copy'])
					$copy = TRUE;
				
				$charter = $this->context->projectCharterRepository->find($entry['ProjectCharterID']);
				if ($charter)
				{					
					try
					{
						// set final approval date (actualy timestamp)
						if (!$charter->getDaysForFinalApproval())
						{
							$charter->setDaysForFinalApproval($this->context->globalSetting->get('charters.finalApprovalFirstRoundDays'));
						}
						else
						{
							$charter->setDaysForFinalApproval($this->context->globalSetting->get('charters.finalApprovalNextRoundDays'));
						}
						
						$finalApprovalDate = strtotime('now +' . $charter->getDaysForFinalApproval() . 'day');
						if (!$finalApprovalDate)
						{
							$this->flashMessage("Changing project charter status failed.", 'error');
							return;
						}
						
						$approverHistory->setFinalApprovalDate($finalApprovalDate);
						if (!$this->context->projectCharterApproverHistoryRepository->update($approverHistory))
						{
							$this->flashMessage("Changing project charter status failed.", 'error');
							return;
						}
						
						$charter->setStatus(ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL);
						if (!$this->context->projectCharterRepository->update($charter))
						{
							$this->flashMessage('Changing project charter status failed.', 'error');
							return;
						}
						$this->context->projectCharterMailer->sendApprovalRequest($charter, $entry['EmailContent'], $this->link('//ProjectCharterApproval:approver', array('id' => $entry['ProjectCharterID'])), $copy);
					}
					catch (Exception $e)
					{
						$this->flashMessage('Sending approval request failed.', 'error');
						return;
					}
				}
				
				$this->flashMessage('Request for approval sent.', 'success');
				$this->redirect('ProjectCharter:detail', array('id' => $entry['ProjectCharterID']));
			}
		}
		
		$this->flashMessage("Error occured during submitting the form.", 'error');
        return;
	}

    /**
     * Creates form for disapprove received.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentDisapproveReceivedForm($name)
    {
        $form = new AppForm($this, $name);
        
        $form->addSubmit('SubmitForApproval', 'Submit For Approval')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'submitForApprovalClicked');
        $form['SubmitForApproval']->setAttribute('onclick', '
                    return confirm("Are you sure to submit for approval?");
                ');
        $form->addSubmit('FinalApproversUpdate', 'Final Approvers Update')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'finalApproversUpdateClicked');
        $form['FinalApproversUpdate']->setAttribute('onclick', '
                    return confirm("Are you sure to update final approvers?");
                ');
		$form->addSubmit('Definition', 'Redefine Charter')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'definitionClicked');
        $form['Definition']->setAttribute('onclick', '
                    return confirm("All received approvals will be cancelled. Do you want to continue?");
                ');
        $form->addSubmit('CancelProjectCharter', 'Cancel Project Charter')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'cancelProjectCharterClicked');
        $form['CancelProjectCharter']->setAttribute('onclick', '
                    return confirm("Are you sure to cancel the charter?");
                ');
        
        $form->addSubmit('Back', 'Go back')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'backClicked');

        return $form;
    }
    
    /**
     * Submit For Approval button.
     * 
     * @param SubmitButton $button.
     */
    public function submitForApprovalClicked(SubmitButton $button)
    {
        $projectCharterID = $this->getHttpRequest()->getQuery('id');
		
		// check accessibility for user
		$this->secureAccess($projectCharterID, array(ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED), TRUE);
		
        $charter = $this->context->projectCharterRepository->find($projectCharterID);
        $charter->Status = ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION;
        if($this->context->projectCharterRepository->update($charter))
        {
            $creation_date = date("Y-m-d H:i:s");
            $oldApproverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($projectCharterID);
            
            // Creates new ProjectCharterApproverHistory from old one
            $newApproverHistory = new ProjectCharterApproverHistory();
            $newApproverHistory->ProjectCharterID = $oldApproverHistory->ProjectCharterID;
            $newApproverHistory->Version = $oldApproverHistory->Version + 1;
            $newApproverHistory->CreationDateTime = $creation_date;
            $newApproverHistory->Responsible = $oldApproverHistory->Responsible;
            $newApproverHistory->LatestVersion = 1;
            $newApproverHistory->BudgetVersion = $oldApproverHistory->BudgetVersion;
            $newApproverHistory->FileVersion = $oldApproverHistory->FileVersion;
            $newApproverHistory->EmailContent = null;
            $newApproverHistory->FinalApprovalDate = null;
            
            // Update old ProjectCharterApproverHistory;
            $oldApproverHistory->LatestVersion = 0;
            
            /* New list of final approvers */
            $oldApprovers = $this->context->projectCharterApproverRepository->findAllApprover($projectCharterID, $newApproverHistory->Version - 1);
            $newApprovers = array();
            foreach($oldApprovers as $oldApprover)
            {
                $newApprovers[$oldApprover->projectCharterApproverID] = new ProjectCharterApprover();
                $newApprovers[$oldApprover->projectCharterApproverID]->ProjectCharterID = $oldApprover->ProjectCharterID;
                $newApprovers[$oldApprover->projectCharterApproverID]->Version = ($oldApprover->Version + 1);
                $newApprovers[$oldApprover->projectCharterApproverID]->ApproverEID = $oldApprover->ApproverEID;
                $newApprovers[$oldApprover->projectCharterApproverID]->Surname = $oldApprover->surname;
                $newApprovers[$oldApprover->projectCharterApproverID]->Name = $oldApprover->name;
                $newApprovers[$oldApprover->projectCharterApproverID]->Email = $oldApprover->email;
				$newApprovers[$oldApprover->projectCharterApproverID]->Role = $oldApprover->role;
                $newApprovers[$oldApprover->projectCharterApproverID]->Approval = ProjectCharterApprover::DECISION_PENDING;
                $newApprovers[$oldApprover->projectCharterApproverID]->DecisionDateTime = null;
                $newApprovers[$oldApprover->projectCharterApproverID]->Comment = null;
                $newApprovers[$oldApprover->projectCharterApproverID]->AutoApproved = false;
            }
            
            $success = false;
            if($this->context->projectCharterApproverHistoryRepository->update($oldApproverHistory) && $this->context->projectCharterApproverHistoryRepository->save($newApproverHistory))
            {
                $success = true;
                $approverSucces = true;
                foreach($newApprovers as $approver)
                {
                    if($this->context->projectCharterApproverRepository->save($approver))
                    {
                        
                    }
                    else
                    {
                        $approverSucces = false;
                    }
                }
            }
            if($success && $approverSucces)
            {
                $this->flashMessage('Project charter is prepared for send to final approvers. Write the email.', 'success');
                $this->redirect("ProjectCharterApproval:email", $projectCharterID);
            }
            else
            {
                $this->flashMessage('Sending project charter to final approvars failed.', 'error');
                $this->redirect("ProjectCharter:detail", $projectCharterID);
            }
        }
        else
        {
            $this->flashMessage('Sending project charter to final approvars failed.', 'error');
            $this->redirect("ProjectCharter:detail", $projectCharterID);
        }
    }
    
    /**
     * Final Approvers Update button.
     * 
     * @param SubmitButton $button.
     */
    public function finalApproversUpdateClicked(SubmitButton $button)
    {
        $projectCharterID = $this->getHttpRequest()->getQuery('id');
		
		// check accessibility for user
		$this->secureAccess($projectCharterID, array(ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED), TRUE);
		
        $charter = $this->context->projectCharterRepository->find($projectCharterID);
        $charter->Status = ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION;
        if($this->context->projectCharterRepository->update($charter))
        {
            $creation_date = date("Y-m-d H:i:s");
            $oldApproverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($projectCharterID);
            
            // Creates new ProjectCharterApproverHistory from old one
            $newApproverHistory = new ProjectCharterApproverHistory();
            $newApproverHistory->ProjectCharterID = $oldApproverHistory->ProjectCharterID;
            $newApproverHistory->Version = $oldApproverHistory->Version + 1;
            $newApproverHistory->CreationDateTime = $creation_date;
            $newApproverHistory->Responsible = $oldApproverHistory->Responsible;
            $newApproverHistory->LatestVersion = 1;
            $newApproverHistory->BudgetVersion = $oldApproverHistory->BudgetVersion;
            $newApproverHistory->FileVersion = $oldApproverHistory->FileVersion;
            $newApproverHistory->EmailContent = null;
            $newApproverHistory->FinalApprovalDate = null;
            // Update old ProjectCharterApproverHistory;
            $oldApproverHistory->LatestVersion = 0;
            
            /* New list of final approvers */
            $oldApprovers = $this->context->projectCharterApproverRepository->findAllApprover($projectCharterID, $newApproverHistory->Version - 1);
            $newApprovers = array();
            foreach($oldApprovers as $oldApprover)
            {
                $newApprovers[$oldApprover->projectCharterApproverID] = new ProjectCharterApprover();
                $newApprovers[$oldApprover->projectCharterApproverID]->ProjectCharterID = $oldApprover->ProjectCharterID;
                $newApprovers[$oldApprover->projectCharterApproverID]->Version = ($oldApprover->Version + 1);
                $newApprovers[$oldApprover->projectCharterApproverID]->ApproverEID = $oldApprover->ApproverEID;
                $newApprovers[$oldApprover->projectCharterApproverID]->Surname = $oldApprover->Surname;
                $newApprovers[$oldApprover->projectCharterApproverID]->Name = $oldApprover->Name;
                $newApprovers[$oldApprover->projectCharterApproverID]->Email = $oldApprover->Email;
				$newApprovers[$oldApprover->projectCharterApproverID]->Role = $oldApprover->Role;
                $newApprovers[$oldApprover->projectCharterApproverID]->Approval = ProjectCharterApprover::DECISION_PENDING;
                $newApprovers[$oldApprover->projectCharterApproverID]->DecisionDateTime = null;
                $newApprovers[$oldApprover->projectCharterApproverID]->Comment = null;
                $newApprovers[$oldApprover->projectCharterApproverID]->AutoApproved = false;
            }
            $success = false;
            if($this->context->projectCharterApproverHistoryRepository->update($oldApproverHistory) && $this->context->projectCharterApproverHistoryRepository->save($newApproverHistory))
            {
                $success = true;
                $approverSucces = true;
                foreach($newApprovers as $approver)
                {
                    if($this->context->projectCharterApproverRepository->save($approver))
                    {
                        
                    }
                    else
                    {
                        $approverSucces = false;
                    }
                }
            }
            if($success && $approverSucces)
            {
                $this->redirect("ProjectCharterApproval:definition", $projectCharterID);
            }
            else
            {
                $this->flashMessage('Updating list of final approvers failed.', 'error');
                $this->redirect("ProjectCharter:detail", $projectCharterID);
            }
        }
        else
        {
            $this->flashMessage('Updating list of final approvers failed.', 'error');
            $this->redirect("ProjectCharter:detail", $projectCharterID);
        }
    }
    
    /**
     * Definition button.
     * 
     * @param SubmitButton $button.
     */
    public function definitionClicked(SubmitButton $button)
    {
        $projectCharterID = $this->getHttpRequest()->getQuery('id');
		
		// check accessibility for user
		$this->secureAccess($projectCharterID, array(ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED), TRUE);
		
        $charter = $this->context->projectCharterRepository->find($projectCharterID);
        $charter->Status = ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION;
        if($this->context->projectCharterRepository->update($charter))
        {
            $creation_date = date("Y-m-d H:i:s");
            $oldApproverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($projectCharterID);
            
            // Creates new ProjectCharterApproverHistory from old one
            $newApproverHistory = new ProjectCharterApproverHistory();
            $newApproverHistory->ProjectCharterID = $oldApproverHistory->ProjectCharterID;
            $newApproverHistory->Version = $oldApproverHistory->Version + 1;
            $newApproverHistory->CreationDateTime = $creation_date;
            $newApproverHistory->Responsible = $oldApproverHistory->Responsible;
            $newApproverHistory->LatestVersion = 1;
            $newApproverHistory->BudgetVersion = $oldApproverHistory->BudgetVersion;
            $newApproverHistory->FileVersion = $oldApproverHistory->FileVersion;
            $newApproverHistory->EmailContent = null;
            $newApproverHistory->FinalApprovalDate = null;
            // Update old ProjectCharterApproverHistory;
            $oldApproverHistory->LatestVersion = 0;
            
            /* New list of final approvers */
            $oldApprovers = $this->context->projectCharterApproverRepository->findAllApprover($projectCharterID, $newApproverHistory->Version - 1);
            $newApprovers = array();
            foreach($oldApprovers as $oldApprover)
            {
                $newApprovers[$oldApprover->projectCharterApproverID] = new ProjectCharterApprover();
                $newApprovers[$oldApprover->projectCharterApproverID]->ProjectCharterID = $oldApprover->ProjectCharterID;
                $newApprovers[$oldApprover->projectCharterApproverID]->Version = ($oldApprover->Version + 1);
                $newApprovers[$oldApprover->projectCharterApproverID]->ApproverEID = $oldApprover->ApproverEID;
                $newApprovers[$oldApprover->projectCharterApproverID]->Surname = $oldApprover->Surname;
                $newApprovers[$oldApprover->projectCharterApproverID]->Name = $oldApprover->Name;
                $newApprovers[$oldApprover->projectCharterApproverID]->Email = $oldApprover->Email;
				$newApprovers[$oldApprover->projectCharterApproverID]->Role = $oldApprover->Role;
                $newApprovers[$oldApprover->projectCharterApproverID]->Approval = ProjectCharterApprover::DECISION_PENDING;
                $newApprovers[$oldApprover->projectCharterApproverID]->DecisionDateTime = null;
                $newApprovers[$oldApprover->projectCharterApproverID]->Comment = null;
                $newApprovers[$oldApprover->projectCharterApproverID]->AutoApproved = false;
            }
            $success = false;
            if($this->context->projectCharterApproverHistoryRepository->update($oldApproverHistory) && $this->context->projectCharterApproverHistoryRepository->save($newApproverHistory))
            {
                $success = true;
                $approverSucces = true;
                foreach($newApprovers as $approver)
                {
                    if($this->context->projectCharterApproverRepository->save($approver))
                    {
                        
                    }
                    else
                    {
                        $approverSucces = false;
                    }
                }
            }
            if($success && $approverSucces)
            {
                $this->flashMessage('Project charter status was changed to definition.', 'success');
                $this->redirect("ProjectCharter:edit", $projectCharterID);
            }
            else
            {
                $this->flashMessage('Changing project charter status to definition failed.', 'error');
                $this->redirect("ProjectCharter:detail", $projectCharterID);
            }
        }
        else
        {
            $this->flashMessage('Changing project charter status to definition failed.', 'error');
            $this->redirect("ProjectCharter:detail", $projectCharterID);
        }
    }
    
    /**
     * Cancel Project Charter button.
     * 
     * @param SubmitButton $button.
     */
    public function cancelProjectCharterClicked(SubmitButton $button)
    {
        $projectCharterID = $this->getHttpRequest()->getQuery('id');
		
		// check accessibility for user
		$this->secureAccess($projectCharterID, array(ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED), TRUE);
		
        $charter = $this->context->projectCharterRepository->find($projectCharterID);
        $charter->Status = ProjectCharter::PROJECT_CHARTER_STATUS_CANCELLED;
        if($this->context->projectCharterRepository->update($charter))
        {
            $creation_date = date("Y-m-d H:i:s");
            $oldApproverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($projectCharterID);
            
            // Creates new ProjectCharterApproverHistory from old one
            $newApproverHistory = new ProjectCharterApproverHistory();
            $newApproverHistory->ProjectCharterID = $oldApproverHistory->ProjectCharterID;
            $newApproverHistory->Version = $oldApproverHistory->Version + 1;
            $newApproverHistory->CreationDateTime = $creation_date;
            $newApproverHistory->Responsible = $oldApproverHistory->Responsible;
            $newApproverHistory->LatestVersion = 1;
            $newApproverHistory->BudgetVersion = $oldApproverHistory->BudgetVersion;
            $newApproverHistory->FileVersion = $oldApproverHistory->FileVersion;
            $newApproverHistory->EmailContent = null;
            $newApproverHistory->FinalApprovalDate = null;
            // Update old ProjectCharterApproverHistory;
            $oldApproverHistory->LatestVersion = 0;
            
            /* New list of final approvers */
            $oldApprovers = $this->context->projectCharterApproverRepository->findAllApprover($projectCharterID, $newApproverHistory->Version - 1);
            $newApprovers = array();
            foreach($oldApprovers as $oldApprover)
            {
                $newApprovers[$oldApprover->projectCharterApproverID] = new ProjectCharterApprover();
                $newApprovers[$oldApprover->projectCharterApproverID]->ProjectCharterID = $oldApprover->ProjectCharterID;
                $newApprovers[$oldApprover->projectCharterApproverID]->Version = ($oldApprover->Version + 1);
                $newApprovers[$oldApprover->projectCharterApproverID]->ApproverEID = $oldApprover->ApproverEID;
                $newApprovers[$oldApprover->projectCharterApproverID]->Surname = $oldApprover->Surname;
                $newApprovers[$oldApprover->projectCharterApproverID]->Name = $oldApprover->Name;
                $newApprovers[$oldApprover->projectCharterApproverID]->Email = $oldApprover->Email;
				$newApprovers[$oldApprover->projectCharterApproverID]->Role = $oldApprover->Role;
                $newApprovers[$oldApprover->projectCharterApproverID]->Approval = ProjectCharterApprover::DECISION_PENDING;
                $newApprovers[$oldApprover->projectCharterApproverID]->DecisionDateTime = null;
                $newApprovers[$oldApprover->projectCharterApproverID]->Comment = null;
                $newApprovers[$oldApprover->projectCharterApproverID]->AutoApproved = null;
            }
            $success = false;
            if($this->context->projectCharterApproverHistoryRepository->update($oldApproverHistory) && $this->context->projectCharterApproverHistoryRepository->save($newApproverHistory))
            {
                $success = true;
                $approverSucces = true;
                foreach($newApprovers as $approver)
                {
                    if($this->context->projectCharterApproverRepository->save($approver))
                    {
                        
                    }
                    else
                    {
                        $approverSucces = false;
                    }
                }
            }
            if($success && $approverSucces)
            {
                $this->flashMessage('Project charter was canceled.', 'success');
                $this->redirect("ProjectCharter:detail", $projectCharterID);
            }
            else
            {
                $this->flashMessage('Canceling of project charter failed.', 'error');
                $this->redirect("ProjectCharter:detail", $projectCharterID);
            }
        }
        else
            $this->flashMessage('Canceling of project charter failed.', 'error');
        $this->redirect("ProjectCharter:detail", $projectCharterID);
    }
    
    /**
     * Back button.
     * 
     * @param SubmitButton $button.
     */
    public function backClicked(SubmitButton $button)
    {
        $this->redirect("ProjectCharter:myAction");
    }
    
    /**
     * Creates form for disapprove received.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentFileForm($name)
    {
        $form = new AppForm($this, $name);
        
        $form->addUpload('File', 'Add charter document')
                ->setAttribute('size', '30')
                ->addRule(Form::FILLED, 'Select charter document!');
        
        $form->addSubmit('Submit', 'Submit')
                ->onClick[] = array($this, 'updateFileSubmitted');
        
        return $form;
    }
    
    /**
     * File form submitted.
     * 
     * @param SubmitButton $button
     */
    public function updateFileSubmitted(SubmitButton $button)
    {        
        if ($button->form->isValid())
        {
            $entry = $button->form->getValues();
            // processing of charter file
            if ($entry['File'])
            {
                // update latest file version
                $lastFile = $this->context->projectCharterFileHistoryRepository->findLatestVersion($this->getHttpRequest()->getQuery('id'));
                $lastFile->latestVersion = 0;
                
                // basic file informations
                $newFile = new ProjectCharterFileHistory();
                $newFile->setProjectCharterID($this->getHttpRequest()->getQuery('id'));
                $newFile->setVersion($lastFile->getVersion() + 1);
                $newFile->setCreationDateTime(new DateTime);
                $newFile->setFileName($entry['File']->name);
                $newFile->setResponsible($this->getUser()->getId());
                $newFile->setLatestVersion(1);
                
                // set path to storage
                $storage = 
                    __DIR__ . $this->charters_storage_dir .
                    $this->charter_dir_prefix . $newFile->getProjectCharterID() . '/' .
                    $this->version_dir_prefix . $newFile->getVersion() . '/'
                ;
                
                // project charter approver history                
                $projectCharterApproverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($this->getHttpRequest()->getQuery('id'));
                $projectCharterApproverHistory->FileVersion = $newFile->Version;
                // save file & update project charter approver history file version
                if($this->context->projectCharterFileHistoryRepository->save($newFile) && $this->context->projectCharterFileHistoryRepository->update($lastFile) && $this->context->projectCharterApproverHistoryRepository->update($projectCharterApproverHistory))
                {
                    if (!is_dir($storage))
                        mkdir($storage, '0777', TRUE);
                    if($entry['File']->move($storage . $entry['File']->name))
                    {
                        $this->flashMessage("Charter document was updated.", 'success');
                    }
                    else
                    {
                        $this->flashMessage("Saving charter document failed.", 'error');
                    }
                }
                else
                {
                    $this->flashMessage("Saving charter document failed.", 'error');
                }
            }
        }
        else
        {
            $this->flashMessage("No charter document selected.", 'error');
        }
        $this->redirect('ProjectCharterApproval:disapproveReceived', $this->getHttpRequest()->getQuery('id'));
    }
	
	/**
	 * Restores charter from end states.
	 * @param SubmitButton $button
	 */
	public function restoreDetailCharterFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues(TRUE);
			
			// important TODO: remove this hard-coded hack!
			if($this->context->projectCharterRepository->find($entry['ProjectCharterID'])->year <= 2015)
			{
				$this->flashMessage('Modifying of project charters from year 2015 and older has been disabled. '
					. 'Please, create new charter instead.', 'error');
				return;
			}
			
			// check accessibility for user
			$this->secureAccess($entry['ProjectCharterID'], array(ProjectCharter::PROJECT_CHARTER_STATUS_APPROVED, ProjectCharter::PROJECT_CHARTER_STATUS_CANCELLED, ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION, ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED), TRUE);
			
			$charter = $this->context->projectCharterRepository->find($entry['ProjectCharterID']);
			if ($charter)
			{
				$succes = true;
				if($charter->status == ProjectCharter::PROJECT_CHARTER_STATUS_APPROVED)
				{
					$creation_date = date("Y-m-d H:i:s");
					$oldApproverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);

					// Creates new ProjectCharterApproverHistory from old one
					$newApproverHistory = new ProjectCharterApproverHistory();
					$newApproverHistory->ProjectCharterID = $oldApproverHistory->ProjectCharterID;
					$newApproverHistory->Version = $oldApproverHistory->Version + 1;
					$newApproverHistory->CreationDateTime = $creation_date;
					$newApproverHistory->Responsible = $oldApproverHistory->Responsible;
					$newApproverHistory->LatestVersion = 1;
					$newApproverHistory->BudgetVersion = $oldApproverHistory->BudgetVersion;
					$newApproverHistory->FileVersion = $oldApproverHistory->FileVersion;
					$newApproverHistory->EmailContent = null;
					$newApproverHistory->FinalApprovalDate = null;
					// Update old ProjectCharterApproverHistory;
					$oldApproverHistory->LatestVersion = 0;

					/* New list of final approvers */
					$oldApprovers = $this->context->projectCharterApproverRepository->findAllApprover($entry['ProjectCharter_ID'], $newApproverHistory->Version - 1);
					$newApprovers = array();
					foreach($oldApprovers as $oldApprover)
					{
						$newApprovers[$oldApprover->projectCharterApproverID] = new ProjectCharterApprover();
						$newApprovers[$oldApprover->projectCharterApproverID]->ProjectCharterID = $oldApprover->ProjectCharterID;
						$newApprovers[$oldApprover->projectCharterApproverID]->Version = ($oldApprover->Version + 1);
						$newApprovers[$oldApprover->projectCharterApproverID]->ApproverEID = $oldApprover->ApproverEID;
						$newApprovers[$oldApprover->projectCharterApproverID]->Surname = $oldApprover->Surname;
						$newApprovers[$oldApprover->projectCharterApproverID]->Name = $oldApprover->Name;
						$newApprovers[$oldApprover->projectCharterApproverID]->Email = $oldApprover->Email;
						$newApprovers[$oldApprover->projectCharterApproverID]->Role = $oldApprover->Role;
						$newApprovers[$oldApprover->projectCharterApproverID]->Approval = ProjectCharterApprover::DECISION_PENDING;
						$newApprovers[$oldApprover->projectCharterApproverID]->DecisionDateTime = null;
						$newApprovers[$oldApprover->projectCharterApproverID]->Comment = null;
						$newApprovers[$oldApprover->projectCharterApproverID]->AutoApproved = null;
					}
					if($this->context->projectCharterApproverHistoryRepository->update($oldApproverHistory) && $this->context->projectCharterApproverHistoryRepository->save($newApproverHistory))
					{
						foreach($newApprovers as $approver)
						{
							if(!$this->context->projectCharterApproverRepository->save($approver))
							{
								$succes = false;
							}
						}
					}
					else
					{
						$succes = false;
					}
				}
				$charter->setStatus(ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION);
				if ($this->context->projectCharterRepository->update($charter) && $succes)
					$this->flashMessage("Project charter '" . $charter->getName() . "' restored.", 'success');
				else
					$this->flashMessage("Restoring of project charter '" . $charter->getName() . "' failed.", 'error');
			}
			$this->redirect('ProjectCharter:detail', $entry['ProjectCharterID']);
		}

		$this->flashMessage("Error occured during submitting the form.", 'error');
		$this->redirect('this');
	}
}
