<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for administration of projects.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectPresenter extends SecuredPresenter
{
	/** @var session */
	private $session;
	/** @persistent */
	public $year;
	/** @const If this expires, we assume the session mutex is no longer acquired.
	 *  Value is in seconds. */
	const AJAX_BEACON_TIMEOUT = 3;

	/**
	 * Initialization
	 */
	public function startup()
	{
		parent::startup();
		
		$this->session = $this->getSession('Project');
        $this->session->setExpiration(0);
        if (!isset($this->session->form))
            $this->session->form = array();

		if (!isset($this->session->poAttributesAndBudget))
			$this->session->poAttributesAndBudget = array();
        
        $this->setYear($this->year, $this->session);
        
        // Forget year before opening another project detail
        if ($this->getAction() == 'default')
        {
        	$this->dropYear($this->session);
        	$this->year = NULL;
        }

        // Forget year before opening another workpackage detail
        $wp_session = $this->getSession('Workpackage');
        $this->dropYear($wp_session);
	}
    
    /**
     * Handles creation of AssignedResourcesAndActuals report
     */
    public function handleYearProjectReport($downloadTokenValue = null)
    {
        $projectID = $this->getHttpRequest()->getQuery('id');
		$year = $this->getHttpRequest()->getQuery('year');

		$multilog = new MultiLogger();
		$batch_id = 'ProjectReport-'.time().'-'. rand(1, 9);
		$multilog->add($this->context->createLogger($batch_id, "Project Report"));
		$downloadTokenValue = (is_null($downloadTokenValue)) ? time() : $downloadTokenValue;
        $projectReportService = new ProjectReportService($projectID, $this->context, $multilog, array($year), $downloadTokenValue);
		$importResult = true;
        try
        {
			$projectReportService->runProjectReportGeneration();
		}
        catch (Exception $e)
        {
			$importResult = false;
            Debugger::log('Project Report generation failed: ' . $e->getMessage(), Debugger::ERROR);
			$multilog->log('Project Report generation failed: ' . $e->getMessage(), ILogger::ERROR);
			$this->flashMessage('Project Report generation failed: ' . $e->getMessage(), 'error');
		}
		if($importResult)
		{
			$multilog->log('Project Report generation completed.', ILogger::NOTICE);
			$this->flashMessage('Project Report generation was successful.', 'success');
		}
		$this->redirect('this');
    }
    
    /**
     * Handles creation of AssignedResourcesAndEAC report
     */
    public function handleWholeTimeProjectReport($downloadTokenValue = null)
    {
        $projectID = $this->getHttpRequest()->getQuery('id');

		$multilog = new MultiLogger();
		$batch_id = 'ProjectReport-'.time().'-'. rand(1, 9);
		$multilog->add($this->context->createLogger($batch_id, "Project Report"));
		$downloadTokenValue = (is_null($downloadTokenValue)) ? time() : $downloadTokenValue;
        $projectReportService = new ProjectReportService($projectID, $this->context, $multilog, null, $downloadTokenValue);
		$importResult = true;
        try
        {
			$projectReportService->runProjectReportGeneration();
		}
        catch (Exception $e)
        {
			$importResult = false;
            Debugger::log('Project Report generation failed: ' . $e->getMessage(), Debugger::ERROR);
			$multilog->log('Project Report generation failed: ' . $e->getMessage(), ILogger::ERROR);
			$this->flashMessage('Project Report generation failed: ' . $e->getMessage(), 'error');
		}
		if($importResult)
		{
			$multilog->log('Project Report generation completed.', ILogger::NOTICE);
			$this->flashMessage('Project Report generation was successful.', 'success');
		}
		$this->redirect('this');
    }
	
	/**
	 * Returns only cutout of programs or calls according to selected customer.
	 * @param string $json_values Values of combo boxes encoded by JSON.stringify()
	 * @return array Creates associative array indexed by name of combo boxes in payload. Each index contains related options.
	 */
	public function actionFilterCustomerProgramCall($json_values)
	{
		$values = json_decode($json_values, TRUE);
		
		if ($values['Customer'])
		{
			$related_programs = $this->context->customerProgramRepository->findByCustomer($values['Customer']);
			$customer_related_calls = $this->context->customerProgramCallRepository->findByCustomer($values['Customer']);
		}
		else
		{
			$related_programs = $customer_related_calls = array();
		}
		
		// Programs
		$programs = $programs_list = array();
		foreach ($related_programs as $program)
		{
			$programs[$program->ProgramID] = $program->Program;
			$programs_list[] = $program->ProgramID;
		}
		
		if (($values['name'] == 'Program_ID') || (in_array($values['Program_ID'], $programs_list)))
		{
			$program_related_calls = $this->context->customerProgramCallRepository->findByProgram($values['Program_ID']);
		}
		
		// Calls
		if (isset($program_related_calls))
			$related_calls = $program_related_calls;
		else
			$related_calls = $customer_related_calls;
		
		$calls = array();
		foreach ($related_calls as $call)
			$calls[$call->CallID] = $call->Call;
		
		$this->payload->customerProgramCallFiltered = array(
			'Program_ID' => $programs,
			'Call_ID' => $calls
		);
		
		$this->terminate();
	}
	
	/**
	 * Is called when user has open an instance of PO Attributes & Budget - Edit window
	 * and so holds the mutex.
	 * @return
	 */
	public function actionUserRefreshEditingStatus($removeFlag = FALSE)
	{
		if($this->isAjax())
		{
			if($removeFlag)
			{
				$this->session->creatingOrEditingFlag = NULL;
			}
			else
			{
				// updates last timestamp
				$this->session->creatingOrEditingFlag = time();
			}
			$this->terminate();
		}
	}
    
    /**
     * Global setting of templates
     */
    public function beforeRender()
    {
        parent::beforeRender();
        
        $this->template->year = $this->year;
    }
	
	/**
	 * Shows projects overview
	 */
    public function renderDefault()
    {
		$this->template->form = $this['filterProjectForm'];
		$filtered_projects = array();
		
		if ($this->isAjax()) // is ajax request
		{
			$filter = new ProjectFilter();
			if (isset($this->payload->filter)) // filter is defined
			{
				$entry = $this->payload->filter; // $entry contains content of filterProjectForm

				$filter->Name = $entry->Name;
				$filter->SAPID = $entry->SAP_ID;
				$filter->KeyCode = $entry->KeyCode;
				$filter->POValueMin = $entry->POValueMin;
				$filter->POValueMax = $entry->POValueMax;
				$filter->PONumber = $entry->PO_number;
				$filter->Start = $entry->Start;
				$filter->End = $entry->End;
				$filter->PIPM = $entry->PIPM;
				$filter->TM = $entry->TM;
				$filter->PMOE = $entry->PMOE;
				$filter->PPaC = $entry->PPaC;
				$filter->PMO = $entry->PMO;
				$filter->Customer = $entry->Customer;
				$filter->Program = $entry->Program_ID;
				$filter->Call = $entry->Call_ID;
				$filter->Responsibility = $entry->Responsibility;
				$filter->ResponsibleUnit = $entry->ResponsibleUnit;
				$filter->chargeNumber = $entry->ChargeNumber;
				if ($entry->Type)
				{
					$types = array();
					foreach ($entry->Type as $type)
						$types[] = $type;
					$filter->Type = $types;
				}
				if($entry->Phase)
				{
					$phases = array();
					foreach ($entry->Phase as $phase)
						$phases[] = $phase;
					$filter->Phase = $phases;
				}
				if($entry->Fake) 
				{ 
					$fakes = array(); 
					foreach ($entry->Fake as $fake) 
						$fakes[] = $fake; 
					$filter->Fake = $fakes; 
				}
				$filter->State = $entry->State;
			}
			
			$this->context->formStateRepository->save('filterProjectForm', $this->template->form); // save filter state
			$filtered_projects = $this->context->projectRepository->findByFilter($filter);
		}
		else
		{
			if ($this->context->formStateRepository->exists('filterProjectForm')) // exists saved filter
				$this->context->formStateRepository->load('filterProjectForm', $this->template->form);
			$this->setCustomerDependencies($this->template->form);
		}
		
		// get project states
		$all_states = $this->context->projectStateRepository->findAll();
        $states = array();
        foreach ($all_states as $state)
            $states[$state->Number] = $state->NumberWithName;
		
		// setting displayed content
		$all_setting = $this->context->settingDisplayedContentRepository->findByEID($this->getUser()->getId());
		$allowed_responsible_units = array();
		foreach ($all_setting as $setting)
			$allowed_responsible_units[] = $setting->OrganizationUnitID;
		
		// export control
		$all_licences = $this->context->ecLicenceRepository->findAll();
		
		// prepare projects to be shown
		$projects = array();
		foreach ($filtered_projects as $project)
		{
			// filter projects according to setting of displayed content by excluding from list
			// projects without filled responsible unit will be displayed
			if ($project->ResponsibleUnit && !in_array($project->ResponsibleUnit, $allowed_responsible_units))
			    continue;
			
			$item = array();
			$item['ProjectID'] = $project->ID;
			$item['Name'] = $project->Name;
			$item['SAPID'] = $project->SAP_ID;
			    $poValue  = $this->context->poAttributesAndBudgetRepository->poValueInCurrentYearByProjectID($project->ID);
			$item['POValueSum'] = ($poValue) ? $poValue : '-';
			    $poNumber = $this->context->poAttributesAndBudgetRepository->poNumberInCurrentYearByProjectID($project->ID);
			$item['PONumber'] = ($poNumber) ? substr($poNumber, 0, 15) : '-';
			if (strlen($poNumber) > 15)
				$item['PONumber'] .= '..';
			$item['PONumber_Full'] = ($poNumber) ? $poNumber : '-';
			$item['KeyCode'] = ($project->KeyCode) ? $project->KeyCode : '-';
			    $person = $this->context->personRepository->find($project->PIPM);
			$item['PIPM'] = ($person) ? $person->ReversedShortName : '-';
			$item['PIPM_Full'] = ($person) ? $person->ReversedFullNameWithEID : '-';
			    $person = $this->context->personRepository->find($project->TM);
			$item['TM'] = ($person) ? $person->ReversedShortName : '-';
			$item['TM_Full'] = ($person) ? $person->ReversedFullNameWithEID : '-';
			    $person = $this->context->personRepository->find($project->PMOE);
			$item['PMOE'] = ($person) ? $person->ReversedShortName : '-';
			$item['PMOE_Full'] = ($person) ? $person->ReversedFullNameWithEID : '-';
			    $person = $this->context->personRepository->find($project->PPaC);
			$item['PPaC'] = ($person) ? $person->ReversedShortName : '-';
			$item['PPaC_Full'] = ($person) ? $person->ReversedFullNameWithEID : '-';
			$item['PMOSupport'] = ($project->PMOSupport) ? $project->PMOSupport : '-';
			$item['Phase'] = $project->Phase;
			$item['Type'] = $project->Type;
			$item['State'] = $project->State;
			$item['State_Full'] = (count($states) ? $states[$project->State] : '-');
			$item['Start'] = $project->Start;
			$item['End'] = $project->End;
			$item['ECLicence'] = (isset($all_licences[$project->ECLicenceID])) ? $all_licences[$project->ECLicenceID]->Name : '-';
			$item['Fake'] = $project->Fake;
			$projects[] = $item;
		}	
		
		$this->template->projects = $projects;			
    }
    
	/**
	 * Shows project detail
	 * @param integer $id Should be project id
	 */
    public function renderDetail($id, $year = NULL)
    {
		$project = $this->context->projectRepository->find($id);
		if (!$project)
		{
			$this->flashMessage('Invalid project ID. You have been redirected.', 'warning');
			$this->redirect('Project:default');
		}

        if ($year && is_numeric($year))
			$this->year = $year;

		$properties = array('EU_SAP_ID','PIPM','TM','PMOE','PPaC','Name',
			'Type','Customer', 'Responsibility','Description',
			'Phase','State','Remark', 'ECORemark', 'KeyCode','EuKeyCode','PMOSupport',
			'Start','End','SAP_ID', 'ResponsibleUnit', 'Plant');
		$form_defaults = array(
			'PO' => $this->context->poAttributesAndBudgetRepository->poValueByProjectIDAndYear($project->ID, $this->year),
			'TotalBudget' => $this->context->poAttributesAndBudgetRepository->budgetByProjectIDAndYear($project->ID, $this->year),
			'Project_ID' => $project->ID,
			'Program_ID' => $project->ProgramID, // program related to customer \_ filled this way, because of mismatches in system
			'Call_ID' => $project->CallID, // call related to customer program   /
			'RealCostShare' => $project->RealCostSharePercentage,
			'Plant' => $project->Plant
		);
		foreach ($properties as $property)
			$form_defaults[$property] = $project->$property;
		$PIPM   = $this->context->personRepository->find($project->PIPM);
		if (isset($PIPM))
			$form_defaults['PIPM_info'] = $PIPM->Surname . ', ' . $PIPM->Firstname;
		$TM     = $this->context->personRepository->find($project->TM);
		if (isset($TM))
			$form_defaults['TM_info'] = $TM->Surname . ', ' . $TM->Firstname;
		$PMOE   = $this->context->personRepository->find($project->PMOE);
		if (isset($PMOE))
			$form_defaults['PMOE_info'] = $PMOE->Surname . ', ' . $PMOE->Firstname;
		$PPaC   = $this->context->personRepository->find($project->PPaC);
		if (isset($PPaC))
			$form_defaults['PPaC_info'] = $PPaC->Surname . ', ' . $PPaC->Firstname;
		$licence  = $this->context->ecLicenceRepository->find($project->ECLicenceID);
		if (isset($licence))
			$form_defaults['ECLicenceID_info'] = $licence->Name;
		$this->template->project = $project;
		$this->template->form = $this['detailProjectForm'];
		if($project->Fake)
		{
			$form_defaults['ECLicenceID'] = $licence->ECLicenceID;
			$this->template->form['save']->setDisabled(false);
			$this->template->form['delete']->setDisabled(false);
			$this->template->form['Name']->setRequired('Please select project name!');
			$this->template->form['Plant']->addRule(Form::RANGE, '%label should be number in range <1000,9999>', array(1000, 9999));
		}
		else
		{
			$this->template->form['Plant']->setAttribute('readonly', 'readonly');
		}
		$this->template->form->setDefaults($form_defaults);
		$this->setCustomerDependencies($this->template->form);
			$poNumber = $this->context->poAttributesAndBudgetRepository->poNumberByProjectIDAndYear($project->ID, $this->year);
		$projectInfo['PONumber'] = ($poNumber) ? substr($poNumber, 0, 15) : '-';
		if (strlen($poNumber) > 15)
				$projectInfo['PONumber'] .= '..';
		$projectInfo['PONumber_Full'] = ($poNumber) ? $poNumber : '-';
		$this->template->projectInfo = $projectInfo;
		
		$this->template->projectPlanningYearSelectForm = $this['projectPlanningYearSelectForm'];
		
		$userID = strtoupper($this->getUser()->getId());
		if($project->Fake == 1)
			$this->template->isPHTaskCreationEnabled = $this->isAllowed(SecuredPresenter::RESOURCE_PLACEHOLDER_TASK, SecuredPresenter::ACTION_CREATE) || 
					($userID == $project->PIPM || $userID == $project->PMOE);
		else
			$this->template->isPHTaskCreationEnabled = $this->isAllowed(SecuredPresenter::RESOURCE_PLACEHOLDER_TASK, SecuredPresenter::ACTION_CREATE);
		
		$this->template->eacLink = $this->link('eAC:AjaxCall!');
		$this->template->wbsLink = $this->link('wBS:AjaxCall!');
		$this->template->ppLink = $this->link('projectPlanningCharter:AjaxCall!');
    }
	
	/**
	 * Shows form for project creation
	 */
	public function renderCreate()
	{
		$form = $this['createProjectForm'];
		
		$this->template->form = $form;
	}

	/**
	 * Shows maintenance part for PO attributes and budget
	 * @param integer $id Should be project id
	 */
	public function renderPoAttributesAndBudget($id)
	{
		$this->creatingOrEditingFlagRedirect();
		$project = $this->context->projectRepository->find($id);
		if (!$project)
		{
			$this->flashMessage('Invalid Project ID. You have been redirected.', 'warning');
            $this->redirect('Project:default');
		}
		
		$form = $this['poAttributesAndBudgetForm'];
		if($project->Fake == true)
		{
			$form['POValue']->addCondition(Form::FILLED)
								->addRule(Form::PATTERN, 'PO Value must be integer or decimal number in format NN or NN.NN.', '-?[0-9]+(\.[0-9]+)?');
			
			$form['Budget']->addCondition(Form::FILLED)
								->addRule(Form::PATTERN, 'Budget must be integer or decimal number in format NN or NN.NN.', '-?[0-9]+(\.[0-9]+)?');
		}
		else
		{
			$form['POValue']->addCondition(Form::FILLED)
								->addRule(Form::PATTERN, 'PO Value must be integer or decimal number in format NN or NN.NN.', '[0-9]+(\.[0-9]+)?');

			$form['Budget']->addCondition(Form::FILLED)
								->addRule(Form::PATTERN, 'Budget must be integer or decimal number in format NN or NN.NN.', '[0-9]+(\.[0-9]+)?');
		}
		
		if (count($this->session->form))
		{
			$form->setDefaults($this->session->form);
			$this->session->form = array();
		}
		
		// if id does not exist set default values
		if (!$form['ProjectID']->getValue())
		{
			// PO attributes & budget
			$all_poAttributesAndBudget = $this->context->poAttributesAndBudgetRepository->findByProjectID($id);
			$arrPOAttributesAndBudget = array();
			foreach ($all_poAttributesAndBudget as $poAttributesAndBudget)
			{
				$poAttr = array();
				$poAttr['POAttributesAndBudgetID'] = $poAttributesAndBudget->POAttributesAndBudgetID;
				$poAttr['Year'] = $poAttributesAndBudget->Year;
				$poAttr['POValue'] = $poAttributesAndBudget->POValue;
				$poAttr['PONumber'] = $poAttributesAndBudget->PONumber;
				$poAttr['Budget'] = $poAttributesAndBudget->Budget;
				$arrPOAttributesAndBudget[] = $poAttr;
			}
			$this->session->poAttributesAndBudget = $arrPOAttributesAndBudget;
			
			$defaults = array();
			$defaults['ProjectID'] = $id;
			$this->session->form = $defaults;
			$this->redirect('this');
		}
		
		foreach ($this->session->poAttributesAndBudget as $key => $poAttr)
		{
			$form['POAttributesAndBudgetID' . $key]->setDefaultValue($poAttr['POAttributesAndBudgetID']);
			$form['Year' . $key]->setDefaultValue($poAttr['Year']);
			if($project->Fake == true)
			{
				$form['POValue' . $key]->addCondition(Form::FILLED)
									->addRule(Form::PATTERN, 'Budget must be integer or decimal number in format NN or NN.NN.', '-?[0-9]+(\.[0-9]+)?');
				$form['Budget' . $key]->addCondition(Form::FILLED)
									->addRule(Form::PATTERN, 'Budget must be integer or decimal number in format NN or NN.NN.', '-?[0-9]+(\.[0-9]+)?');
			}
			else
			{
				$form['POValue' . $key]->addCondition(Form::FILLED)
									->addRule(Form::PATTERN, 'Budget must be integer or decimal number in format NN or NN.NN.', '[0-9]+(\.[0-9]+)?');
				$form['Budget' . $key]->addCondition(Form::FILLED)
									->addRule(Form::PATTERN, 'Budget must be integer or decimal number in format NN or NN.NN.', '[0-9]+(\.[0-9]+)?');
			}
			$form['POValue' . $key]->setDefaultValue($poAttr['POValue']);
			$form['PONumber' . $key]->setDefaultValue($poAttr['PONumber']);
			$form['Budget' . $key]->setDefaultValue($poAttr['Budget']);
		}
		$this->template->project = $project;
		$this->template->poAttributesAndBudget = $this->session->poAttributesAndBudget;
	}
	
    public function createComponentFilterProjectForm($name)
    {
		$setting = $this->context->userSettingRepository->find($this->getUser()->getId());
        $form = new AppForm($this, $name);

        $form->addHidden('Project_ID');
        $form->addText('Name', 'Name');
        $form->addText('Description', 'Description');
        $form->addText('SAP_ID', 'SAP ID');
        $form->addText('EU_SAP_ID', 'EU SAP ID');
        $form->addText('KeyCode', 'Key Code');
        $form->addText('EuKeyCode', 'EU Key Code');
        $form->addText('POValueMin', 'PO Value Min');
        $form->addText('POValueMax', 'PO Value Max');
        $form->addText('PO_number', 'PO Number');
		$form->addText('ChargeNumber', 'Task Charge Number'); //Task SAP ID
        
        
        $all_displayed_content = $this->context->settingDisplayedContentRepository->findByEID($this->getUser()->getId());
		$allowed_responsible_units = array();
		foreach ($all_displayed_content as $displayed_content)
			$allowed_responsible_units[] = $displayed_content->OrganizationUnitID;
        
        $all_sbes = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
		$sbe = array('' => '*');
		foreach ($all_sbes as $unit)
        {
            if (in_array($unit->ID, $allowed_responsible_units))
			    $sbe[$unit->ID] = $unit->Name;
        }
        $form->addMultiSelect('ResponsibleUnit', 'Responsible Unit', $sbe, 5)
                ->setDefaultValue('');

        $form->addText('PIPM', 'PI', 7);
        $PIPM = $form['PIPM']->getControlPrototype();
        $PIPM->addClass('autocomplete');
        $PIPM_info = $form->addText('PIPM_info', 'PI_info');
        $PIPM_info->setDisabled();
        $form->addText('TM', 'CL', 7);
        $TM = $form['TM']->getControlPrototype();
        $TM->addClass('autocomplete');
        $TM_info = $form->addText('TM_info', 'TM_info');
        $TM_info->setDisabled();
        $form->addText('PMOE', 'PM', 7);
        $PMOE = $form['PMOE']->getControlPrototype();
        $PMOE->addClass('autocomplete');
        $PMOE_info = $form->addText('PMOE_info', 'PM_info');
        $PMOE_info->setDisabled();
        $form->addText('PPaC', 'PP&C', 7);
        $PPaC = $form['PPaC']->getControlPrototype();
        $PPaC->addClass('autocomplete');
        $PPaC_info = $form->addText('PPaC_info', 'PPaC_info');
        $PPaC_info->setDisabled();

        $form->addSelect('PMO', 'PMO Support Level', array('' => '*', 'Full' => 'Full', 'Basic' => 'Basic'));
        $Phase = $form->addCheckboxList('Phase', 'Project Phase', array('Active' => 'Active', 'Planned' => 'Planned', 'Closed' => 'Closed', 'Cancelled' => 'Cancelled'));
        $Phase->setDefaultValue(array('Active', 'Planned'));
        $Phase->getSeparatorPrototype()->setName(NULL);
        $Type = $form->addCheckboxList('Type', 'Project Type', array('B&P' => 'B&P', 'Internal' => 'Internal (U)', 'External' => 'External (F)', 'Capital' => 'Capital', 'Cost-share' =>'Cost-share (U)', 'CSC' => 'CSC'));
        $Type->setDefaultValue(array('B&P', 'Internal', 'External', 'Capital', 'Cost-share', 'CSC'));
        $Type->getSeparatorPrototype()->setName(NULL);
		
		$fake = $form->addCheckboxList('Fake', 'Placeholder projects', array('1' => 'Placeholder', '0' => 'Real project')); 
		$fake->setDefaultValue(array('0')); 
		$fake->getSeparatorPrototype()->setName(NULL);

		/** 
		 * Program_ID and Call_ID are dynamically related with Customer.
		 * Each customer has some related programs and each program has some related calls.
		 * Here are fetched all of programs as well as all calls, but displayed items are
		 * filtered in render method according to current selection.
		 * It is simpler to obtain values from form if there are fetched all items.
		 */
        $all_customers = $this->context->customerRepository->findAll();
        $customers = array('' => '*');
        foreach ($all_customers as $customer)
            $customers[$customer->ID] = $customer->Name;
        $form->addSelect('Customer', 'Customer', $customers)
                ->setAttribute('style', 'width: 201px;');
		
		$all_customerPrograms = $this->context->customerProgramRepository->findAll();
		$customerPrograms = array('' => '*');
		foreach ($all_customerPrograms as $program)
			$customerPrograms[$program->ProgramID] = $program->Program;
        $form->addSelect('Program_ID', 'Program', $customerPrograms)
				->setAttribute('style', 'width: 201px;');
		
		$all_customerProgramCalls = $this->context->customerProgramCallRepository->findAll();
		$customerProgramCalls = array('' => '*');
		foreach ($all_customerProgramCalls as $call)
			$customerProgramCalls[$call->CallID] = $call->Call;
        $form->addSelect('Call_ID', 'Call', $customerProgramCalls)
				->setAttribute('style', 'width: 201px;');

        $all_responsibilities = $this->context->responsibilityRepository->findAll();
        $responsibilities = array('' => '*');
        foreach ($all_responsibilities as $responsibility)
            $responsibilities[$responsibility->ID] = $responsibility->Name;
        $form->addSelect('Responsibility', 'Responsibility', $responsibilities);

        $form->addText('Start', 'Starts After')
            ->addCondition(Form::FILLED, TRUE)
            ->addRule(Form::PATTERN, '%value is not valid %label!', '2[0-9]{3}-(0[0-9]{1}|1[012]{1})-([012]{1}[0-9]|3[01]{1})');
        $form['Start']->getControlPrototype()->addClass('datepicker');
        $form->addText('End', 'Ends Before')
            ->addCondition(Form::FILLED, TRUE)
            ->addRule(Form::PATTERN, '%value is not valid %label!', '2[0-9]{3}-(0[0-9]{1}|1[012]{1})-([012]{1}[0-9]|3[01]{1})');
        $form['End']->getControlPrototype()->addClass('datepicker');

        $all_states = $this->context->projectStateRepository->findAll();
        $states = array('' => '*');
        foreach ($all_states as $state)
            $states[$state->Number] = $state->NumberWithName;
        $form->addSelect('State', 'State', $states)
                ->setAttribute('style', 'width: 201px;');
		
        $form->addSubmit('create', 'Create placeholder Project')
				->onClick[] = callback($this, 'createProjectClicked');
		if(!$setting->ShowFakeProjects)
			$form['create']->setDisabled(true);

        $form->addSubmit('reset', 'Reset Filter')
            ->onClick[] = callback($this, 'filterProjectFormReset');
        $form->getElementPrototype()->class('ajax');

        $form->onSuccess[] = array($this, 'filterProjectFormSubmitted');

        return $form;
    }

    public function createComponentProjectForm($name)
    {
        $form = new AppForm($this, $name);

        $form->addHidden('Project_ID');
        $form->addText('Name', 'Project Name', null, 500);

        $form->addTextArea('Description', 'Description', 28, 3);
        $form->addText('SAP_ID', HPTForm::createLabel('SAP ID', 'Automatically generated'));
        $form->addText('EU_SAP_ID', 'EU SAP ID');
        $form->addText('KeyCode', 'Key Code');
        $form->addText('EuKeyCode', 'EU Key Code');
        $form->addText('PO', HPTForm::createLabel('PO Value', 'Sum of PO Values defined in current year.'))
				->setAttribute('readonly', 'readonly');
	    $form->addText('PONumber', HPTForm::createLabel('PO Number', 'Shows all PO Numbers defined in current year.'));

        $form->addText('PIPM', 'PI', 7)
			->getControlPrototype()
				->addClass('autocomplete');
        $form->addText('PIPM_info', 'PI_info')
			->setAttribute('readonly', 'readonly');
        $form->addText('TM', 'CL', 7)
			->getControlPrototype()
				->addClass('autocomplete');
        $form->addText('TM_info', 'TM_info')
			->setAttribute('readonly', 'readonly');
        $form->addText('PMOE', 'PM', 7)
			->getControlPrototype()
				->addClass('autocomplete');
		$form->addText('PMOE_info', 'PM_info')
			->setAttribute('readonly', 'readonly');
        $form->addText('PPaC', 'PP&C', 7)
			->getControlPrototype()
				->addClass('autocomplete');
        $form->addText('PPaC_info', 'PPaC_info')
			->setAttribute('readonly', 'readonly');

        $form->addSelect('PMOSupport', 'PMO Support Level', array('' => '*', 'Full' => 'Full', 'Basic' => 'Basic'));
        $Phase = $form->addRadioList('Phase', 'Project Phase', array('Active' => 'Active', 'Planned' => 'Planned', 'Closed' => 'Closed', 'Cancelled' => 'Cancelled'));
        $Phase->getSeparatorPrototype()->setName(NULL);
        $form->addSelect('Type', 'Project Type', array('B&P' => 'B&P', 'Internal' => 'Internal (U)', 'External' => 'External (F)', 'Capital' => 'Capital', 'Cost-share' =>'Cost-share (U)', 'CSC' => 'CSC'))
                ->setAttribute('style', 'width: 201px;');

		/** 
		 * Program_ID and Call_ID are dynamically related with Customer.
		 * Each customer has some related programs and each program has some related calls.
		 * Here are fetched all of programs as well as all calls, but displayed items are
		 * filtered in render method according to current selection.
		 * It is simpler to obtain values from form if there are fetched all items.
		 */
        $all_customers = $this->context->customerRepository->findAll();
        $customers = array('' => '*');
        foreach ($all_customers as $customer)
            $customers[$customer->ID] = $customer->Name;
        $form->addSelect('Customer', 'Customer', $customers)
                ->setAttribute('style', 'width: 201px;');
		
		$all_customerPrograms = $this->context->customerProgramRepository->findAll();
		$customerPrograms = array('' => '*');
		foreach ($all_customerPrograms as $program)
			$customerPrograms[$program->ProgramID] = $program->Program;
        $form->addSelect('Program_ID', 'Program', $customerPrograms)
				->setAttribute('style', 'width: 201px;');
		
		$all_customerProgramCalls = $this->context->customerProgramCallRepository->findAll();
		$customerProgramCalls = array('' => '*');
		foreach ($all_customerProgramCalls as $call)
			$customerProgramCalls[$call->CallID] = $call->Call;
        $form->addSelect('Call_ID', 'Call', $customerProgramCalls)
				->setAttribute('style', 'width: 201px;');

        $all_responsibilities = $this->context->responsibilityRepository->findAll();
        $responsibilities = array('' => '*');
        foreach ($all_responsibilities as $responsibility)
            $responsibilities[$responsibility->ID] = $responsibility->Name;
        $form->addSelect('Responsibility', 'Responsibility', $responsibilities);

        $form->addText('Start', 'Start Date')
			->addCondition(Form::FILLED, TRUE)
				->addRule(Form::PATTERN, '%value is not valid %label!', '2[0-9]{3}-(0[0-9]{1}|1[012]{1})-([012]{1}[0-9]|3[01]{1})');
        $form['Start']->getControlPrototype()->addClass('datepicker');
        $form->addText('End', 'End Date')
			->addCondition(Form::FILLED, TRUE)
				->addRule(Form::PATTERN, '%value is not valid %label!', '2[0-9]{3}-(0[0-9]{1}|1[012]{1})-([012]{1}[0-9]|3[01]{1})');
        $form['End']->getControlPrototype()->addClass('datepicker');
        $form->addText('TotalBudget', HPTForm::createLabel('Total Budget', 'Sum of Budgets defined in current year.'))
				->setAttribute('readonly', 'readonly');

        $all_states = $this->context->projectStateRepository->findAll();
        $states = array('' => '*');
        foreach ($all_states as $state)
            $states[$state->Number] = $state->Number.': '.$state->Name;
        $form->addSelect('State', 'State', $states)
                ->setAttribute('style', 'width: 201px;');

        $form->addTextArea('Remark', 'Remark', 32, 3);
		
		$form->addTextArea('ECORemark', 'EC Remark', 28, 3);

		$licences = array();
		$all_licences = $this->context->ecLicenceRepository->findAll();
		foreach ($all_licences as $licence)
			$licences[$licence->ECLicenceID] = $licence->Name;
 	  	$form->addSelect('ECLicenceID', 'Licence', $licences)
				->setDefaultValue(5)
			    ->setAttribute('style', 'width: 140px;');
		
		$all_SBE = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
		$SBE = array();
		foreach ($all_SBE as $unit)
        {
            if (!$unit->Virtual)
			    $SBE[$unit->ID] = $unit->Name;
        }
		$form->addSelect('ResponsibleUnit', 'Responsible Unit', $SBE)
				->setPrompt('')
				->addRule(Form::FILLED, 'Responsible Unit must be filled.');

		$form->addText('RealCostShare', HPTForm::createLabel('Cost Share', 'Percentage of costs sharing among customer and supplier.<br>If value is set, Cost Share of each task belonging to the project will be temporary overwritten by this value.'))
		        ->addCondition(Form::FILLED, TRUE)
		            ->addRule(Form::RANGE, '%label should be number in range <0,999>', array(0, 999));
		if (!$this->isAllowed(SecuredPresenter::RESOURCE_PROJECT_REAL_COST_SHARE, SecuredPresenter::ACTION_EDIT))
			$form['RealCostShare']->setAttribute('readonly', 'readonly');
		
		$form->addText('Plant', 'Plant');
		$form['Plant']->setRequired('Please select project Plant!');
				
        $form->addSubmit('poAttributesAndBudget', 'PO Attributes & Budget')
		    ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'poAttributesAndBudgetClicked');

        $form->addSubmit('discard', 'Discard changes')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'discardClicked');

        return $form;
    }
	
    public function createComponentDetailProjectForm($name)
    {
        $form = $this->createComponentProjectForm($name);

		$form['SAP_ID']->setAttribute('readonly', 'readonly');
		
		$form->addText('ECLicenceID_info', 'Licence')
				->setAttribute('readonly', 'readonly');
		
		$delete_button = $form->addSubmit('delete', 'Delete project')
				->setValidationScope(FALSE);
		if (!$this->isAllowed(SecuredPresenter::RESOURCE_PROJECT, SecuredPresenter::ACTION_DELETE))
			$delete_button->setDisabled(true);
		
        $submit_button = $form->addSubmit('save', 'Save project');
		if (!$this->isAllowed(SecuredPresenter::RESOURCE_PROJECT, SecuredPresenter::ACTION_EDIT))
			$submit_button->setDisabled(true);
		
		$form->onSuccess[] = array($this, 'detailProjectFormSubmitted');
		
        return $form;
    }
	
    public function createComponentCreateProjectForm($name)
    {
        $form = $this->createComponentProjectForm($name);

		$form['SAP_ID']->setAttribute('readonly','readonly');		
		$form['SAP_ID']->setValue('PH-' . $this->context->projectRepository->getNextProjectID());
		
        $submit_button = $form->addSubmit('create', 'Save');
		
		$form['poAttributesAndBudget']->setDisabled(true);
		
		$form['ECLicenceID']->addRule(Form::FILLED, 'Licence must be filled.');
		
		$form['Type']->setPrompt('*');
		$form['Type']->setRequired('Please select project type!');
		$form['Phase']->setRequired('Please select project phase!');
		$form['State']->setRequired('Please select project state!');
		$form['Name']->setRequired('Please select project name!');
		$form['Plant']->addRule(Form::RANGE, '%label should be number in range <1000,9999>', array(1000, 9999));
		
        $form->onSuccess[] = array($this, 'createProjectFormSubmitted');

        return $form;
    }

    public function poAttributesAndBudgetClicked(SubmitButton $button)
    {
		$entry = $button->form->getValues();
        $this->redirect('Project:poAttributesAndBudget', array('id' => $entry->Project_ID));
    }
	
    public function createProjectClicked(SubmitButton $button)
    {
        $this->redirect('create');
    }

    public function discardClicked(SubmitButton $button)
    {
        $this->flashMessage('Changes discarded.', 'info');
        $this->redirect('this');
    }

    /**
     * Handler triggered when Reset button is pressed. It will drop
     * remembered filter.
     */
    public function filterProjectFormReset()
    {
		$this->context->formStateRepository->drop('filterProjectForm');
		$this->redirect('this');
		Debugger::log('Project filter form reset.', Debugger::DEBUG);
    }

    public function filterProjectFormSubmitted(AppForm $form)
    {
		$this->payload->filter = $form->getValues();
		$this->invalidateControl('projects');
    }

    public function detailProjectFormSubmitted(AppForm $form)
    {
		$entry = $form->getValues();
		$project = $this->context->projectRepository->find($entry['Project_ID']);
		if($project == NULL)
		{
			$this->flashMessage('Project update failed. Cannot find project with ID='.$entry['Project_ID'], 'error');
			Debugger::log("Project update with ID ".$entry['Project_ID']." failed.", Debugger::ERROR);
			$this->redirect('this');
		}
		if ($form['save']->isSubmittedBy())
		{
			if ($this->isAllowed(SecuredPresenter::RESOURCE_PROJECT, SecuredPresenter::ACTION_EDIT) || ($project->Fake == 1))
			{
				if ($form->isValid())
				{
					try
					{
						// not all project properties can be changes here
						$properties = array('EU_SAP_ID','PIPM','TM','PMOE','PPaC',
							'Type','Customer','Responsibility','Description',
							'Phase','State','Remark', 'ECORemark', 'KeyCode','EuKeyCode','PMOSupport',
							'Start','End', 'ResponsibleUnit');
						foreach ($properties as $property)
							$project->$property = $entry[$property];
						// Placeholder project name can be changed
						if($project->Fake)
						{
							$project->Name = $entry['Name'];
							$project->Plant = $entry['Plant'];
						}
						$project->ProgramID = $entry['Program_ID'] ? $entry['Program_ID'] : NULL; // program related to customer \_ filled this way, because of mismatches in system
						$project->CallID = $entry['Call_ID'] ? $entry['Call_ID'] : NULL; // call related to customer program     /
						$project->RealCostSharePercentage = (strlen($entry['RealCostShare'])) ? $entry['RealCostShare'] : null;
						$persons = $this->context->personRepository->findAllPersonsAssignedToProjectInYearToFuture($project->ID, Date('Y'));
						if($entry['ECLicenceID'])
						{
							if($entry['ECLicenceID'] <> $project->ECLicenceID) // if EC licence is changed, EC check must follow
							{//findProjectByTaskId
								$tasks = $this->context->taskRepository->findAllProjectTasksInYear($project->ID, Date('Y'));
								$persons = $this->context->personRepository->findAllPersonsAssignedToProjectInYearToFuture($project->ID, Date('Y'));
								foreach($persons as $person)
								{
									if(!$this->context->ecUtility->hasAccess($person->NationalityID, $entry['ECLicenceID']))
									{
										foreach($tasks as $task)
										{
											if($task->LabourTaskFlag == 1)
											{
												$assignment = 0;
												$assigns = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, Date('Y'), $person->ID);
												foreach($assigns as $assign)
												{
													if($assign->Month >= Date('m'))
														$assignment += $assign->Hours;
												}
												if($assignment > 0)
												{
													$this->flashMessage('Licence can not be changed. There is assigned person: ' . $person->getReversedFullNameWithEID() . ' who breaks security rules based on licence.', 'error');
													$this->redirect('this');
												}
											}
										}
									}
								}
								$project->setECLicenceID($entry['ECLicenceID']);
							}
						}					

						if ($this->context->projectRepository->update($project))
						{
							if (!$this->context->taskRepository->updateRealCostShare($project->RealCostShare, $project->ID))
							{
								$this->flashMessage("Cannot update cost share for subordinate tasks.", 'error');
							}

							Debugger::log("Project ".$project->Name." with ID=".$project->ID." updated.", Debugger::INFO);
							$this->flashMessage("Project ".$project->Name." updated.", 'success');
							$this->redirect('this');
						}
						else
						{
							Debugger::log("Project update with ID ".$entry['Project_ID']." failed.", Debugger::ERROR);
							$this->flashMessage('Project update with ID '.$entry['Project_ID'].' failed. See log for more information.', 'error');
						}
					}
					catch (InvalidArgumentException $e)
					{
						Debugger::log("Project update with ID=".$entry['Project_ID']." failed. Invalid data." . $e, Debugger::ERROR);
						$this->redirect('this');
					}
				}
				else
				{
					$this->flashMessage('Invalid input. Project was not updated!', 'error');
					//$this->redirect('this');
				}
			}
			else
			{
				$this->flashMessage("You are not authorized for this operation. PMO role required.", 'error');
				$this->redirect('this');
			}
		}
		elseif ($form['delete']->isSubmittedBy())
		{
			if ($this->isAllowed(SecuredPresenter::RESOURCE_PROJECT, SecuredPresenter::ACTION_DELETE) || ($project->Fake == 1))
			{
				try
				{
					if ($this->context->projectRepository->delete($entry['Project_ID']))
						$this->flashMessage("Project ".$entry['Name']." deleted.", 'success');
					else
						$this->flashMessage("Failed to delete project ".$entry['Name']." .", 'error');
					$this->redirect('Project:default');
				}
				catch (DibiDriverException $e)
				{
					$this->flashMessage('Error: '.$e->getCode());
				}
			}
			else
			{
				$this->flashMessage("You are not authorized for this operation. ADMIN role required.", 'error');
				$this->redirect('this');
			}
		}
    }
	
    public function createProjectFormSubmitted(AppForm $form)
    {
		$entry = $form->getValues();
		if ($form['create']->isSubmittedBy())
		{
			if ($form->isValid())
			{
				if($this->context->projectRepository->findProjectBySAPID($entry['SAP_ID']))
				{
					$this->flashMessage("Project with SAP ID: '" . $entry['SAP_ID'] . "' already exists.", 'error');
					$this->redirect('this');
				}
				$project = new Project();
				$properties = array('Name', 'SAP_ID', 'EU_SAP_ID','PIPM','TM','PMOE','PPaC',
					'Type','Customer','Responsibility','Description',
					'Phase','State','Remark', 'ECORemark', 'KeyCode','EuKeyCode','PMOSupport',
							'Start','End', 'ResponsibleUnit', 'Plant');
				foreach ($properties as $property)
					$project->$property = $entry[$property];

				$project->ProgramID = $entry['Program_ID'] ? $entry['Program_ID'] : NULL; // program related to customer \_ filled this way, because of mismatches in system
				$project->CallID = $entry['Call_ID'] ? $entry['Call_ID'] : NULL; // call related to customer program     /
				$project->Fake = 1;
				$project->RealCostSharePercentage = (strlen($entry['RealCostShare'])) ? $entry['RealCostShare'] : null;
				if($entry['ECLicenceID'])
					$project->ECLicenceID = $entry['ECLicenceID'];
				
				if ($this->context->projectRepository->save($project))
				{
					if (!$this->context->taskRepository->updateRealCostShare($project->RealCostShare, $project->ID))
					{
						$this->flashMessage("Cannot update cost share for subordinate tasks.", 'error');
					}

					$this->flashMessage("Project " . $project->Name . " was saved.", 'success');
					$this->redirect('detail', $project->ID);
				}
				else
				{
					$this->flashMessage('Saving project ' . $project->Name . ' failed.', 'error');
					$this->redirect('this');
				}
			}
			else
			{
				$this->flashMessage('Invalid input. Project was not created!', 'error');
				$this->redirect('this');
			}
		}
    }
	
	/**
	 * Redirects user if he tries to do more things (edit PO Attributes & Budget)
	 * simultaneously. That way those tabs won't interfere and break session information,
	 * for instance Pursuit Risk. Better way of doing this would be using HTML5's sessionStorage via
	 * Javascript to store all data in a form on client's side.
	 * @return
	 */
	private function creatingOrEditingFlagRedirect()
	{
		// limit PO Attributes & Budget - Edit instance to only one
		if(!empty($this->session->creatingOrEditingFlag) && (time() - $this->session->creatingOrEditingFlag) < self::AJAX_BEACON_TIMEOUT)
		{
			$this->flashMessage('You can edit only one Project\'s PO Attributes and Budget at a time. We apologize for any inconvenience caused.', 'warning');
			$this->redirect('Project:default');
		}
	}

    public function handleAutoComplete()
    {
		$this->payload->autoComplete = array();

		$text = trim($_GET['q']);

		if ($text !== '')
		{
			$filter = new PersonFilter();
			$filter->setFirstname($text)->setSurname($text)->setID($text);
			$persons = $this->context->personRepository->getPersonsByOrFilter($filter);
			foreach ($persons as $person) 
			{
				$person_item = array($person->getReversedFullNameWithEID(), strtoupper(trim($person->ID)), $person->getReversedFullName());
				$this->payload->autoComplete[] = $person_item;
			}
		}

		$this->terminate();
	}
	
	/**
	 * Filters and sets programs as well as calls according to selected customer.
	 * @param AppForm $form Form with fields dynamically related with field Customer.
	 */
	protected function setCustomerDependencies($form)
	{
		// capture selected customer and related program. To be able to filter list of programs and list of calls
		$customerID = $form['Customer']->getValue();
		$programID = $form['Program_ID']->getValue();
		
		// filter related programs according to selected customer
		$programs = $this->context->customerProgramRepository->findByCustomer($customerID);
		$customerPrograms = array('' => '*');
		foreach ($programs as $program)
			$customerPrograms[$program->ProgramID] = $program->Program;
		$form['Program_ID']->setItems($customerPrograms);
		
		// filter related calls according to selected program
		$calls = $this->context->customerProgramCallRepository->findByProgram($programID);
		$customerProgramCalls = array('' => '*');
		foreach ($calls as $call)
			$customerProgramCalls[$call->CallID] = $call->Call;
		$form['Call_ID']->setItems($customerProgramCalls);
	}

    public function renderOldAdministration()
    {
 		if ($this->testUserRoles('PMO'))
            header('Location: /'.TOOL_DIR.'/app2/?page=project');
		else
			$this->flashMessage("You are not authorized to use this functionality.", 'info');
	}
	
	/**
	 * Creates form for changing PO attributes & budget
	 * @param string $name
	 */
	public function createComponentPoAttributesAndBudgetForm($name)
	{
		$form = new AppForm($this, $name);
		
		$form->addHidden('ProjectID');
		
		foreach ($this->session->poAttributesAndBudget as $key => $poAttr)
		{
			$form->addHidden('POAttributesAndBudgetID' . $key);
			$form->addText('POValue' . $key, NULL);
//					->addCondition(Form::FILLED)
//					        ->addRule(Form::PATTERN, 'PO Value must be integer or decimal number in format NN or NN.NN.', '[0-9]+(\.[0-9]+)?');
			$form->addText('PONumber' . $key, NULL);
			$form->addText('Budget' . $key, NULL);
//					->addCondition(Form::FILLED)
//					        ->addRule(Form::PATTERN, 'Budget must be integer or decimal number in format NN or NN.NN.', '[0-9]+(\.[0-9]+)?');
			$form->addText('Year' . $key, NULL)
					->addConditionOn($form['POValue' . $key], Form::FILLED)
				        ->addRule(Form::FILLED, 'Year must be filled.');
					$form['Year' . $key]
		                    ->addConditionOn($form['PONumber' . $key], Form::FILLED)
				                    ->addRule(Form::FILLED, 'Year must be filled.');
					$form['Year' . $key]
							->addConditionOn($form['Budget' . $key], Form::FILLED)
									->addRule(Form::FILLED, 'Year must be filled.');
					$form['Year' . $key]
							->addCondition(Form::FILLED)
									->addRule(Form::RANGE, 'Year must be in format YYYY with range <2000;2155>.', array(2000, 2155));
			$form->addButton('Remove' . $key, 'Remove')
					->setAttribute('onclick', 'removeAttributesAndBudget(' . $key . ')');
		}
		
		$form->addText('POValue', NULL);
//				->addCondition(Form::FILLED)
//					    ->addRule(Form::PATTERN, 'PO Value must be integer or decimal number in format NN or NN.NN.', '[0-9]+(\.[0-9]+)?');
		$form->addText('PONumber', NULL);
		$form->addText('Budget', NULL);
//				->addCondition(Form::FILLED)
//					    ->addRule(Form::PATTERN, 'Budget must be integer or decimal number in format NN or NN.NN.', '[0-9]+(\.[0-9]+)?');
		$form->addText('Year', NULL)
				->addConditionOn($form['POValue'], Form::FILLED)
				        ->addRule(Form::FILLED, 'Year must be filled.');
				$form['Year']
		                ->addConditionOn($form['PONumber'], Form::FILLED)
				                ->addRule(Form::FILLED, 'Year must be filled.');
				$form['Year']
		                ->addConditionOn($form['Budget'], Form::FILLED)
				                ->addRule(Form::FILLED, 'Year must be filled.');
				$form['Year']
				        ->addCondition(Form::FILLED)
				                ->addRule(Form::RANGE, 'Year must be in format YYYY with range <2000;2155>.', array(2000, 2155));
		
		$form->addSubmit('Add', 'Add')
				->setValidationScope(FALSE)
				->onClick[] = callback($this, 'addPoAttributesAndBudgetFormSubmitted');
		
		$form->addHidden('POAttributesAndBudgetToRemove');
		
		$form->addSubmit('Remove', NULL)
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'removePoAttributesAndBudgetFormSubmitted');
		        $form['Remove']
		                ->setAttribute('style', 'display: none;');
				
		$form->addSubmit('Back', 'Back to Project')
				->setValidationScope(FALSE)
				->onClick[] = callback($this, 'backPoAttributesAndBudgetFormSubmitted');
				
	    $form->addSubmit('Save', 'Save Changes')
				->onClick[] = callback($this, 'savePoAttributesAndBudgetFormSubmitted');
		        $form['Save']->setDisabled(!$this->testUserRoles('PMO'));
		
		return $form;
	}
	
	/**
	 * Copies current state of PO attributes & budget into temp storage
	 * @param ArrayHash $entry
	 */
	private function copyPOAttributesAndBudgetEntry($entry)
	{
		$poAttributesAndBudgetKeys = array_keys($this->session->poAttributesAndBudget);
		foreach ($poAttributesAndBudgetKeys as $key)
		{
			$this->session->poAttributesAndBudget[$key]['POAttributesAndBudgetID'] = $entry->{'POAttributesAndBudgetID' . $key};
			$this->session->poAttributesAndBudget[$key]['Year'] = $entry->{'Year' . $key};
			$this->session->poAttributesAndBudget[$key]['POValue'] = $entry->{'POValue' . $key};
			$this->session->poAttributesAndBudget[$key]['PONumber'] = $entry->{'PONumber' . $key};
			$this->session->poAttributesAndBudget[$key]['Budget'] = $entry->{'Budget' . $key};
		}
	}
	
	/**
	 * Processing of POAttributesAndBudget form submitted by add button.
	 * @param SubmitButton $button
	 */
	public function addPoAttributesAndBudgetFormSubmitted(SubmitButton $button)
	{		
		$entry = $button->form->getValues();
		
		$this->copyPOAttributesAndBudgetEntry($entry);
		
		if ($entry->Year || $entry->POValue || $entry->PONumber || $entry->Budget)
		{
			$keys = array_keys($this->session->poAttributesAndBudget);
			$lastIndex = end($keys);
			$newIndex = ($lastIndex !== FALSE) ? $lastIndex + 1 : 0;
			$this->session->poAttributesAndBudget[$newIndex]['POAttributesAndBudgetID'] = NULL;
			$this->session->poAttributesAndBudget[$newIndex]['Year'] = $entry->Year;
			$this->session->poAttributesAndBudget[$newIndex]['POValue'] = $entry->POValue;
			$this->session->poAttributesAndBudget[$newIndex]['PONumber'] = $entry->PONumber;
			$this->session->poAttributesAndBudget[$newIndex]['Budget'] = $entry->Budget;
		}
		
		unset($entry->Year);
		unset($entry->POValue);
		unset($entry->PONumber);
		unset($entry->Budget);
		$this->session->form = $entry;
		$this->redirect('this');
	}
	
	/**
	 * Processing of POAttributesAndBudget form submitted by remove button.
	 * @param SubmitButton $button
	 */
	public function removePoAttributesAndBudgetFormSubmitted(SubmitButton $button)
	{		
		$entry = $button->form->getValues();
		
		$this->copyPOAttributesAndBudgetEntry($entry);
		
		if ($entry->POAttributesAndBudgetToRemove != '')
		{
			unset($this->session->poAttributesAndBudget[$entry->POAttributesAndBudgetToRemove]);
		}
		
		unset($entry->POAttributesAndBudgetToRemove);
		$this->session->form = $entry;
		$this->redirect('this');
	}

	/**
	 * Processing of POAttributesAndBudget form submitted by back button.
	 * @param SubmitButton $button
	 */
	public function backPoAttributesAndBudgetFormSubmitted(SubmitButton $button)
	{
		$entry = $button->form->getValues();
		$this->redirect('Project:detail', array('id' => $entry->ProjectID));
	}

	/**
	 * Processing of POAttributesAndBudget form submitted by save button.
	 * @param SubmitButton $button
	 */
	public function savePoAttributesAndBudgetFormSubmitted(SubmitButton $button)
	{	
		if ($button->form->isValid() && $this->testUserRoles('PMO'))
		{
			$result = TRUE;
			$entry = $button->form->getValues();
			
			$this->copyPOAttributesAndBudgetEntry($entry);
			
			$poAttributesAndBudgetKeys = array_keys($this->session->poAttributesAndBudget);
			$all_poAttributesAndBudget = $this->context->poAttributesAndBudgetRepository->findByProjectID($entry->ProjectID);

			$oldPOAttributesAndBudget = array();
			foreach ($all_poAttributesAndBudget as $poAttr)
				$oldPOAttributesAndBudget[] = $poAttr->POAttributesAndBudgetID;

			$actualPOAttributesAndBudget = array();
			foreach ($poAttributesAndBudgetKeys as $key)
			{
				if (!$entry->{'Year' . $key}) // empty row can not be included into actual set of PO attributes
					continue;
				$actualPOAttributesAndBudget[$key] = $entry->{'POAttributesAndBudgetID' . $key};
			}

			$poAttributesAndBudgetToRemove = array_diff($oldPOAttributesAndBudget, $actualPOAttributesAndBudget);
			foreach ($poAttributesAndBudgetToRemove as $id)
				$result = $result && $this->context->poAttributesAndBudgetRepository->delete($id);
			
		    // save or update rows
			foreach ($actualPOAttributesAndBudget as $key => $id)
			{					
				if (!$entry->{'Year' . $key}) // empty row won't be saved
					continue;

				$poAttr = new POAttributesAndBudget();
				if ($id)
					$poAttr->POAttributesAndBudgetID = $id;
				$poAttr->ProjectID = $entry->ProjectID;
				$poAttr->Year = $entry->{'Year' . $key};
				$poAttr->POValue = ($entry->{'POValue' . $key}) ? $entry->{'POValue' . $key} : NULL;
				$poAttr->PONumber = ($entry->{'PONumber' . $key}) ? $entry->{'PONumber' . $key} : NULL;
				$poAttr->Budget = ($entry->{'Budget' . $key}) ? $entry->{'Budget' . $key} : NULL;

				if ($id)
					$result = $result && $this->context->poAttributesAndBudgetRepository->update($poAttr);
				else
					$result = $result && $this->context->poAttributesAndBudgetRepository->save($poAttr);
			}
			
			// store content of default line
			if ($entry->Year)
			{
				$poAttr = new POAttributesAndBudget();
				$poAttr->ProjectID = $entry->ProjectID;
				$poAttr->Year = $entry->Year;
				$poAttr->POValue = ($entry->POValue) ? $entry->POValue : NULL;
				$poAttr->PONumber = ($entry->PONumber) ? $entry->PONumber : NULL;
				$poAttr->Budget = ($entry->Budget) ? $entry->Budget : NULL;
				$result = $result && $this->context->poAttributesAndBudgetRepository->save($poAttr);
			}
			
			
			if ($result) // check result of operation
				$this->flashMessage('PO Attributes & Budget saved successfully.', 'success');
			else
			{
				$this->flashMessage('PO Attributes & Budget can not be completely saved. Try it again.', 'error');
				return;
			}
			
			unset($this->session->poAttributesAndBudget);
			$this->redirect('this');
		}
	}
    
    /**
     * Creates EAC table
     * @return \EACControl
     */
    public function createComponentEAC()
    {
        $project = $this->context->projectRepository->find((int) $this->getParameter('id'));
        $control = new EACControl($project, $this->year, $this->getParameter('expandMonths'));
        return $control;
    }
    
    /**
     * Creates base WBS filter form
     * @return WBSFilterForm
     */
    public function createComponentWBSFilterForm()
    {
        $control = new WBSFilterForm();
        return $control;
    }
    
    /**
     * Creates WBS
     * @return \WBSControl
     */
    public function createComponentWBS()
    {
		$projectID = (int)$this->getParameter('id');
		$projectYear = (int)$this->getParameter('year');
        $project = $this->context->projectRepository->find($projectID);
        $control = new WBSControl($project, $this->year, 'project' . '+' . $projectID . '+' . $projectYear);
        return $control;
    }
	
    /**
     * Creates ProjectPlanningCharter component
     * @return \ProjectPlanningCharterControl
     */
    public function createComponentProjectPlanningCharter()
    {
		$projectID = (int)$this->getParameter('id');
		$projectYear = (int)$this->getParameter('year');
        $project = $this->context->projectRepository->find($projectID);
        $control = new ProjectPlanningCharterControl($project, $projectYear);
        return $control;
    }
	
	/**
	 * Create project planning form.
	 * @param string $name Name of component.
     * @return AppForm Returns AppForm.
	 */
	public function createComponentProjectPlanningYearSelectForm($name)
	{
		$form = new AppForm($this, $name);
		
		$years = array();
		for ($i = (int)date('Y'); $i <= (int)date('Y') + 10; $i++)
		{
			$years[$i] = $i;
		}
		
		$form->addSelect('year', 'Year:', $years)
				->setDefaultValue(date('Y'))
				->setAttribute('style', 'width: 185px;');
		
		$form->addButton('cancel', 'Cancel')
		        ->setAttribute('onclick', '$.fancybox.close()')
				->setAttribute('class', 'rightFloat');
		
		$form->addButton('create', 'Create / Open')
				->setAttribute('onclick', 'projectPlanning()');
		
		return $form;
	}
	
	/**
	 * Sets planning state to 'planning is ongoing', if it isn't already in that state
	 * (we are plannning this year for the first time).
	 * Otherwise, it just redirects us.
	 * 
	 * @param int $planningForYear The year chosen from the 'Planning for year: ' textbox.
	 */
	public function handleProjectPlanningYearSelectForm($planningForYear)
	{
		$projectID = (int)$this->getParameter('id');
		
		// attempt to find if planning is on
		$state = $this->presenter->context->projectPlanningCharterStateRepository->find($projectID, $planningForYear);
		
		// if it isn't, then create a new one
		if ($state === NULL)
		{
			$planningState = new ProjectPlanningCharterState();
			$planningState->ProjectID = $projectID;
			$planningState->Year = $planningForYear;
			$planningState->State = ProjectPlanningCharterState::PROJECT_PLANNING_CHARTER_STATE_PLANNINGONGOING;

			$this->presenter->context->projectPlanningCharterStateRepository->save($planningState);
		}
		else if ($state->State = ProjectPlanningCharterState::PROJECT_PLANNING_CHARTER_STATE_FINISHED)
		{
			if ($state->ProjectCharterID !== NULL)
				$state->State = ProjectPlanningCharterState::PROJECT_PLANNING_CHARTER_STATE_CHARTEREXISTS;
			else
				$state->State = ProjectPlanningCharterState::PROJECT_PLANNING_CHARTER_STATE_PLANNINGONGOING;
				
			$this->presenter->context->projectPlanningCharterStateRepository->update($state);
		}
		
		$this->redirect('Project:detail', array('id' => $projectID, 'year' => $planningForYear));
	}
}
