<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * SettingsPresenter presents forms for setting upp
 * the application.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class SettingsPresenter extends SecuredPresenter
{
    
	/**
	 * Renders Empty output. Used only for redirecting when user does
     * not have enough rights.
	 */
	public function renderDefault()
	{
        
    }
    
	/**
	 * Renders User Settings form.
	 */
	public function renderUserSettings()
	{
        $setting = $this->context->userSettingRepository->find(Environment::getUser()->getId());
        $formDefaultSetting = array();
        if($setting->EXPDecimalMark == false)
            $formDefaultSetting['EXPDecimalMark'] = 0;
        else
            $formDefaultSetting['EXPDecimalMark'] = 1;
		if($setting->ShowFakeProjects == false)
            $formDefaultSetting['ShowFakeProjects'] = 0;
        else
            $formDefaultSetting['ShowFakeProjects'] = 1;
		if($setting->ShowFakeTasks == false)
            $formDefaultSetting['ShowFakeTasks'] = 0;
        else
            $formDefaultSetting['ShowFakeTasks'] = 1;
        $formSetting = $this['userSettingForm'];
        $formSetting->setDefaults($formDefaultSetting);
        $this->template->formSetting = $formSetting;
    }
    
	/**
	 * Renders User Settings form.
	 */
	public function renderGlobalSettings()
	{
 		$this->checkCredentials();
		// get all years, for that charters are defined
		// important for rendering EditChartersApprovalLock form.
		$all_years = $this->context->projectCharterRepository->getYears();
		$this->template->years = $all_years;
    }
    
	/**
	 * Factory method for creating component into template.
	 * Creates form for eCharge Credentials Setup. 
	 */
	public function createComponentEChargePasswordForm()
	{
		$form = new AppForm();
 //       $form->addGroup('ECharge Credentials');
        $form->addText('EChargeUser', 'EID')
            ->addCondition(Form::FILLED, TRUE)
            ->addRule(Form::PATTERN, '%value is not valid EID!', Person::ID_PREG_EXP);
        $form->addPassword('EChargePass', 'Password')
            ->addCondition(Form::FILLED, TRUE);
		$form->addSubmit('Update', 'Update Credentials');
        $form->onSuccess[] = callback($this, 'eChargePasswordFormSubmitted');
		return $form;
	}	

	/**
	 * Renders ECharge credentials form.
	 */
	public function renderEChargePasswordUpdate()
	{
        $form_defaults = array();
        $form_defaults['EChargeUser'] = $this->context->globalSetting->get('eCharge.credentials.user');
        $form_defaults['EChargePass'] = '';
        $this->template->form = $this['eChargePasswordForm'];
        $this->template->form->setDefaults($form_defaults);		
	}
    
    
	/**
	 * Updates credentials for ECharge access.
	 * @param AppForm	$form
	 */
	public function eChargePasswordFormSubmitted(AppForm $form)
	{
 		if (!$this->testUserRoles('ADMIN'))
		{
			$this->flashMessage("You are not authorized to use this functionality. ADMIN role required.", 'info');
			$this->redirect('default');
		}
		$values = $form->getValues();
		$logger = $this->context->createLogger(NULL, "SETTINGS UPDATE");
        try
        {
            $this->context->globalSetting->set('eCharge.credentials.user', $values['EChargeUser']);
            $encrypted_password = EChargePasswordHandler::encrypt($values['EChargePass']);
            $this->context->globalSetting->set('eCharge.credentials.password', $encrypted_password);
            $logger->log('eCharge access credentials were updated. User ID is '.$values['EChargeUser'], ILogger::NOTICE);
			$this->flashMessage("eCharge access credentials updated.", 'success');
        } 
        catch (Exception $e) 
        {
            $logger->log('Failed to update eCharge credentials: ' . $e->getMessage(), ILogger::ERROR);
			$this->flashMessage("eCharge credentials update failed!", 'error');
        }
		$this->redirect('eChargePasswordUpdate');
	}
	
	/**
	 * Factory method for creating component into template.
	 * Creates a form for testing eCharge access. 
	 */
	public function createComponentTestEChargePasswordForm()
	{
		$form = new AppForm();
		$form->addSubmit('Test', 'Test Credentials');
        $form->onSuccess[] = callback($this, 'eChargePasswordTestSubmitted');
		return $form;
	}	

	/**
	 * Updates credentials for ECharge access.
	 * @param AppForm	$form
	 */
	public function eChargePasswordTestSubmitted(AppForm $form)
	{
        if (!$this->testUserRoles('ADMIN'))
		{
			$this->flashMessage("You are not authorized to use this functionality. ADMIN role required.", 'info');
			$this->redirect('default');
		}
        $user = $this->context->globalSetting->get('eCharge.credentials.user');
        $password = $this->context->globalSetting->get('eCharge.credentials.password');
        $url  = $this->context->globalSetting->get('eCharge.urlBase');
        
        $logger = new MemoryLogger();
        $echarge = new EChargeWrapper($url, $logger);
        try
        {
            $echarge->login($user, $password);
            $this->flashMessage('Test success: Login to eCharge was successful. Current credentials are functional.', 'success');
        }
        catch (EChargeWrapperException $e)
        {
            $this->flashMessage('Test failed: '. $e->getMessage(), 'error');
        }
    }    

    /**
     * Creates form for user options(setting).
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentUserSettingForm($name)
    {
        $form = new AppForm($this, $name);
        
        $options = array(            
            '1' => '. point',
            '0' => ', comma',
        );        

        $form->addRadioList('EXPDecimalMark', 'Decimal mark:', $options)
                ->getSeparatorPrototype()->setName(NULL);
		
        $options = array(            
            '1' => 'Yes',
            '0' => 'No',
        );
		
		$form->addRadioList('ShowFakeProjects', 'Show placeholder projects:', $options)
                ->getSeparatorPrototype()->setName(NULL);
		
		$form->addRadioList('ShowFakeTasks', 'Show placeholder tasks:', $options)
                ->getSeparatorPrototype()->setName(NULL);

        $form->addSubmit('Save', 'Save')
                ->setValidationScope(FALSE);
        $form->onSuccess[] = array($this, 'userSettingFormSubmittedSave');
    }
    
    /**
     * User setting form submitted.
     * 
     * @param AppForm $form 
     */
    public function userSettingFormSubmittedSave($form)
    {
        $eid = Environment::getUser()->getId();
        $data = $form->getValues();
        $setting = new UserSetting();
        $setting = $this->context->userSettingRepository->find($eid);        
        $setting->setEXPDecimalMark($data['EXPDecimalMark']);
		$setting->setShowFakeProjects($data['ShowFakeProjects']);
		$setting->setShowFakeTasks($data['ShowFakeTasks']);
        if($form->isValid())
        {                                     
            if ($this->context->userSettingRepository->save($setting))
            {              
                Debugger::log("Setting of person: ".$eid." updated.", Debugger::INFO);
                $this->flashMessage("Setting updated.", 'success');
            }            
        }
        else
        {  
            $this->flashMessage("Invalid setting values.");
        }

        $this->redirect('Settings:userSettings');
    }
	
	/**
	 * Form for creation of locks of charter final approval process.
	 * @param string $name
	 * @return \AppForm
	 */
	public function createComponentCreateChartersApprovalLock($name)
	{
		$this->checkCredentials();
		
		$form = new AppForm($this, $name);
		
		$all_years = $this->context->projectCharterRepository->getYears();
		$form->addSelect('Year', 'Lock for Year', $all_years)
				->setPrompt('')
				->addRule(Form::FILLED, 'Year must be filled.');
		$form->addSubmit('Create', 'Create lock');
		
		$form->onSuccess[] = array($this, 'createChartersApprovalLockSubmitted');
		return $form;
	}
	
	/**
	 * Processing of CreateChartersApprovalLock form.
	 * @param AppForm $form
	 */
	public function createChartersApprovalLockSubmitted(AppForm $form)
	{
		$this->checkCredentials();
		
		if ($form->isValid())
		{
			$entry = $form->getValues();
			$this->context->globalSetting->set('charters.finalApprovalLocked.' . $entry->Year, TRUE, TRUE, IGlobalSetting::OPTVAL_BOOLEAN);
			$this->flashMessage('Lock for charter approval process for year ' . $entry->Year . ' has been created.', 'success');
			$this->redirect('this');
		}
		
		$this->flashMessage('Error occured during submitting the form.', 'error');
		$this->redirect('this');
	}
	
	/**
	 * Form for maintenance of locks of charter final approval process.
	 * @param string $name
	 * @return \AppForm
	 */
	public function createComponentEditChartersApprovalLock($name)
	{
		$this->checkCredentials();
		
		$form = new AppForm($this, $name);
		
		$items = array(0 => 'Unlocked', 1 => 'Locked');
		$all_years = $this->context->projectCharterRepository->getYears();
		foreach ($all_years as $year)
		{
		    try
			{
				$locked = $this->context->globalSetting->get('charters.finalApprovalLocked.' . $year);
				$default = ($locked) ? 1 : 0; // 1 => Locked, 0 => Unlocked
				$form->addRadioList('Lock' . $year, NULL, $items)
						->getSeparatorPrototype()->setName('');
				        $form['Lock' . $year]
						        ->setDefaultValue($default);
			}
			catch (Exception $e) {}
		}
		$form->addSubmit('Save', 'Save');
		
		$form->onSuccess[] = array($this, 'editChartersApprovalLockSubmitted');
		return $form;
	}

	/**
	 * Processing of EditChartersApprovalLock form.
	 * @param AppForm $form
	 */
	public function editChartersApprovalLockSubmitted(AppForm $form)
	{
		$this->checkCredentials();
		
		if ($form->isValid())
		{
			$entry = $form->getValues();
			foreach ($entry as $key => $value)
			{
				if (preg_match('/Lock[0-9]{4}/', $key))
				{
					$year = preg_replace('/Lock/', '', $key);
					$value = ($value) ? TRUE : FALSE; // TRUE => lock, FALSE => unlock
					$this->context->globalSetting->set('charters.finalApprovalLocked.' . $year, $value, TRUE, IGlobalSetting::OPTVAL_BOOLEAN);
				}
			}
			$this->flashMessage('Locks have been updated.', 'success');
			$this->redirect('this');
		}
		
		$this->flashMessage('Error occured during submitting the form.', 'error');
		$this->redirect('this');
	}
	
	protected function checkCredentials()
	{
		if (!$this->testUserRoles('ADMIN'))
		{
			$this->flashMessage("You are not authorized to use this functionality. ADMIN role required.", 'info');
			$this->redirect('default');
		}
	}
}