<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for preparing view with current (logged) user tasks
 * to be done in current month.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class UserTasksReportPresenter extends SecuredPresenter
{
	/**
     * Renders report containing current month's task report
     * for current user (logged user) tasks he should be working on.
     */
    public function renderDefault() 
    {
		$current_date_ts = time();
		$this->setView("monthlyReport");
		$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp($current_date_ts);	
		if ($fiscal_month === NULL)
			$this->flashMessage('No fiscal month defined for current date:' . date('r', $current_date_ts));
		else
			try
			{
				$this->prepareMonthlyReport($fiscal_month);
			}
			catch (InvalidArgumentException $e)
			{
				$this->flashMessage('Application error.', 'error');
			}
    }

    /**
     * Render report containing current month's task report
     * for current user (logged user) tasks he should be working on.
     * 
     * @param	$month	int Fiscal month index.
     * @param	$year	int Fiscal year index.
     */
    public function renderMonthlyReport($month, $year) 
    {
		// process input
		$fiscal_month = FiscalCalendar::getFiscalMonth($month, $year);
		if ($fiscal_month === NULL)
		{
			$this->flashMessage('Fiscal month  '.FiscalCalendar::fiscalMonthIndexToName($month, true) .' is not defined for year '.$year, 'error');
//			throw new BadRequestException('Fiscal month  '.FiscalCalendar::fiscalMonthIndexToName($month, true) .' is not defined for year '.$year);
//			$this->redirect(404, 'Error:default');
		}
		else
		{
			try
			{
				$this->prepareMonthlyReport($fiscal_month);
			}
			catch (InvalidArgumentException $e)
			{
				$this->flashMessage('Application error.', 'error');
			}
		}
	}
    
    /**
     * Generates report containing current month's task report
     * for current user (logged user) tasks he should be working on
	 * and populates the presenter template. 
     * 
     * @param	$fiscal_month	FiscalMonth Fiscal month.
     */
    private function prepareMonthlyReport(FiscalMonth $fiscal_month) 
    {
		$report_service = new UserMonthlyReportService($this->context);
		$report_service->prepareUserMonthlyTaskReport(Environment::getUser()->getId(), $fiscal_month);
		$this->template->report = $report_service->getReport();
		$this->template->totals = $report_service->getTotals();
		$this->template->month = $fiscal_month;
		$this->template->currentMonth = $fiscal_month->includesTimestamp(time());
		$this->template->closedMonth = (FiscalCalendar::dateToTimestamp($fiscal_month->To) < time());
		$this->template->navPrevMonth = ($fiscal_month->Index == FiscalMonth::FIRST_INDEX) ? FiscalMonth::LAST_INDEX : ($fiscal_month->Index - 1);
		$this->template->navPrevYear = ($fiscal_month->Index == FiscalMonth::FIRST_INDEX) ? ($fiscal_month->Year - 1) : $fiscal_month->Year;
		$this->template->navNextMonth = ($fiscal_month->Index == FiscalMonth::LAST_INDEX) ? FiscalMonth::FIRST_INDEX : ($fiscal_month->Index + 1);
		$this->template->navNextYear = ($fiscal_month->Index == FiscalMonth::LAST_INDEX) ? ($fiscal_month->Year + 1) : $fiscal_month->Year;
	}
    
	/**
	 * Overrides default createTemplate and registers new helper 
	 * 'fiscalDateToTimestamp' for fiscal date parsing to UNIX timestamp.
	 * 
	 * @param  string|NULL
	 * @return ITemplate
	 */
	protected function createTemplate($class = NULL)
    {
		$template = parent::createTemplate($class);
		$template->registerHelper('fiscalDateToTimestamp', 'FiscalCalendar::dateToTimestamp');
		return $template;
    }
	
}