<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for reading and writing exchange rates into database
 * using Dibi libarary.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class ExchangeRateDibiMapper extends DibiMapper implements IExchangeRateMapper
{
    
    /**
     * Saves exchange rate.
     * 
     * @param CostCenter $currencyExchangeRate exchange rate.
     * @return boolean Returns true if exchange rate is successfully saved, otherwise false.
     */
    public function save(CurrencyExchangeRate $currencyExchangeRate)
    {
        try
        {
            $array = array();
			$array['SourceCurrency'] = $currencyExchangeRate->SourceCurrencyCode;
			$array['TargetCurrency'] = $currencyExchangeRate->TargetCurrencyCode;
            $array['Year'] = $currencyExchangeRate->Year;
			$array['Month'] = $currencyExchangeRate->Month;
			$array['Value'] = $currencyExchangeRate->Value;

            
            if (!CurrencyExchangeRate::validCurrencyCode($array['SourceCurrency']))
                throw new InvalidArgumentException('Invalid source currency code: ' . $array['SourceCurrency']);
            if (!CurrencyExchangeRate::validCurrencyCode($array['TargetCurrency']))
                throw new InvalidArgumentException('Invalid target currency code: ' . $array['TargetCurrency']);
            
            $this->conn->insert('ExchangeRate', $array)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Exchange rate insert of year = " . $currencyExchangeRate->getYear() . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        } 
    }
    
    /**
     * Updates exchange rate.
     * 
     * @param CostCenter $currencyExchangeRate exchange rate.
     * @return boolean Returns true if exchange rate is successfully updated, otherwise false.
     */
    public function update(CurrencyExchangeRate $currencyExchangeRate)
    {
        try
        {
            $array = array();
			$array['SourceCurrency'] = $currencyExchangeRate->SourceCurrencyCode;
			$array['TargetCurrency'] = $currencyExchangeRate->TargetCurrencyCode;
			$array['Year'] = $currencyExchangeRate->Year;
			$array['Month'] = $currencyExchangeRate->Month;
            $array['Value'] = $currencyExchangeRate->Value;
            
            if (!CurrencyExchangeRate::validCurrencyCode($array['SourceCurrency']))
                throw new InvalidArgumentException('Invalid source currency code: ' . $array['SourceCurrency']);
            if (!CurrencyExchangeRate::validCurrencyCode($array['TargetCurrency']))
                throw new InvalidArgumentException('Invalid target currency code: ' . $array['TargetCurrency']);
            
            $exchange_rate_db_result = $this->conn->update('ExchangeRate', $array)->where('[SourceCurrency] = %s AND [TargetCurrency] = %s AND [Year] = %i AND [Month] = %i', $array['SourceCurrency'], $array['TargetCurrency'], $array['Year'], $array['Month'])->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Exchange rate update of year = " . $currencyExchangeRate->getYear() . " failed. " . $e->getMessage());
            return false;
        }
    }
    
	/**
	 * Find exchange rate between currencies in defined month in a year.
	 * 
	 * @param string $source_curr	Source currency (3-letter ISO code)
	 * @param string $target_curr	Target currency (3-letter ISO code)
	 * @param int $month Must be number from 1 to 12.
	 * @param int $year
	 * @return CurrencyExchangeRate or NULL in unknown.
	 */
	public function find($source_curr, $target_curr, $month, $year)
	{
		$month_index = 0 + $month;
		if (($month_index <  1) || ($month_index > 12))
			throw new InvalidArgumentException('Month index must be number from 1 to 12. Invalid value: ' . $month);
		if (!CurrencyExchangeRate::validCurrencyCode($source_curr))
			throw new InvalidArgumentException('Invalid source currency code: ' . $source_curr);
		if (!CurrencyExchangeRate::validCurrencyCode($target_curr))
			throw new InvalidArgumentException('Invalid target currency code: ' . $target_curr);
		$row = $this->conn->query('SELECT * FROM [ExchangeRate] 
			WHERE [SourceCurrency] = %s AND [TargetCurrency] = %s AND [Month] = %u AND [Year] = %u', $source_curr, $target_curr, $month_index, $year)->fetch();
		if ($row->count() > 0)
			return new CurrencyExchangeRate($row->SourceCurrency, $row->TargetCurrency, $row->Value, $row->Year);
		else
			return NULL;
	}

	/**
	 * Finds exchange rates between currencies in defined year.
	 * Objects with undefined rates may be returned if rate is unknown
	 * of undefined for a month. 
	 * 
	 * @param string $source_curr	Source currency (3-letter ISO code)
	 * @param string $target_curr	Target currency (3-letter ISO code)
	 * @param int $year
	 * @return MonthArray of CurrencyExchangeRate
	 */
	public function findYearRates($source_curr, $target_curr, $year)
	{
		if (!CurrencyExchangeRate::validCurrencyCode($source_curr))
			throw new InvalidArgumentException('Invalid source currency code: ' . $source_curr);
		if (!CurrencyExchangeRate::validCurrencyCode($target_curr))
			throw new InvalidArgumentException('Invalid target currency code: ' . $target_curr);
		$rates = new MonthArray(new CurrencyExchangeRate($source_curr, $target_curr, CurrencyExchangeRate::RATE_UNDEFINED, $year), TRUE);
		for ($i = MonthArray::LOWER_INDEX; $i <= MonthArray::UPPER_INDEX; $i++)
			$rates[$i] = new CurrencyExchangeRate($source_curr, $target_curr, CurrencyExchangeRate::RATE_UNDEFINED, $year);
		$db_rates = $this->conn->query('SELECT * FROM [ExchangeRate] 
			WHERE [SourceCurrency] = %s AND [TargetCurrency] = %s AND [Year] = %u', $source_curr, $target_curr, $year)->fetchAll();
		foreach ($db_rates as $row)
			$rates[$row->Month]->Value = $row->Value;
		return $rates;
	}
    
    /**
     * Gets all years of ExchangeRate.
     * 
	 * @param string $source_curr	Source currency (3-letter ISO code)
	 * @param string $target_curr	Target currency (3-letter ISO code)
	 * 
     * @return array Array of available years
     */
    public function findExchangeRateYears($source_curr, $target_curr)
    {
		$db_result = $this->conn->query('SELECT DISTINCT [Year] FROM [ExchangeRate] WHERE [SourceCurrency] = %s AND [TargetCurrency] = %s', $source_curr, $target_curr)->fetchAll();
        $years = array();        
		foreach ($db_result as $year)		
			array_push($years, $year->Year);      
        return $years;
    }
	
}


