<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for importing data from eCharge.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class EChargeImportService extends Object
{
    
    /** @const php.ini maximum execution time */
    const MAXIMUM_EXECUTION_TIME = 10800;
   
    /** @var DIContainer DI context with application repositories access */
    private $context;

	/** @var ILogger Logging facility */
    private $logger;
    
    /** @var resource EChargeWrapper */
    private $echarge;

    /** @var String eCharge URL **/
    private $echarge_url;
    
    /** @var String eCharge EID **/
    private $echarge_username;
    
    /** @var String eCharge password **/
    private $echarge_password;

    /** @var Date eCharge start date **/
    private $echarge_startDate;
	
    /** @var Date eCharge start date **/
    private $origin_echarge_startDate;

    /** @var Date eCharge end date **/
    private $echarge_endDate;

	/** @var boolean Whether to report progress of running **/
    private $report_progress = false;

	/**
	 * Creates service for importing data from eCharge.
	 *  
	 * @param $context Application context.
	 * @param ILogger Object for logging the import progress.
	 */
    public function  __construct(DIContainer $context, ILogger $logger)
    {
        
        $this->context  = $context;
		$this->logger   = $logger;

        $this->echarge_url          = GlobalSettingReaderHelper::get('eCharge.urlBase');
        $this->echarge_username     = GlobalSettingReaderHelper::get('eCharge.credentials.user');
        $this->echarge_password     = GlobalSettingReaderHelper::get('eCharge.credentials.password');
        $this->echarge_startDate    = new DateTime(GlobalSettingReaderHelper::get('eCharge.actualsUpdate.startDate'));
		$this->origin_echarge_startDate    = new DateTime(GlobalSettingReaderHelper::get('eCharge.actualsUpdate.startDate'));
        $this->echarge_endDate      = new DateTime(); // today
        
        $this->echarge = new EChargeWrapper($this->echarge_url, $this->logger);
        
    }
    
    /**
     * Runs import actuals from eCharge
     * 
     * @return boolean
     * @throws EChargeImportServiceException 
     */
    public function runImport()
    {
        
        // adjust maximum execution time (php.ini)
        $maximum_execution_time = EChargeImportService::MAXIMUM_EXECUTION_TIME;
		ini_set('max_execution_time', $maximum_execution_time);
        if (ini_get('max_execution_time') != $maximum_execution_time)
            throw new EChargeImportServiceException("Setting maximum execution time to '$maximum_execution_time' failed, check server (php.ini) settings");
        
		$lock = $this->context->createLock('ECHARGE_IMPORT');
		if (!$lock->lock())
            throw new EChargeImportServiceException("ECharge import is already running.");
		
        // open logfile
        $this->logger->log('Echarge import start: ' . date('m-d-Y H:i:s'), ILogger::INFO);
        
        try
        {
            // log into eCharge
            $this->echarge->login($this->echarge_username, $this->echarge_password);

            //Deletes actuals from temp table
			if($this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_ECHARGE))
			{
				$this->logger->log('Temp actuals was deleted successfully.', ILogger::INFO);
			}
			else
			{
				$lock->unlock();
				throw new EChargeImportServiceException('Temp actuals was not deleted successfully.');
			}

            // get all distinct task sap ids
            $distinctTaskSAPIDs = $this->context->taskRepository->findDistinctSAPIDs();
			
			$this->reportProgress("Progress:");
            while ($this->echarge_startDate <= $this->echarge_endDate)
            {

                $startDate = clone $this->echarge_startDate;
                $this->echarge_startDate->modify('+1 month -1 day');
                $endDate = ($this->echarge_startDate > $this->echarge_endDate) ? $this->echarge_endDate : $this->echarge_startDate;

                $succ = 0; // number of successfully imported records
                $fail = 0; // number of records that couldn't be stored in DB

                foreach ($this->echarge->readActuals($startDate, $endDate) as $record)
                {
					$this->reportProgress("*");
					try
					{
						//$date = new DateTime($record['CHARGE DAY']); // incompatible with SAP decision date but compataible to eCharge dates
						$date = new DateTime($record['MODIFIED_AT']); // compatible with SAP decision date
					}
					catch (Exception $ex)
					{
						$this->logger->log("EChargeImportService: date validation failed. " . var_export($record, true), ILogger::ERROR);
						$fail++;
						continue;
					}

                    $actual = new ActualsTemp();
                    $actual->Eid = $record['USERID'];
                    $actual->Charge_number = str_replace('-', ' ', $record['ORDER/CHRG#']);
					$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $date->format('U'));
					if ($fiscal_month === NULL)
					{
						$lock->unlock();
			            throw new EChargeImportServiceException("ECharge data are invalid, cannot determine fiscal month for: ". var_export($record, true));
					}
                    $actual->Fiscal_month = $fiscal_month->getIndex();
                    $actual->Year = $fiscal_month->getYear();
                    $actual->Hours = (float) $record['HOURS'];
                    $actual->Project_sap_id = $record['Project'];
					$actual->Source = ActualsTemp::SOURCE_ECHARGE;

                    if (in_array($actual->Charge_number, $distinctTaskSAPIDs))
                    {
                        if ($this->context->actualsTempRepository->add($actual))
                        {
                            //$this->logger->log('Written data metadata: CHARGE DAY = ' . $record['CHARGE DAY'] . ', MODIFIED_AT = ' . $record['MODIFIED_AT'] . ', Selected Fiscal Month (January index is 0) = ' . $actual->Fiscal_month, ILogger::DEBUG);
                            //$this->logger->log(var_export($actual, true), ILogger::DEBUG);
                            $succ++;
                        }
                        else
                        {
                            $fail++;
                            $this->logger->log("EChargeImportService: actuals db insert failed. " . var_export($actual, true), ILogger::ERROR);
                        }
                    }

                }
                
                if ($fail == 0)
                {
                    $msg = "EChargeImportService: All $succ actuals from " . $startDate->format('m-d-Y') . " till " . $endDate->format('m-d-Y') . " imported successfully";
                    $this->logger->log($msg, ILogger::NOTICE);
                    Debugger::log($msg, Debugger::INFO);
                }
                else
                {
                    $msg = "EChargeImportService: $succ actuals imported, $fail actuals thrown away (" . $startDate->format('m-d-Y') . " - " . $endDate->format('m-d-Y') . ")";
                    $this->logger->log($msg, ILogger::WARNING);
                    Debugger::log($msg, Debugger::ERROR);
                }

                $this->echarge_startDate->modify('+1 day');
            }
			$actualsTemp = $this->context->actualsTempRepository->countActualsByRangeAndType($this->origin_echarge_startDate, $this->echarge_endDate, ActualsTemp::SOURCE_ECHARGE);
			$actualsCurrent = $this->context->actualsRepository->countActualsByRangeAndType($this->origin_echarge_startDate, $this->echarge_endDate, ActualsTemp::SOURCE_ECHARGE);
			$this->logger->log('Current actuals records: ' . $actualsCurrent, ILogger::INFO);
			$this->logger->log('Temp actuals records: ' . $actualsTemp, ILogger::INFO);
			// Compares temp actuals to current actuals
			if($actualsCurrent <= $actualsTemp)
			{
				//Deletes actuals from current table by range
				if($this->context->actualsRepository->deleteByRange($this->origin_echarge_startDate, $this->echarge_endDate))
				{
					$this->logger->log('Current actuals was deleted successfully.', ILogger::INFO);
				}
				else
				{
					$lock->unlock();
					$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_ECHARGE);
					throw new EChargeImportServiceException('Current actuals was not deleted successfully.');
				}
				//Moves actuals
				if($this->context->actualsTempRepository->moveAll(ActualsTemp::SOURCE_ECHARGE))
				{
					$this->logger->log('Temp actuals were moved successfully. E-charge import ends.', ILogger::NOTICE);
				}
				else
				{
					$lock->unlock();
					$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_ECHARGE);
					throw new EChargeImportServiceException('Temp actuals were not moved successfully.');
				}
			}
			else
			{
				$lock->unlock();
				$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_ECHARGE);
				throw new EChargeImportServiceException('Count of current actuals is greater than count of temp actuals.');
			}
			$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_ECHARGE);
			$this->reportProgress("\n");
			$lock->unlock();
        }
        catch (EChargeWrapperException $e)
        {
            $this->logger->log($e->getMessage(), ILogger::ERROR);
            Debugger::log($e->getMessage(), Debugger::ERROR);
			$lock->unlock();
			$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_ECHARGE);
            return FALSE;
        }
        
        return TRUE;
        
    }
	
    /**
     * Runs import actuals from eCharge for given task
     * 
	 * @param $task Task object.
     * @return boolean
     * @throws EChargeImportServiceException 
     */
    public function runImportforTask($task)
    {
        
        // adjust maximum execution time (php.ini)
        $maximum_execution_time = EChargeImportService::MAXIMUM_EXECUTION_TIME;
		ini_set('max_execution_time', $maximum_execution_time);
        if (ini_get('max_execution_time') != $maximum_execution_time)
            throw new EChargeImportServiceException("Setting maximum execution time to '$maximum_execution_time' failed, check server (php.ini) settings");
        
		$lock = $this->context->createLock('ECHARGE_IMPORT');
		if (!$lock->lock())
            throw new EChargeImportServiceException("ECharge import is already running.");
		
        // open logfile
        $this->logger->log('Echarge import for task start: ' . date('m-d-Y H:i:s'), ILogger::INFO);
        
        try
        {
            // log into eCharge
            $this->echarge->login($this->echarge_username, $this->echarge_password);

            // delete from actuals table by range
            $this->context->actualsRepository->deleteByRangeForTask($this->echarge_startDate, $this->echarge_endDate, $task);

            // get all distinct task sap ids
            $distinctTaskSAPIDs = $this->context->taskRepository->findDistinctSAPIDs();

			$this->reportProgress("Progress:");
            while ($this->echarge_startDate <= $this->echarge_endDate)
            {

                $startDate = clone $this->echarge_startDate;
                $this->echarge_startDate->modify('+1 month -1 day');
                $endDate = ($this->echarge_startDate > $this->echarge_endDate) ? $this->echarge_endDate : $this->echarge_startDate;

                $succ = 0; // number of successfully imported records
                $fail = 0; // number of records that couldn't be stored in DB

                foreach ($this->echarge->readActualsForTask($startDate, $endDate, $task) as $record)
                {
					
                    //$date = new DateTime($record['CHARGE DAY']); // incompatible with SAP decision date but compataible to eCharge dates
                    $date = new DateTime($record['MODIFIED_AT']); // compatible with SAP decision date

                    $actual = new Actuals();
                    $actual->Eid            = $record['USERID'];
                    $actual->Charge_number  = str_replace('-', ' ', $record['ORDER/CHRG#']);
					$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $date->format('U'));
					if ($fiscal_month === NULL)
					{
						$lock->unlock();
			            throw new EChargeImportServiceException("ECharge data are invalid, cannot determine fiscal month for: ". var_export($record, true));
					}
                    $actual->Fiscal_month   = $fiscal_month->getIndex();
                    $actual->Year           = $fiscal_month->getYear();
                    $actual->Hours          = (float) $record['HOURS'];
                    $actual->Project_sap_id = $record['Project'];
					$actual->Source = ActualsTemp::SOURCE_ECHARGE;

                    if (in_array($actual->Charge_number, $distinctTaskSAPIDs))
                    {
                        if ($this->context->actualsRepository->add($actual))
                        {
                            //$this->logger->log('Written data metadata: CHARGE DAY = ' . $record['CHARGE DAY'] . ', MODIFIED_AT = ' . $record['MODIFIED_AT'] . ', Selected Fiscal Month (January index is 0) = ' . $actual->Fiscal_month, ILogger::DEBUG);
                            //$this->logger->log(var_export($actual, true), ILogger::DEBUG);
                            $succ++;
                        }
                        else
                        {
                            $fail++;
                            $this->logger->log(var_export($actual, true), ILogger::ERROR);
                        }
                    }

                }
                
                if ($fail == 0)
                {
                    $msg = "EChargeImportService: All $succ actuals from " . $startDate->format('m-d-Y') . " till " . $endDate->format('m-d-Y') . " imported successfully";
                    $this->logger->log($msg, ILogger::NOTICE);
                    Debugger::log($msg, Debugger::INFO);
                }
                else
                {
                    $msg = "EChargeImportService: $succ actuals imported, $fail actuals thrown away (" . $startDate->format('m-d-Y') . " - " . $endDate->format('m-d-Y') . ")";
                    $this->logger->log($msg, ILogger::WARNING);
                    Debugger::log($msg, Debugger::ERROR);
                }

                $this->echarge_startDate->modify('+1 day');
				$this->reportProgress("*");
            }
			$this->reportProgress("\n");
			$lock->unlock();
        }
        catch (EChargeWrapperException $e)
        {
            $this->logger->log($e->getMessage(), ILogger::ERROR);
            Debugger::log($e->getMessage(), Debugger::ERROR);
			$lock->unlock();
            return FALSE;
        }
        
        return TRUE;
        
    }

    /**
     * If set to TRUE, import echos character "*" for every
	 * downloaded block of data so to keep the connection alive.
     */
    public function setReportProgress($report)
    {
        $this->report_progress = ($report) ? TRUE : FALSE;
    }

    /**
	 * Reports progress to output and flushes buffers.
	 * Used as a workaround to keep clients alive because whole process can take minutes
     */
    private function reportProgress($report)
    {
		if ($this->report_progress)
		{
			echo $report;					
			ob_flush();					
			flush();
		}
    }
	
    /**
     * Gets maximum execution time.
     * 
     * @return int
     */
    public static function getMaximumExecutionTime()
    {
        return (int) constant('EChargeImportService::MAXIMUM_EXECUTION_TIME');
    }
    
}	

class EChargeImportServiceException extends Exception {}
