<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Organization Unit definition.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class OrganizationUnit extends Object
{
    /* possible organization unit types */
    const ORG_UNIT_TYPE_SBE = 'SBE'; // SBE - ???
    const ORG_UNIT_TYPE_DEP = 'DEP'; // Department
    const ORG_UNIT_TYPE_CLUS = 'CLUS'; // Cluster
    const ORG_UNIT_TYPE_TG = 'TG'; // Technical group
    
    /* full names of types */
    const FULL_NAME_TYPE_SBE = 'SBE';
    const FULL_NAME_TYPE_DEP = 'Department';
    const FULL_NAME_TYPE_CLUS = 'Cluster';
    const FULL_NAME_TYPE_TG = 'Technical group';

    /** @var int ID */
	private $id;
    /** @var int ID of parent node */
    private $parentID;
    /** @var string Type */
    private $type;
    /** @var string EID of leader */
    private $leader;
    /** @var int Sign of leadership status */
    private $actingLeader = 0;
    /** @var string Name of organization unit */
    private $name;
    /** @var string Short of organization unit */
    private $short;
    /** @var String Node path in organization unit (e.g. 001002003001 - each level has 3 Chars) */
    private $path;
    /** @var boolean Mark if org unit is virtual or not.*/
    private $virtual;
	/** @var boolean Person assignment lock */
	private $locked = FALSE;
	/** @var boolean Mark if org unit is active or inactive.*/
    private $status = TRUE;
    
   /**
     * Sets ID of organization unit.
     * @param int $id Must be positive integer.
     */
	public function setID($id)
	{
		if ($id > 0)
			$this->id = $id;
		else
			throw new InvalidArgumentException('Organization unit ID must be integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of organization unit.
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}

    /**
     * Sets ID of parent node.
     * @param int $id Must be positive integer.
     */
	public function setParent_ID($id)
	{
		if (($id > 0) || ($id === NULL))
			$this->parentID = $id;
		else
			throw new InvalidArgumentException('Organization unit parent ID must be integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of parent node.
     * @return int
     */
	public function getParent_ID()
	{
		return $this->parentID;
	}
    
	/**
     * Sets type of organization unit.
     * 
     * @param string $type
     */
	public function setType($type)
	{
		if (in_array($type, array_keys(OrganizationUnit::getTypes())))
			$this->type = $type;
		else
			throw new InvalidArgumentException('Unknown organization unit type.');
	}

    /**
     * Gets type of organization unit.
     * 
     * @return string
     */
	public function getType()
	{
		return $this->type;
	}
    
    /**
     * Sets leader EID.
     * @param string $eid
     */
	public function setLeader($eid)
	{
		$this->leader = Person::processEID($eid);
	}

    /**
     * Gets leader EID.
     * @return string
     */
	public function getLeader()
	{
		return $this->leader;
	}

    /**
     * Sets sign of leadership status
     * @param boolean $value True in case leader is acting, false otherwise.
     */
    public function setActingLeader($value)
    {
        $this->actingLeader = ($value) ? 1 : 0;
    }

    /**
     * Gets sign of leadership status
     * @return int 1 in case leader is acting, 0 otherwise.
     */
    public function getActingLeader()
    {
        return $this->actingLeader;
    }

    /**
     * Gets information about leadership [Leadership status]: e.g. (Act.)
     * @return string
     */
    public function getLeadershipInfo()
    {
        return ($this->getActingLeader()) ? ' (Act.)' : '';
    }
    
    /**
     * Sets name.
     * @param string $name
     */
	public function setName($name)
	{
		$this->name = $name;
	}

    /**
     * Gets name.
     * @return string
     */
	public function getName()
	{
		return $this->name;
	}	
    
    /**
     * Sets short.
     * @param string $name
     */
	public function setShort($name)
	{
		$this->short = $name;
	}

    /**
     * Gets short.
     * @return string
     */
	public function getShort()
	{
		return $this->short;
	}	
    
    /**
     * Sets short.
     * @param string $name
     */
	public function setPath($name)
	{
		$this->path = $name;
	}

    /**
     * Gets short.
     * @return string
     */
	public function getPath()
	{
		return $this->path;
	}
    
    /**
     * Sets virtual flag.
     * @param boolean $virtual
     */
	public function setVirtual($virtual)
	{
		$this->virtual = $virtual;
	}

    /**
     * Gets virtual flag.
     * @return boolean
     */
	public function getVirtual()
	{
		return ($this->virtual) ? TRUE : FALSE;
	}
    
    /**
     * Returns array of available types.
     * 
     * @return array
     */
     public static function getTypes()
    {
            return array(
                self::ORG_UNIT_TYPE_SBE => self::ORG_UNIT_TYPE_SBE, 
                self::ORG_UNIT_TYPE_DEP => self::ORG_UNIT_TYPE_DEP, 
                self::ORG_UNIT_TYPE_CLUS => self::ORG_UNIT_TYPE_CLUS,
                self::ORG_UNIT_TYPE_TG => self::ORG_UNIT_TYPE_TG
            );
    }
	
    /**
     * Returns array of available types with full names.
     * @return array
     */
     public static function getFullTypeNames()
    {
            return array(
                self::ORG_UNIT_TYPE_SBE => self::FULL_NAME_TYPE_SBE, 
                self::ORG_UNIT_TYPE_DEP => self::FULL_NAME_TYPE_DEP, 
                self::ORG_UNIT_TYPE_CLUS => self::FULL_NAME_TYPE_CLUS,
                self::ORG_UNIT_TYPE_TG => self::FULL_NAME_TYPE_TG
            );
    }
    
    /**
     * Returns full name of type of organization unit according to shortcut.
     * @param string Type of organization unit in range of self::ORG_UNIT_TYPE_*
     * @return string Full name of type or empty string if type doesn't exist.
     */
    public static function getFullTypeName($type)
    {
        switch($type)
        {
            case self::ORG_UNIT_TYPE_SBE:
                return self::FULL_NAME_TYPE_SBE;
            case self::ORG_UNIT_TYPE_DEP:
                return self::FULL_NAME_TYPE_DEP;
            case self::ORG_UNIT_TYPE_CLUS:
                return self::FULL_NAME_TYPE_CLUS;
            case self::ORG_UNIT_TYPE_TG:
                return self::FULL_NAME_TYPE_TG;
            default:
                return '';
        }
    }
	
	/**
	 * Sets org unit assignment lock.
     * @param boolean $locked
     */
    public function setLocked($locked)
    {
        $this->locked = ($locked) ? TRUE : FALSE;
    }

    /**
	 * Gets org unit assignment lock.
     * @return boolean
     */
    public function getLocked()
    {
        return $this->locked;
    }
	
	/**
	 * Sets org unit status.
     * @param boolean $status
     */
    public function setStatus($status)
    {
        $this->status = ($status) ? TRUE : FALSE;
    }

    /**
	 * Gets org unitstatus.
     * @return boolean
     */
    public function getStatus()
    {
        return $this->status;
    }
}