<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing organization unit from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class OrganizationUnitDibiMapper extends DibiMapper implements IOrganizationUnitMapper
{
    /**
	 * Creates OrganizationUnit instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	OrganizationUnit
	 */
    private function load(DibiRow $row)
    {
		$organizationUnit = new OrganizationUnit();
		$organizationUnit->ID = $row->OrganizationUnit_ID;
		$organizationUnit->Parent_ID = $row->Parent_ID;
		$organizationUnit->Type = $row->Type;
		$organizationUnit->Leader = $row->Leader;
		$organizationUnit->ActingLeader = $row->ActingLeader;
		$organizationUnit->Name = $row->Name;
		$organizationUnit->Short = $row->Short;
		$organizationUnit->Path = $row->Path;
		$organizationUnit->Virtual = $row->Virtual;
		$organizationUnit->Locked = $row->Locked;
		$organizationUnit->Status = $row->Status;
		return $organizationUnit;
    }	

	/**
	 * Saves organization unit.
	 * 
	 * @param OrganizationUnit $organizationUnit
	 */
	public function save(OrganizationUnit $organizationUnit)
	{
        try
        {
            $array = array();
            $array['Parent_ID'] = $organizationUnit->getParent_ID();
            $array['Type'] = $organizationUnit->getType();
            $array['Leader'] = $organizationUnit->getLeader();
            $array['ActingLeader'] = $organizationUnit->getActingLeader();
            $array['Name'] = $organizationUnit->getName();
            $array['Short'] = $organizationUnit->getShort();
            $array['Path'] = $organizationUnit->getPath();
			$array['Virtual'] = $organizationUnit->getVirtual();
			$array['Locked'] = 0;
			$array['Status'] = 1;
            $organization_unit_db_result = $this->conn->insert('OrganizationUnit', $array)->execute();
			$organizationUnit->ID = $this->conn->getInsertId();
			// update paths
			$this->conn->nativeQuery('CALL GenerateOrgUnitPaths(3)');
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving organization unit with parent id = " . $organizationUnit->getParent_ID() . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Updates organization unit.
	 * 
	 * @param OrganizationUnit $organizationUnit
	 */
	public function update(OrganizationUnit $organizationUnit)
	{
        try
        {
            $array = array();
            $array['Parent_ID'] = $organizationUnit->getParent_ID();
            $array['Type'] = $organizationUnit->getType();
            $array['Leader'] = $organizationUnit->getLeader();
            $array['ActingLeader'] = $organizationUnit->getActingLeader();
            $array['Name'] = $organizationUnit->getName();
            $array['Short'] = $organizationUnit->getShort();
            $array['Path'] = $organizationUnit->getPath();
			$array['Virtual'] = $organizationUnit->getVirtual();
			$array['Locked'] = $organizationUnit->getLocked();
			$array['Status'] = $organizationUnit->getStatus();
            $organization_unit_db_result = $this->conn->update('OrganizationUnit', $array)->where('[OrganizationUnit_ID] = %i', $organizationUnit->ID)->execute();
			    // update paths
			    $this->conn->nativeQuery('CALL GenerateOrgUnitPaths(3)');
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Organization unit update with id = " . $organizationUnit->ID . " failed. ".$e->getMessage());
            return false;
        }
	}
	
	/**
	 * Updates parent id of set of organization units.
	 * Used for move of children to another parent after delete of original parent.
	 * 
	 * @param array $children Of organization unit ids.
	 * @param int $parent_id New parent organization unit id.
	 */
	public function updateParentInBatch(array $children, $parent_id)
	{
		try
        {
            $array = array();
            $array['Parent_ID'] = $parent_id;
            $db_result = $this->conn->update('OrganizationUnit', $array)->where('[OrganizationUnit_ID] IN (%i)', $children)->execute();
			    // update paths
			    $this->conn->nativeQuery('CALL GenerateOrgUnitPaths(3)');
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Organization unit batch update of parent id failed. ".$e->getMessage());
            return false;
        }
	}
	
	/**
	 * Deletes existing organization unit.
	 * 
	 * @param int $OrganizationUnit_id
	 * @return boolean
	 */
	public function delete($organizationUnit_id)
	{
		try
		{
		    $result = $this->conn->delete('OrganizationUnit')->where('[OrganizationUnit_ID] = %i', $organizationUnit_id)->execute();
			if ($result)
				Debugger::log("Organization unit with ID = " . $organizationUnit_id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of organization unit with ID = " . $organizationUnit_id . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of organization unit with ID = " . $organizationUnit_id . " failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
    
	/**
	 * Finds organization unit by id.
	 * 
	 * @param int $id organization unit ID.
	 * @return	OrganizationUnit or NULL if not found.
	 */
	public function find($id)
	{
		$organizationUnit_db_result = $this->conn->select('*')->from('OrganizationUnit')->where('[OrganizationUnit_ID] = %i', $id)->fetch();
		if ($organizationUnit_db_result !== FALSE)
			return $this->load($organizationUnit_db_result);
		else
			return NULL;
	}
    
    /**
	 * Finds all organization units.
	 * 
	 * @param int $id organization unit ID.
	 * @return	array of OrganizationUnit where id is key or NULL if not found.
	 */
	public function findAll()
	{
        $orgUnit_array = array();
		$orgUnit_db_result = $this->conn->select('*')->from('OrganizationUnit')->fetchAll();        
        if ($orgUnit_db_result !== FALSE)
        {
            foreach ($orgUnit_db_result as $orgUnit)
                $orgUnit_array[$orgUnit->OrganizationUnit_ID] = $this->load($orgUnit);
            return $orgUnit_array;
        }
        else
            return null;
	}
    
    /**
	 * Finds all children unit of organization unit given by id.
	 * 
	 * @param int $id organization unit ID.
	 * @return	array of OrganizationUnit where id is key or NULL if not found.
	 */
	public function findAllChildren($id)
	{
        $orgUnit_array = array();
		$orgUnit_db_result = $this->conn->select('*')->from('OrganizationUnit')->where('[Parent_ID] = %i', $id)->fetchAll();        
        if ($orgUnit_db_result !== FALSE)
        {
            foreach ($orgUnit_db_result as $orgUnit)
                $orgUnit_array[$orgUnit->OrganizationUnit_ID] = $this->load($orgUnit);
            return $orgUnit_array;
        }
        else
            return null;
	}
	
    /**
	 * Finds all children unit of organization unit by given path.
	 * 
	 * @param int $path organization unit path.
	 * @return	array of OrganizationUnit where id is key or NULL if not found.
	 */
	public function findAllChildrenByPath($path)
	{
        $orgUnit_array = array();
		$orgUnit_db_result = $this->conn->select('*')->from('OrganizationUnit')->where('[Path] LIKE %like~', $path)->fetchAll();        
        if ($orgUnit_db_result !== FALSE)
        {
            foreach ($orgUnit_db_result as $orgUnit)
                $orgUnit_array[$orgUnit->OrganizationUnit_ID] = $this->load($orgUnit);
            return $orgUnit_array;
        }
        else
            return null;
	}
	
	/**
	 * Finds all children unit of organization unit given by id and type of children.
	 * 
	 * @param int $id organization unit ID.
	 * @param string $type Type of organization unit in range of OrganizationUnit::ORG_UNIT_TYPE_*.
	 * @return	array of OrganizationUnit where id is key or empty array.
	 */
	public function findAllChildrenByType($id, $type)
	{
		$orgUnit_array = array();
		
		$parent = $this->find($id);
		if ($parent)
		{
			$db_result = $this->conn->select('*')->from('[OrganizationUnit]')->where('[Type] = %s AND [Path] LIKE %like~', $type, $parent->Path)->orderBy('[Name]')->fetchAll();        
			if ($db_result !== FALSE)
			{
				foreach ($db_result as $orgUnit)
					$orgUnit_array[$orgUnit->OrganizationUnit_ID] = $this->load($orgUnit);
			}
        }
        
		return $orgUnit_array;
	}
    
    /**
	 * Finds all subordinate of organization unit given by id.
	 * 
	 * @param int $id organization unit ID.
	 * @return	array of string(EID) or NULL if not found.
	 */
	public function findAllSubordinate($id)
	{
        $array = array();
		$EID_db_result = $this->conn->query("SELECT [Person].[EID] FROM [OrganizationUnit] 
                                                    JOIN [Person] ON [OrganizationUnit].[OrganizationUnit_ID] = [Person].[OrganizationUnit_ID]
                                                        WHERE [OrganizationUnit].[OrganizationUnit_ID] = %i", $id)->fetchAll();

        if ($EID_db_result !== FALSE)
        {
            foreach ($EID_db_result as $eid)
                $array[] = $eid->EID;
            return $array;
        }
        else
            return null;
	}
    
	/**
	 * Gets all Organization units by type.
	 * 
     * @param string Type of org unit
	 * @return array Array of OrganizationUnit where Organization unit ID is key.
	 */
	public function findAllOrgUnitByType($type)
	{
        $orgUnit_array = array();
		$orgUnit_db_result = $this->conn->select('*')->from('OrganizationUnit')->where('[Type] = %s', $type)->orderBy('[Name]')->fetchAll();        
		foreach ($orgUnit_db_result as $orgUnit)
			$orgUnit_array[$orgUnit->OrganizationUnit_ID] = $this->load($orgUnit);
        return $orgUnit_array;
	}
    
	/**
	 * Finds all technical groups working on given project charter.
	 * 
	 * @param int $charter_ID ProjectCharter ID.
	 * @return array Array of OrganizationUnit where key is OrganizationUnit_ID or NULL if not found.
	 */
	public function findAllTechnicalGroupOnCharter($charter_ID)
	{
		$organizationUnit_db_result = $this->conn->query('SELECT [OrganizationUnit].* FROM [ProjectCharterBudget] 
                                                            JOIN [OrganizationUnit] ON [OrganizationUnit].[OrganizationUnit_ID] = [ProjectCharterBudget].[TechnicalGroup]
                                                                WHERE [ProjectCharterBudget].[ProjectCharter_ID] = %i', $charter_ID)->fetchAll();
        $organizationUnit_array = array();
        if ($organizationUnit_db_result !== FALSE)
        {
            foreach ($organizationUnit_db_result as $organizationUnit)
                $organizationUnit_array[$organizationUnit->OrganizationUnit_ID] = $this->load($organizationUnit);
            return $organizationUnit_array;
        }
        else
            return null;
	}
	
	/**
	 * Finds all Organization units by type and leader.
	 * 
     * @param string $type Type of organization unit
     * @param string $eid EID of leader of organization unit
	 * @return array Array of OrganizationUnit where Organization unit ID is key.
	 */
	public function findByTypeAndLeader($type, $eid)
	{
        $orgUnit_array = array();
		$orgUnit_db_result = $this->conn->select('*')->from('OrganizationUnit')->where('[Type] = %s AND [Leader] = %s', $type, $eid)->orderBy('[Name]')->fetchAll();        
		foreach ($orgUnit_db_result as $orgUnit)
			$orgUnit_array[$orgUnit->OrganizationUnit_ID] = $this->load($orgUnit);
        return $orgUnit_array;
	}
	
	/**
	 * Finds first ancestor of given organization unit
	 * 
	 * @param int $id Organization unit ID
	 * @return OrganizationUnit Or NULL in case that no ancestor was found.
	 */
	public function findFirstAncestor($id)
	{
		$db_result = $this->conn->select('*')->from('OrganizationUnit')->where('[Path] = (SELECT SUBSTR([Path], 1, 3)
			                                                                              FROM [OrganizationUnit]
																						  WHERE [OrganizationUnit_ID] = %i)', $id)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds organization unit by path.
	 * 
	 * @param int $id organization unit Path.
	 * @return	OrganizationUnit or NULL if not found.
	 */
	public function findSBEByPath($id)
	{
		$organizationUnit_db_result = $this->conn->select('*')->from('OrganizationUnit')->where('[Path] = %i', $id)->fetch();
		if ($organizationUnit_db_result !== FALSE)
			return $this->load($organizationUnit_db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds organization unit by short.
	 * 
	 * @param string $short organization unit Short.
	 * @return	OrganizationUnit or NULL if not found.
	 */
	public function findSBEByShort($short)
	{
		$organizationUnit_db_result = $this->conn->select('*')->from('OrganizationUnit')->where('[Short] = %s', $short)->and('CHAR_LENGTH([Path]) = %i', 3)->fetch();
		if ($organizationUnit_db_result !== FALSE)
			return $this->load($organizationUnit_db_result);
		else
			return NULL;
	}

    /**
     * Finds all organization units where given person is leader
     *
     * @param string $person_id
     * @return array Array of OrganizationUnits indexed by their id or empty array.
     */
	public function findByLeader($person_id)
	{
        $db_result = $this->conn->query('
            SELECT *
            FROM [OrganizationUnit]
            WHERE [Leader] = %s
        ', $person_id)
        ->fetchAll();

        $org_units = array();

        foreach ($db_result as $unit)
            $org_units[$unit->OrganizationUnit_ID] = $this->load($unit);

        return $org_units;
	}

    /**
     * Tells if given person is leader of at least one organization unit
     *
     * @param string $person_id
     * @return boolean True if given person is leader, false otherwise
     */
	public function isLeader($person_id)
	{
        $db_result = $this->conn->query('
        	SELECT [OrganizationUnit_ID]
        	FROM [OrganizationUnit]
        	WHERE [Leader] = %s
        ', $person_id)
        ->fetch();

        if ($db_result->OrganizationUnit_ID)
        	return true;

        return false;
	}
	
}