<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project charter approver repository access class.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 * 
 */
class ProjectCharterApproverRepository extends Object
{
    /** @var IProjectCharterApproverMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IProjectCharterApproverMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ProjectCharterApprover-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Project Charter Approver"));
    }

	/**
	 * Saves project charter approver.
	 * 
	 * @param ProjectCharterApprover $projectCharterApprover
	 * @return boolean
	 */
	public function save(ProjectCharterApprover $projectCharterApprover)
	{
		if($this->mapper->save($projectCharterApprover))
		{
			$this->multilog->log('Project Charter Approver for charter with ID: ' . $projectCharterApprover->ProjectCharterID . ' and version: ' . $projectCharterApprover->Version . ' and Approver EID: ' . $projectCharterApprover->ApproverEID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Updates project charter approver.
	 * 
	 * @param ProjectCharterApprover $projectCharterApprover
         * @return boolean
	 */
	public function update(ProjectCharterApprover $projectCharterApprover)
	{
		$oldObject = $this->find($projectCharterApprover->ProjectCharterApproverID);
		if($this->mapper->update($projectCharterApprover))
		{
			$this->multilog->log('Project Charter Approver for charter with ID: ' . $projectCharterApprover->ProjectCharterID . ' and version: ' . $projectCharterApprover->Version . ' and Approver EID: ' . $projectCharterApprover->ApproverEID . ' updated. ' . $this->multilog->differences($oldObject, $projectCharterApprover), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing project charter approver.
	 * 
	 * @param int $projectCharterApprover_id ProjectCharterApprover ID.
	 * @return boolean
	 */
	public function delete($projectCharterApprover_id)
	{
		if($this->mapper->delete($projectCharterApprover_id))
		{
			$this->multilog->log('Project Charter Approver History with ID ' . $projectCharterApprover_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Finds project charter approver by  project charter id.
	 * 
	 * @param int $projectCharterApprover_id ProjectCharter ID.
	 * @return	ProjectCharterApprover or NULL if not found.
	 */
	public function find($projectCharterApprover_id)
	{
        return $this->mapper->find($projectCharterApprover_id);
	}
    
	/**
	 * Finds all project charter approver by project charter id and version.
	 * 
	 * @param int $projectCharter_id ProjectCharter ID.
         * @param int $version Version of round of approval.
	 * @return	array Associative array of ProjectCharterApprover where key is approver eid or NULL if not found.
	 */
	public function findAllApprover($projectCharter_id, $version)
	{
        return $this->mapper->findAllApprover($projectCharter_id, $version);
	}
    
	/**
	 * Finds all latest version project charter approver by approver eid.
	 * 
     * @param string $eid Approver EID.
	 * @return	array Associative array of ProjectCharterApprover where key is approver eid or NULL if not found.
	 */
	public function findAllProjectCharterByApprover($eid)
	{
        return $this->mapper->findAllProjectCharterByApprover($eid);
	}
    
	/**
	 * Find out if project charter is approved by all approvers.
	 * Returns TRUE if there are no approvers defined for the charter.
	 * 
	 * @param int $projectCharter_id ProjectCharter ID.
	 * @param int $version Version of round of approval.
	 * @return	boolean True if project charter is approved otherwise false.
	 */
	public function isApproved($projectCharter_id, $version)
	{
        return $this->mapper->isApproved($projectCharter_id, $version);
	}
    
	/**
	 * Finds latest project charter approver version by approver EID.
	 * 
	 * @param int $projectCharter_id ProjectCharter ID.
     * @param int $version Version of round of approval.
	 * @param String $eid Approver EID.
	 * @return	ProjectCharterApprover or NULL if not found.
	 */
    public function findByApprover($projectCharter_id, $version, $eid)
	{
        return $this->mapper->findByApprover($projectCharter_id, $version, $eid);
	}
	
	/**
	 * Finds project charter approver by role of approver.
	 * 
	 * @param int $projectCharter_id ProjectCharter ID.
     * @param int $version Version of round of approval.
	 * @param string $role Role of approver in company structure.
	 * @return	ProjectCharterApprover or NULL if not found.
	 */
    public function findByRole($projectCharter_id, $version, $role)
	{
        return $this->mapper->findByApprover($projectCharter_id, $version, $role);
	}
}
