<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing projectCharterBudgetHistory from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterBudgetHistoryDibiMapper extends DibiMapper implements IProjectCharterBudgetHistoryMapper
{
        /**
	 * Creates ProjectCharterBudgetHistory instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return ProjectCharterBudgetHistory 
	 */
        private function load(DibiRow $row)
        {
	    $projectCharterBudgetHistory = new ProjectCharterBudgetHistory();
            $projectCharterBudgetHistory->projectCharterID = $row->ProjectCharter_ID;
            $projectCharterBudgetHistory->version = $row->Version;
            $projectCharterBudgetHistory->creationDateTime = $row->CreationDateTime;
            $projectCharterBudgetHistory->responsible = $row->Responsible;		
            $projectCharterBudgetHistory->comment = $row->Comment;		
            $projectCharterBudgetHistory->latestVersion = $row->LatestVersion;		
	    return $projectCharterBudgetHistory;
	}
        
        /**
	 * Saves new project charter budget history record.
	 * 
	 * @param ProjectCharterBudgetHistory $projectCharterBudgetHistory
	 * @return boolean
	 */
	public function save(ProjectCharterBudgetHistory $projectCharterBudgetHistory)
	{
        try
        {
            $array = array();
            $array['ProjectCharter_ID'] = $projectCharterBudgetHistory->getProjectCharterID();
            $array['Version'] = $projectCharterBudgetHistory->getVersion();
            $array['CreationDateTime'] = new DateTime;
            $array['Responsible'] = $projectCharterBudgetHistory->getResponsible();
            $array['Comment'] = $projectCharterBudgetHistory->getComment();
            $array['LatestVersion'] = $projectCharterBudgetHistory->getLatestVersion();
            
            $this->conn->insert('ProjectCharterBudgetHistory', $array)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert of projectCharterBudgetHistory with ProjectCharter_ID = ".$projectCharterBudgetHistory->getProjectCharterID()." and Version = ".$projectCharterBudgetHistory->getVersion()." failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
        
        /**
	 * Updates existing project charter budget history record.
	 * 
	 * Updates only attributes that are updatable.
	 * 
	 * @param ProjectCharterBudgetHistory $projectCharterBudgetHistory
	 * @return boolean
	 */
	public function update(ProjectCharterBudgetHistory $projectCharterBudgetHistory)
	{
        try
        {
            $array = array();
            $array['Comment'] = $projectCharterBudgetHistory->getComment(); 
            $array['LatestVersion'] = $projectCharterBudgetHistory->getLatestVersion(); 
            $this->conn->update('ProjectCharterBudgetHistory', $array)->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterBudgetHistory->getProjectCharterID(), $projectCharterBudgetHistory->getVersion())->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of projectCharterBudgetHistory with ProjectCharter_ID = ".$projectCharterBudgetHistory->getProjectCharterID()." and Version = ".$projectCharterBudgetHistory->getVersion()." failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
        
        /**
	 * Deletes existing project charter budget history record.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return boolean
	 */
	public function delete($projectCharterID, $version)
	{
		try
		{
		    $result = $this->conn->delete('ProjectCharterBudgetHistory')->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterID, $version)->execute();
			if ($result)
				Debugger::log("ProjectCharterBudgetHistory with ProjectCharter_ID = ".$projectCharterID." and Version = ".$version." was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of projectCharterBudgetHistory with ProjectCharter_ID = ".$projectCharterID." and Version = ".$version." failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of projectCharterBudgetHistory with ProjectCharter_ID = ".$projectCharterID." and Version = ".$version." failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
        
        /**
	 * Gets project charter budget history record by ids.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return ProjectCharterBudgetHistory
	 */
	public function find($projectCharterID, $version)
	{
		$project_db_result = $this->conn->select('*')->from('ProjectCharterBudgetHistory')->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterID, $version)->fetch();
		if ($project_db_result !== FALSE)
			return $this->load($project_db_result);
		else
			return NULL;
	}
        
        /**
	 * Gets project charter budget history record with latest version.
	 * 
	 * @param int $projectCharterID
	 * @return ProjectCharterBudgetHistory
	 */
	public function findLatestVersion($projectCharterID)
	{
		$budgetHistory_db_result = $this->conn
                        ->select('*')
                        ->from('ProjectCharterBudgetHistory')
                        ->where('[ProjectCharter_ID] = %i AND [LatestVersion] = 1', $projectCharterID)
                        ->fetch();
		if ($budgetHistory_db_result !== FALSE)
			return $this->load($budgetHistory_db_result);
		else
			return NULL;
	}
	
	/**
	 * Gets project charter budget history record, which is the last one
	 * submitted by certain person (indentified by EID). It's possible this record
	 * won't be the latest one from all histories for this charter,
	 * as someone else (CL - line manager) could have created a new budget
	 * version and attain the latest budget status.
	 * 
	 * But for the EID supplied, it will.
	 * 
	 * @param int $projectCharterID
	 * @param int $EID
	 * @return ProjectCharterBudgetHistory
	 */
	public function findLatestVersionByEID($projectCharterID, $EID)
	{
		$budgetHistory_db_result = $this->conn->query(
			'SELECT * FROM `ProjectCharterBudgetHistory`
				WHERE `ProjectCharter_ID` = %i AND `Responsible` = %s
					ORDER BY `Version` DESC', $projectCharterID, $EID)
					->fetch();
		if ($budgetHistory_db_result !== FALSE)
			return $this->load($budgetHistory_db_result);
		else
			return NULL;
	}
	
	/**
	 * Gets all project charter history records by project charter id.
	 * 
	 * @param int $projectCharterID
	 * @return array Array of ProjectCharterBudgetHistory or empty array if error occured.
	 */
	public function findByProjectCharterID($projectCharterID)
	{
		$db_result = $this->conn
		    ->select('*')
			->from('ProjectCharterBudgetHistory')
			->where('[ProjectCharter_ID] = %i', $projectCharterID)
			->fetchAll();
		
		$array_budget_history = array();
		if ($db_result !== FALSE)
		{
			foreach ($db_result as $budget_history)
				$array_budget_history[] = $this->load($budget_history);
			return $array_budget_history;
		}
		else
		    return $array_budget_history;
	}
        
	/**
	 * Initializes flag LatestVersion
	 * @return boolean
	 */
	public function initLatestVersion($projectCharterID)
	{
		try {
			$array = array();
			$array['LatestVersion'] = 0;
			$this->conn->update('ProjectCharterBudgetHistory', $array)->where('[ProjectCharter_ID] = %i AND [Version] IS NOT NULL', $projectCharterID)->execute();
				return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("Initializing of LatestVersion in projectCharterBudgetHistory failed: ".$e->getMessage(), Debugger::ERROR);
			return false;
		}
	}
}
