<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project planning charter repository access class.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ProjectPlanningCharterRepository extends Object
{
	/** @var IProjectPlanningCharterMapper */
	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

	public function __construct(IProjectPlanningCharterMapper $mapper)
	{
		$this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ProjectPlanningCharter-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Project Planning Charter"));
	}
	
	/**
	 * Saves project planning charter.
	 * 
     * @param ProjectPlanningCharter $projectPlanningCharter Project planning charter
	 * @return boolean
	 */
	public function save(ProjectPlanningCharter $projectPlanningCharter)
	{
		if($this->mapper->save($projectPlanningCharter))
		{
			$this->multilog->log('Project Planning Charter with ID: ' . $projectPlanningCharter->ID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Updates project planning charter.
	 * 
	 * @param ProjectPlanningCharter $projectPlanningCharter Project planning charter
	 * @return boolean
	 */
	public function update(ProjectPlanningCharter $projectPlanningCharter)
	{
		$oldObject = $this->find($projectPlanningCharter->ID);
		if($this->mapper->update($projectPlanningCharter))
		{
			$this->multilog->log('Project Planning Charter with ID: ' . $projectPlanningCharter->ID . ' updated. ' . $this->multilog->differences($oldObject, $projectPlanningCharter), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Deletes existing project planning charter.
	 * 
	 * @param int $projectPlanningCharter_id
	 * @return boolean
	 */
	public function delete($projectPlanningCharter_id)
	{
		if($this->mapper->delete($projectPlanningCharter_id))
		{
			$this->multilog->log('Project planning charter with ID: ' . $projectPlanningCharter_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing project planning charters by project id and by planning year.
	 * 
	 * @param int $projectID Project ID.
	 * @param int $planningYear Year.
	 * @return boolean
	 */
	public function deleteByProjectIDAndYear($projectID, $planningYear)
	{
		if($this->mapper->deleteByProjectIDAndYear($projectID, $planningYear))
		{
			$this->multilog->log('Project planning charters with Project_ID = ' . $projectID . ' and Year = ' . $planningYear . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Returns project planning charter by its id.
	 * 
	 * @param int $id ProjectPlanningCharter ID.
	 * @return ProjectPlanningCharter
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
	
	/**
	 * Returns project planning charter by project id.
	 * 
	 * @param int $projectID Project ID.
	 * @return array of ProjectPlanningCharter or empty array if not found
	 */
	public function findByProjectID($projectID)
	{
		return $this->mapper->findByProjectID($projectID);
	}
	
	/**
	 * Returns project planning charter by project id and by planning year.
	 * 
	 * @param int $projectID Project ID.
	 * @param int $planningYear Year.
	 * @return array of ProjectPlanningCharter or empty array if not found
	 */
	public function findByProjectIDAndYear($projectID, $planningYear)
	{
		return $this->mapper->findByProjectIDAndYear($projectID, $planningYear);
	}
	
	/**
	 * Returns rows related to Project ID with specified Type
	 * 
	 * @param int $projectID Project ID.
	 * @param int $year The year of planning.
	 * @param string $type Type of the records to find, e.g. 'HRS', 'SUB', 'TRV'.
	 * @return 
	 */
	public function findByProjectIDAndYearAndType($projectID, $year, $type)
	{
		return $this->mapper->findByProjectIDAndYearAndType($projectID, $year, $type);
	}
	
	/**
	 * Returns rows related to Project ID with specified Type
	 * 
	 * @param int $projectID Project ID.
	 * @param int $year The year of planning.
	 * @param string $type Type of the records to find, e.g. 'HRS', 'SUB', 'TRV'.
	 * @param string $name Name column. Used for labor tables.
	 * @return 
	 */
	public function findByProjectIDAndYearAndTypeAndName($projectID, $year, $type, $name)
	{
		return $this->mapper->findByProjectIDAndYearAndTypeAndName($projectID, $year, $type, $name);
	}
	
	/*
	 * Returns years, which are planned for project identified by $projectID
	 * 
	 * @param int $projectID Project ID.
	 * @return array
	 */
	public function findYearsByProjectId($projectID)
	{
		return $this->mapper->findYearsByProjectID($projectID);
	}

	/**
	 * Returns all project planning charters.
	 * 
	 * @return array of ProjectPlanningCharter or empty array if not found
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
}
