<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing project planning charter state from database using dibi.
 *
 * @author Leos Navratil <leos.navratil@honeywell.com>
  */
class ProjectPlanningCharterStateDibiMapper extends DibiMapper implements IProjectPlanningCharterStateMapper
{
	/**
	 * Creates ProjectPlanningCharterState instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return ProjectPlanningCharterState
	 */
	private function load(DibiRow $row)
	{
		$projectPlanningCharterState = new ProjectPlanningCharterState();
		$projectPlanningCharterState->ProjectID = $row->Project_ID;
		$projectPlanningCharterState->Year = $row->Year;
		$projectPlanningCharterState->WorkpackageID = $row->WorkPackage_ID;
		$projectPlanningCharterState->ProjectCharterID = $row->ProjectCharter_ID;
		$projectPlanningCharterState->State = $row->State;
		$projectPlanningCharterState->AtLeastOnceFinished = ($row->AtLeastOnceFinished) ? TRUE : FALSE;
		return $projectPlanningCharterState;
	}

	/**
	 * Saves project planning state.
	 * 
     * @param ProjectPlanningCharterState $projectPlanningCharterState Project planning charter state
	 * @return boolean
	 */
	public function save(ProjectPlanningCharterState $projectPlanningCharterState)
	{
		try
		{
			$array = array();
			$array['Project_ID'] = $projectPlanningCharterState->ProjectID;
			$array['Year'] = $projectPlanningCharterState->Year;
			$array['WorkPackage_ID'] = $projectPlanningCharterState->WorkpackageID;
			$array['ProjectCharter_ID'] = $projectPlanningCharterState->ProjectCharterID;
			$array['State'] = $projectPlanningCharterState->State;
			$array['AtLeastOnceFinished'] = ($projectPlanningCharterState->AtLeastOnceFinished) ? TRUE : FALSE;
			$db_result = $this->conn->insert('ProjectPlanningCharterState', $array)->execute();
			return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("Saving ProjectPlanningCharterState failed. " . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Updates project planning state.
	 * 
     * @param ProjectPlanningCharterState $projectPlanningCharterState Project planning charter state
	 * @return boolean
	 */
	public function update(ProjectPlanningCharterState $projectPlanningCharterState)
	{
		try
		{
			$array = array();
			$array['Project_ID'] = $projectPlanningCharterState->ProjectID;
			$array['Year'] = $projectPlanningCharterState->Year;
			$array['WorkPackage_ID'] = $projectPlanningCharterState->WorkpackageID;
			$array['ProjectCharter_ID'] = $projectPlanningCharterState->ProjectCharterID;
			$array['State'] = $projectPlanningCharterState->State;
			$array['AtLeastOnceFinished'] = ($projectPlanningCharterState->AtLeastOnceFinished) ? TRUE : FALSE;
			$db_result = $this->conn->update('ProjectPlanningCharterState', $array)->where('[Project_ID] = %i AND [Year] = %i',
					$projectPlanningCharterState->ProjectID, $projectPlanningCharterState->Year)->execute();
			return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("ProjectPlanningCharterState update with Project ID = " . $projectPlanningCharterState->getProjectID() . " and Year = " . $projectPlanningCharterState->getYear()
					. " failed. " . $e->getMessage());
			return false;
		}
	}
	
	/**
	 * Deletes existing project planning state.
	 * 
	 * @param int $projectID Project's ID
	 * @param int $year Year
	 * @return boolean
	 */
	public function delete($projectID, $year)
	{
		try
		{
			$result = $this->conn->delete('ProjectPlanningCharterState')->where('[Project_ID] = %i AND [Year] = %i',
					$projectID, $year)->execute();
			if ($result)
				Debugger::log("Project planning charter state with Project ID = " . $projectID . " and Year = " . $year
						. " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of project planning charter state with Project ID = " . $projectID . " and Year = " . $year
						. " failed.", Debugger::ERROR);
			return $result;
		}
		catch (DibiException $e)
		{
			Debugger::log("Deleting of project planning charter state with Project ID = " . $projectID . " and Year = " . $year
					. " failed: " . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Returns project planning charter state by its Project ID and Year.
	 * 
	 * @param int $projectID Project's ID
	 * @param int $year Year
	 * @return ProjectPlanningCharterState
	 */
	public function find($projectID, $year)
	{
		$db_result = $this->conn->select('*')->from('ProjectPlanningCharterState')->where('[Project_ID] = %i AND [Year] = %i ', $projectID, $year)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Returns project planning charter state by Project Charter ID and Year.
	 * 
	 * @param int $projectCharterID Project Charter's ID
	 * @param int $year Year
	 * @return ProjectPlanningCharterState
	 */
	public function findByProjectCharterIDAndYear($projectCharterID, $year)
	{
		$db_result = $this->conn->select('*')->from('ProjectPlanningCharterState')->where('[ProjectCharter_ID] = %i AND [Year] = %i ', $projectCharterID, $year)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Returns all project planning charter states.
	 * 
	 * @return array of ProjectPlanningCharterState or empty array if not found
	 */
	public function findAll()
	{
		$states_array = array();
		$db_result = $this->conn->select('*')->from('ProjectPlanningCharterState')->fetchAll();
		foreach ($db_result as $projectPlanningCharterState)
		{
			$states_array[] = $this->load($projectPlanningCharterState);
		}
		return $states_array;
	}
}
