<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for assignment vs workable hours report.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class AssignmentWHReportService extends object
{
    /** @var DIContainer DI context with application repositories access */
    private $context;

	/** @var ILogger Logging facility */
    private $logger;
	
	/** @var int Year to generation */
    private $year;
	
/** @var int Organizaiton unit ID */
    private $orgUnitID;
	
	/** @var String Value for freeze user interface during report generation. */
    private $uiFreezeToken;
	
	/**
	 * Creates service for tracking table export.
	 *  
	 * @param $context Application context.
	 * @param ILogger Object for logging the import progress.
	 */
    public function  __construct(DIContainer $context, ILogger $logger, $year, $orgUnitID, $uiFreezeToken = null)
    {
        $this->context = $context;
		$this->logger = $logger;
		$this->year = $year;
		$this->orgUnitID = $orgUnitID;
		$this->uiFreezeToken = $uiFreezeToken;
    }
	
    /**
	 * 
     * Generates assignemnt vs workable hours report as a XLSX (MS-EXCEL file).
	 * File can be downloaded through browser.
	 * 
     */
    public function runAssignmentWHReportGeneration()
    {
		$objPHPExcel = new PHPExcel();
		
        $exec_time = 600;
	    if (!ini_set("max_execution_time", $exec_time)) 
		{
			$this->flashMessage('Setup of maximum execution time failed. Contact administrator.', 'error');
			trigger_error('Unable to setup maximum execution time to '.$exec_time.'.', E_USER_ERROR);
		}

		$this->logger->log('Assignment vs Workable hours report generation start: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		// find org unit
		$selectedOrgUnit = $this->context->organizationUnitRepository->find($this->orgUnitID);
		if($selectedOrgUnit == null)
			throw new EAssignmentWHReportServiceException('Org Unit with ID: ' . $this->orgUnitID . ' not found!');
		$reportedOrgUnit = $this->context->organizationUnitRepository->findAllChildrenByPath($selectedOrgUnit->Path);
		/**
		 *  Prepare all workers and their workable hours and assignment to report
		 *  Array structure:
		 *  $reportedData
		 *  {
		 *		OrgUnitID => array(2)
		 *		{
		 *			OrgUnitObject => OrganizationUnit
		 *			workers => array(number of workers)
		 *			{
		 *				EID => array(3)
		 *				{
		 *					PersonObject => Person
		 *					WorkableHours => AnnualOperatingPlan
		 *					Assignment => array(12)
		 *					{
		 *						jan .. dec
		 *					}
		 *				}
		 *			}
		 *		} 
		 */
		$reportedData = array();
		foreach($reportedOrgUnit as $orgUnit)
		{
			$reportedData[$orgUnit->ID]['OrgUnitObject'] = $orgUnit;
			$reportedData[$orgUnit->ID]['workers'] = array();
			$workers = $this->context->organizationUnitRepository->findAllSubordinate($orgUnit->ID);
			foreach($workers as $workerEID)
			{
				$personObject = $this->context->personRepository->find($workerEID);
				if($personObject->Status == 1)
				{
					$reportedData[$orgUnit->ID]['workers'][$workerEID]['PersonObject'] = $personObject;
					$reportedData[$orgUnit->ID]['workers'][$workerEID]['WorkableHours'] = $this->context->aopRepository->findForPerson($this->year, $workerEID);
					$reportedData[$orgUnit->ID]['workers'][$workerEID]['Assignment'] = $this->context->personTaskAssignmentRepository->findPersonTotalYearAssignments($workerEID, $this->year);
					$reportedData[$orgUnit->ID]['workers'][$workerEID]['Actuals'] = $this->context->workRepository->findPersonYearWork($workerEID, $this->year);
				}
			}
		}
		$actualMonth = Date('m');
		$monthPlus1Limit = 5;
		$monthPlus2Limit = 10;
		$whArray = $assignArray = $actualsArray = array();
		
		for ($i = 1; $i <= 12; $i++)
			$whArray[$i] = $assignArray[$i] = $actualsArray[$i] = array();

		$sheet = $objPHPExcel->getActiveSheet();
        $rowIterator = $sheet->getRowIterator();
        $cellIterator = $rowIterator->current()->getCellIterator();
		//setting
		$firstFormatedColumn = 'A';
		$lastFormatedColumn = 'P';
		$firstPercentageValueColumn = 'E';
		$lastPercentageValueColumn = 'P';
		$firstPercentageValueColumnASCII = 69;
		$warningColorRed = 'FFC8C8';
		$warningColorYellow = 'FFEB9B';
		$lineColor1 = 'DBE5F1';
		$lineColor2 = 'F2F2F2';
		$colorValue = 1;
        $borderMediumLeft = array(
        	'borders' => array(
                'left' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderMediumRight = array(
        	'borders' => array(
                'right' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderMediumBottom = array(
        	'borders' => array(
                'bottom' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
		//set conditional formatting for upcoming month
		$objConditional1Less = new PHPExcel_Style_Conditional();
		$objConditional1Less->setConditionType(PHPExcel_Style_Conditional::CONDITION_CELLIS);
		$objConditional1Less->setOperatorType(PHPExcel_Style_Conditional::OPERATOR_LESSTHAN);
		$objConditional1Less->addCondition('(100 - C1)');
		$objConditional1Less->getStyle()->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getEndColor()->setARGB($warningColorRed);
		
		$objConditional1Greater = new PHPExcel_Style_Conditional();
		$objConditional1Greater->setConditionType(PHPExcel_Style_Conditional::CONDITION_CELLIS);
		$objConditional1Greater->setOperatorType(PHPExcel_Style_Conditional::OPERATOR_GREATERTHAN);
		$objConditional1Greater->addCondition('(100 + C1)');
		$objConditional1Greater->getStyle()->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getEndColor()->setARGB($warningColorRed);
		$objConditionalStyles1 = array();
		array_push($objConditionalStyles1, $objConditional1Greater);
		array_push($objConditionalStyles1, $objConditional1Less);
		
		//set conditional formatting for month + 2 till the end of the year
		$objConditional2Less = new PHPExcel_Style_Conditional();
		$objConditional2Less->setConditionType(PHPExcel_Style_Conditional::CONDITION_CELLIS);
		$objConditional2Less->setOperatorType(PHPExcel_Style_Conditional::OPERATOR_LESSTHAN);
		$objConditional2Less->addCondition('(100 - C2)');
		$objConditional2Less->getStyle()->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getEndColor()->setARGB($warningColorYellow);
		
		$objConditional2Greater = new PHPExcel_Style_Conditional();
		$objConditional2Greater->setConditionType(PHPExcel_Style_Conditional::CONDITION_CELLIS);
		$objConditional2Greater->setOperatorType(PHPExcel_Style_Conditional::OPERATOR_GREATERTHAN);
		$objConditional2Greater->addCondition('(100 + C2)');
		$objConditional2Greater->getStyle()->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getEndColor()->setARGB($warningColorYellow);
		$objConditionalStyles2 = array();
		array_push($objConditionalStyles2, $objConditional2Greater);
		array_push($objConditionalStyles2, $objConditional2Less);
				
		//collapsing values columns
		$collapseFrom = $firstPercentageValueColumn;
		for ($collapseFrom; $collapseFrom != chr($firstPercentageValueColumnASCII + $actualMonth); $collapseFrom++)
		{
			$sheet->getColumnDimension($collapseFrom)->setOutlineLevel(1);
			$sheet->getColumnDimension($collapseFrom)->setVisible(false);
			$sheet->getColumnDimension($collapseFrom)->setCollapsed(true);
		}
		
		// set column width
		$sheet->getColumnDimension('A')->setWidth(41);
        $sheet->getColumnDimension('B')->setWidth(35);
		$sheet->getColumnDimension('C')->setWidth(35);
		$sheet->getColumnDimension('D')->setWidth(15);
		
		//Limits setting
		$cellIterator->current()->setValue("Actual month + 1 warning for value percentage greater or less then 100 by:");
        $cellIterator->next();
		//$cellIterator->current()->setValue($monthPlus1Limit);
        $cellIterator->next();
		$cellIterator->current()->setValue($monthPlus1Limit);
        $cellIterator->next();
		$rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		
		$cellIterator->current()->setValue("Actual month + 2 till the end of year warning for percentage value greater or less then 100 by:");
        $cellIterator->next();
		//$cellIterator->current()->setValue($monthPlus2Limit);
        $cellIterator->next();
		$cellIterator->current()->setValue($monthPlus2Limit);
        $cellIterator->next();
		$rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		
        // Header
        $cellIterator->current()->setValue("Organization Unit");
        $cellIterator->next();
        $cellIterator->current()->setValue("Person");
        $cellIterator->next();
        $cellIterator->current()->setValue("Resource Owner");
        $cellIterator->next();
        $cellIterator->current()->setValue("Type");
        $cellIterator->next();
        $cellIterator->current()->setValue("Jan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Feb");
        $cellIterator->next();
        $cellIterator->current()->setValue("Mar");
        $cellIterator->next();
        $cellIterator->current()->setValue("Apr");
        $cellIterator->next();
        $cellIterator->current()->setValue("May");
        $cellIterator->next();
        $cellIterator->current()->setValue("Jun");
        $cellIterator->next();
        $cellIterator->current()->setValue("Jul");
        $cellIterator->next();
        $cellIterator->current()->setValue("Aug");
        $cellIterator->next();
        $cellIterator->current()->setValue("Sep");
        $cellIterator->next();
        $cellIterator->current()->setValue("Oct");
        $cellIterator->next();
        $cellIterator->current()->setValue("Nov");
        $cellIterator->next();
        $cellIterator->current()->setValue("Dec");
		// first row formating
		$firstRow = $cellIterator->current()->getRow();
		$sheet->getStyle($firstFormatedColumn . $firstRow . ':' . $lastFormatedColumn . $firstRow)->applyFromArray($borderMediumBottom);
		$sheet->getStyle($firstFormatedColumn . $firstRow . ':' . $lastFormatedColumn . $firstRow)->getFont()->setBold(true);
		// freeze pane
		$sheet->freezePane($firstPercentageValueColumn . ($firstRow + 1));
        $cellIterator->next();
        $rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
                                   
        // Data generating
		foreach ($reportedData as $orgUnitArr)
		{
			foreach($orgUnitArr['workers'] as $worker)
			{
				$hours = 0;
				$hours = array_sum($worker['WorkableHours']->getAOP()) + array_sum($worker['Assignment']) + $worker['Actuals']->sum();
				if($worker['PersonObject']->Status == 1 && $hours > 0)
				{
					//Actuals hours row
					$row = $cellIterator->current()->getRow();
					if($colorValue/2 != floor($colorValue/2))
						$sheet->getStyle($firstFormatedColumn . $row . ':' . $lastFormatedColumn . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($lineColor1);
					else
						$sheet->getStyle($firstFormatedColumn . $row . ':' . $lastFormatedColumn . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($lineColor2);
                    
                    // Find person supervisor
                    $supervisor = $this->context->personRepository->find($worker['PersonObject']->getSupervisorID());
                    $supervisor_name = ($supervisor) ? $supervisor->getReversedFullNameWithEID() : 'Undefined';
                    
					$cellIterator->current()->setValue($orgUnitArr['OrgUnitObject']->Type . ' - ' . $orgUnitArr['OrgUnitObject']->Name);
					$cellIterator->next();
					$cellIterator->current()->setValue($worker['PersonObject']->getReversedFullNameWithEID());
					$cellIterator->next();
					$cellIterator->current()->setValue($supervisor_name);
					$cellIterator->next();
					$cellIterator->current()->setValue('Actuals');
					$cellIterator->next();
					for($i = 1; $i <= 12; $i++)
					{
						$cellIterator->current()->setValue($worker['Actuals'][$i]);
						$col = $cellIterator->current()->getColumn();
						$actualsArray[$i][] = $col . $row; 
						$cellIterator->next();
					}
					$rowIterator->next();
					$cellIterator = $rowIterator->current()->getCellIterator();
					
					//Assigned hours row
					$row = $cellIterator->current()->getRow();
					if($colorValue/2 != floor($colorValue/2))
						$sheet->getStyle($firstFormatedColumn . $row . ':' . $lastFormatedColumn . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($lineColor1);
					else
						$sheet->getStyle($firstFormatedColumn . $row . ':' . $lastFormatedColumn . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($lineColor2);
                    
					$cellIterator->current()->setValue($orgUnitArr['OrgUnitObject']->Type . ' - ' . $orgUnitArr['OrgUnitObject']->Name);
					$cellIterator->next();
					$cellIterator->current()->setValue($worker['PersonObject']->getReversedFullNameWithEID());
					$cellIterator->next();
					$cellIterator->current()->setValue($supervisor_name);
					$cellIterator->next();
					$cellIterator->current()->setValue('Assignment');
					$cellIterator->next();
					for($i = 1; $i <= 12; $i++)
					{
						$cellIterator->current()->setValue($worker['Assignment'][$i]);
						$col = $cellIterator->current()->getColumn();
						$assignArray[$i][] = $col . $row; 
						$cellIterator->next();
					}
					$rowIterator->next();
					$cellIterator = $rowIterator->current()->getCellIterator();
					
					//Workable hoours row
					$row = $cellIterator->current()->getRow();
					if($colorValue/2 != floor($colorValue/2))
						$sheet->getStyle($firstFormatedColumn . $row . ':' . $lastFormatedColumn . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($lineColor1);
					else
						$sheet->getStyle($firstFormatedColumn . $row . ':' . $lastFormatedColumn . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($lineColor2);
					$cellIterator->current()->setValue($orgUnitArr['OrgUnitObject']->Type . ' - ' . $orgUnitArr['OrgUnitObject']->Name);
					$cellIterator->next();
					$cellIterator->current()->setValue($worker['PersonObject']->getReversedFullNameWithEID());
					$cellIterator->next();
					$cellIterator->current()->setValue($supervisor_name);
					$cellIterator->next();
					$cellIterator->current()->setValue('Workable Hours');
					$cellIterator->next();
					for($i = 1; $i <= 12; $i++)
					{
						$cellIterator->current()->setValue($worker['WorkableHours']->getMonthAOPHours($i));
						$col = $cellIterator->current()->getColumn();
						$whArray[$i][] = $col . $row; 
						$cellIterator->next();
					}
					$rowIterator->next();
					$cellIterator = $rowIterator->current()->getCellIterator();
					
					//Percentage row
					$row = $cellIterator->current()->getRow();
					if($colorValue/2 != floor($colorValue/2))
						$sheet->getStyle($firstFormatedColumn . $row . ':' . $lastFormatedColumn . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($lineColor1);
					else
						$sheet->getStyle($firstFormatedColumn . $row . ':' . $lastFormatedColumn . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($lineColor2);
					$cellIterator->current()->setValue($orgUnitArr['OrgUnitObject']->Type . ' - ' . $orgUnitArr['OrgUnitObject']->Name);
					$cellIterator->next();
					$cellIterator->current()->setValue($worker['PersonObject']->getReversedFullNameWithEID());
					$cellIterator->next();
					$cellIterator->current()->setValue($supervisor_name);
					$cellIterator->next();
					$cellIterator->current()->setValue('Percentage');
					$cellIterator->next();
					for($i = 1; $i <= 12; $i++)
					{
						if($worker['WorkableHours']->getMonthAOPHours($i) != 0)
						{
							$col = $cellIterator->current()->getColumn();
							$cellIterator->current()->setValue('=ROUND((' . chr($firstPercentageValueColumnASCII - 1 + $i) . ($row - 2) . '/' . chr($firstPercentageValueColumnASCII - 1 + $i) . ($row - 1) . ')*100,0)');
							if(($actualMonth + 1) == $i)
								$sheet->getStyle($col . $row)->setConditionalStyles($objConditionalStyles1);
							elseif(($actualMonth + 2) <= $i)
								$sheet->getStyle($col . $row)->setConditionalStyles($objConditionalStyles2);
							$cellIterator->next();
						}
						else
						{
							$cellIterator->current()->setValue(0);
							$cellIterator->next();
						}
					}
					$sheet->getStyle($firstFormatedColumn . $row . ':' . $lastFormatedColumn . $row)->applyFromArray($borderMediumBottom);
					$rowIterator->next();
					$cellIterator = $rowIterator->current()->getCellIterator();
					$colorValue++;
				}
			}
		}
		$rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->next();
		$cellIterator->next();
		$cellIterator->next();
		$cellIterator->current()->setValue('Actuals');
		$cellIterator->next();
		// Actuals TOTAL
		$row = $cellIterator->current()->getRow();
		for($i = 1; $i <= 12; $i++)
		{
			$cellIterator->current()->setValue('=SUBTOTAL(9,' . join($actualsArray[$i], ',') . ')');
			$cellIterator->next();
		}
		$rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->next();
		$cellIterator->next();
		$cellIterator->next();
		$cellIterator->current()->setValue('Assignment');
		$cellIterator->next();
		// Assignment TOTAL
		$row = $cellIterator->current()->getRow();
		for($i = 1; $i <= 12; $i++)
		{
			$cellIterator->current()->setValue('=SUBTOTAL(9,' . join($assignArray[$i], ',') . ')');
			$cellIterator->next();
		}
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->next();
		$cellIterator->next();
		$cellIterator->next();
		$cellIterator->current()->setValue('Workable Hours');
		$cellIterator->next();
		// WH TOTAL
		$row = $cellIterator->current()->getRow();
		for($i = 1; $i <= 12; $i++)
		{
			$cellIterator->current()->setValue('=SUBTOTAL(9,' . join($whArray[$i], ',') . ')');
			$cellIterator->next();
		}
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->next();
		$cellIterator->next();
		$cellIterator->next();
		$cellIterator->current()->setValue('Percentage');
		$cellIterator->next();
		// Percentage TOTAL
		$row = $cellIterator->current()->getRow();
		for($i = 1; $i <= 12; $i++)
		{
			$col = $cellIterator->current()->getColumn();
			$cellIterator->current()->setValue('=ROUND((' . chr($firstPercentageValueColumnASCII - 1 + $i) . ($row - 2) . '/' . chr($firstPercentageValueColumnASCII - 1 + $i) . ($row - 1) . ')*100,0)');
			if(($actualMonth + 1) == $i)
				$sheet->getStyle($col . $row)->setConditionalStyles($objConditionalStyles1);
			elseif(($actualMonth + 2) <= $i)
				$sheet->getStyle($col . $row)->setConditionalStyles($objConditionalStyles2);
			$cellIterator->next();
		}
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		
		// actual month highlighted
		$row = $cellIterator->current()->getRow();
		$sheet->getStyle(chr($firstPercentageValueColumnASCII + $actualMonth) . $firstRow . ':' . chr($firstPercentageValueColumnASCII + $actualMonth) . ($row - 1))->applyFromArray($borderMediumRight);
		$sheet->getStyle(chr($firstPercentageValueColumnASCII + $actualMonth) . $firstRow . ':' . chr($firstPercentageValueColumnASCII + $actualMonth) . ($row - 1))->applyFromArray($borderMediumLeft);
        
		if(!is_null($this->uiFreezeToken))
			setcookie('fileDownloadToken', $this->uiFreezeToken);

        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment; filename=AssignmentVSWorkableHoursReport.xlsx');
        header('Cache-Control: max-age=0');
        $objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
		ob_end_clean();
        $objWriter->save("php://output");
		$this->logger->log('Assignment vs Workable hours report generation end: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		exit;
    }
}

class EAssignmentWHReportServiceException extends Exception {}