<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for person details export.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PersonReportService extends object
{
    /** @var DIContainer DI context with application repositories access */
    private $context;

	/** @var ILogger Logging facility */
    private $logger;
	
	/** @var int Year to generation */
    private $year;
	
	/** @var Array of SBE Organizaiton unit objects */
    private $sbe;
	
	/** @var String Value for freeze user interface during report generation. */
    private $uiFreezeToken;
	
    /** @var string Path to storage EAC files. */
    private $person_report_storage_dir;
	
	/** @var array Labour Centers */
	private $cachedLabourCenters;
	
	/** @var array Organization Units */
	private $cachedOrgUnits;
	
	/** @var array Persons */
	private $cachedPersons;
	
	/**
	 * Creates service for person report export.
	 *  
	 * @param $context Application context.
	 * @param ILogger Object for logging the import progress.
	 */
    public function  __construct(DIContainer $context, ILogger $logger, $year, $sbe, $uiFreezeToken = null)
    {
        $this->context = $context;
		$this->logger = $logger;
		$this->year = $year;
		$this->sbe = $sbe;
		$this->uiFreezeToken = $uiFreezeToken;
		$this->person_report_storage_dir = Environment::getConfig('personReport_storage_dir');
		
		$this->cachedLabourCenters = $this->context->costCenterRepository->findAllCostCenters();
		$this->cachedOrgUnits = $this->context->organizationUnitRepository->findAll(true);
		$this->cachedPersons = $this->context->personRepository->findAll();
    }
	
    /**
	 * 
     * Generates Person Details as a XLSX (MS-EXCEL file).
	 * File can be downloaded through browser and also file is saved to the public/PersonReport folder.
	 * 
     */
    public function runPersonReportGeneration()
    {
		$cacheMethod = PHPExcel_CachedObjectStorageFactory::cache_to_phpTemp;
		$cacheSettings = array('memoryCacheSize' => '1024MB', 'cacheTime' => '1000'); 

		PHPExcel_Settings::setCacheStorageMethod($cacheMethod, $cacheSettings);
		
		$objPHPExcel = new PHPExcel();
		
        $exec_time = 1800;
	    if (!ini_set("max_execution_time", $exec_time)) 
		{
			$this->flashMessage('Setup of maximum execution time failed. Contact administrator.', 'error');
			trigger_error('Unable to setup maximum execution time to '.$exec_time.'.', E_USER_ERROR);
		}
		
		$mem_limit = '2048M';
		if (!ini_set("memory_limit", $mem_limit)) 
		{
			$this->flashMessage('Setup of maximum allowed memory failed. Contact administrator.', 'error');
			trigger_error('Unable to setup maximum allowd memory to '.$mem_limit.'.', E_USER_ERROR);
		}
		$lock = $this->context->createLock('Person_Report', 600);
		if (!$lock->lock())
		{
			$this->logger->log('Person Report generation start failed - active lock detected: ' . date('m-d-Y H:i:s'), ILogger::INFO);
			throw new EPersonReportServiceException("Export of Person Report is already running.");
		}
		$this->logger->log('Person Report generation start: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		
		$actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index+1;
        $sheet = $objPHPExcel->getActiveSheet();
		$sheet->setTitle('Persons');
        $rowIterator = $sheet->getRowIterator();
        $cellIterator = $rowIterator->current()->getCellIterator();        
        // Header     
        $cellIterator->current()->setValue("SBU");
        $cellIterator->next();
        $cellIterator->current()->setValue("Department");
        $cellIterator->next();
        $cellIterator->current()->setValue("Department Short");
        $cellIterator->next();
        $cellIterator->current()->setValue("DL");
        $cellIterator->next();
        $cellIterator->current()->setValue("DL EID");
        $cellIterator->next();
        $cellIterator->current()->setValue("Cluster");
        $cellIterator->next();
        $cellIterator->current()->setValue("Cluster Short");
        $cellIterator->next();
        $cellIterator->current()->setValue("CL");
        $cellIterator->next();
        $cellIterator->current()->setValue("CL EID");
        $cellIterator->next();
        $cellIterator->current()->setValue("Technical group");
        $cellIterator->next();
        $cellIterator->current()->setValue("RO");
        $cellIterator->next();
        $cellIterator->current()->setValue("RO EID");
        $cellIterator->next();
		$cellIterator->current()->setValue("Supervisor");
        $cellIterator->next();
		$cellIterator->current()->setValue("Supervisor EID");
        $cellIterator->next();
        $cellIterator->current()->setValue("Surname");
        $cellIterator->next();
        $cellIterator->current()->setValue("Name");
        $cellIterator->next();
        $cellIterator->current()->setValue("Person EID");
        $cellIterator->next();
        $cellIterator->current()->setValue("Work Center");
        $cellIterator->next();
        $cellIterator->current()->setValue("Comment");
        $cellIterator->next();
        $cellIterator->current()->setValue("Status");
        $cellIterator->next();
        $cellIterator->current()->setValue("Yield");
        $cellIterator->next();
        $cellIterator->current()->setValue("Contract");
        $cellIterator->next();
        $cellIterator->current()->setValue("Licence");
        $cellIterator->next();
		$cellIterator->current()->setValue("CZI");
        $cellIterator->next();
        $cellIterator->current()->setValue("DUAL");
        $cellIterator->next();
        $cellIterator->current()->setValue("EAR");
        $cellIterator->next();
        $cellIterator->current()->setValue("ITAR");
        $cellIterator->next();
        $cellIterator->current()->setValue("MILITARY");
        $cellIterator->next();
        $cellIterator->current()->setValue("NLR");
        $cellIterator->next();
        $cellIterator->current()->setValue("Per");
		$cellIterator->next();
		$cellIterator->current()->setValue("Workable Hours");
        $cellIterator->next();
		$cellIterator->current()->setValue("Total Assigned");
        $cellIterator->next();
		$cellIterator->current()->setValue("Total Actuals");
        $cellIterator->next();
		$cellIterator->current()->setValue("Total Actuals_Assigned");
        $cellIterator->next();
		$cellIterator->current()->setValue("Unit");
        $cellIterator->next();
		$cellIterator->current()->setValue('Person Plant');
		$cellIterator->next();
		$cellIterator->current()->setValue("Generation time");
		$row = $cellIterator->current()->getRow();
		$col = $cellIterator->current()->getColumn();
		$sheet->setCellValue($col . ($row + 1), date('m-d-Y H:i:s'));
        $rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
                                   
        // Data generation
		foreach($this->sbe as $sbe)
		{
			$allPersons = $this->context->personRepository->findAllByOrgUnitPath($sbe->Path);
			foreach ($allPersons as $person)
			{
				$orgUnit = $this->cachedOrgUnits[$person->OrganizationUnitID];
				
				// get Department data
				$dep = null;
				$depDL = null;
				$depDLEID = null;
				$depObj = null;
				$depShort = null;
				if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE)
				{
					$dep = 'none';
					$depDL = 'none';
					$depDLEID = 'none';
					$depShort = 'none';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP)
				{
					$dep = $orgUnit->Name;
					$dl = $this->cachedPersons[$orgUnit->Leader];
					//DL
					if($dl)
					{
						$depDL = $dl->getReversedFullName();
						$depDLEID = $dl->ID;
					}
					else
					{
						$depDL = 'Not defined';
						$depDLEID = 'Not defined';
					}
					// Short
					if($orgUnit->Short)
						$depShort = $orgUnit->Short;
					else
						$depShort = 'Not defined';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				{
					$depObj = $this->cachedOrgUnits[$orgUnit->Parent_ID];
					$dep = $depObj->Name;
					$dl = $this->cachedPersons[$depObj->Leader];
					//DL
					if($dl)
					{
						$depDL = $dl->getReversedFullName();
						$depDLEID = $dl->ID;
					}
					else
					{
						$depDL = 'Not defined';
						$depDLEID = 'Not defined';
					}
					//Short
					if($depObj->Short)
						$depShort = $depObj->Short;
					else
						$depShort = 'Not defined';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
				{
					$depObj = $this->cachedOrgUnits[$this->cachedOrgUnits[$orgUnit->Parent_ID]->Parent_ID];
					$dep = $depObj->Name;
					$dl = $this->cachedPersons[$depObj->Leader];
					//DL
					if($dl)
					{
						$depDL = $dl->getReversedFullName();
						$depDLEID = $dl->ID;
					}
					else
					{
						$depDL = 'Not defined';
						$depDLEID = 'Not defined';
					}
					//Short
					if($depObj->Short)
						$depShort = $depObj->Short;
					else
						$depShort = 'Not defined';
				}
				// get Department data END

				// get Cluster data
				$clus = null;
				$clusCL = null;
				$clusCLEID = null;
				$clusObj = null;
				$clusShort = null;
				if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE || $orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP)
				{
					$clus = 'none';
					$clusCL = 'none';
					$clusCLEID = 'none';
					$clusShort = 'none';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				{
					$clus = $orgUnit->Name;
					$cl = $this->cachedPersons[$orgUnit->Leader];
					//CL
					if($cl)
					{
						$clusCL = $cl->getReversedFullName();
						$clusCLEID = $cl->ID;
					}
					else
					{
						$clusCL = 'Not defined';
						$clusCLEID = 'Not defined';
					}
					// Short
					if($orgUnit->Short)
						$clusShort = $orgUnit->Short;
					else
						$clusShort = 'Not defined';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
				{
					$clusObj = $this->cachedOrgUnits[$orgUnit->Parent_ID];
					$clus = $clusObj->Name;
					$cl = $this->cachedPersons[$clusObj->Leader];
					//CL
					if($cl)
					{
						$clusCL = $cl->getReversedFullName();
						$clusCLEID = $cl->ID;
					}
					else
					{
						$clusCL = 'Not defined';
						$clusCLEID = 'Not defined';
					}
					// Short
					if($clusObj->Short)
						$clusShort = $clusObj->Short;
					else
						$clusShort = 'Not defined';
				}
				// get Cluster data END

				// Technical Group
				$tg = null;
				$tgRO = null;
				$tgROEID = null;
				if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE || $orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP || $orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				{
					$tg = 'none';
					$tgRO = 'none';
					$tgROEID = 'none';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
				{
					$tg = $orgUnit->Name;
					$tgTL = $this->cachedPersons[$orgUnit->Leader];
					//TG leader
					if($tgTL)
					{
						$tgRO = $tgTL->getReversedFullName();
						$tgROEID = $tgTL->ID;
					}
					else
					{
						$tgRO = 'Not defined';
						$tgROEID = 'Not defined';
					}
				}
				// Technical Group END

				//super visor
				$supervisor = $this->cachedPersons[$person->SupervisorID];
				if($supervisor)
				{
					$supervisorName = $supervisor->getReversedFullName();
					$supervisorEID = $supervisor->ID;
				}
				else 	
				{
					$supervisorName = 'Not defined';
					$supervisorEID = 'Not defined';
				}
				// Workable hours
				$yield = $this->context->workableHourRepository->getYield($person->ID, $this->year);
				if($yield)
					$yield = $yield*100;
				else
					$yield = 'null';

				$contract = $this->context->workableHourRepository->getContract($person->ID, $this->year);
				if($contract)
					$contract = $contract*100; 
				else
					$contract = 'null';

				$nationality = $this->context->nationalityRepository->find($person->NationalityID);
				$all_licences = $this->context->ecLicenceRepository->findAll();
				$licensed = $this->context->personECLicenceRepository->findByEID($person->ID);
				$workableHours = $this->context->aopRepository->findForPerson($this->year, $person->ID);
				$totalAssignment = $this->context->personTaskAssignmentRepository->findPersonTotalYearAssignments($person->ID, $this->year);
				$totalActuals = $this->context->workRepository->findPersonYearWork($person->ID, $this->year);

				//Data export
				for($i = 1; $i <= 12; $i++)
				{
					$cellIterator->current()->setValue($sbe->Name);
					$cellIterator->next();
					$cellIterator->current()->setValue($dep);
					$cellIterator->next();
					$cellIterator->current()->setValue($depShort);
					$cellIterator->next();
					$cellIterator->current()->setValue($depDL);
					$cellIterator->next();
					$cellIterator->current()->setValue($depDLEID);
					$cellIterator->next();
					$cellIterator->current()->setValue($clus);
					$cellIterator->next();
					$cellIterator->current()->setValue($clusShort);
					$cellIterator->next();
					$cellIterator->current()->setValue($clusCL);
					$cellIterator->next();
					$cellIterator->current()->setValue($clusCLEID);
					$cellIterator->next();
					$cellIterator->current()->setValue($tg);
					$cellIterator->next();
					$cellIterator->current()->setValue($tgRO);
					$cellIterator->next();
					$cellIterator->current()->setValue($tgROEID);
					$cellIterator->next();
					$cellIterator->current()->setValue($supervisorName);
					$cellIterator->next();
					$cellIterator->current()->setValue($supervisorEID);
					$cellIterator->next();
					$cellIterator->current()->setValue($person->Surname);
					$cellIterator->next();
					$cellIterator->current()->setValue($person->Firstname);
					$cellIterator->next();
					$cellIterator->current()->setValue($person->ID);
					$cellIterator->next();
					$cellIterator->current()->setValue($person->CostCenterID);
					$cellIterator->next();
					$cellIterator->current()->setValue($person->Comment);
					$cellIterator->next();
					if($person->Status)
						$cellIterator->current()->setValue('Active');
					else
						$cellIterator->current()->setValue('Inactive');
					$cellIterator->next();
					$cellIterator->current()->setValue($yield);
					$cellIterator->next();
					$cellIterator->current()->setValue($contract);
					$cellIterator->next();
					$cellIterator->current()->setValue($nationality->Name);
					$cellIterator->next();
					foreach ($all_licences as $licence)
					{
						if (isset($licensed[$licence->ECLicenceID]))
						{
							$cellIterator->current()->setValue(1);
						}
						else
						{
							$cellIterator->current()->setValue(0);
						}
						$cellIterator->next();
					}
					$cellIterator->current()->setValue($i);
					$cellIterator->next();
					if ($workableHours != NULL)
					{
						$cellIterator->current()->setValue($workableHours->getMonthAOPHours($i));
					}
					else
					{			
						$cellIterator->current()->setValue(0);
					}
					$cellIterator->next();
					$cellIterator->current()->setValue($totalAssignment[$i]);
					$cellIterator->next();
					$cellIterator->current()->setValue($totalActuals[$i]);
					$cellIterator->next();
					if($this->year == Date('Y'))
					{
						if($i >= $actualMonth)
							$cellIterator->current()->setValue($totalAssignment[$i]);
						else
							$cellIterator->current()->setValue($totalActuals[$i]);
					}
					elseif($this->year > Date('Y'))
					{
						$cellIterator->current()->setValue($totalAssignment[$i]);
					}
					else
					{
						$cellIterator->current()->setValue($totalActuals[$i]);
					}
					$cellIterator->next();
					$cellIterator->current()->setValue("hrs");
					$cellIterator->next();
					if($this->cachedLabourCenters[$person->costCenterID])
						$cellIterator->current()->setValue($this->cachedLabourCenters[$person->costCenterID]->Plant);
					$rowIterator->next();
					$cellIterator = $rowIterator->current()->getCellIterator();
				}
			}
		}
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// assignment sheet
		$sheet1 = $objPHPExcel->createSheet(1);
		$sheet1->setTitle('Assignment');
		$rowIterator = $sheet1->getRowIterator();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Header
		$cellIterator->current()->setValue("SBU");
        $cellIterator->next();
        $cellIterator->current()->setValue("Department");
        $cellIterator->next();
        $cellIterator->current()->setValue("Department Short");
        $cellIterator->next();
        $cellIterator->current()->setValue("Cluster");
        $cellIterator->next();
        $cellIterator->current()->setValue("Cluster Short");
        $cellIterator->next();
        $cellIterator->current()->setValue("Technical group");
        $cellIterator->next();
        $cellIterator->current()->setValue("Person EID");
		$cellIterator->next();
        $cellIterator->current()->setValue("Surname");
        $cellIterator->next();
        $cellIterator->current()->setValue("Name");
        $cellIterator->next();
        $cellIterator->current()->setValue("Work Center");
        $cellIterator->next();
        $cellIterator->current()->setValue("Comment");
        $cellIterator->next();
        $cellIterator->current()->setValue("Status");
        $cellIterator->next();
        $cellIterator->current()->setValue("Utilization");
        $cellIterator->next();
        $cellIterator->current()->setValue("Contract");
        $cellIterator->next();
        $cellIterator->current()->setValue("SAP ID");
        $cellIterator->next();
        $cellIterator->current()->setValue("Project Name");
        $cellIterator->next();
        $cellIterator->current()->setValue("CO Object");
        $cellIterator->next();
        $cellIterator->current()->setValue("Object");
        $cellIterator->next();
        $cellIterator->current()->setValue("Per");
		$cellIterator->next();
		$cellIterator->current()->setValue("Assignment");
        $cellIterator->next();
		$cellIterator->current()->setValue("Actuals");
        $cellIterator->next();
		$cellIterator->current()->setValue("Actuals_Assigned");
		$cellIterator->next();
		$cellIterator->current()->setValue("Workable Hours");
        $cellIterator->next();
		$cellIterator->current()->setValue("Unit");
        $cellIterator->next();
		$cellIterator->current()->setValue("Task type");
        $cellIterator->next();
		$cellIterator->current()->setValue('Person Plant');
		$cellIterator->next();
		$cellIterator->current()->setValue('Project Plant');
		$cellIterator->next();
		$cellIterator->current()->setValue("Generation time");
		$row = $cellIterator->current()->getRow();
		$col = $cellIterator->current()->getColumn();
		$sheet1->setCellValue($col . ($row + 1), date('m-d-Y H:i:s'));
        $rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		//Data generation and export
		foreach($this->sbe as $sbe)
		{
			$allPersons = $this->context->personRepository->findAllByOrgUnitPath($sbe->Path);
			foreach($allPersons as $person)
			{
				$orgUnit = $this->cachedOrgUnits[$person->OrganizationUnitID];
				
				// get Department data
				$dep = null;
							$depObj = null;
				$depShort = null;
				if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE)
				{
					$dep = 'none';
					$depShort = 'none';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP)
				{
					$dep = $orgUnit->Name;
					// Short
					if($orgUnit->Short)
						$depShort = $orgUnit->Short;
					else
						$depShort = 'Not defined';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				{
					$depObj = $this->cachedOrgUnits[$orgUnit->Parent_ID];
					$dep = $depObj->Name;
					//Short
					if($depObj->Short)
						$depShort = $depObj->Short;
					else
						$depShort = 'Not defined';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
				{
					$depObj = $this->cachedOrgUnits[$this->cachedOrgUnits[$orgUnit->Parent_ID]->Parent_ID];
					$dep = $depObj->Name;
					//Short
					if($depObj->Short)
						$depShort = $depObj->Short;
					else
						$depShort = 'Not defined';
				}
				// get Department data END

				// get Cluster data
				$clus = null;
				$clusObj = null;
				$clusShort = null;
				if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE || $orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP)
				{
					$clus = 'none';
					$clusShort = 'none';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				{
					$clus = $orgUnit->Name;
					// Short
					if($orgUnit->Short)
						$clusShort = $orgUnit->Short;
					else
						$clusShort = 'Not defined';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
				{
					$clusObj = $this->cachedOrgUnits[$orgUnit->Parent_ID];
					$clus = $clusObj->Name;
					// Short
					if($clusObj->Short)
						$clusShort = $clusObj->Short;
					else
						$clusShort = 'Not defined';
				}
				// get Cluster data END

				// Technical Group
				$tg = null;
				if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE || $orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP || $orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				{
					$tg = 'none';
				}
				elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
				{
					$tg = $orgUnit->Name;
				}
				// Technical Group END
				if(!$this->context->workableHourRepository->isAOP($this->year, $person->ID))
					$yield = $this->context->workableHourRepository->getYield($person->ID, $this->year)*100;
				else
					$yield = 'null';

				if(!$this->context->workableHourRepository->isAOP($this->year, $person->ID))
					$contract = $this->context->workableHourRepository->getContract($person->ID, $this->year)*100; 
				else
					$contract = 'null';
				//Workable hours
				$workableHours = $this->context->aopRepository->findForPerson($this->year, $person->ID);
				$projects = array();
				$projectName = '';
				$projectSAP_ID = '';
				$projects = $this->context->projectRepository->findPersonAssignedAtProjectsInYear($person->ID, $this->year); 
				// lest find projects where persons are not assigned
				$notAssignedProjects = $this->context->projectRepository->findPersonNotAssignedAtProjectsInYear($person->ID, $this->year);
				foreach($notAssignedProjects as $project)
				{
					if(!isset($projects[$project->ID]))
						$projects[$project->ID] = $project;  
				}

				foreach ($projects as $project)
				{
					$projectName = $project->Name;
					$projectSAP_ID = $project->SAP_ID;
					$tasks = $this->context->taskRepository->findPersonTasksInYear($person->ID, $project->ID, $this->year);
					$activeTasks = $this->context->taskRepository->findPersonActiveTasksInYear($person->ID, $project->ID, $this->year);
					foreach($activeTasks as $task)
					{
						if(!isset($tasks[$task->ID]))
						{
							$tasks[$task->ID] = $task;
						}
					}
					$taskName = '';
					$taskSAP_ID = '';
					$assigned = null;
					$actual = null;
					foreach($tasks as $task)
					{
						$taskName = $task->Name;
						$taskSAP_ID = $task->SAPID;
						$assigned = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, $this->year, $person->ID);
						$actual = $this->context->workRepository->findPersonTaskYearWork($person->ID, $task->ID, $this->year);
						for($i = 1; $i <= 12; $i++)
						{
							$cellIterator->current()->setValue($sbe->Name);
							$cellIterator->next();
							$cellIterator->current()->setValue($dep);
							$cellIterator->next();
							$cellIterator->current()->setValue($depShort);
							$cellIterator->next();
							$cellIterator->current()->setValue($clus);
							$cellIterator->next();
							$cellIterator->current()->setValue($clusShort);
							$cellIterator->next();
							$cellIterator->current()->setValue($tg);
							$cellIterator->next();
							$cellIterator->current()->setValue($person->ID);
							$cellIterator->next();
							$cellIterator->current()->setValue($person->Surname);
							$cellIterator->next();
							$cellIterator->current()->setValue($person->Firstname);
							$cellIterator->next();
							$cellIterator->current()->setValue($person->CostCenterID);
							$cellIterator->next();
							$cellIterator->current()->setValue($person->Comment);
							$cellIterator->next();
							if($person->Status)
								$cellIterator->current()->setValue('Active');
							else
								$cellIterator->current()->setValue('Inactive');
							$cellIterator->next();
							$cellIterator->current()->setValue($yield);
							$cellIterator->next();
							$cellIterator->current()->setValue($contract);
							$cellIterator->next();
							$cellIterator->current()->setValue($projectSAP_ID);
							$cellIterator->next();
							$cellIterator->current()->setValue($projectName);
							$cellIterator->next();
							$cellIterator->current()->setValue($taskSAP_ID);
							$cellIterator->next();
							$cellIterator->current()->setValue($taskName);
							$cellIterator->next();
							$cellIterator->current()->setValue($i);
							$cellIterator->next();
							$cellIterator->current()->setValue($assigned[$i]->Hours);
							$cellIterator->next();
							$cellIterator->current()->setValue($actual[$i]);
							$cellIterator->next();
							if($this->year == Date('Y'))
							{
								if($i >= $actualMonth)
									$cellIterator->current()->setValue($assigned[$i]->Hours);
								else
									$cellIterator->current()->setValue($actual[$i]);
							}
							elseif($this->year > Date('Y'))
							{
								$cellIterator->current()->setValue($assigned[$i]->Hours);
							}
							else
							{
								$cellIterator->current()->setValue($actual[$i]);
							}
							$cellIterator->next();
							if ($workableHours != NULL)
							{
								$cellIterator->current()->setValue($workableHours->getMonthAOPHours($i));
							}
							else
							{			
								$cellIterator->current()->setValue(0);
							}
							$cellIterator->next();
							$cellIterator->current()->setValue("hrs");
							$cellIterator->next();
							if($task->Fake == Task::TASK_TYPE_SAP)
								$cellIterator->current()->setValue('SAP');
							elseif($task->Fake == Task::TASK_TYPE_PLACEHOLDER)
								$cellIterator->current()->setValue('Placeholder');
							else
								$cellIterator->current()->setValue('Planning');
							$cellIterator->next();
							if($this->cachedLabourCenters[$person->costCenterID])
								$cellIterator->current()->setValue($this->cachedLabourCenters[$person->costCenterID]->Plant);
							$cellIterator->next();
							$cellIterator->current()->setValue($project->Plant);
							$cellIterator->next();
							$rowIterator->next();
							$cellIterator = $rowIterator->current()->getCellIterator();
						}
					}
				}
			}
		}
		// Labour Rates sheet
		$sheet2 = $objPHPExcel->createSheet(2);
		$sheet2->setTitle('Labour Rates');
		$rowIterator = $sheet2->getRowIterator();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Header
		$cellIterator->current()->setValue("Work Center");
		$cellIterator->next();
		$cellIterator->current()->setValue("Work Center ID");
		$cellIterator->next();
		$cellIterator->current()->setValue("Year");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jan");
		$cellIterator->next();
		$cellIterator->current()->setValue("Feb");
		$cellIterator->next();
		$cellIterator->current()->setValue("Mar");
		$cellIterator->next();
		$cellIterator->current()->setValue("Apr");
		$cellIterator->next();
		$cellIterator->current()->setValue("May");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jun");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jul");
		$cellIterator->next();
		$cellIterator->current()->setValue("Aug");
		$cellIterator->next();
		$cellIterator->current()->setValue("Sep");
		$cellIterator->next();
		$cellIterator->current()->setValue("Oct");
		$cellIterator->next();
		$cellIterator->current()->setValue("Nov");
		$cellIterator->next();
		$cellIterator->current()->setValue("Dec");
		$cellIterator->next();
		$cellIterator->current()->setValue("Unit");
		$cellIterator->next();
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();

		foreach ($this->cachedLabourCenters as $center)
		{
			$cellIterator->current()->setValue($center->Name);
			$cellIterator->next();
			$cellIterator->current()->setValue($center->ID);
			$cellIterator->next();
			$rate = $center->getYearLabourRate($this->year);
			if ($rate != NULL)
			{
				$cellIterator->current()->setValue($this->year);
				$cellIterator->next();			
				for ($i = 1; $i <= 12; $i++)
				{
					$cellIterator->current()->setValue($rate->getMonthlyHourRate($i));
					$cellIterator->next();	
				}
			}
			else
			{
				$cellIterator->current()->setValue($this->year);
				$cellIterator->next();			
				for ($i = 1; $i <= 12; $i++)
				{
					$cellIterator->current()->setValue(0);
					$cellIterator->next();	
				}
			}
			$cellIterator->current()->setValue("USD/hrs");
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
		}
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$this->logger->log('Person Report generation end: ' . date('m-d-Y H:i:s'), ILogger::INFO);

		if(!is_null($this->uiFreezeToken))
			setcookie('fileDownloadToken', $this->uiFreezeToken);

        $objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
        if (!file_exists($this->person_report_storage_dir)) 
          mkdir($this->person_report_storage_dir, 0777, TRUE);
		
		if(count($this->sbe) > 1)
		{
			$fileName = 'AEE';
		}
		else
		{
			$fileName = $this->sbe[0]->Short;
		}
		
		if($this->year == date('Y'))
			$objWriter->save($this->person_report_storage_dir . $fileName . '-PersonReport.xlsx');
		else
			$objWriter->save($this->person_report_storage_dir . $fileName . '-PersonReport_' . $this->year . '.xlsx');
		
		$this->logger->log('Person Report generation end: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		
		$lock->unlock();
    }
}

class EPersonReportServiceException extends Exception {}