<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for tracking table export.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class TTReportService extends object
{
    /** @var DIContainer DI context with application repositories access */
    private $context;

	/** @var ILogger Logging facility */
    private $logger;
	
	/** @var int Year to generation */
    private $year;
	
	/** @var OrganizaitonUnit SBE Organizaiton unit object */
    private $sbe;
	
	/** @var String Value for freeze user interface during report generation. */
    private $uiFreezeToken;
	
    /** @var string Path to storage TT files. */
    private $tt_storage_dir;
	
	/** @var array Persons */
	private $cachedPersons;
	
	/** @var array Organization Units */
	private $cachedOrgUnits;
	
	/** @const Possible results of report */
	const RESULT_SUCCESS = 0,
		  RESULT_WARNING = 1,
		  RESULT_ERROR = 2,
		  RESULT_NO_DATA = 3;
	
	/**
	 * Creates service for tracking table export.
	 *  
	 * @param $context Application context.
	 * @param ILogger Object for logging the import progress.
	 */
    public function  __construct(DIContainer $context, ILogger $logger, $year, $sbe, $uiFreezeToken = null)
    {
        $this->context = $context;
		$this->logger = $logger;
		$this->year = $year;
		$this->sbe = $sbe;
		$this->uiFreezeToken = $uiFreezeToken;
		$this->tt_storage_dir = Environment::getConfig('trackingTable_storage_dir');
		$this->cachedPersons = $this->context->personRepository->findAll();
		$this->cachedOrgUnits = $this->context->organizationUnitRepository->findAll(true);
    }
	
    /**
	 * 
     * Generates Tracking table as a XLSX (MS-EXCEL file).
	 * File can be downloaded through browser and also file is saved to the public/TrackingTable folder.
	 * 
	 * @return int Result of operation in range of TTReportService::RESULT_*
     */
    public function runTrackingTableGeneration()
    {
		$objPHPExcel = new PHPExcel();
		
        $memory_limit = "2048M";
        if (!ini_set("memory_limit", $memory_limit)) 
		{
			$this->flashMessage('Setup of memory limit failed. Contact administrator.', 'error');
			trigger_error('Unable to setup memory limit to ' . $memory_limit . '.', E_USER_ERROR);
		}

        $exec_time = 1800;
	    if (!ini_set("max_execution_time", $exec_time)) 
		{
			$this->flashMessage('Setup of maximum execution time failed. Contact administrator.', 'error');
			trigger_error('Unable to setup maximum execution time to '.$exec_time.'.', E_USER_ERROR);
		}
		
		$lock = $this->context->createLock('TT_Report', $exec_time);
		if (!$lock->lock())
		{
			$this->logger->log('TT generation start failed - active lock detected: ' . date('m-d-Y H:i:s'), ILogger::INFO);
			throw new ETTReportServiceException("Export of tracking table is already running.");
		}
		$this->logger->log('TT generation start: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		
		$allProjects = $this->context->projectRepository->findAllSBEProjectsInYear($this->year, $this->sbe->ID, true);
		if(empty($allProjects))
		{
			$lock->unlock();
			$this->logger->log('TT has no data for SBE: ' . $this->sbe->Name . ' and year: ' . $this->year . 'to export', ILogger::WARNING);
			return self::RESULT_NO_DATA;
		}
        $labour_centers = $this->context->costCenterRepository->findAllCostCenters();
		$error = false;
        //$labourTasksPrefix = $this->context->costCenterRepository->findAllCostCentersPrefix();

        $sheet = $objPHPExcel->getActiveSheet();        
        $rowIterator = $sheet->getRowIterator();
        $cellIterator = $rowIterator->current()->getCellIterator();        
        $currency = "USD";
        // Header     
        $cellIterator->current()->setValue("Project Def.");
        $cellIterator->next();
        $cellIterator->current()->setValue("Project Name");
        $cellIterator->next();
        $cellIterator->current()->setValue("Per");
        $cellIterator->next();
        $cellIterator->current()->setValue("CO object");
        $cellIterator->next();
        $cellIterator->current()->setValue("Object");
        $cellIterator->next();
        $cellIterator->current()->setValue("Hrs plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Hrs actuals/EAC");
        $cellIterator->next();
        $cellIterator->current()->setValue("USD plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("USD actual/EAC");
        $cellIterator->next();
        $cellIterator->current()->setValue("Labor Rate");
        $cellIterator->next();
        $cellIterator->current()->setValue("Category");
        $cellIterator->next();
        $cellIterator->current()->setValue("Budget");
        $cellIterator->next();
        $cellIterator->current()->setValue("HON PO value");
        $cellIterator->next();
        $cellIterator->current()->setValue("State");        
        $cellIterator->next();
        $cellIterator->current()->setValue("CL");
        $cellIterator->next();
        $cellIterator->current()->setValue("Cluster");
        $cellIterator->next();
        $cellIterator->current()->setValue("PI");
        $cellIterator->next();
        $cellIterator->current()->setValue("PM");
        $cellIterator->next();
        $cellIterator->current()->setValue("PP&C");
        $cellIterator->next();
        $cellIterator->current()->setValue("Project type");
        $cellIterator->next();
        $cellIterator->current()->setValue("Project Phase");
        $cellIterator->next();
        $cellIterator->current()->setValue("External Customer");
        $cellIterator->next();
        $cellIterator->current()->setValue("Part Number");
        $cellIterator->next();
        $cellIterator->current()->setValue("Co-Funding");
        $cellIterator->next();
        $cellIterator->current()->setValue("Co-Funding Plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("HON financing");
        $cellIterator->next();
        $cellIterator->current()->setValue("HON financing Plan");
        $cellIterator->next();
        $cellIterator->current()->setValue("Percentage from Current Year Budget");
        $cellIterator->next();
        $cellIterator->current()->setValue("Delta comulative USD");
        $cellIterator->next();
        $cellIterator->current()->setValue("Delta last month USD");
        $cellIterator->next();
        $cellIterator->current()->setValue("RTC");
        $cellIterator->next();
        $cellIterator->current()->setValue("PMO sup. lvl");
        $cellIterator->next();
		$cellIterator->current()->setValue("Placeholder project");
		$cellIterator->next();
		$cellIterator->current()->setValue("CL EID");
		$cellIterator->next();
		$cellIterator->current()->setValue("PI EID");
		$cellIterator->next();
		$cellIterator->current()->setValue("PM EID");
		$cellIterator->next();
		$cellIterator->current()->setValue("PP&C EID");
		$cellIterator->next();
		$cellIterator->current()->setValue("License");
        $cellIterator->next();
		$cellIterator->current()->setValue("Labor type");
		$cellIterator->next();
		$cellIterator->current()->setValue("Task Type");
		$cellIterator->next();
		$cellIterator->current()->setValue("HRS Actuals");
		$cellIterator->next();
		$cellIterator->current()->setValue("USD Actuals");
		$cellIterator->next();
		$cellIterator->current()->setValue("HON Financing Actuals");
		$cellIterator->next();
		$cellIterator->current()->setValue("Part Number SAP");
		$cellIterator->next();
		$cellIterator->current()->setValue("Key code");
		$cellIterator->next();
		$cellIterator->current()->setValue("Work center");
		$cellIterator->next();
		$cellIterator->current()->setValue("Program");
        $cellIterator->next();
		$cellIterator->current()->setValue("Call");
        $cellIterator->next();
		$cellIterator->current()->setValue("WC Plant code");
        $cellIterator->next();
		$cellIterator->current()->setValue("Project Plant code");
        $cellIterator->next();
		$cellIterator->current()->setValue("Task Remark");
        $cellIterator->next();
		$cellIterator->current()->setValue("Generation time");
		$row = $cellIterator->current()->getRow();
		$col = $cellIterator->current()->getColumn();
		$sheet->setCellValue($col . ($row + 1), date('m-d-Y H:i:s'));
        $rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
        
		foreach ($allProjects as $project)
        {
			$tasks = array();
			$allTasks = $this->context->taskRepository->findAllProjectTasks($project->ID);
			//check every task if he has some data to report.
			foreach($allTasks as $task)
			{
				//has some plan
				if($this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year))
				{
					$tasks[$task->ID] = $task;
					continue;
				}
				//has some actuals
				// This approche is way faster then taskRepository->findActiveProjectTasksInYear function
				if($task->isLabourTask())
				{
					if($this->context->workRepository->findPeopleWorkingOnTaskInYear($task->ID, $this->year))
					{
						$tasks[$task->ID] = $task;
					}
				}
				else
				{
					if($this->context->costCenterRepository->findActualCostsYearSummary($task->ID, $this->year, 'USD')->sum() != 0)
					{
						$tasks[$task->ID] = $task;
					}
				}
			}
            
            $reporter = new ProjectEACReporter($this->context, $project, $this->year);
            $sumPlanFirstRow = $cellIterator->current()->getRow();          
            $sumPlanLastRow = count($tasks)*12 + $sumPlanFirstRow - 1;            
            $count = count($tasks);           
            $x = 0;
            $sum = 0;
            $deltaPlanActual = array();
            
			if(!array_key_exists($project->PIPM, $this->cachedPersons))
				$pipm = 'UNKNOWN WORKER ' . $project->PIPM;
			else
			{
				$pipmPerson = $this->cachedPersons[$project->PIPM];
				$pipm = $pipmPerson->surname. ', ' . $pipmPerson->firstname;
			}
           
            if(!array_key_exists($project->TM, $this->cachedPersons))
                $tm = 'UNKNOWN WORKER ' . $project->TM;
            else
			{
				$tmPerson = $this->cachedPersons[$project->TM];
                $tm = $tmPerson->surname. ', ' . $tmPerson->firstname;
			}
            
            $clus = "";
            if($tmPerson == null)
                $clus = 'UNKNOWN CLUSTER';
            else
            {
                $length = $this->cachedOrgUnits[$tmPerson->OrganizationUnitID]->Path;
				// TM from departmner
                if(strlen($length) == 6)
                {
                    $orgUnits = $this->context->organizationUnitRepository->findAllChildren($tmPerson->OrganizationUnitID);
                    foreach ($orgUnits as $orgUnit)
                        if ($orgUnit->leader == $project->TM)
                            $clus = $orgUnit->name;
                    if($clus == "")
                        $clus = $this->cachedOrgUnits[$tmPerson->OrganizationUnitID]->name;
                }
				// TM from technical group
				elseif(strlen($length) == 12)
				{
					$clusID = $this->cachedOrgUnits[$tmPerson->OrganizationUnitID]->Parent_ID;
					$clus = $this->cachedOrgUnits[$clusID]->name;
                    if($clus == "")
                        $clus = $this->cachedOrgUnits[$tmPerson->OrganizationUnitID]->name;
				}
                else
					$clus = $this->cachedOrgUnits[$tmPerson->OrganizationUnitID]->name;
            }
            
            if(!array_key_exists($project->PMOE, $this->cachedPersons))
                $pmoe = 'UNKNOWN WORKER ' . $project->PMOE;
            else
			{
				$pmoePerson = $this->cachedPersons[$project->PMOE];
				$pmoe = $pmoePerson->surname. ', ' . $pmoePerson->firstname;
			}
           
            if(!array_key_exists($project->PPaC, $this->cachedPersons))
                $ppac = 'UNKNOWN WORKER ' . $project->PPaC;
            else
			{
				$ppacPerson = $this->cachedPersons[$project->PPaC];
                $ppac = $ppacPerson->surname. ', ' . $ppacPerson->firstname;
			}
            
            if($project->Customer == null)
                $customer = 'UNKNOWN CUSTOMER ' . $project->Customer;
            else
                $customer = $this->context->customerRepository->find($project->Customer)->Name;
			
			if($project->ProgramID == null)
				$program = 'UNKNOWN PROGRAM';
			else
				$program = $this->context->customerProgramRepository->find($project->ProgramID)->Program;
			
			if($project->CallID == null)
				$call = 'UNKNOWN CALL';
			else
				$call = $this->context->customerProgramCallRepository->find($project->CallID)->Call;
			
			if($project->responsibility == null)
				$responsibility = 'UNKNOWN RESPONSIBILITY';
			else
				$responsibility = $this->context->responsibilityRepository->find($project->responsibility)->name;
			
			if($project->ECLicenceID)
				$ecLicence = $this->context->ecLicenceRepository->find($project->ECLicenceID)->Name;
			else
				$ecLicence = 'UNKNOWN LICENCE';
			
			$budget = $this->context->poAttributesAndBudgetRepository->budgetByProjectIDAndYear($project->ID, $this->year);
			if($budget != null)
				$setBudget = $budget;
			else
				$setBudget = 0;

			$POValue = $this->context->poAttributesAndBudgetRepository->poValueByProjectIDAndYear($project->ID, $this->year);
			if($POValue != null)
				$setPOValue = $POValue;
			else
				$setPOValue = 0;
			
            foreach ($tasks as $task)
            {
				$separators = array('.', '-', '_', ' ');
				$taskPrefix = '';
				if($task->isLabourTask())
				{
					if(in_array($task->name[3], $separators))
					{
						$taskPrefix = substr($task->name, 0, 3);
					}
					elseif(in_array($task->name[4], $separators))
					{
						$taskPrefix = substr($task->name, 0, 4);
					}
				}
				if($taskPrefix == 'BRN')
					$labourType = 'ATE';
				else
					$labourType = $taskPrefix;
				
				$category = '';
				if($task->isLabourTask() && in_array($labour_centers[$task->costCenterID]->Plant, array('3330', '3320', '3104')))
				{
					$category = "LABOR";
				}
				elseif(substr($task->name, 0, 3) == "MAT")
					$category = "MATERIAL";
				elseif(substr($task->name, 0, 3) == "TRV")
					$category = "TRAVEL";
				elseif(substr($task->name, 0, 3) == "RES")
					$category = "RESERVE";
				else
					$category = "SUBCONTRACTING";
				
                $task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year);  
                if($task_plan != NULL)                
                    $task_plan_id = $task_plan->taskPlanID;                
                else
                    $task_plan_id = NULL;
                if ($task_plan === NULL) // task is not planned but has some work/costs for this year
                    $task_plan = new TaskYearPlan($this->year);
                
                $labour_EAC = $nonLabour_EAC = array();  
                $rate = null;
				
                if($task->isLabourTask())
                {
					if(is_null($task->costCenterID))
					{
						$this->logger->log('Project: ' . $project->Name . ', labour task: ' . $task->Name . ' has not defined work center!', ILogger::ERROR);
						$rate = new CostCenterLabourRates();
						$rate->Year = $this->year;
						$error = true;
//						throw new InvalidStateException('Project: ' . $project->Name . ', labour task: ' . $task->Name . ' has not defined work center!');
//						$lock->unlock();
					}
					else
					{
						$rate = $labour_centers[$task->costCenterID]->getYearLabourRate($this->year);
					}
					if(is_null($rate))
					{
						$error = true;
						$this->logger->log('Labour rates for Work Center with id: ' . $task->costCenterID . ' and year: ' . $this->year . ' doesnt exist!', ILogger::ERROR);
						$rate = new CostCenterLabourRates();
						$rate->Year = $this->year;
//						$lock->unlock();
//						throw new InvalidStateException('Labour rates for Work Center with id: ' . $task->costCenterID . ' and year: ' . $this->year . ' doesnt exist!');
					}
                    $labour_EAC = $reporter->getTaskLabourEACHoursMonthTotals($task);
                }
                else
                {
                    $nonLabour_EAC = $reporter->getTaskNonLabourEACCostsMonthTotals($task, $currency);
                }
                
                for ($i = 1; $i <= 12; $i++)
                {
                    if(!$task->isLabourTask())
                    {
                        $deltaPlanActual[$i][] = $task_plan->getMonthPlan($i)->Budget - $nonLabour_EAC[$i];
						$costActuals[$i] = $this->context->costCenterRepository->findActualTaskCostsMonthSummary($task->ID, $i, $this->year, $currency);
                    }
                    else
                    {
						$labourActuals = $this->context->workRepository->findTaskYearWork($task->ID, $this->year);
                        $deltaPlanActual[$i][] = ($task_plan->getMonthPlan($i)->Hours - $labour_EAC[$i])*$rate->getMonthlyHourRate($i);
                    }
                }
                for ($i = 1; $i <= 12; $i++)
                {
                    $row = $cellIterator->current()->getRow();
                    $cellIterator->current()->setValue($project->SAPID);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($project->Name);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($i);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($task->name);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($task->SAPID);
                    $cellIterator->next();
                    if (!$task->isLabourTask())
                    {
                        $cellIterator->current()->setValue(0);
                        $cellIterator->next();
                        $cellIterator->current()->setValue(0);
                        $cellIterator->next();
                        $cellIterator->current()->setValue($task_plan->getMonthPlan($i)->Budget);
                        $cellIterator->next();
                        $cellIterator->current()->setValue($nonLabour_EAC[$i]);
                        $cellIterator->next();
                    }
                    else
                    {
                        $cellIterator->current()->setValue($task_plan->getMonthPlan($i)->Hours);
                        $cellIterator->next();
                        $cellIterator->current()->setValue($labour_EAC[$i]);
                        $cellIterator->next();
                        $cellIterator->current()->setValue("=F" . $row . '*J' . $row);
                        $cellIterator->next();
                        $cellIterator->current()->setValue("=G" . $row . '*J' . $row);
                        $cellIterator->next();
                    }  
                    if ($rate != NULL)
                        $cellIterator->current()->setValue($rate->getMonthlyHourRate($i));
                    else
                        $cellIterator->current()->setValue(0);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($category);
                    $cellIterator->next();
					$cellIterator->current()->setValue($setBudget);
					$cellIterator->next();
					$cellIterator->current()->setValue($setPOValue);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($project->State);//$cellIterator->current()->setValue($project->State + 1);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($tm);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($clus);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($pipm);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($pmoe);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($ppac);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($project->Type);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($project->Phase);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($customer);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($task->costShare);
                    $cellIterator->next();                    
                    $cellIterator->current()->setValue("=I" . $row . '*W' . $row);
                    $cellIterator->next();
                    $cellIterator->current()->setValue("=H" . $row . '*W' . $row);
                    $cellIterator->next();
                    $cellIterator->current()->setValue("=(1-W" . $row . ')*I' . $row);
                    $cellIterator->next();
                    $cellIterator->current()->setValue("=(1-W" . $row . ')*H' . $row);
                    $cellIterator->next();
                       
                    //Percentage from Current Year Budget
                    if ($x == $count - 1) // last iteration
                    {                                                
                        foreach ($deltaPlanActual[$i] as $value)
                        {
                            $sum = $sum + $value;
                        }
                        $cellIterator->current()->setValue("=IFERROR(" . $sum . "/SUM(H" . $sumPlanFirstRow . ":H" . $sumPlanLastRow . "),0)");
                    }
                    else
                        $cellIterator->current()->setValue(0);
                    $cellIterator->next();
                    
                    // Comulative delta USD
                    switch ($i) 
                    {
                        case 1:
                            $cellIterator->current()->setValue("=AD" . $row);
                            break;
                        case 2:
                            $cellIterator->current()->setValue("=AD" . ($row - 1) . "+AD" . $row);
                            break;
                        case 3:
                            $cellIterator->current()->setValue("=AD" . ($row - 2) . "+AD" .($row - 1) . "+AD" . $row);
                            break;
                        case 4:
                            $cellIterator->current()->setValue("=AD" . ($row - 3) . "+AD" . ($row - 2) . "+AD" .($row - 1) . "+AD" . $row);
                            break;
                        case 5:
                            $cellIterator->current()->setValue("=AD" . ($row - 4) . "+AD" . ($row - 3) . "+AD" . ($row - 2) . "+AD" .($row - 1) . "+AD" . $row);
                            break;
                        case 6:
                            $cellIterator->current()->setValue("=AD" . ($row - 5) . "+AD" . ($row - 4) . "+AD" . ($row - 3) . "+AD" . ($row - 2) . "+AD" .($row - 1) . "+AD" . $row);
                            break;
                        case 7:
                            $cellIterator->current()->setValue("=AD" . ($row - 6) . "+AD" . ($row - 5) . "+AD" . ($row - 4) . "+AD" . ($row - 3) . "+AD" . ($row - 2) . "+AD" .($row - 1) . "+AD" . $row);
                            break;
                        case 8:
                            $cellIterator->current()->setValue("=AD" . ($row - 7) . "+AD" . ($row - 6) . "+AD" . ($row - 5) . "+AD" . ($row - 4) . "+AD" . ($row - 3) . "+AD" . ($row - 2) . "+AD" .($row - 1) . "+AD" . $row);
                            break;
                        case 9:
                            $cellIterator->current()->setValue("=AD" . ($row - 8) . "+AD" . ($row - 7) . "+AD" . ($row - 6) . "+AD" . ($row - 5) . "+AD" . ($row - 4) . "+AD" . ($row - 3) . "+AD" . ($row - 2) . "+AD" .($row - 1) . "+AD" . $row);
                            break;
                        case 10:
                            $cellIterator->current()->setValue("=AD" . ($row - 9) . "+AD" . ($row - 8) . "+AD" . ($row - 7) . "+AD" . ($row - 6) . "+AD" . ($row - 5) . "+AD" . ($row - 4) . "+AD" . ($row - 3) . "+AD" . ($row - 2) . "+AD" .($row - 1) . "+AD" . $row);
                            break;
                        case 11:
                            $cellIterator->current()->setValue("=AD" . ($row - 10) . "+AD" . ($row - 9) . "+AD" . ($row - 8) . "+AD" . ($row - 7) . "+AD" . ($row - 6) . "+AD" . ($row - 5) . "+AD" . ($row - 4) . "+AD" . ($row - 3) . "+AD" . ($row - 2) . "+AD" .($row - 1) . "+AD" . $row);
                            break;
                        case 12:
                            $cellIterator->current()->setValue("=AD" . ($row - 11) . "+AD" . ($row - 10) . "+AD" . ($row - 9) . "+AD" . ($row - 8) . "+AD" . ($row - 7) . "+AD" . ($row - 6) . "+AD" . ($row - 5) . "+AD" . ($row - 4) . "+AD" . ($row - 3) . "+AD" . ($row - 2) . "+AD" .($row - 1) . "+AD" . $row);
                            break;
                    }
                    $cellIterator->next();
                    $cellIterator->current()->setValue("=I" . $row . '-H' . $row);
                    $cellIterator->next();
                    $cellIterator->current()->setValue($responsibility);
                    $cellIterator->next();
                    if(is_null($project->PMOSupport))
                        $pmoSupport = 'UNKNOWN PMO SUP. LVL';
                    else
                        $pmoSupport = $project->PMOSupport;
					$cellIterator->current()->setValue($pmoSupport);
					$cellIterator->next();
					if($project->Fake)
						$cellIterator->current()->setValue('Yes');
					else
						$cellIterator->current()->setValue('No');
                    $cellIterator->next();
					if(isset($tmPerson))
						$cellIterator->current()->setValue($tmPerson->ID);
					else
						$cellIterator->current()->setValue($tm);
                    $cellIterator->next();
					if(isset($pipmPerson))
						$cellIterator->current()->setValue($pipmPerson->ID);
					else
						$cellIterator->current()->setValue($pipm);
                    $cellIterator->next();
					if(isset($pmoePerson))
						$cellIterator->current()->setValue($pmoePerson->ID);
					else
						$cellIterator->current()->setValue($pmoe);
                    $cellIterator->next();
					if(isset($ppacPerson))
						$cellIterator->current()->setValue($ppacPerson->ID);
					else
						$cellIterator->current()->setValue($ppac);
                    $cellIterator->next();
					$cellIterator->current()->setValue($ecLicence);
                    $cellIterator->next();
					if($task->isLabourTask())
						$cellIterator->current()->setValue($labourType);
					$cellIterator->next();
					if($task->Fake == Task::TASK_TYPE_SAP)
						$cellIterator->current()->setValue('SAP');
					elseif($task->Fake == Task::TASK_TYPE_PLACEHOLDER)
						$cellIterator->current()->setValue('Placeholder');
					else
						$cellIterator->current()->setValue('Planning');
					$cellIterator->next();
					if($task->isLabourTask())
					{
						$cellIterator->current()->setValue($labourActuals[$i]);
						$cellIterator->next();
						$cellIterator->current()->setValue("=AO". $row . "*J" . $row);
					}
					else
					{
						$cellIterator->current()->setValue(0);
						$cellIterator->next();
						$cellIterator->current()->setValue($costActuals[$i]);
					}
					$cellIterator->next();
					$cellIterator->current()->setValue("=(1-W" . $row . ')*AP' . $row);
					$cellIterator->next();
					$cellIterator->current()->setValue($task->SAPCostShare);
					$cellIterator->next();
					$cellIterator->current()->setValue($project->keyCode);
					$cellIterator->next();
					$cellIterator->current()->setValue($task->costCenterID);
					$cellIterator->next();
					$cellIterator->current()->setValue($program);
					$cellIterator->next();
					$cellIterator->current()->setValue($call);
					$cellIterator->next();
					if($task->isLabourTask())
					{
						$wc = $labour_centers[$task->costCenterID];
						if($wc)
							$cellIterator->current()->setValue($wc->Plant);
					}
					$cellIterator->next();
					$cellIterator->current()->setValue($project->Plant);
					$cellIterator->next();
					$cellIterator->current()->setValue($task->Remark);
					$cellIterator->next();
					$rowIterator->next();
                    $cellIterator = $rowIterator->current()->getCellIterator();  
                }
                $x++;
            }
        }
		
		if(!is_null($this->uiFreezeToken))
			setcookie('fileDownloadToken', $this->uiFreezeToken);

        $objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
		$objWriter->setPreCalculateFormulas(true);
        if (!file_exists($this->tt_storage_dir)) 
          mkdir($this->tt_storage_dir, 0777, TRUE);

		if(is_null($this->uiFreezeToken))
		{
			if(date('Y-m-d') == FiscalCalendar::currentSecondWednesday())
				$objWriter->save($this->tt_storage_dir . $this->sbe->Short . '-TrackingTable-Frozen.xlsx');
		}
		if($this->year == date('Y'))
			$objWriter->save($this->tt_storage_dir . $this->sbe->Short . '-TrackingTable.xlsx');
		else
			$objWriter->save($this->tt_storage_dir . $this->sbe->Short . '-TrackingTable_' . $this->year . '.xlsx');
		
		$this->logger->log('TT generation end: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		$lock->unlock();
		if($error)
			return self::RESULT_ERROR;
		else
			return self::RESULT_SUCCESS;
    }
}

class ETTReportServiceException extends Exception {}

