<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of STRAP ItemActivity.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ItemActivityRepository extends Object
{
	/** @var IItemActivityMapper */
    private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IItemActivityMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'Activity-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "STRAP Activity"));
    }
    
	/**
	 * Saves new STRAP item activity.
	 * 
	 * @param ItemActivity $item
	 * @return boolean
	 */
	public function save(ItemActivity $item)
	{
		if($this->mapper->save($item))
		{
			$this->multilog->log('Strap Activity with ID: ' . $item->ID . ' and Name: ' . $item->Name . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Saves linkage between Strap ItemActivity and Pursuit
	 * 
	 * @param int $activityID
	 * @param int $pursuitID
	 * @return boolean
	 */
	public function	savePursuitLinkage($activityID, $pursuitID)
	{
		return $this->mapper->savePursuitLinkage($activityID, $pursuitID);
	}
	
	/**
	 * Deletes linkage between Strap ItemActivity and Pursuit
	 * 
	 * @param int $activityID
	 * @param int $pursuitID
	 * @return boolean
	 */
	public function	deletePursuitLinkage($activityID, $pursuitID)
	{
		return $this->mapper->deletePursuitLinkage($activityID, $pursuitID);
	}
	
	/**
	 * Updates existing STRAP item activity.
	 * 
	 * @param ItemActivity $item
	 * @return boolean
	 */
	public function update(ItemActivity $item)
	{
		$oldObject = $this->find($item->ID);
		if($this->mapper->update($item))
		{
			$this->multilog->log('Strap Activity with ID: ' . $item->ID . ' and Name: ' . $item->Name . ' updated. ' . $this->multilog->differences($oldObject, $item), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing STRAP ItemActivity.
	 * 
	 * @param int $item
	 * @return boolean
	 */
	public function delete($item_id)
	{
		if($this->mapper->delete($item_id))
		{
			$this->multilog->log('Strap Activity with ID: ' . $item_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Gets STRAP ItemActivity by id.
	 * 
	 * @param int $item_id
	 * @return ItemActivity or null if not found
	 */
	public function find($item_id)
	{
		return $this->mapper->find($item_id);
	}
	
	/**
	 * Gets STRAP ItemActivity by custom id.
	 * 
	 * @param int $item_id
	 * @return ItemActivity or null if not found
	 */
	public function findByCustomID($item_id)
	{
		return $this->mapper->findByCustomID($item_id);
	}

	/**
	 * Gets all Strap Item Activities.
	 * 
	 * @return array Array of ItemActivity where Activity_ID is key or null if not found.
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
	
	/**
	 * Gets all Strap Item Activities belonging to STRAP item.
	 * 
	 * @param int $item_id
	 * @return array Array of ItemActivity where Activity_ID is key or null if not found.
	 */
	public function findByItemID($item_id)
	{
		return $this->mapper->findByItemID($item_id);
	}
	
	/**
	 * Gets all active Strap Item Activities belonging to STRAP item.
	 * 
	 * @param int $item_id
	 * @return array Array of ItemActivity where Activity_ID is key or empty array if not found.
	 */
	public function findActiveByItemID($item_id)
	{
		return $this->mapper->findActiveByItemID($item_id);
	}
	
	/**
	 * Gets project allocaiotn from all strap item activities.
	 * 
	 * @param int $project_ID
	 * @return int $allocation
	 */
	public function findProjectAllocation($project_ID)
	{
        return $this->mapper->findProjectAllocation($project_ID);
	}
	
	/**
	 * Finds all strap item activities connected to one pursuit.
	 * 
	 * @param int $id Should be pursuit ID
	 * @return array Array of ItemActivity or empty array.
	 */
	public function findByPursuitID($id)
	{
        return $this->mapper->findByPursuitID($id);
	}
	
	/**
	 * Gets all Activities for given Pursuit.
	 * 
	 * @param int $pursuitID
	 * @return array Array of Activities where Activity ID is key or null if not found.
	 */
	public function findAllActivitiesByPursuitID($pursuitID)
	{
        return $this->mapper->findAllActivitiesByPursuitID($pursuitID);
	}
	
	/**
	 * Gets highest Custom_ID for givin strap item id.
	 * 
	 * @param int $strapItem_ID StrapItem ID
	 * @return int Highest custom id
	 */
	public function getNextCustomIDNumber($strapItem_ID)
	{
        return $this->mapper->getNextCustomIDNumber($strapItem_ID);
	}
	
	/**
	 * Gets all strap item activities where is project allocated.
	 * 
	 * @param int $project_ID
	 * @return array Array of ItemActivity where Activity_ID is key or null if not found.
	 */
	public function findProjectActivities($project_ID)
	{
        return $this->mapper->findProjectActivities($project_ID);
	}
}