<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing PursuitPlan from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PursuitPlanDibiMapper extends DibiMapper implements IPursuitPlanMapper
{
	/**
	 * Creates PursuitPlan instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	PursuitPlan 
	 */
    private function load(DibiRow $row)
    {
		$rursuitPlan = new PursuitPlan();
		$rursuitPlan->PursuitID = $row->Pursuit_ID;
		$rursuitPlan->Milestone = $row->Milestone;
		$rursuitPlan->PlannedDate = $this->dateOnly($row->PlannedDate);
		$rursuitPlan->ActualDate = $this->dateOnly($row->ActualDate);
		$rursuitPlan->Link = $row->Link;
		$rursuitPlan->BnPPlan = $row->BnP_Plan;
		$rursuitPlan->BnPActuals = $row->BnP_Actuals;
		$rursuitPlan->Remarks = $row->Remarks;
		return $rursuitPlan;
    }
	
	/**
	 * Saves new PursuitPlan.
	 * 
	 * @param PursuitPlan $pursuitPlan
	 * @return boolean
	 */
	public function save(PursuitPlan $pursuitPlan)
	{
        try
        {
            $array = array();
			$array['Pursuit_ID'] = $pursuitPlan->PursuitID;
			$array['Milestone'] = $pursuitPlan->Milestone;
			$array['PlannedDate'] = $pursuitPlan->PlannedDate;
			$array['ActualDate'] = $pursuitPlan->ActualDate;
			$array['Link'] = $pursuitPlan->Link;
			$array['BnP_Plan'] = $pursuitPlan->BnPPlan;
			$array['BnP_Actuals'] = $pursuitPlan->BnPActuals;
			$array['Remarks'] = $pursuitPlan->Remarks;
            $this->conn->insert('PursuitPlan', $array)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert new PursuitPlan for pursuit with ID: " . $pursuitPlan->PursuitID . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates PursuitPlan.
	 * 
	 * @param PursuitPlan $pursuitPlan
	 * @return boolean
	 */
	public function update(PursuitPlan $pursuitPlan)
	{
        try
        {
            $array = array();
			$array['Pursuit_ID'] = $pursuitPlan->PursuitID;
			$array['Milestone'] = $pursuitPlan->Milestone;
			$array['PlannedDate'] = $pursuitPlan->PlannedDate;
			$array['ActualDate'] = $pursuitPlan->ActualDate;
			$array['Link'] = $pursuitPlan->Link;
			$array['BnP_Plan'] = $pursuitPlan->BnPPlan;
			$array['BnP_Actuals'] = $pursuitPlan->BnPActuals;
			$array['Remarks'] = $pursuitPlan->Remarks;
            $this->conn->update('PursuitPlan', $array)->where('[Pursuit_ID] = %i AND [Milestone] = %s', $pursuitPlan->PursuitID, $pursuitPlan->Milestone)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of PursuitPlan for pursuit with ID: " . $pursuitPlan->PursuitID . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing PursuitPlan.
	 * 
	 * @param int $pursuit Pursuit ID
	 * @param string $milestone Milestone
	 * @return boolean
	 */
	public function delete($pursuitID, $milestone)
	{
		try
		{
		    $result = $this->conn->delete('PursuitPlan')->where('[Milestone] = %s AND [Pursuit_ID] = %i', $milestone, $pursuitID)->execute();
			if ($result)
				Debugger::log("PursuitPlan for pursuit with ID: " . $pursuitID . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of PursuitPlan for pursuit with ID: " . $pursuitID . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of PursuitPlan for pursuit with ID: " . $pursuitID . " failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Gets PursuitPlan by purusit ID and milestone.
	 * 
	 * @param int $pursuit Pursuit ID
	 * @param string $milestone Milestone
	 * @return PursuitPlan or null if not found
	 */
	public function find($pursuitID, $milestone)
	{
		$pursuitPlan_db_result = $this->conn->select('*')->from('PursuitPlan')->where('[Milestone] = %s AND [Pursuit_ID] = %i', $milestone, $pursuitID)->fetch();
		if ($pursuitPlan_db_result !== FALSE)
			return $this->load($pursuitPlan_db_result);
		else
			return NULL;
	}
	
	/**
	 * Gets PursuitPlan by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return array Of PursuitPlan or empty array if not found
	 */
	public function findByPursuitID($pursuitID)
	{
		$pursuitPlan_db_result = $this->conn->select('*')->from('PursuitPlan')->where('[Pursuit_ID] = %i', $pursuitID)->fetchAll();
		
		$pursuitPlan_array = array();
		if ($pursuitPlan_db_result !== FALSE)
		{
			foreach ($pursuitPlan_db_result as $pursuitPlan)
				$pursuitPlan_array[] = $this->load($pursuitPlan);
		}
		return $pursuitPlan_array;
	}
	
	/**
	 * Gets nearest PursuitPlan by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return PursuitPlan or null if not found
	 */
	public function findNearestByPursuitID($pursuitID)
	{
		$now = new DateTime();
		$pursuitPlan_db_result = $this->conn->query('SELECT * FROM [PursuitPlan]
														WHERE [PlannedDate] > %d
															AND [Pursuit_ID] = %i', $now, $pursuitID)->fetch();
		if ($pursuitPlan_db_result !== FALSE)
			return $this->load($pursuitPlan_db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds out if exists missed milestone.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return True or false
	 */
	public function existsMissedMilestone($pursuitID)
	{
		$now = new DateTime();
		$pursuitPlan_db_result = $this->conn->query('SELECT * FROM [PursuitPlan]
														WHERE [PlannedDate] < %d
															AND [Pursuit_ID] = %i
																AND [ActualDate] IS NULL', $now, $pursuitID)->fetch();
		if ($pursuitPlan_db_result !== FALSE)
			return true;
		else
			return false;
	}
	
	/**
	 * Gets missed PursuitPlan by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return array Of PursuitPlan or empty array if not found
	 */
	public function findMissedByPursuitID($pursuitID)
	{
		$now = new DateTime();
		$pursuitPlan_db_result = $this->conn->query('SELECT * FROM [PursuitPlan]
														WHERE [PlannedDate] < %d
															AND [Pursuit_ID] = %i
																AND [ActualDate] IS NULL', $now, $pursuitID)->fetchAll();
		$pursuitPlan_array = array();
		if ($pursuitPlan_db_result !== FALSE)
		{
			foreach ($pursuitPlan_db_result as $pursuitPlan)
				$pursuitPlan_array[] = $this->load($pursuitPlan);
		}
		return $pursuitPlan_array;
	}
}