<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for authentication of a user via LDAP.
 */
class LDAPAuthenticator extends ValidUserAuthenticator
{
	/** server parameters for LDAP authentication */
    private $server = '';
    private $port = '';
    private $basedn = '';
    private $basepass = '';
    private $searchdn = '';
    private $filter = '';

	/**
	 * Options injection from application configuration.
	 */
	public function setOptions($options)
	{
		$this->server = (isset($options['server'])) ? $options['server'] : '';
		$this->port  = (isset($options['port'])) ? $options['port'] : '';
		$this->basedn = (isset($options['basedn'])) ? $options['basedn'] : '';
		$this->basepass = (isset($options['basepass'])) ? $options['basepass'] : '';
		$this->searchdn = (isset($options['searchdn'])) ? $options['searchdn'] : '';
		$this->filter = (isset($options['filter'])) ? $options['filter'] : '';
	}
	
    /**
     * Performs authentication against LDAP and
     * updates user's email from LDAP.
     *
     * @param  array
     * @return IIdentity
     * @throws AuthenticationException
	 * @todo Test options setting (should throw errors on missing settings).
     */
    public function authenticate(array $credentials)
    {
        $username = strtolower($credentials[self::USERNAME]);
        $password = $credentials[self::PASSWORD];
        try 
        {
            $person = $this->validateUserAccount($username);
            $is_person = true;
        } 
        catch (PersonNotFoundAuthenticationException $e)//(PersonNotFoundAuthenticationException $e) 
        {
            $is_person = false;
        }
		
		$connect = ldap_connect($this->server, $this->port);
		if (!$connect)
			throw new AuthenticationException("LDAP connect failed.", self::FAILURE);
		$bind = ldap_bind($connect, $this->basedn, $this->basepass);
		if (!$bind)
			throw new AuthenticationException("LDAP bind failed.".$this->basedn.";".$this->basepass, self::FAILURE);

		$search = ldap_search($connect, $this->searchdn, $this->filter.$username);
		$info = ldap_get_entries($connect, $search);
		if ($info['count'] == 0)
			throw new AuthenticationException("Wrong username. ".$username." not found via LDAP.", self::IDENTITY_NOT_FOUND);
        
        if ($is_person) 
        {
            // update user's email
            $user_email = strtolower($info[0]['mail'][0]);
            if ($user_email <> $person->Email)
            {
                $person->Email = $user_email;
                if ($this->userRepository->update($person))
                    Debugger::log("$person->Firstname $person->Surname's email was updated to '$person->Email'.", Debugger::INFO);
                else
                    Debugger::log("$person->Firstname $person->Surname's email could not be updated to '$person->Email'.", Debugger::WARNING);
            }
        }
        
		$dn = $info[0]["dn"];
		
		if(Environment::getContext()->parameters['serverType'] == 'Production')
			$bind_user = @ldap_bind($connect, $dn, $password);
		else
		{
			if(!Environment::getContext()->globalSetting->get('login.developmentMode'))
				$bind_user = @ldap_bind($connect, $dn, $password);
			else
				$bind_user = true;
		}
		
		if (!$bind_user)
		{
			throw new AuthenticationException("Wrong password.", self::INVALID_CREDENTIAL);
		}

        if ($is_person)
        {
            return new ValidPersonIdentity($username);
        }
        else
        {
            $LDAPValidIdentity = new LDAPValidIdentity($username); 
            $LDAPValidIdentity->firstname = $info[0]['givenname'][0];
            $LDAPValidIdentity->surname = $info[0]['sn'][0];
            $LDAPValidIdentity->email = $info[0]['mail'][0];
            return $LDAPValidIdentity;
        }
    }

}
