<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of TaskPONumber
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class TaskPONumberRepository extends Object
{
	/** @var ITaskPONumberMapper */
    private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(ITaskPONumberMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'TaskPONumber-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Task PO Number"));
    }
	
	/**
	 * Saves TaskPONumber.
	 * @param TaskPONumber $poAttr
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(TaskPONumber $poAttr)
	{
		if($this->mapper->save($poAttr))
		{
			$this->multilog->log('POTaskNumber with ID: ' . $poAttr->ID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Updates TaskPONumber.
	 * @param TaskPONumber $poAttr
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(TaskPONumber $poAttr)
	{
		$oldObject = $this->find($poAttr->ID);
		if($this->mapper->update($poAttr))
		{
			$this->multilog->log('TaskPONumber with ID: ' . $poAttr->ID . ' updated. ' . $this->multilog->differences($oldObject, $poAttr), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing TaskPONumber.
	 * @param int $id TaskPONumber ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		if ($this->mapper->delete($id))
		{
			$this->multilog->log('TaskPONumber with ID: ' . $id . ' deleted.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Finds TaskPONumber by ID.
	 * @param int $id TaskPONumber ID.
	 * @return TaskPONumber or NULL if not found.
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
	
	/**
	 * Finds all TaskPONumbers.
	 * @return array Array of TaskPONumber or empty array.
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
	
	/**
	 * Finds all TaskPONumbers for task.
	 * @param int $id Task ID
	 * @return array Array of TaskPONumber or empty array.
	 */
	public function findByTaskID($id)
	{
		return $this->mapper->findByTaskID($id);
	}
	
	/**
	 * Find TaskPONumber for task and year.
	 * @param int $id Task ID
	 * @param int $year Year
	 * @return TaskPONumber or NULL if not found.
	 */
	public function findByTaskIDandYear($id, $year)
	{
		return $this->mapper->findByTaskIDandYear($id, $year);
	}
	
	/**
	 * Find TaskPONumber by PO number.
	 * @param int $po_number
	 * @return TaskPONumber or NULL if not found.
	 */
	public function findByPONumber($po_number)
	{
		return $this->mapper->findByPONumber($po_number);
	}
	
	/**
	 * Find TaskPONumber by year.
	 * @param int $year
	 * @return array Array of TaskPONumber or empty array.
	 */
	public function findByYear($year)
	{
		return $this->mapper->findByYear($year);
	}
	
	/**
	 * Finds relations between TaskPONumber, Task and Project by given PO numbers.
	 * @param array $po_numbers List of PO numbers
	 * @param int $year Year
	 * @return array
	 *     $array[PONumber] = array(Year => val, TaskSAP_ID => val, ProjectSAP_ID => val)
	 *     or NULL if not found.
	 */
	public function findIASAPActualsImportRelationsByPONumbers(array $po_numbers, $year)
	{
		return $this->mapper->findIASAPActualsImportRelationsByPONumbers($po_numbers, $year);
	}
}
