<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing task plans from database using Dibi.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class TaskPlanDibiMapper extends DibiMapper implements ITaskPlanMapper
{
	
	private function hoursBudgetTableColumns()
	{
		return array(
			'January' => 'JanHrs',
			'February' => 'FebHrs',
			'March' => 'MarHrs',
			'April' => 'AprHrs',
			'May' => 'MayHrs',
			'June' => 'JunHrs',
			'July' => 'JulHrs',
			'August' => 'AugHrs',
			'September' => 'SepHrs',
			'October' => 'OctHrs',
			'November' => 'NovHrs',
			'December' => 'DecHrs'
		);
	}
	
	/**
	 * Creates TaskYearPlan instance from DibiRow.
	 * 
	 * @param $row DibiRow
	 * @return	TaskYearPlan 
	 */
    private function load(DibiRow $row)
    {
		$plan = new TaskYearPlan();
		$plan->Year = $row->Year;
		$plan->TaskID = $row->Task_ID;
		$plan->TaskPlanID = $row->TaskPlan_ID;
		$plan->Date = $row->Date;
				
		$column_names = $this->hoursBudgetTableColumns();
		$i = 0;
        foreach ($column_names as $budget_col => $hours_col) 
		{
			$i++;
			$plan->setMonthPlan($i, new MonthPlan($row->$hours_col, $row->$budget_col));
        }
		return $plan;
    }	
	
	/**
	 * Saves task year plan.
	 * 
	 * @param TaskYearPlan $plan
	 * @return boolean
	 */
	public function save(TaskYearPlan $plan)
	{
        try
        {
            $array = array();
            $array['Task_ID'] = $plan->getTaskID();
            $array['Year'] = $plan->getYear();
            $array['Date'] = $plan->getDate();
			$column_names = $this->hoursBudgetTableColumns();
			$i = 0;
			foreach ($column_names as $budget_col => $hours_col) 
			{
				$i++;
				$array[$budget_col] = $plan->getMonthPlan($i)->getBudget();
				$array[$hours_col] = $plan->getMonthPlan($i)->getHours();
			}
            $this->conn->insert('TaskPlan', $array)->execute();
			$plan->ID = $this->conn->insertId();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert of new task plan with task ID = ".$plan->getTaskID()." for year ".$plan->getYear()." failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates task year plan.
	 * 
	 * @param TaskYearPlan $plan
	 * @return boolean
	 */
	public function update(TaskYearPlan $plan)
	{
        try
        {
            $array = array();
			$column_names = $this->hoursBudgetTableColumns();
			$i = 0;
			foreach ($column_names as $budget_col => $hours_col) 
			{
				$i++;
				$array[$budget_col] = $plan->getMonthPlan($i)->getBudget();
				$array[$hours_col] = $plan->getMonthPlan($i)->getHours();
			}
            $this->conn->update('TaskPlan', $array)->where('[TaskPlan_ID] = %u', $plan->getID())->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of task plan with ID = ".$plan->getID()." failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes task year plan by ID.
	 * 
	 * @param int $plan_id
	 * @return boolean
	 */
	public function delete($plan_id)
	{
        try
        {
            $this->conn->delete('TaskPlan')->where('[TaskPlan_ID] = %u', $plan_id)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Task plan deletion with id = $id failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Returns task plan by id.
	 * 
	 * @param int $id Task plan ID.
	 * @return	TaskYearPlan or NULL if not found.
	 */
    public function find($id)
    {
		$result = $this->conn->select('*')->from('TaskPlan')->where('[TaskPlan_ID] = %i', $id)->fetch();
        if ($result)
            return $this->load($result);
        else
            return NULL;
    }
    
	/**
	 * Returns all task year plans by task id.
	 * 
	 * @param int $id Task ID.
	 * @return	array Returns associative array of year plans (key is year), value TaskYearPlan.
	 */
	public function findTaskPlans($id)
	{
		$plans_db_result = $this->conn->query('SELECT * FROM [TaskPlan] WHERE [Task_ID] = %u ORDER BY [Year] ASC', $id)->fetchAll();
		$plans = array();
		foreach ($plans_db_result as $plan_db_result)
		{
			$task_year_plan = $this->load($plan_db_result);
			$plans[$task_year_plan->getYear()] = $task_year_plan;
		}
		return $plans;
	}
	
	/**
	 * Returns all task years by task id.
	 * 
	 * @param int $id Task ID.
	 * @return	array Returns year of years.
	 */
	public function findTaskYears($id)
	{
		$plans_db_result = $this->conn->query('SELECT DISTINCT [Year] FROM [TaskPlan] WHERE [Task_ID] = %u ORDER BY [Year] ASC', $id)->fetchAll();
		$years = array();
		foreach ($plans_db_result as $plan_db_result)
		{
			$years[] = $plan_db_result['Year'];
		}
		return $years;
	}

	/**
	 * Returns task year plan by task id and year.
	 * 
	 * @param int $id Task ID.
	 * @param int $year year.
	 * @return	TaskYearPlan
	 */
	public function findTaskYearPlan($id, $year)
	{
		$plan_db_result = $this->conn->select('*')->from('TaskPlan')->where('[Task_ID] = %u AND [Year] = %u', $id, $year)->fetch();
		if ($plan_db_result !== FALSE)
			return $this->load($plan_db_result);
		else
			return NULL;
	}
}
