<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009 - 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\TaskResourcePlan
 */

/**
 * TaskResourcePlanner provides necessary functinality for planning individual resources.
 * Protects consistency of task resource plan according to EAC usage:
 * - no changes to task year plan (PMB/SAP plan)
 * - no changes to planned work for resources in the past 
 * - possible to change resource plans only for current fiscal and future fiscal months
 * - no group unassigning if worker still assigned
 * - solves persistence (no need to call "save to repository" functions")
 *
 * WARNING: Works only for Labour tasks.
 * 
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @todo Should there be ancestor to all services with set/getContext methods to make DI of context into services standardized?
 */
class TaskResourceYearPlanner extends Object
{

    /** @var TaskYearPlan Task ID */
    private $taskYearPlan;

    /** @var Task */
	private $cachedTask = null;

    /** @var int Don't use directly - use getFirstUpdatableMonth() function */
	private $cachedFirstUpdatableMonth = null;
	
	/** @var DIContainer Access to model repositories */
	private $context;

    public function __construct(DIContainer $context, TaskYearPlan $year_plan)
    {
		$this->taskYearPlan = $year_plan;
		$this->context = $context;

		$this->cachedTask = $this->context->taskRepository->find($this->taskYearPlan->getTaskID());
		if ($this->cachedTask === NULL)
			throw new InvalidArgumentException('Task not found for year plan. Task ID: ' . $this->taskYearPlan->getTaskID());
		if (!$this->cachedTask->isLabourTask())
			throw new InvalidStateException('Non-labour task plan cannot plan resources. Task ID: ' . $this->taskYearPlan->getTaskID());
    }

	/**
	 * Returns a copy of task.
	 * 
	 * @return Task
	 */
	public function getTask()
	{
		if ($this->cachedTask == NULL)
			$this->cachedTask = $this->context->taskRepository->find($this->taskYearPlan->getTaskID());
		if ($this->cachedTask === NULL)
			throw new InvalidArgumentException('Task not found for year plan. ID is no longer valid: ' . $this->taskYearPlan->getTaskID());
		return clone $this->cachedTask;
	}
    

    /**
     * Returns task year plan.
     * 
     * @return TaskYearPlan
     */
    public function getTaskYearPlan()
    {
        return $this->taskYearPlan;
    }
	
	/**
	 * Updates remark of a task.
	 * 
	 * @param string $remark Remark to update.
	 * @return Task
	 */
	public function updateTaskRemark($remark)
	{
		$task = $this->getTask();
		$task->setRemark($remark);
		$this->context->taskRepository->update($task);
		// invalidate cached task
		$this->cachedTask = NULL;
	}
	
	/**
	 * Returns task planned resource totals (SAP plan / PMB) for task for every month in planned year.
	 * 
	 * @return MonthArray Array of 12 values for every month.
	 */
	public function getTaskYearPlanHours()
	{
		// initialize array
		$hours = new MonthArray((float) 0.0, false);
		for ($i = 1; $i <= 12; $i++)
			$hours[$i] = $this->taskYearPlan->getMonthPlan($i)->getHours();
		return $hours;
	}
	
	/**
	 * Returns task EAC totals for task for every month in planned year.
	 * 
	 * EAC corresponds to total Acuals for task for fiscal months
	 * that are already finished and SAP/PMB plan for current and next months. 
	 * (REALLY?!? or should EAC for unfinished months be sum of assigned hours?)
	 * 
	 * Warning: For previous years only Actuals are returned.
	 * 
	 * @return MonthArray Array of 12 floats for every month.
	 */
	public function getTaskYearEACHours()
	{
		$plan_from = $this->getFirstUpdatableMonth();
		if ($plan_from == 0) // year is closed
			$plan_from = 13;
		// initialize array
		$hours = new MonthArray((float) 0.0, false);
		// actuals are shown as EAC for finished months
		$actuals = $this->context->workRepository->findTaskYearWork($this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear());
		for ($i = 1; $i < $plan_from; $i++)
			$hours[$i] = $actuals[$i];
		// if there are still unfinished months
		// show totals of assigned hours from current month until the end of year
		$task_year_assignments = $this->context->personTaskAssignmentRepository->findTaskYearAssignments($this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear());
		foreach ($task_year_assignments as $eid => $person_task_assignments)
			for ($i = $plan_from; $i <= 12; $i++) 
				$hours[$i] += $person_task_assignments[$i]->getHours();
		return $hours;
	}	

	/**
	 * Returns number <1, 12> of updatable month in current year
	 * according to current date and fiscal calendar.
	 * If task plan is for previous years, 0 si returned.
	 * 
	 * @return int value in <1,12> or 0 if no month is updatable (for plan for previous years)
	 */
	public function getFirstUpdatableMonth()
	{
		if (FiscalCalendar::currentYear() <= $this->taskYearPlan->getYear())
		{
			if ($this->cachedFirstUpdatableMonth === NULL) // not found yet
			{
				if (FiscalCalendar::currentYear() < $this->taskYearPlan->getYear())
					$this->cachedFirstUpdatableMonth = 1;
				else
				{
					$current_fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp(time());
					$this->cachedFirstUpdatableMonth = $current_fiscal_month->getIndex() + 1;
				}
			}
			return $this->cachedFirstUpdatableMonth;
		}
		else
			return 0;
	}

	/**
	 * Returns array of EACs for all assigned workers on task.
	 * It is an associative array of arrays: outer array's key is worker ID,
	 * and inner array's key are month indexes (1 to 12).
	 * 
	 * @return array (associative, key is worker ID) of arrays of float with EAC
	 */
	public function getAssignedWorkerEACArray()
	{
		$plan_from = $this->getFirstUpdatableMonth();
		if ($plan_from == 0) // year is closed
			$plan_from = 13;
		$worker_assignments = $this->context->personTaskAssignmentRepository->findTaskYearAssignments($this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear());
		$eac = array();
		foreach ($worker_assignments as $eid => $year_assignments)
		{
			$eac[$eid] = new MonthArray((float) 0.0, false);
			$year_work = $this->context->workRepository->findPersonTaskYearWork($eid, $this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear());
			for ($i = 1; $i < $plan_from; $i++)
				$eac[$eid][$i] = $year_work[$i];
			for ($i = $plan_from; $i <= 12; $i++) 
				$eac[$eid][$i] = $year_assignments[$i]->getHours();
		}
		return $eac;
        
		// initialize array
//		$aop_available = new MonthArray((float) 0.0, false);
		// delta => available
//		for ($i = 1; $i <= 12; $i++)
//			$aop_available[$i] = $aop->getMonthAOPHours($i) - $total_assignments[$i];
//		return $aop_available;
	}	

	/**
	 * Returns array of EACs for all assigned workers on task.
	 * It is an associative array of arrays: outer array's key is worker ID,
	 * and inner array's key are month indexes (1 to 12.
	 * 
	 * @return array (associative, key is worker ID) of arrays of float with EAC
	 */
	public function getUnassignedWorkersEACArray()
	{
		$assigned_workers = $this->context->personTaskAssignmentRepository->findTaskYearAssignments($this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear());
		$all_workers = $this->context->workRepository->findPeopleWorkingOnTaskInYear($this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear());
		$unassigned_workers_eac = array();
		foreach ($all_workers as $eid)
		{
			if (!isset($assigned_workers[$eid])) // unassigned but working
				$unassigned_workers_eac[$eid] = $this->context->workRepository->findPersonTaskYearWork($eid, $this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear());
		}
		return $unassigned_workers_eac;
	}	

	/**
	 * Compares two persons by surname alphabetically.
	 * 
	 * @param array of Person that can be assigned to task
	 * @param array of Person that can be assigned to task
	 * @return int -1, 0, 1
	 */
	private function comparePersonBySurname($person1, $person2)
	{
		return strcasecmp($person1->getSurname(), $person2->getSurname());
	}
	
	/**
	 * Returns associative array (keyed by person id) of assignable
	 * workers to task plan. Returns only still unassigned workers.
	 * 
	 * Workers are sorted by names.
	 * 
	 * @param array of Person that can be assigned to task
	 */
	public function getAssignableWorkers()
	{
		$project = $this->context->projectRepository->findProjectByTaskId($this->taskYearPlan->getTaskID());
		$assigned_workers = $this->context->personTaskAssignmentRepository->findTaskYearAssignments($this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear());
		$assignable_workers = array();
        $persons = $this->context->personRepository->findAll();
        foreach ($persons as $person)
            if (!isset($assignable_workers[$person->getID()]))
            {
                if (!isset($assigned_workers[$person->getID()]))
				{
					$assignable_workers[$person->getID()] = $person;
				}
            }
		// sort array by surnames
		uasort($assignable_workers, array($this, 'comparePersonBySurname'));
		return $assignable_workers;
	}
	
	/**
	 * Assigns new year plan for worker. If worker doesn't have a plan yet,
	 * new assignment plan is created otherwise it is updated.
	 * 
	 * Warning: Values that are updatable (from next month after actual month) must be non-negative integers.
	 * Warning: Values for closed months are not updated and are not tested for input validity. If year is closed, returns FALSE.
	 * 
	 * @param string $person_id ID (EID) of the worker.
	 * @param MonthArray $year_assignment array of containing worker hours plan
	 * @return boolean Returns TRUE if saving was successful, otherwise FALSE.
	 */
	public function assignWorkerPlan($person_id, MonthArray $year_assignment)
	{
		$planable_from = $this->getFirstUpdatableMonth();
		if ($planable_from == 0) // year is closed - no planning
			return false;
		$current_assignments = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear(), $person_id);
		$changes_ok = true;
		for ($i = $planable_from; $i <= 12; $i++)
		{
			if (!NumericUtils::isNonNegativeInteger($year_assignment[$i]))
				throw new InvalidArgumentException("Invalid data in 'year_assignment' parameter. Value on position $i is not non-negative integer value.");
			if ($current_assignments[$i]->getHours() != $year_assignment[$i]) // change in assignment
			{
				$current_assignments[$i]->setHours($year_assignment[$i]);
				if (!$this->context->personTaskAssignmentRepository->save($current_assignments[$i]))
					$changes_ok = false;
			}
		}
		return $changes_ok;
	}	
	
	/**
	 * Unassigns worker from task plan.
	 * 
	 * Worker cannot be unnassigned if he already worked on the task in that
	 * method throws InvalidStateException.
	 * @param string $person_id ID (EID) of the worker.
	 */
	public function unassignWorker($person_id)
	{
		if (!$this->context->personTaskAssignmentRepository->isPersonAssignedToTaskInYear($this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear(), $person_id))
			throw new InvalidStateException("Worker with EID '".$person_id."' is not assigned to the task.");
		if ($this->context->workRepository->findPersonTaskYearWorkSum($person_id, $this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear()) == 0)
			$this->context->personTaskAssignmentRepository->deleteByPersonAndYear($person_id, $this->taskYearPlan->getTaskID(), $this->taskYearPlan->getYear());
		else
			throw new InvalidStateException("Worker with EID '".$person_id."' cannot be unassigned because he already worked on the task.");
	}	

	/**
	 * Returns array of still available hours (from his AOP) for worker for
	 * every month in a year.
	 * 
	 * @param string $person_id ID (EID) of the worker.
	 * @return MonthArray 12 float values containing workers available/unassigned hours 
	 */
	public function getWorkersAvailableAOP($person_id)
	{
		// basic aop for person
		$aop = $this->context->aopRepository->findForPerson($this->taskYearPlan->getYear(), $person_id);
		// total assigned hours for person
		$total_assignments = $this->context->personTaskAssignmentRepository->findPersonTotalYearAssignments($person_id, $this->taskYearPlan->getYear());
		// initialize array
		$aop_available = new MonthArray((float) 0.0, false);
		// delta => available
		for ($i = 1; $i <= 12; $i++)
			$aop_available[$i] = $aop->getMonthAOPHours($i) - $total_assignments[$i];
		return $aop_available;
	}	
	

}


