<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\WBS
 */

/**
 * WBS Tree Element Item for holding WBS tree data.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class WBSTreeElementItem extends Object implements WBSTreeElement
{
	/** Possible element types */
	const 
		WBS_TYPE_TASK = 'task',
		WBS_TYPE_WORK_PACKAGE = 'workpackage';
	
    /** @var WBSTree Root of the WBSTreeElements */
	private $tree = NULL;

    /** @var string V4 UUID */
	private $uuid;
	
    /** @var WBSElement data object */
	private $data = NULL;

    /** @var TypedArray of WBSTreeElementItem. */
	private $subitems;
	
	public function __construct(WBSElement $elem, TypedArray $subitems = NULL) 
	{
		if (is_null($elem))
			throw new InvalidArgumentException('NULL data are not allowed.');
		$this->data = $elem;
		if ($subitems !== NULL)
		{
			if ($subitems->getClass() == 'WBSTreeElementItem')
				$this->subitems = $subitems;
			else
				throw new InvalidArgumentException('Subitems must be TypedArray with elements that are WBSTreeElementItem class');
		}
		$this->uuid = UUID::generate(UUID::UUID_RANDOM);
	}	

    /**
     * Injects tree in which this Item is included.
	 * 
	 * @param WBSTree $tree
     */
    public function setTree(WBSTree $tree)
	{
		if (is_null($tree))
			throw new InvalidArgumentException('Tree cannot be NULL');
		$this->tree = $tree;
	}
	
    /**
     * Returns tree.
	 * 
     * @return WBSTree.
     */
    public function getTree()
	{
		return $this->tree;
	}
	
    /**
     * Returns level of element nesting in the whole tree.
	 * 
	 * 0 is top level.
     * 
     * @return int Non-negative level.
     */
    public function getUUID()
	{
		return $this->uuid;
	}

    /**
     * Gets data element name.
     * 
     * @return string
     */
    public function getName()
	{
		return $this->data->getName();
	}
	
    /**
     * Gets data element SAP ID.
     * 
     * @return string
     */
    public function getSAPID()
	{
		return $this->data->getSAPID();
	}
	
    /**
     * Gets data element type (constants TASK or WORK_PACKAGE).
     * 
     * @return string
     */
    public function getWBSType()
	{
		if ($this->data instanceof Task)
			return self::WBS_TYPE_TASK;
		elseif ($this->data instanceof WorkPackage)
			return self::WBS_TYPE_WORK_PACKAGE;
		else
			throw new InvalidArgumentException('Unknown data type in WBS element.');
	}
	
    /**
     * Returns level of element nesting in the whole tree.
	 * 
	 * 0 is top level.
     * 
     * @return int Non-negative level.
     */
    public function getLevel()
	{
		if ($this->tree == NULL)
			throw new InvalidStateException('Tree is NULL. Item not in tree.');
		return $this->tree->getElementLevel($this);
	}

    /**
     * Returns data object.
	 * 
     * @return WBSElement
     */
    public function getData()
	{
		return $this->data;
	}

    /**
     * Returns subitems.
	 * 
     * @return TypedArray
     */
    public function getChildren()
	{
		if ($this->subitems == NULL)
			return new TypedArray (new WBSTreeElementItem (new Task())); // leaves should be tasks
		else
			return $this->subitems;
	}


    /**
     * Returns whether this item has children.
	 * 
     * @return boolean
     */
    public function hasChildren()
	{
		return (count($this->getChildren()) > 0);
	}

    /**
     * Adds child in the end of subitem list.
	 * 
     * @return boolean
     */
    public function addChild(WBSTreeElementItem $child)
	{
		if ($this->subitems == NULL) // first subitem
			$this->subitems = new TypedArray ($child);
		$this->subitems[count($this->subitems)] = $child;
	}
	
}
