<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Database access to work effort of persons/employees using Dibi.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class WorkDibiMapper extends DibiMapper implements IWorkMapper
{
	/**
	 * Returns hours a worker spent on a given task in given month and year.
	 * 
	 * @param string $id person ID.
	 * @param int $id task ID.
	 * @param int $month month in a year (0-11)
	 * @param int $year year
	 * @return	float
	 */
	public function findPersonTaskMonthWork($person_id, $task_id, $month, $year)
	{
		$hours = 0.0;
		$db_hours = $this->conn->query('SELECT [Actuals].[Hours] FROM [Actuals],[Task] 
				WHERE [Actuals].[ChargeNumber] = [Task].[SAP_ID]
				AND [Actuals].[EID] = %s 
				AND [Task].[Task_ID] = %u 
				AND [Actuals].[FiscalMonth] = %u
				AND [Actuals].[Year] = %u ', 
				$person_id, $task_id, $month, $year)->fetchAll();
		foreach($db_hours as $row)
			$hours += $row->Hours;
		return $hours;
	}

	/**
	 * Returns array of hours a worker spent on a given task in given year.
	 *   
	 * @param string $id person ID.
	 * @param int $id task ID.
	 * @param int $year year.
	 * @return	MonthArray associative array of twelve floats where key is month.
	 */
	public function findPersonTaskYearWork($person_id, $task_id, $year)
	{
		$db_hours = $this->conn->query('SELECT [Actuals].[FiscalMonth],[Actuals].[Hours] 
				FROM [Actuals],[Task] 
				WHERE [Actuals].[ChargeNumber] = [Task].[SAP_ID]
				AND [Actuals].[EID] = %s 
				AND [Task].[Task_ID] = %u 
				AND [Actuals].[Year] = %u', 
				$person_id, $task_id, $year)->fetchAll();
		$hours = new MonthArray((float) 0.0, false);
		foreach ($db_hours as $row)
			$hours[1 + $row->FiscalMonth] += $row->Hours;
		return $hours;
	}
	
	/**
	 * Returns array of hours a worker spent on year.
	 *   
	 * @param string $id person ID.
	 * @param int $year year.
	 * @return	MonthArray associative array of twelve floats where key is month.
	 */
	public function findPersonYearWork($person_id, $year)
	{
		$db_hours = $this->conn->query('SELECT [Actuals].[FiscalMonth],[Actuals].[Hours] 
				FROM [Actuals]
				WHERE [Actuals].[EID] = %s 
				AND [Actuals].[Year] = %u', 
				$person_id, $year)->fetchAll();
		$hours = new MonthArray((float) 0.0, false);
		foreach ($db_hours as $row)
			$hours[1 + $row->FiscalMonth] = $hours[1 + $row->FiscalMonth] + $row->Hours;
		return $hours;
	}
	
	/**
	 * Returns array of hours a worker spent on a given task in given year for all worker.
	 * 
	 * @param int $task_id
	 * @param int $year
	 * @return array Associative array hours[EID of worker][index of month (1..12)] => number of hours a worker spent on task in selected month
	 * @throws InvalidArgumentException
	 */
	public function findPeopleTaskYearWork($task_id, $year)
	{
		if (!(NumericUtils::isInteger($task_id) && ($task_id > 0)))
			throw new InvalidArgumentException("Task ID parameter must be integer greater than 0. Given value:" . $task_id);
		if (!NumericUtils::isInteger($year))
			throw new InvalidArgumentException("Year parameter must be integer. Given value:" . $year);
		
		$db_result = $this->conn->query('
	            SELECT [Actuals].[EID], [Actuals].[FiscalMonth], [Actuals].[Hours]
				FROM [Actuals], [Task]
				WHERE [Task].[Task_ID] = %i AND
				      [Task].[SAP_ID] = [Actuals].[ChargeNumber] AND
					  [Actuals].[Year] = %u', 
				$task_id, $year)->fetchAll();
		$hours = array();
		foreach ($db_result as $row)
		{
			for ($i = 1; $i < 13; $i++)
			{
				$hours[$row->EID][$i] = 0;
			}
		}
		foreach ($db_result as $row)
		{
			$hours[$row->EID][1 + $row->FiscalMonth] += $row->Hours;
		}
		return $hours;
	}

	/**
	 * Returns array of sums of hours that workers spent on a given task in given year.
	 *   
	 * @param int $id task ID.
	 * @param int $year year.
	 * @return	array associative array of twelve floats where key is month (indexes 1 to 12).
	 */
	public function findTaskYearWork($task_id, $year)
	{
		$db_hours = $this->conn->query('SELECT [Actuals].[FiscalMonth],SUM([Actuals].[Hours]) AS [Hours]
				FROM [Actuals],[Task] 
				WHERE [Actuals].[ChargeNumber] = [Task].[SAP_ID]
				AND [Task].[Task_ID] = %u 
				AND [Actuals].[Year] = %u
				GROUP BY [FiscalMonth]', 
				$task_id, $year)->fetchAll();
		$hours = array();
		for ($i = 1; $i <= 12; $i++)
			$hours[$i] = 0.0;
		foreach ($db_hours as $row)
			$hours[1 + $row->FiscalMonth] += $row->Hours;
		return $hours;
	}

	/**
	 * Returns array of ids of workers that worked on a task in given year (Actuals table).
	 *   
	 * @param int $id task ID.
	 * @param int $year year.
	 * @return	array Array of ids of workers (EIDs).
	 */
	public function findPeopleWorkingOnTaskInYear($task_id, $year)
	{
		$db_ids = $this->conn->query('SELECT DISTINCT [EID] 
				FROM [Actuals], [Task]
				WHERE [Actuals].[Year] = %u
				AND [Actuals].[ChargeNumber] = [Task].[SAP_ID]
				AND [Task].[Task_ID] = %u', 
				$year, $task_id)->fetchAll();
		$worker_ids = array();
		foreach ($db_ids as $row)
			$worker_ids[] = $row->EID;
		return $worker_ids;
	}
    
    /**
     * Returns array of ids of workers that are scheduled to work on a task in given year (Person_Task table).
     * 
     * @param int $task_id
     * @param int $year
     * @return  array Array of ids of workers (EIDs).
     */
    public function findPeopleScheduledToWorkOnTaskInYear($task_id, $year)
    {
		$db_ids = $this->conn->query('SELECT DISTINCT [EID] 
				FROM [Person_Task]
				WHERE [Person_Task].[Year] = %u
				AND [Person_Task].[Task_ID] = %u', 
				$year, $task_id)->fetchAll();
		$worker_ids = array();
		foreach ($db_ids as $row)
			$worker_ids[] = $row->EID;
		return $worker_ids;
    }

    /**
     * Returns associative array of effort/hours/actuals spent on tasks in month.
	 * Key in the array is Task ID and value is spent hours / actuals 
	 * (it can be even negative number!).
     * 
     * @param int $task_id
     * @param int $month Month number (1-12).
     * @param int $year
     * @return  array Associative array.
     */
    public function findTasksPersonWorkedOnInMonth($person_id, $month, $year)
    {
		if (!(NumericUtils::isInteger($month) && ($month >= 1) && ($month <= 12)))
			throw new InvalidArgumentException("Month parameter must be integer in range <1,12>. Given value:" . $month);
		if (!NumericUtils::isInteger($year))
			throw new InvalidArgumentException("Year parameter must be integer. Given value:" . $year);
		
		$db_result = $this->conn->query('SELECT [Actuals].[Hours] as [Hours],[Task].[Task_ID],[Task].[Project_ID] FROM [Actuals],[Task] 
			WHERE [Actuals].[EID] = %s AND [Actuals].[FiscalMonth] = %u AND [Actuals].[Year] = %u
			AND [Actuals].[Hours] != 0 AND ([Actuals].[ChargeNumber] = [Task].[SAP_ID])
			ORDER BY [Project_ID]', 
				$person_id, $month - 1, $year)->fetchAll();
		$work = array();
		foreach ($db_result as $row)
		{
			$work[$row->Task_ID] = 0;
		}
		foreach ($db_result as $row)
		{
			$work[$row->Task_ID] += $row->Hours;
		}
		return $work;
    }
	
	/**
	 * Returns array of hours a worker spent on a given project per month in given year for all worker.
	 * 
	 * @param string $person_id
	 * @param int $project_id
	 * @param int $year
	 * @return array Associative array hours[index of month (1..12)] => number of hours a worker spent on project in selected month
	 * @throws InvalidArgumentException
	 */
	public function findPersonProjectMonthWork($person_id, $project_id, $year)
	{
		if (!(NumericUtils::isInteger($project_id) && ($project_id > 0)))
			throw new InvalidArgumentException("Project ID parameter must be integer greater than 0. Given value:" . $project_id);
		if (!NumericUtils::isInteger($year))
			throw new InvalidArgumentException("Year parameter must be integer. Given value:" . $year);
		
		$db_result = $this->conn->query('
	            SELECT [Actuals].[FiscalMonth], [Actuals].[Hours]
				FROM [Actuals], [Task]
				WHERE [Task].[Project_ID] = %i AND
				      [Task].[SAP_ID] = [Actuals].[ChargeNumber] AND
					  [Actuals].[Year] = %u AND
					  [Actuals].[EID] = %s', 
				$project_id, $year, $person_id)->fetchAll();
		$hours = array();
		for ($i = 1; $i < 13; $i++)
			$hours[$i] = 0;
		foreach ($db_result as $row)
			$hours[1 + $row->FiscalMonth] += $row->Hours;
		return $hours;
	}
	
}
