<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Database access to work packages using Dibi.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class WorkPackageDibiMapper extends DibiMapper implements IWorkPackageMapper
{

	/**
	 * Creates WorkPackage class instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	WorkPackage 
	 */
    private function load(DibiRow $row)
    {
		// TODO: define Annotation e.g. DbField for automatic population of object using Reflection of Object predecessor if possible.
		$wp = new WorkPackage();
		$wp->ID = $row->WorkPackage_ID;
		$wp->ProjectID = $row->Project_ID;
		$wp->SAPID = $row->SAP_ID;
		$wp->ParentID = $row->Parent_ID;
		$wp->Name = $row->Name;
		$wp->Date = $row->Date;
		return $wp;
    }	

	/**
	 * Saves new WorkPackage.
	 * 
	 * @param WorkPackage $wp New Workpackage to add.
	 * @return	boolean 
	 */
	public function save(WorkPackage $wp)
	{
        try
        {
            $array = array();
            $array['SAP_ID'] = $wp->getSAPID();
			if ($wp->getProjectID() > 0) // missing ids will stay NULL by default
				$array['Project_ID'] = $wp->getProjectID();
			if ($wp->getParentID() > 0) // missing ids will stay NULL by default
	            $array['Parent_ID'] = $wp->getParentID();
            $array['Name'] = $wp->getName();
            $array['Date'] = $wp->getDate();
            $this->conn->insert('WorkPackage', $array)->execute();
			$wp->ID = $this->conn->insertId();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert of WorkPackage with SAP ID = ".$wp->getSAPID()." failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates existing WorkPackage.
	 * 
	 * @param WorkPackage $wp Existing WorkPackage.
	 * @return	boolean 
	 */
	public function update(WorkPackage $wp)
	{
        try
        {
            $array = array();
            $array['Name'] = $wp->getName();
			$array['SAP_ID'] = $wp->getSAPID();
			if ($wp->getProjectID() > 0) // missing ids will stay NULL by default
				$array['Project_ID'] = $wp->getProjectID();
			if ($wp->getParentID() > 0) // missing ids will stay NULL by default
	            $array['Parent_ID'] = $wp->getParentID();
            $array['Name'] = $wp->getName();
            $array['Date'] = $wp->getDate();
            $wp_db_result = $this->conn->update('WorkPackage', $array)->where('[WorkPackage_ID] = %i', $wp->getID())->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("WorkPackage update with id = ".$wp->getID()." failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Finds work package by id.
	 * 
	 * @param int $id work package ID.
	 * @return	WorkPackage or NULL if not found.
	 */
	public function find($workpackage_id) 
	{
		if (!is_numeric($workpackage_id) || (!($workpackage_id > 0)))
			throw new InvalidArgumentException('WorkPackage id must be a positive integer.');
		
		$wp_db_result = dibi::select('*')->from('WorkPackage')->where('[WorkPackage_ID] = %i', $workpackage_id)->fetch();
		if ($wp_db_result !== FALSE)
			return $this->load($wp_db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds work package by work package parent id.
	 * 
	 * @param int $workpackageParent_id work package parent ID.
	 * @return	WorkPackage or empty array if not found.
	 */
	public function findByParentID($workpackageParent_id) 
	{
		if (!is_numeric($workpackageParent_id) || (!($workpackageParent_id > 0)))
			throw new InvalidArgumentException('WorkPackage id must be a positive integer.');
		
		$wp_db_result = dibi::select('*')->from('WorkPackage')->where('[Parent_ID] = %i', $workpackageParent_id)->fetchAll();
		$wps = array();
		foreach ($wp_db_result as $row)
		{
			$wps[$row->WorkPackage_ID] = $this->load($row);
		}
		return $wps;
	}

	/**
	 * Finds work package by SAP id.
	 * 
	 * @param string $id work package SAP ID.
	 * @return	WorkPackage or NULL if not found.
	 */
	public function findBySAPID($workpackage_id) 
	{
		$wp_db_result = dibi::select('*')->from('WorkPackage')->where('[SAP_ID] = %s', $workpackage_id)->fetch();
		if ($wp_db_result !== FALSE)
			return $this->load($wp_db_result);
		else
			return NULL;
	}

	/**
	 * Returns array of all WorkPackages that are in Project WBS.
	 * 
	 * @param int $project_id Project ID.
	 * @return	array of WorkPackage 
	 */
	public function getProjectWorkPackages($project_id)
	{
		$wps_db_result = $this->conn->query('SELECT * 
				FROM [WorkPackage] 
				WHERE GetProjectIDFromWorkPackageID([WorkPackage_ID]) = %i
				ORDER BY [WorkPackage_ID] ASC', 
				$project_id)->fetchAll();
		$work_packages = array();
		foreach ($wps_db_result as $wp_db_result)
		{
			$work_package = $this->load($wp_db_result);
			$work_packages[$work_package->getID()] = $work_package;
		}
		return $work_packages;
	}
	
	/**
	 * Returns array of WorkPackages on the path
	 * in the WorkPackage tree from root to the task
	 * specified by $task_id.
	 * 
	 * @param int $task_id Task ID.
	 * @return	array of WorkPackage 
	 * @throws InvalidArgumentException if task ID is invalid.
	 * @throws InvalidStateException if work package tree is corrupted.
	 */
	public function getWPTreePathByTaskID($task_id)
	{
		$wp_id = dibi::select('WorkPackage_ID')->from('Task')->where('[Task_ID] = %i', $task_id)->fetchSingle();
		if (!$wp_id)
			throw new InvalidArgumentException("Task with ID ".$task_id." does not exist.");
		$wp_path = array();
		do 
		{
			$wp = $this->find($wp_id);
			if ($wp === NULL)
				throw InvalidStateException("Invalid WorkPackage ID " . $wp_id . " on WP path for task ID  ". $task_id);
			if ($wp->getSAPID() != '') // dummy WorkPackages are dropped
				$wp_path[] = $wp;
			$wp_id = $wp->getParentID();
		} while ($wp_id > 0);
		return array_reverse($wp_path);
	}
    
    /**
     * Gets project id
     * @param int $wp_id Should be workpackage id
     * @return int Return project id or NULL
     */
    public function getProjectID($wp_id)
    {
        $wp = $this->find($wp_id);
        
        while (($wp !== NULL) && (!$wp->ProjectID))
            $wp = $this->find($wp->ParentID);
        
        return $wp->ProjectID;
    }
	
    /**
     * Gets all years of work package.
     * 
     * @param int $workpackage_ID
     * @return array Array of available years
     */
    public function findWorkPackageYears($workpackage_ID)
    {
		$db_result = $this->conn->query('SELECT DISTINCT [TaskPlan].[Year] FROM [Task]
                                            JOIN [TaskPlan] ON [Task].[Task_ID] = [TaskPlan].[Task_ID]
                                                WHERE [Task].[WorkPackage_ID] = %u ORDER BY [TaskPlan].[Year] ASC', $workpackage_ID)->fetchAll();
        $years = array();        
		foreach ($db_result as $year)		
			array_push($years, $year->Year);       
        return $years;
    }
	
	/**
	 * Deletes existing WorkPackage.
	 * 
	 * @param int $workpackage_ID
	 * @return boolean
	 */
	public function delete($workpackage_ID)
	{
		try
		{
			$result = $this->conn->delete('WorkPackage')->where('[WorkPackage_ID] = %i', $workpackage_ID)->execute();
			if ($result)
				Debugger::log("WorkPackage with ID = " . $workpackage_ID . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of WorkPackage with ID = " . $workpackage_ID . " failed.", Debugger::ERROR);
			return $result;
		}
		catch (DibiException $e)
		{
			Debugger::log("Deleting of WorkPackage with ID = " . $workpackage_ID . " failed: " . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}
}