<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing workable hours from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class WorkableHourDibiMapper extends DibiMapper implements IWorkableHourMapper
{
    
	/**
	 * Creates WorkableHour instance from DibiRow.
	 * 
	 * @param $row DibiRow
	 * @return	WorkableHour 
	 */
    private function load(DibiRow $row)
    {
		$wh = new WorkableHour();
		$wh->ID = $row->EID;
		$wh->Year = $row->Year;
		$wh->Yield = $row->Yield;
		$wh->Contract = $row->Contract;
		$wh->IsAOP = $row->IsAOP;
		$wh->setMonthHours($row->Jan, 1);
		$wh->setMonthHours($row->Feb, 2);
		$wh->setMonthHours($row->Mar, 3);
		$wh->setMonthHours($row->Apr, 4);
		$wh->setMonthHours($row->May, 5);
		$wh->setMonthHours($row->Jun, 6);
		$wh->setMonthHours($row->Jul, 7);
		$wh->setMonthHours($row->Aug, 8);
		$wh->setMonthHours($row->Sep ,9);
		$wh->setMonthHours($row->Oct, 10);
		$wh->setMonthHours($row->Nov, 11);
		$wh->setMonthHours($row->Dec, 12);
		$wh->setRemark($row->Remark);
		return $wh;
    }
	
    /**
     * Search if workable hours of person for selected year exists.
     * 
     * @param int $year
     * @param string $person_id
     * @return Return true if search successfull, otherwise returns false. 
     */
    public function search($year, $person_id)
    {     
		$db_result = $this->conn->query('SELECT * FROM [Hour] WHERE [Year] = %u AND [EID] = %s', $year, $person_id)->fetch();
		if ($db_result !== FALSE)
			return true;
		else
			return false;	
    }
    
    /**
     * Search if workable hours of person for selected year is AOP.
     * 
     * @param int $year
     * @param string $person_id
     * @return Return true if search successfull, otherwise returns false. 
     */
    public function isAop($year, $person_id)
    {
		$db_result = $this->conn->query('SELECT [IsAOP] FROM [Hour] WHERE [Year] = %u AND [EID] = %s', $year, $person_id)->fetch();
		if ($db_result !== FALSE)
			return $db_result->IsAOP;
    }
	
    /**
     * Updates WorkableHour.
     * 
     * @param WorkableHour $workableHour
     * @return boolean Returns true if update successfull, otherwiste returns false.
     */
	public function update(WorkableHour $workableHour)
	{
        try
        {
            $array = array();
            $array['EID'] = $workableHour->getID();
            $array['Year'] = $workableHour->getYear();
            for($i = 1; $i<13 ; $i++)
            {
                $array[ FiscalCalendar::fiscalMonthIndexToName($i-1, false) ] = $workableHour->getMonthHours($i);
            }            
            if(is_numeric($workableHour->getYield()))
                $array['Yield'] = $workableHour->getYield()/100;
            else
                $array['Yield'] = null;
            if(is_numeric($workableHour->getContract()))
                $array['Contract'] = $workableHour->getContract()/100;
            else
                $array['Contract'] = null;
			$array['Remark'] = $workableHour->getRemark();
			$array['IsAOP'] = $workableHour->IsAOP;
            $this->conn->update('Hour', $array)->where('[EID] = %s', $array['EID'])->and('[Year] = %s', $array['Year'])->execute();
            return true;      
        }
        catch (DibiException $e)
        {
            Debugger::log("WorkableHour update with id = ".$workableHour->ID." failed. ".$e->getMessage());
            return false;
        }
	}
    
    /**
     * Creates WorkableHour.
     * 
     * @param WorkableHour $workableHour
     * @return boolean Returns true if create successfull, otherwiste returns false.
     */
    public function create(WorkableHour $workableHour)
    {
        try
        {          
            $array = array();
            $array['EID'] = $workableHour->getID();
            $array['Year'] = $workableHour->getYear();            
            for($i = 1; $i<13 ; $i++)
            {
                $array[ FiscalCalendar::fiscalMonthIndexToName($i-1, false) ] = $workableHour->getMonthHours($i);
            }
            if(is_numeric($workableHour->getYield()))
                $array['Yield'] = $workableHour->getYield()/100;
            else
                $array['Yield'] = null;
            if(is_numeric($workableHour->getContract()))
                $array['Contract'] = $workableHour->getContract()/100;
            else
                $array['Contract'] = null;
			$array['Remark'] = $workableHour->getRemark();
			$array['IsAOP'] = $workableHour->IsAOP;
            $hour_db_result = $this->conn->insert('Hour', $array)->execute(); 
            return true;       
        }
        catch (DibiException $e)
        {
            Debugger::log("WorkableHour insert with id = ".$workableHour->ID." failed. ".$e->getMessage());
            return false;
        }    
    }
    
    /**
     * Deletes WorkableHour.
     * 
     * @param int $year
     * @param string $person_id
     * @return boolean Return true if delete successfull, otherwise returns false.
     */
    public function delete ($year, $person_id)
    {
        try
        {
            $this->conn->delete('Hour')->where('[EID] = %s AND [Year] = %u', $person_id, $year)->execute();
            Debugger::log(print_r(dibi::$sql, true), Debugger::DEBUG);
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("WorkableHour delete with id = $person_id failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
    }
    
    /**
     * Get yield of workanle hours for user and for selected year.
     *
     * @param string $id
     * @param int $year
     * @return float
     */
    public function getYield ($id, $year)
    {
        $db_getYield_result = $this->conn->query('SELECT Yield FROM [Hour] WHERE [EID] = %s AND [Year] = %u', $id, $year)->fetch();
		if(is_null($db_getYield_result['Yield']))
			return 1;
		else
			return $db_getYield_result['Yield'];
    }
	
    /**
     * Get contract of workanle hours for user and for selected year.
     *
     * @param string $id
     * @param int $year
     * @return float
     */
    public function getContract ($id, $year)
    {
        $db_getContract_result = $this->conn->query('SELECT Contract FROM [Hour] WHERE [EID] = %s AND [Year] = %u', $id, $year)->fetch();
		if(is_null($db_getContract_result['Contract']))
			return 1;
		else
			return $db_getContract_result['Contract'];
    }
	
    /**
     * Get remark of workanle hours for user and for selected year.
     *
     * @param string $id
     * @param int $year
     * @return string
     */
    public function getRemark ($id, $year)
    {
        $db_getContract_result = $this->conn->query('SELECT Remark FROM [Hour] WHERE [EID] = %s AND [Year] = %u', $id, $year)->fetch();
		if(is_null($db_getContract_result['Remark']))
			return '';
		else
			return $db_getContract_result['Remark'];
    }
	
    /**
     * Gets WH for person and given year.
     * 
     * @param int $year
     * @param string $person_id
     * @return Return WorkableHour or NULL if not found
     */
    public function find($year, $person_id)
    {     
		$result = $this->conn->query('SELECT * FROM [Hour] WHERE [Year] = %u AND [EID] = %s', $year, $person_id)->fetch();
        if ($result)
            return $this->load($result);
        else
            return NULL;
    }
}