<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils
 */

/**
 * Class controls worker task assignment.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class AssignmentControl extends Object
{
	/** @const forbiden assignment messages */
	const FORBIDEN_ASSIGNMENT_LICENCE_NATIONALITY_MESSAGE = 'Resource is not authorized to work on this task (license or declaration signature missing)',
			FORBIDEN_ASSIGNMENT_LOCKED_PERSON_MESSAGE = 'Resource is Locked',
			FORBIDEN_ASSIGNMENT_LOCKED_TASK_MESSAGE = 'Task is locked',
			FORBIDEN_ASSIGNMENT_INACTIVE_PERSON_MESSAGE = 'Resource is no longer active',
			FORBIDEN_ASSIGNMENT_WRONG_WORK_CENTER_MESSAGE = 'Resource has wrong work center',
			FORBIDEN_ASSIGNMENT_PROJECT_STATE_MESSAGE = 'Project is not in Resource planning state',
			FORBIDEN_ASSIGNMENT_PERSON_INACTIVE_WORK_CENTER = 'Resource has inactive work center',
			FORBIDEN_ASSIGNMENT_TASK_INACTIVE_WORK_CENTER = 'Task has inactive work center',
			FORBIDEN_ASSIGNMENT_TASK_UNKNOWN_WORK_CENTER = 'Task has unknown work center',
			FORBIDEN_ASSIGNMENT_PLANNING_TASK = 'Planning task cannot be edited',
			FORBIDEN_ASSIGNMENT_FINAL_MESSAGE = 'For further information contact PMO.';
	
    /** @var DIContainer DI context with application repositories access */
    protected $context;
	/** @var Person Worker to assign */
	protected $person;
	/** @var Task Task to assign */
	private $task;
	/** @var Projct Task's project */
	protected $project;
	/** @var array Info messages why cannot be worker assigned */
	protected $tempMessageArray;
	/** @var string Info message why cannot be worker assigned */
	protected $message = 'Resource can be assigned.';
	/** @var boolean Assignmet flag */
	protected $assignable = true;
	/** @var boolean Person is locked, warning for PMO user*/
	protected $lockedPersonPMOWarning = false;
	
	/**
	 * Constructor immediately runs assignment control check.
	 * 
	 * @param Person $person
	 * @param Task $task
	 */
	public function __construct(Person $person, Task $task)
	{
		$this->context = Environment::getContext();
		$this->person = $person;
		$this->task = $task;
		$this->project = $this->context->projectRepository->findProjectByTaskId($this->task->ID);
		$this->runAssignmentControl();
		if(!$this->assignable)
			$this->message = implode(', ', $this->tempMessageArray) . '. ' . self::FORBIDEN_ASSIGNMENT_FINAL_MESSAGE;
	}
	/**
	 * Procedure runs all assignment controls.
	 */
	private function runAssignmentControl()
	{
		$this->licenceVsNationalityCheck();
		$this->lockedPersonCheck();
		$this->lockedTaskCheck();
		$this->personStatusCheck();
		$this->projectStateCheck();
		$this->wrongWorkCenterCheck();
		$this->inactivePersonWorkCenterCheck();
		$this->inactiveTaskWorkCenterCheck();
		$this->planningTaskCheck();
	}
	
	/**
	 * Procedure checks if resource is authorized to work on this task.
	 */
	protected function licenceVsNationalityCheck()
	{
		$assignable = false;
		if($this->context->ecUtility->hasAccess($this->person->NationalityID, $this->project->ECLicenceID)) // process check
			$assignable = true;
		if(!$assignable) // sets assignemnt flag and adds info message
		{
			$this->assignable = $assignable;
			$this->tempMessageArray[] = self::FORBIDEN_ASSIGNMENT_LICENCE_NATIONALITY_MESSAGE;
		}
	}
	
	/**
	 * Procedure checks if person is locked.
	 */
	protected function lockedPersonCheck()
	{
		$assignable = false;
		if(!$this->person->Locked) // process check
			$assignable = true;
		if($this->person->SupervisorID == strtoupper(Environment::getUser()->getId())) //Person is not locked for Resource Owner
		{
			$assignable = true;
		}
		if(Environment::getApplication()->getPresenter()->isAllowed(SecuredPresenter::RESOURCE_RESOURCE_LOCK, SecuredPresenter::ACTION_RUN)) //Person is not locked for PMO user, anyway there is needed a warning notice
		{
			$assignable = true;
			$this->lockedPersonPMOWarning = true;
		}
		if(!$assignable) // sets assignemnt flag and adds info message
		{
			$this->assignable = $assignable;
			$this->tempMessageArray[] = self::FORBIDEN_ASSIGNMENT_LOCKED_PERSON_MESSAGE;
		}
		
	}
	
	/**
	 * Procedure checks if task is locked
	 */
	private function lockedTaskCheck()
	{
		$assignable = false;
		if(!$this->task->Locked) // process check
			$assignable = true;
		if(!$assignable) // sets assignemnt flag and adds info message
		{
			$this->assignable = $assignable;
			$this->tempMessageArray[] = self::FORBIDEN_ASSIGNMENT_LOCKED_TASK_MESSAGE;
		}
	}
	
	/**
	 * Procedure checks if person is active
	 */
	protected function personStatusCheck()
	{
		$assignable = false;
		if($this->person->Status) // process check
			$assignable = true;
		if(!$assignable) // sets assignemnt flag and adds info message
		{
			$this->assignable = $assignable;
			$this->tempMessageArray[] = self::FORBIDEN_ASSIGNMENT_INACTIVE_PERSON_MESSAGE;
		}
	}
	
	/**
	 * Procedure checks if resource has correct workcenter to work on this task..
	 */
	private function wrongWorkCenterCheck()
	{
		$assignable = false;
		if($this->person->CostCenterID == $this->task->CostCenterID) // process check
			$assignable = true;
		/**
		 * ---TEMPORARY FIX 1---
		 * If project SAP ID = 'EF%' then task cost center does NOT have to correspond
		 * with person cost center when assigning new resources.
		 * -------------------
		 */
		if (preg_match('/^EF.*/', $this->project->SAP_ID))
			$assignable = true;
		if(!$assignable) // sets assignemnt flag and adds info message
		{
			$this->assignable = $assignable;
			$this->tempMessageArray[] = self::FORBIDEN_ASSIGNMENT_WRONG_WORK_CENTER_MESSAGE;
		}
	}
	
	/**
	 * Procedure checks if resource has active workcenter..
	 */
	private function inactivePersonWorkCenterCheck()
	{
		$assignable = false;
		$workCenter = $this->context->costCenterRepository->findCostCenter($this->person->CostCenterID);
		if($workCenter->Status) // process check
			$assignable = true;

		if(!$assignable) // sets assignemnt flag and adds info message
		{
			$this->assignable = $assignable;
			$this->tempMessageArray[] = self::FORBIDEN_ASSIGNMENT_PERSON_INACTIVE_WORK_CENTER;
		}
	}
	
	/**
	 * Procedure checks if task has active workcenter..
	 */
	private function inactiveTaskWorkCenterCheck()
	{
		$assignable = false;
		$workCenter = $this->context->costCenterRepository->findCostCenter($this->task->CostCenterID);
		
		// a work center, which we don't have in the DB yet
		if($workCenter === NULL)
		{
			$this->assignable = $assignable;
			$this->tempMessageArray[] = self::FORBIDEN_ASSIGNMENT_TASK_UNKNOWN_WORK_CENTER;
			return;
		}
		
		if($workCenter->Status) // process check
			$assignable = true;

		if(!$assignable) // sets assignment flag and adds info message
		{
			$this->assignable = $assignable;
			$this->tempMessageArray[] = self::FORBIDEN_ASSIGNMENT_TASK_INACTIVE_WORK_CENTER;
		}
	}
	
	/**
	 * Procedure checks if project is in resource planning state
	 */
	protected function projectStateCheck()
	{
		$assignable = false;
		if($this->project->State == 1) // process check
			$assignable = true;
		if(!$assignable) // sets assignment flag and adds info message
		{
			$this->assignable = $assignable;
			$this->tempMessageArray[] = self::FORBIDEN_ASSIGNMENT_PROJECT_STATE_MESSAGE;
		}
	}
	
	/**
	 * Procedure checks whatever task is planning or not
	 */
	private function planningTaskCheck()
	{
		$assignable = false;
		if($this->task->Fake != Task::TASK_TYPE_PLANNING) // process check
			$assignable = true;
		if(!$assignable) // sets editable flag and adds info message
		{
			$this->assignable = $assignable;
			$this->tempMessageArray[] = self::FORBIDEN_ASSIGNMENT_PLANNING_TASK;
		}
	}
	
	/**
	 * Gets info message.
	 * 
	 * @return string Info message
	 */
	public function getMessage()
	{
		return $this->message;
	}
	
	/**
	 * Gets assignment flag.
	 * 
	 * @return boolean Assignmet flag
	 */
	public function getAssignable()
	{
		return $this->assignable;
	}
	
	/**
	 * Gets lock warning for pmo user.
	 * 
	 * @return boolean Assignmet flag
	 */
	public function getLockedPersonPMOWarning()
	{
		return $this->lockedPersonPMOWarning;
	}
}
