<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * iASAP access class for Actual hours
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ActualsReaderTLC extends IASAPWrapper
{
	/**
	 * @param array $tasks Array of Tasks
	 * @param ILogger $logger
	 */
	public function __construct(ILogger $logger = NULL)
	{
		parent::__construct($logger);
	}
	
	/**
	 * Reads TLC actual hours from iASAP.
	 * Quantity of readed data is determined by start and end date.
	 * @param DateTime $start_date
	 * @param DateTime $end_date
	 * @return array Contains for each task SAP ID set of people.
	 *     And for each person it contains sum of actual hours in defined fiscal months.
	 *     $table[SAP_ID][Person_EID][Fiscal_month<0..11>] = Sum_of_actual_hours
	 *     For empty input set of tasks or in case of error returns empty array.
	 */
	public function readTimeAdminActuals(DateTime $start_date, DateTime $end_date)
	{
		$table = array();
		$start_date->setTime(0,0,0);
		$end_date->setTime(0,0,0);
		$db_result = $this->conn
			->query("
				SELECT [Emp Num] AS [EID], [Order], [Activity], UNIX_TIMESTAMP([Date]) AS [Date], [Hours]
				FROM [input_asap_yhr_timeadmin]
				WHERE %i <= UNIX_TIMESTAMP([Date]) AND UNIX_TIMESTAMP([Date]) <= %i
				ORDER BY [Emp Num] ASC
			", $start_date->format('U'), $end_date->format('U'))
			->fetchAll();
		
		if ($db_result)
		{
			foreach ($db_result as $row)
			{
				$eid = $this->repairEID($row->EID);
				$charge_date = new DateTime('@' . $row->Date);
				$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $charge_date->format('U'));
				if (isset($table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index]))
				{
					$table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index] += $row->Hours;
				}
				else
					$table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index] = $row->Hours;
			}
		}

		return $table;
	}
	
	/**
	 * Reads TLC actual hours from iASAP.
	 * Quantity of readed data is determined by start and end date.
	 * @param DateTime $start_date
	 * @param DateTime $end_date
	 * @return array Contains for each task SAP ID set of people.
	 *     And for each person it contains sum of actual hours in defined fiscal months.
	 *     $table[SAP_ID][Person_EID][Fiscal_month<0..11>] = Sum_of_actual_hours
	 *     For empty input set of tasks or in case of error returns empty array.
	 */
	public function readYPSDetailActuals(DateTime $start_date, DateTime $end_date)
	{
		$table = array();
		$start_date->setTime(0,0,0);
		$end_date->setTime(0,0,0);
		$db_result = $this->conn
			->query("
				SELECT [User ID] AS [EID], [Order], [Activity], UNIX_TIMESTAMP([PostDate]) AS [Date], [Qty] AS [Hours]
				FROM [input_asap_ypsdetail_csc_in]
				WHERE %i <= UNIX_TIMESTAMP([PostDate]) AND UNIX_TIMESTAMP([PostDate]) <= %i
				ORDER BY [User ID] ASC
			", $start_date->format('U'), $end_date->format('U'))
			->fetchAll();

		if ($db_result)
		{
			foreach ($db_result as $row)
			{
				$eid = $row->EID;
				$charge_date = new DateTime('@' . $row->Date);
				$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $charge_date->format('U'));
				if (isset($table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index]))
				{
					$table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index] += $row->Hours;
				}
				else
					$table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index] = $row->Hours;
			}
		}

		return $table;
	}
	
	/**
	 * Reads TLC actual hours from iASAP for given task.
	 * Quantity of readed data is determined by start and end date.
	 * @param DateTime $start_date
	 * @param DateTime $end_date
	 * @param Task $task
	 * @return array Contains for each task SAP ID set of people.
	 *     And for each person it contains sum of actual hours in defined fiscal months.
	 *     $table[SAP_ID][Person_EID][Fiscal_month<0..11>] = Sum_of_actual_hours
	 *     For empty input set of tasks or in case of error returns empty array.
	 */
	public function readTimeAdminTaskActuals(DateTime $start_date, DateTime $end_date, $task)
	{
		$table = array();
		$start_date->setTime(0,0,0);
		$end_date->setTime(0,0,0);
		$db_result = $this->conn
			->query("
				SELECT [Emp Num] AS [EID], [Order], [Activity], UNIX_TIMESTAMP([Date]) AS [Date], [Hours]
				FROM [input_asap_yhr_timeadmin]
				WHERE %i <= UNIX_TIMESTAMP([Date]) AND UNIX_TIMESTAMP([Date]) <= %i
				AND [Activity] = %s AND [Order] = %s
				ORDER BY [Emp Num] ASC
			", $start_date->format('U'), $end_date->format('U'), $task->definition . '', $task->network . '')
			->fetchAll();
		if ($db_result)
		{
			foreach ($db_result as $row)
			{
				$eid = $this->repairEID($row->EID);
				$charge_date = new DateTime('@' . $row->Date);
				$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $charge_date->format('U'));
				if (isset($table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index]))
				{
					$table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index] += $row->Hours;
				}
				else
					$table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index] = $row->Hours;
			}
		}

		return $table;
	}
	
	/**
	 * Reads TLC actual hours from iASAP.
	 * Quantity of readed data is determined by start and end date.
	 * @param DateTime $start_date
	 * @param DateTime $end_date
	 * @param Task $task
	 * @return array Contains for each task SAP ID set of people.
	 *     And for each person it contains sum of actual hours in defined fiscal months.
	 *     $table[SAP_ID][Person_EID][Fiscal_month<0..11>] = Sum_of_actual_hours
	 *     For empty input set of tasks or in case of error returns empty array.
	 */
	public function readYPSDetailTaskActuals(DateTime $start_date, DateTime $end_date, $task)
	{
		$table = array();
		$start_date->setTime(0,0,0);
		$end_date->setTime(0,0,0);
		$db_result = $this->conn
			->query("
				SELECT [User ID] AS [EID], [Order], [Activity], UNIX_TIMESTAMP([PostDate]) AS [Date], [Qty] AS [Hours]
				FROM [input_asap_ypsdetail_csc]
				WHERE %i <= UNIX_TIMESTAMP([PostDate]) AND UNIX_TIMESTAMP([PostDate]) <= %i
				AND [Activity] = %s AND [Order] = %s
				ORDER BY [User ID] ASC
			", $start_date->format('U'), $end_date->format('U'), $task->definition . '', $task->network . '')
			->fetchAll();

		if ($db_result)
		{
			foreach ($db_result as $row)
			{
				$eid = $row->EID;
				$charge_date = new DateTime('@' . $row->Date);
				$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $charge_date->format('U'));
				if (isset($table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index]))
				{
					$table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index] += $row->Hours;
				}
				else
					$table[$row->Order . ' ' . $row->Activity][$eid][$fiscal_month->Index] = $row->Hours;
			}
		}

		return $table;
	}
}
