<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils\Logging
 */

/**
 * Abstract logger suitable as predessor for loggers which
 * require grouping of logs into batches. Batch ID is then
 * added to actual logging target (file, db, ...).
 * 
 * It also provides statistics of logged messages and drops
 * messages with lower debug level then minimal
 * 
 * E.g. Import batch logs should be groupped and tagged by same identifier.
 * 
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
abstract class BatchLogger implements ILogger
{
	/** Array with statistics of logged messages */
	private $stats = NULL;
	/** Total of logged messages */
	private $statsTotal = 0;

	/** Batch identifier */
	private $batchId = NULL;

	/** Predefined message domain name */
	private $domain = NULL;

	/** Predefined minimum log level (messages with lower level should be are dropped) */
	private $minLogLevel = self::DEBUG;
	
	/**
	 * Creates new logger with batch ID set up.
	 * 
	 * @param string $batch_id Identifier of batch.
	 */    
	public function __construct($batch_id)
    {
		$this->batchId = $batch_id;
		$this->initStats();
	}

	/**
	 * Initializes statistics array.
	 */    
	private function initStats()
    {
		$this->stats = array();
		$this->stats[self::DEBUG] = 0;
		$this->stats[self::INFO] = 0;
		$this->stats[self::NOTICE] = 0;
		$this->stats[self::WARNING] = 0;
		$this->stats[self::ERROR] = 0;
		$this->stats[self::FATAL] = 0;
		$this->statsTotal = 0;
	}

	/**
	 * Returns TRUE if log level is valid.
	 * @return boolean
	 */    
	protected function validLevel($level)
    {
		return (isset($this->stats[$level]));
	}

	/**
	 * Returns whether given log level is lower than minimal log level.
	 * @return boolean
	 * @throws InvalidArgumentException when unknown log level is used.
	 */    
	private function lowerThanMinLogLevel($level_to_compare)
    {
		if (!$this->validLevel($level_to_compare))
			throw new InvalidArgumentException('Cannot compare log level: '.$level_to_compare.'. Only predefined constants can be used!');
		$min_found = false;
		// if you find $level sooner than min level => it is lower
		foreach ($this->stats as $log_level => $msg_no)
		{
			if ($log_level == $this->minLogLevel)
				$min_found = true;
			if (($level_to_compare == $log_level) && (!$min_found))
				return TRUE;
		}		
		return FALSE;
	}
	
	/**
	 * Gets statistics of logging including total under 'total' array.
	 * @return array
	 */    
	public function getStats()
    {
		$res = $this->stats;
		$res['total'] = $this->statsTotal;
		return $res;
	}

	/**
	 * Increases statistics of mesagges.
	 * @param string $level Log level.
	 * @throws InvalidArgumentException when unknown log level is used.
	 */    
	protected function incStats($level)
    {
		if (!$this->validLevel($level))
			throw new InvalidArgumentException('Invalid log level: '.$level.'. Stats not increased. Only predefined constants can be used!');
		$this->stats[$level]++;
		$this->statsTotal++;	
	}
	
	/**
	 * Returns batch identifier.
	 * @return string
	 */    
	public function getBatchID()
    {
		return $this->batchId;
	}

	/**
	 * Needs to be overrided to perform actual loggin of message.
	 * 
	 * Is called from log() method and if successful, statistics is increased.
	 * 
	 * @param string $message Message text.
	 * @param mixed $level message level.
	 * @param string $domain domain of logged message (can be used for grouping or filtering later).
	 * @param mixed $context variable logging context (useful for special log parameters).
	 * @return boolean Returns TRUE if message was written.
	 */
    abstract public function performLog($message, $level = NULL, $domain = NULL, $context = NULL);
	
	/**
	 * Logs message into log.
	 * 
	 * @param string $message Message text.
	 * @param mixed $level message level.
	 * @param string $domain domain of logged message (can be used for grouping or filtering later).
	 * @param mixed $context variable logging context (useful for special log parameters).
	 * @return boolean Returns TRUE if message was written.
	 * @throws InvalidArgumentException when unknown log level is used.
	 */
    public function log($message, $level = self::INFO, $domain = NULL, $context = NULL)
	{
		// test validity of log level
		if (!$this->validLevel($level))
			throw new InvalidArgumentException('Invalid log level: '.$level.'. Only predefined constants can be used!');
		if ($domain == NULL)
			$dmn = $this->domain;
		if (!$this->lowerThanMinLogLevel($level))
		{
			if ($this->performLog($message, $level, $dmn, $context))
			{
				$this->incStats($level);
				return true;
			}
			else
				return false;
		}
		return false;
	}

	/**
	 * Sets predefined message domain.
	 * @param string $domain
	 */    
	public function setDomain($domain)
    {
		$this->domain = $domain;
	}


	/**
	 * Gets predefined message domain.
	 * @return string
	 */    
	public function getDomain()
    {
		return $this->domain;
	}

	/**
	 * Sets minimum log level.
	 * @param string $level
	 * @throws InvalidArgumentException when unknown log level is used.
	 */    
	public function setMinLogLevel($level)
    {
		if (!$this->validLevel($level))
			throw new InvalidArgumentException('Invalid minimum log level: '.$level.'. Only predefined constants can be used!');
		else
			$this->minLogLevel = $level;
	}

	/**
	 * Gets minimum log level.
	 * @return string
	 */    
	public function getMinLogLevel()
    {
		return $this->minLogLevel;
	}
	
	
}

