<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils\Logging
 */

/**
 * Class multiple logging. If you need to pass one logger only
 * and log into two or more different outputs. 
 * 
 * You can add as many loggers you need in MultiLogger instance 
 * via add() method and pass it to service on class that needs/accepts
 * ILogger for logging
 * 
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class MultiLogger extends Object implements ILogger
{
	/** Array of loggers */
	private $loggers = array();
	
	/**
	 * Adds new logger to the list.
	 * Fluent interface - returns self.
	 * 
	 * Notice: method accepts one logger add many times as added (can
	 * lead to duplication of log messages).
	 * 
	 * @param ILogger $logger
	 * @return MultiLogger Returns self for fluent interface.
	 */
	public function add(ILogger $logger)
	{
		$this->loggers[] = $logger;
		return $this;
	}

	/**
	 * Logs message into log file (if opened).
	 * For $level only these values are processed:
	 * ILogger::[INFO|NOTICE|WARNING|ERROR|FATAL]
	 * 
	 * @param string $message Message text.
	 * @param mixed $level message level. Default INFO.
	 * @param string $domain domain of logged message (can be used for grouping or filtering later).
	 * @param mixed $context variable logging context (useful for special log parameters).
	 * @return boolean Returns TRUE if message was written.
	 */
    public function log($message, $level = self::INFO, $domain = NULL, $context = NULL)
	{
		for ($i = 0; $i < count($this->loggers); $i++)
			$this->loggers[$i]->log($message, $level, $domain, $context);
	}
	
	public function differences($newObject, $oldObject)
	{
		$newObjectArray = ConversionUtils::ObjectToArray($newObject);
		$oldObjectArray = ConversionUtils::ObjectToArray($oldObject);
		return join(', ', $this->differencesBetweenArrays($newObjectArray, $oldObjectArray));
	}
	
    /**
     * Compares two arrays indexed by the same indexes.
     * If two different values are found on the index,
     * difference is written to the output array on the same index.
	 * Example:'Name: "OriginName" -> "ChangedName"'
     * 
     * @param array $arr1
     * @param array $arr2
     * @return array $arrResult
     */
    public static function differencesBetweenArrays( array $arr1, array $arr2 )
    {
        $arrResult = array();
        foreach ($arr1 as $key => $val) 
		{
			if(!is_object($arr1[$key]) && !is_null($arr1[$key]))
				if(!is_array($arr1[$key]) && $arr1[$key])
					if (strcmp($arr1[$key], $arr2[$key]) != 0)
					{
						$arrResult[$key] = $key . ': "' . $arr1[$key] . '" -> "' . $arr2[$key] . '"';
					}
        }
        return $arrResult;
    }
}
