<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils
 */

/**
 * MonthArray is an object encapsulating array of 12 values.
 * It is always initialized 
 * 
 * It is a child class of TypedArray so it checks data type
 * of setted values.
 *
 * It enables array access for "for" and "foreach" statements.
 * Indexes accepted are numbers 1 to 12, English month name
 * shorts Jan to Dec and fullnames January to December.
 * 
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class MonthArray extends TypedArray
{
	/** Lowest index of month in array (do not change value!) */
	const LOWER_INDEX = 1;
	/** Highest index of month in array (do not change value!) */
	const UPPER_INDEX = 12;
	
	/**
	 * Creates new array object of twelve same type as $init_value.
	 * NULL values and resources are not accepted as init values.
	 * It initializes all twelve array values with $init_value.
	 * If it is object, it clones init values.
	 * 
	 * For more documentation see TypedArray::__construct API docs.
	 * 
	 * Example of creation of object:
	 * $array_of_int = new MonthArray((int) 0, false);
	 * $array_of_float = new MonthArray((float) 0, true);
	 * $array_of_typed_arrays = new MonthArray(new TypedArray('', false), true);
	 * 
	 * @param mixed $init_value Value for initialization of type of values.
	 * @param boolean $strict Whether testing of data types should be strict.
	 * @return TypedArray
	 */    
	public function __construct($init_value, $strict = false)
    {
		parent::__construct($init_value, $strict);
		if ($this->getDataType() == 'object')
			for ($i = self::LOWER_INDEX; $i <= self::UPPER_INDEX; $i++)
				$this->values[$i] = clone $init_value;
		else
			for ($i = self::LOWER_INDEX; $i <= self::UPPER_INDEX; $i++)
				$this->values[$i] = $init_value;
	}

	/**
	 * Returns sum of values in array.
	 * 
	 * Sum is done from offset value (default 0, all values in array)
	 * 
	 * For int and float (double) arrays returns sum.
	 * For string arrays it concatenates values a returns a string.
	 * For other data types returns null.
	 * 
	 * @param int $offset can be number from 1 to 12 (where to start)
	 * @param int $length how many values from offset to sum (default 0, means all from start)
	 * @return mixed
	 */
	public function sum($offset = self::LOWER_INDEX, $length = 0)
	{
		if (!NumericUtils::isNonNegativeInteger($offset) || ($offset > self::UPPER_INDEX))
			throw new InvalidArgumentException("Offset must be integer in range ".self::LOWER_INDEX." to ".self::UPPER_INDEX." or 0. Invalid value: " . $offset);
		if (!NumericUtils::isNonNegativeInteger($length))
			throw new InvalidArgumentException("Lenght must be integer greater or equeal to 0. Invalid value:" . $length);
		if (($this->getDataType() != 'integer') && ($this->getDataType() != 'double') && ($this->getDataType() != 'string'))
			return NULL;
		if (($offset <= self::LOWER_INDEX) && ($length == 0))
			$values = $this->values;
		elseif (($offset > self::LOWER_INDEX) && ($length == 0))
			$values = array_slice($this->values, $offset - 1, self::UPPER_INDEX, TRUE);
		else
			$values = array_slice($this->values, $offset - 1, $length, TRUE);
		if (($this->getDataType() == 'integer')
			|| ($this->getDataType() == 'double'))
			return array_sum($values);
		elseif ($this->getDataType() == 'string')
			return implode('', $values);
	}	

	/**
	 * Adds array to the current array as addition (plus) of
	 * values of given arrays from values in this array for every
	 * month.
	 * 
	 * Works only for int and float (double) arrays otherwise throws exception.
	 * 
	 * @return MonthArray Returns new array.
	 * @throws InvalidStateException If this array is not float or integer array.
	 * @throws InvalidArgumentException If given array is not float or integer array.
	 */
	public function plus(MonthArray $array)
	{
		if (($this->getDataType() != 'integer')
			&& ($this->getDataType() != 'double'))
			throw new InvalidStateException("This array is not float nor integer. Cannot subtract.");
		if (($array->getDataType() != 'integer')
			&& ($array->getDataType() != 'double'))
			throw new InvalidArgumentException ("Given array is not float nor integer. Cannot subtract.");
		$added = new MonthArray($this->values[self::LOWER_INDEX], false);
		for ($i = self::LOWER_INDEX; $i <= self::UPPER_INDEX; $i++)
			$added[$i] = $this->values[$i] + $array[$i];
		return $added;
	}
	
	/**
	 * Subtracts array from current array as substraction (minus) of
	 * values of given arrays from values in this array for every
	 * month.
	 * 
	 * Works only for int and float (double) arrays otherwise throws exception.
	 * 
	 * @return MonthArray Returns new array.
	 * @throws InvalidStateException If this array is not float or integer array.
	 * @throws InvalidArgumentException If given array is not float or integer array.
	 */
	public function minus(MonthArray $array)
	{
		if (($this->getDataType() != 'integer')
			&& ($this->getDataType() != 'double'))
			throw new InvalidStateException("This array is not float nor integer. Cannot subtract.");
		if (($array->getDataType() != 'integer')
			&& ($array->getDataType() != 'double'))
			throw new InvalidArgumentException ("Given array is not float nor integer. Cannot subtract.");
		$subtracted = new MonthArray($this->values[self::LOWER_INDEX], false);
		for ($i = self::LOWER_INDEX; $i <= self::UPPER_INDEX; $i++)
			$subtracted[$i] = $this->values[$i] - $array[$i];
		return $subtracted;
	}

	/**
	 * Divides array from current array as division (percentage) of
	 * values of given arrays from values in this array for every
	 * month.
	 * 
	 * Works only for int and float (double) arrays otherwise throws exception.
	 * 
	 * @return MonthArray Returns new array.
	 * @throws InvalidStateException If this array is not float or integer array.
	 * @throws InvalidArgumentException If given array is not float or integer array.
	 */
	public function percentage(MonthArray $array)
	{
		if (($this->getDataType() != 'integer')
			&& ($this->getDataType() != 'double'))
			throw new InvalidStateException("This array is not float nor integer. Cannot subtract.");
		if (($array->getDataType() != 'integer')
			&& ($array->getDataType() != 'double'))
			throw new InvalidArgumentException ("Given array is not float nor integer. Cannot subtract.");
		$subtracted = new MonthArray($this->values[self::LOWER_INDEX], false);
		for ($i = self::LOWER_INDEX; $i <= self::UPPER_INDEX; $i++)
        {
            if ($array[$i] != 0)
                $subtracted[$i] = (double) ($this->values[$i] / $array[$i]);
            elseif($array[$i] == 0 AND $this->values[$i] == 0)
                $subtracted[$i] = 1;
            else
                $subtracted[$i] = 0;
        }
		return $subtracted;
	}	
	
	/**
	 * Replaces or appends a item.
	 * 
	 * @param  mixed
	 * @param  mixed
	 * @return void
	 * @throws InvalidArgumentException if data type of $value is not acceptable
	 */
	public function offsetSet($index, $value)
	{
		if (!NumericUtils::isInteger($index) || ($index < self::LOWER_INDEX) || ($index > self::UPPER_INDEX))
			throw new OutOfRangeException("Offset must be integer in range ".self::LOWER_INDEX." to ".self::UPPER_INDEX);
		elseif ($this->testData($value))
			$this->values[$index] = $value;
		else
			throw new InvalidArgumentException("Data are of invalid data type. Expected " . $this->getDataTypeExt());
	}

	/**
	 * Returns a item on defined index (index must be 1 to 12).
	 * @param  int $index
	 * @return mixed
	 * @throws OutOfRangeException
	 * @throws InvalidArgumentException
	 */
	public function offsetGet($index)
	{
		if (!NumericUtils::isInteger($index) || ($index < self::LOWER_INDEX) || ($index > self::UPPER_INDEX))
			throw new OutOfRangeException("Offset must be integer in range ".self::LOWER_INDEX." to ".self::UPPER_INDEX);
		return $this->values[$index];
	}



	/**
	 * Determines whether a item exists.
	 * @param  mixed $index
	 * @return bool
	 */
	public function offsetExists($index)
	{
		if (!NumericUtils::isInteger($index) || ($index < self::LOWER_INDEX) || ($index > self::UPPER_INDEX))
			throw new OutOfRangeException("Offset must be integer in range ".self::LOWER_INDEX." to ".self::UPPER_INDEX);
		return isset($this->values[$index]);
	}



	/**
	 * Removes the element at the specified position in this list.
	 * @param  mixed $index
	 * @return void
	 * @throws OutOfRangeException
	 */
	public function offsetUnset($index)
	{
		if (!NumericUtils::isInteger($index) || ($index < self::LOWER_INDEX) || ($index > self::UPPER_INDEX))
			throw new OutOfRangeException("Offset must be integer in range ".self::LOWER_INDEX." to ".self::UPPER_INDEX);
		$this->values[$index] = NULL;
	}	
	
}


