<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils
 */

/**
 * Class controls task plan editing.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class TaskPlanEditing extends Object
{
	/** @const Setting  */
	const TASK_PLAN_EDITING_MESSAGE_YEAR = 'Editing is not enabled for past years',
			TASK_PLAN_EDITING_MESSAGE_PROJECT_STATE = "Project is not in Resource planning state",
			TASK_PLAN_EDITING_MESSAGE_TASK_LOCKED = 'Task is locked',
			TASK_PLAN_EDITING_MESSAGE_PLANNING_TASK = 'Planning task cannot be edited',
			TASK_PLAN_EDITING_MESSAGE_FINAL = 'For further information contact PMO.';
	
	/** @var DIContainer DI context with application repositories access */
    private $context;
	/** @var Task Task to assign */
	private $task;
	/** @var Projct Task's project */
	private $project;
	/** @var int Year */
	private $year;
	/** @var array Temp info messages why cannot be task plan edited */
	private $tempMessageArray;
	/** @var array Info messages why cannot be task plan edited */
	private $message = 'Task plan can be edited.';
	/** @var boolean Assignmet flag */
	private $editable = true;
	
	/**
	 * Constructor immediately runs edit control check.
	 * 
	 * @param Task $task
	 * @param int $year
	 */
	public function __construct(Task $task, $year)
	{
		$this->context = Environment::getContext();
		$this->year = $year;
		$this->task = $task;
		$this->project = $this->context->projectRepository->findProjectByTaskId($this->task->ID);
		$this->runEditingControl();
		if(!$this->editable)
			$this->message = implode(', ', $this->tempMessageArray) . '. ' . self::TASK_PLAN_EDITING_MESSAGE_FINAL;
	}
	
	/**
	 * Procedure runs all editing controls.
	 */
	private function runEditingControl()
	{
		$this->lockedTaskCheck();
		$this->projectStateCheck();
		$this->pastYearCheck();
		$this->planningTaskCheck();
	}
	
	/**
	 * Procedure checks if task is locked
	 */
	private function lockedTaskCheck()
	{
		$editable = false;
		if(!$this->task->Locked) // process check
			$editable = true;
		if(!$editable) // sets editable flag and adds info message
		{
			$this->editable = $editable;
			$this->tempMessageArray[] = self::TASK_PLAN_EDITING_MESSAGE_TASK_LOCKED;
		}
	}
	
	/**
	 * Procedure checks if project is in resource planning state
	 */
	private function projectStateCheck()
	{
		$editable = false;
		if($this->project->State == 1) // process check
			$editable = true;
		if(!$editable) // sets editable flag and adds info message
		{
			$this->editable = $editable;
			$this->tempMessageArray[] = self::TASK_PLAN_EDITING_MESSAGE_PROJECT_STATE;
		}
	}
	
	/**
	 * Procedure checks whatever task is planning or not
	 */
	private function planningTaskCheck()
	{
		$editable = false;
		if($this->task->Fake != Task::TASK_TYPE_PLANNING) // process check
			$editable = true;
		if(!$editable) // sets editable flag and adds info message
		{
			$this->editable = $editable;
			$this->tempMessageArray[] = self::TASK_PLAN_EDITING_MESSAGE_PLANNING_TASK;
		}
	}
	
	/**
	 * Procedure checks editing for actual and future year
	 */
	private function pastYearCheck()
	{
		$editable = false;
		if($this->year >= Date('Y')) // process check
			$editable = true;
		if(!$editable) // sets editable flag and adds info message
		{
			$this->editable = $editable;
			$this->tempMessageArray[] = self::TASK_PLAN_EDITING_MESSAGE_YEAR;
		}
	}
	
	/**
	 * Gets info message.
	 * 
	 * @return string Info message
	 */
	public function getMessage()
	{
		return $this->message;
	}
	
	/**
	 * Gets editable flag.
	 * 
	 * @return boolean Editable flag
	 */
	public function getEditable()
	{
		return $this->editable;
	}
}
